/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.acte;

import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.acte.NatureActe;
import fr.paris.lutece.plugins.ods.dto.acte.StatutActe;
import fr.paris.lutece.plugins.ods.dto.acte.StatutActeEnum;
import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.direction.Direction;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.utils.commons.DateUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.List;


/**
 *
 * AbstractActeDAO
 *
 * @param <GSeance> seance
 * @param <GFichier> fichier
 * @param <GActeFilter> filtre pour l'acte
 */
public abstract class AbstractActeDAO<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>, GActeFilter extends IActeFilter<GFichier, GSeance>>
    implements IActeDAO<GSeance, GFichier, GActeFilter>
{
    /*
    * Requetes SQL
    */
    private static final String SQL_QUERY_NEW_PK = "SELECT max( id_acte ) FROM ods_acte ";
    private static final String SQL_QUERY_INSERT = "INSERT INTO ods_acte ( " +
        " id_acte, reference, numero_delibere, id_seance, id_formation_conseil, id_entite, id_type_entite, id_nature, " +
        " id_statut, id_categorie_deliberation, id_direction, date_retour_ctrl_legalite, " +
        " id_deliberation_finale, poids, transmission_automatique, id_groupe )VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?) ";
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = "SELECT " +
        " oa.id_acte, oa.reference, oa.numero_delibere,id_seance, oa.id_formation_conseil, oa.id_entite, oa.id_type_entite, oa.id_nature, " +
        " oa.id_statut, oa.id_categorie_deliberation, oa.id_direction, oa.date_retour_ctrl_legalite, " +
        " oa.id_deliberation_finale, oa.poids, oa.transmission_automatique, oa.commentaire_annulation, oa.id_groupe, oa.en_ligne FROM ods_acte oa " +
        " WHERE oa.id_acte = ? ";
    private static final String SQL_QUERY_ACTE_LIST_BY_FILTER = "SELECT " +
        " oa.id_acte, oa.reference, oa.numero_delibere, oa.id_seance, oa.id_formation_conseil, oa.id_entite, oa.id_type_entite, oa.id_nature, " +
        " oa.id_statut, oa.id_categorie_deliberation, oa.id_direction, oa.date_retour_ctrl_legalite, " +
        " oa.id_deliberation_finale, oa.poids, oa.transmission_automatique, oa.commentaire_annulation , oa.id_groupe, oa.en_ligne " +
        " FROM ods_acte oa ";
    private static final String SQL_QUERY_ID_ACTE_LIST_BY_FILTER = " SELECT oa.id_acte FROM ods_acte oa ";
    private static final String SQL_QUERY_DELETE = "DELETE FROM ods_acte WHERE id_acte = ? ";
    private static final String SQL_QUERY_UPDATE = "UPDATE ods_acte SET id_acte = ?, " +
        " reference = ?, numero_delibere = ?, id_seance= ?, id_formation_conseil = ?, id_entite = ?, id_type_entite = ?, id_nature = ?, " +
        " id_statut = ?, id_categorie_deliberation = ?,  id_direction = ?, date_retour_ctrl_legalite = ?," +
        " id_deliberation_finale = ?, poids = ?, transmission_automatique = ?, id_groupe= ?, en_ligne = ? " +
        " WHERE id_acte = ? ";
    private static final String SQL_QUERY_NEW_NUMERO_DELIEBERE = "SELECT max( numero_delibere ) FROM ods_acte " +
        " WHERE reference = ? ";
    private static final String SQL_QUERY_INSERT_COM_ANNULATION = "UPDATE ods_acte SET commentaire_annulation = ? WHERE id_acte = ?";
    private static final String SQL_QUERY_PUBLIER = " UPDATE ods_acte SET en_ligne = ?, id_statut = ? WHERE id_acte = ?  ";
    private static final String SQL_QUERY_ACTE_LIST = "SELECT oa.id_acte, oa.id_formation_conseil, C.code_alpaca, oa.id_statut, S.libelle, oa.transmission_automatique, oa.reference FROM ods_acte oa, ods_acte_statut S, ods_formation_conseil C WHERE oa.id_statut=S.id_statut_acte and oa.id_formation_conseil=C.id_formation_conseil";
    private static final String SQL_FILTER_CATEGORIE = " AND oa.id_categorie_deliberation = ? ";
    private static final String SQL_FILTER_STATUT = " AND oa.id_statut = ? ";
    private static final String SQL_FILTER_STATUT_EXCEPT_ANNULE = " AND oa.id_statut NOT IN ( " +
        StatutActeEnum.ANNULE.getId(  ) + " )";
    private static final String SQL_FILTER_TYPE_ENTITE = " AND oa.id_type_entite = ? ";
    private static final String SQL_FILTER_ENTITE = " AND oa.id_entite = ? ";
    private static final String SQL_FILTER_REFERENCE = " AND oa.reference = ? ";
    private static final String SQL_FILTER_FORMATION_CONSEIL = " AND oa.id_formation_conseil = ? ";
    private static final String SQL_FILTER_DATE_RETOUR_CTRL_LEGALITE = " AND oa.date_retour_ctrl_legalite >= ? AND oa.date_retour_ctrl_legalite <= ?";
    private static final String SQL_FILTER_SEANCE = " AND oa.id_seance = ? ";
    private static final String SQL_FILTER_DELIB_FINALE = " AND oa.id_deliberation_finale = ? ";
    private static final String SQL_ORDER_BY_STATUS = " ORDER BY oa.id_statut ";
    private static final String SQL_ORDER_BY_FORMATION_CONSEIL = " ORDER BY oa.id_formation_conseil ";
    private static final String SQL_ORDER_BY_TRANSMISSION = " ORDER BY oa.transmission_automatique DESC ";
    private static final String SQL_LEFT_JOIN_PDD = " LEFT JOIN ods_pdd entite on (oa.id_entite = entite.id_pdd) ";
    private static final String SQL_LEFT_JOIN_VA = " LEFT JOIN ods_voeu_amendement entite on (oa.id_entite = entite.id_va) ";
    private static final String SQL_LEFT_JOIN_DESIGNATION = " LEFT JOIN ods_designation entite on (oa.id_entite = entite.id_designation) ";
    private static final String SQL_LEFT_JOIN_MEMBRE_PRESENT = " LEFT JOIN ods_liste_membres_presents entite on (oa.id_entite = entite.id_liste) ";
    private static final String SQL_WHERE_1 = " WHERE 1 = 1 ";

    /**
     * Retounre une nouvelle instance ISeance
     * @return une nouvelle instance ISeance
     */
    protected abstract ISeance newSeanceInstance(  );

    /**
     * Retourne une nouvelle instance d'un fichier
     * @return une nouvelle instance d'un fichier
     */
    protected abstract GFichier newFichierInstance(  );

    /**
     * Remplit les filtres spcifiques pour le Select
     * @param acteFilter le filtre
     * @param nIndex l'index de depart
     * @param daoUtil DAOUtil
     * @return l'index de sortie
     */
    protected abstract int setFiltersForSelectByFilterSpec( GActeFilter acteFilter, int nIndex, DAOUtil daoUtil );

    /**
     * Ajoute les paramtres spcifiques de la requte
     * @param acteFilter le filtre
     * @param sbSQL le buffer
     */
    protected abstract void setQueryParametersForSelectByFilterSpec( GActeFilter acteFilter, StringBuffer sbSQL );

    /**
     * Gnre un nouvel identifiant
     * @param plugin plugin
     * @return Retourne lidentifiant gnr
     */
    int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Gnre un nouvau numero de dlibr par rapport  la rfrnce passe en parametre
     * @param strReference la reference
     * @param plugin plugin
     * @return Retourne le nouvau numero gnr
     */
    public int newNumeroDelibere( String strReference, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_NUMERO_DELIEBERE, plugin );
        daoUtil.setString( 1, strReference );

        daoUtil.executeQuery(  );

        int nNumero;

        if ( !daoUtil.next(  ) )
        {
            nNumero = 1;
        }

        nNumero = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nNumero;
    }

    /**
     * Supprime l'acte correspondant  lobjet acte pass en paramtre
     * @param acte l'acte  supprimer
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    public void delete( Acte<GSeance, GFichier> acte, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, acte.getId(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Insre l'acte correspondant  lobjet acte pass en paramtre
     * @param acte l'acte  insrer
     * @param plugin Plugin
     */
    public void insert( Acte<GSeance, GFichier> acte, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT, plugin );
        int nKey = newPrimaryKey( plugin );
        acte.setId( nKey );

        setInsertUpdateQueryParameters( daoUtil, acte );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Cre un nouvel acte  partir des valeurs contenues dans le daoUtil
     * @param daoUtil DAOUtil
     * @return l'acte correspondant
     */
    private Acte<GSeance, GFichier> setActeValues( DAOUtil daoUtil )
    {
        Acte<GSeance, GFichier> acte = new Acte<GSeance, GFichier>(  );
        acte.setId( daoUtil.getInt( "oa.id_acte" ) );
        acte.setReference( daoUtil.getString( "oa.reference" ) );
        acte.setNumeroDelibere( daoUtil.getInt( "oa.numero_delibere" ) );

        ISeance seance = newSeanceInstance(  );
        seance.setIdSeance( daoUtil.getInt( "oa.id_seance" ) );
        acte.setSeance( seance );

        FormationConseil formationConseil = new FormationConseil(  );
        formationConseil.setIdFormationConseil( daoUtil.getInt( "oa.id_formation_conseil" ) );
        acte.setFormationConseil( formationConseil );

        acte.setIdEntite( daoUtil.getInt( "oa.id_entite" ) );
        acte.setIdTypeEntite( daoUtil.getInt( "oa.id_type_entite" ) );

        NatureActe natureActe = new NatureActe(  );
        natureActe.setIdNatureDocument( daoUtil.getInt( "oa.id_nature" ) );
        acte.setNatureDocument( natureActe );

        StatutActe statutActe = new StatutActe(  );
        statutActe.setIdStatut( daoUtil.getInt( "oa.id_statut" ) );
        acte.setStatutActe( statutActe );

        CategorieDeliberation categorie = new CategorieDeliberation(  );
        categorie.setIdCategorie( daoUtil.getInt( "oa.id_categorie_deliberation" ) );
        acte.setCategorie( categorie );

        Direction direction = new Direction(  );
        direction.setIdDirection( daoUtil.getInt( "oa.id_direction" ) );

        if ( direction.getIdDirection(  ) != 0 )
        {
            acte.setDirection( direction );
        }
        else
        {
            acte.setDirection( null );
        }

        acte.setDateRetourControleLegalite( daoUtil.getTimestamp( "oa.date_retour_ctrl_legalite" ) );

        GFichier deliberationFinale = newFichierInstance(  );

        if ( daoUtil.getObject( "oa.id_deliberation_finale" ) != null )
        {
            deliberationFinale.setId( daoUtil.getInt( "oa.id_deliberation_finale" ) );
            acte.setDeliberationFinale( deliberationFinale );
        }

        acte.setPoids( daoUtil.getInt( "oa.poids" ) );

        acte.setIdTransmission( daoUtil.getInt( "oa.transmission_automatique" ) );

        acte.setComAnnulation( daoUtil.getString( "oa.commentaire_annulation" ) );

        if ( daoUtil.getInt( "oa.id_groupe" ) != 0 )
        {
            acte.setIdGroupe( daoUtil.getInt( "oa.id_groupe" ) );
        }

        acte.setEnLigne( daoUtil.getBoolean( "oa.en_ligne" ) );

        return acte;
    }

    /**
     * Rcupre l'acte correspondant  l'id nKey
     * @param nKey l'id de l'acte  rcuprer
     * @param plugin Plugin
     * @return l'acte correspondant  l'id nKey
     */
    public Acte<GSeance, GFichier> load( int nKey, Plugin plugin )
    {
        Acte<GSeance, GFichier> acte = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            acte = setActeValues( daoUtil );
        }

        daoUtil.free(  );

        return acte;
    }

    /**
     * Ajoute les jointures correspondant aux filtres dans le buffer
     * @param acteFilter le filtre
     * @param sbSQL le buffer
     */
    private void setLeftJoinForSelectByFilter( GActeFilter acteFilter, StringBuffer sbSQL )
    {
        if ( acteFilter.containsTypeEntiteCriteria(  ) )
        {
            if ( acteFilter.getIdTypeEntite(  ) == ITypeEntite.PROJETS_DE_DELIBERATIONS )
            {
                sbSQL.append( SQL_LEFT_JOIN_PDD );
            }
            else if ( acteFilter.getIdTypeEntite(  ) == ITypeEntite.PROPOSITIONS_DE_DELIBERATIONS )
            {
                sbSQL.append( SQL_LEFT_JOIN_PDD );
            }
            else if ( acteFilter.getIdTypeEntite(  ) == ITypeEntite.VOEUX )
            {
                sbSQL.append( SQL_LEFT_JOIN_VA );
            }
            else if ( acteFilter.getIdTypeEntite(  ) == ITypeEntite.DESIGNATIONS_DE_REPRESENTANTS )
            {
                sbSQL.append( SQL_LEFT_JOIN_DESIGNATION );
            }
            else if ( acteFilter.getIdTypeEntite(  ) == ITypeEntite.LISTE_DES_MEMBRES_PRESENTS )
            {
                sbSQL.append( SQL_LEFT_JOIN_MEMBRE_PRESENT );
            }
        }

        sbSQL.append( SQL_WHERE_1 );
    }

    /**
     * Rcupre la liste des actes correspondants au filtre
     * @param acteFilter ActeFilter
     * @param plugin Plugin
     * @return la liste des statuts
     */
    public List<Acte<GSeance, GFichier>> selectByFilter( GActeFilter acteFilter, Plugin plugin )
    {
        List<Acte<GSeance, GFichier>> listActes = new ArrayList<Acte<GSeance, GFichier>>(  );
        StringBuffer sbSQL = new StringBuffer(  );
        sbSQL.append( SQL_QUERY_ACTE_LIST_BY_FILTER );

        setLeftJoinForSelectByFilter( acteFilter, sbSQL );

        sbSQL.append( ( ( acteFilter.containsCategorieDeliberationCriteria(  ) ) ? SQL_FILTER_CATEGORIE
                                                                                 : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        if ( acteFilter.containsStatutCriteria(  ) )
        {
            if ( acteFilter.getStatut(  ) == IActeFilter.ALL_EXCEPT_ANNULE )
            {
                sbSQL.append( SQL_FILTER_STATUT_EXCEPT_ANNULE );
            }
            else
            {
                sbSQL.append( SQL_FILTER_STATUT );
            }
        }

        setQueryParametersForSelectByFilter( acteFilter, sbSQL );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        int nIndex = 1;

        if ( acteFilter.containsCategorieDeliberationCriteria(  ) )
        {
            daoUtil.setInt( nIndex, acteFilter.getCategorieDeliberation(  ) );
            nIndex++;
        }

        if ( acteFilter.containsStatutCriteria(  ) && ( acteFilter.getStatut(  ) != IActeFilter.ALL_EXCEPT_ANNULE ) )
        {
            daoUtil.setInt( nIndex, acteFilter.getStatut(  ) );
            nIndex++;
        }

        nIndex = setFiltersForSelectByFilter( acteFilter, nIndex, daoUtil );

        if ( acteFilter.containsDeliberationFinaleCriteria(  ) )
        {
            daoUtil.setInt( nIndex, acteFilter.getFichierDeliberationFinale(  ).getId(  ) );
            nIndex++;
        }

        if ( acteFilter.containsDateRetourControleDeLegaliteCriteria(  ) )
        {
            //Date de retour du contrle de lgalit est toujours  00:00:00
            Timestamp tsDatRetourCtrlLegaliteBegin = acteFilter.getDateRetourControleDeLegalite(  );
            daoUtil.setTimestamp( nIndex, tsDatRetourCtrlLegaliteBegin );
            nIndex++;

            //Cration de la date de retour du contrle de lgalit  23:59:59
            String strDate = DateUtils.getDate( tsDatRetourCtrlLegaliteBegin, OdsConstants.CONSTANTE_PATTERN_DATE );
            Timestamp tsDatRetourCtrlLegaliteEnd = DateUtils.getDate( strDate, false );
            daoUtil.setTimestamp( nIndex, tsDatRetourCtrlLegaliteEnd );
            nIndex++;
        }

        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            Acte<GSeance, GFichier> acte = setActeValues( daoUtil );
            listActes.add( acte );
        }

        daoUtil.free(  );

        return listActes;
    }

    /**
     * Rcupre la liste des ids des actes correspondants au filtre
     * @param acteFilter ActeFilter
     * @param plugin Plugin
     * @return la liste des ids des actes
     */
    public List<Integer> selectIdsByFilter( GActeFilter acteFilter, Plugin plugin )
    {
        List<Integer> listIdsActes = new ArrayList<Integer>(  );
        StringBuffer sbSQL = new StringBuffer(  );
        sbSQL.append( SQL_QUERY_ID_ACTE_LIST_BY_FILTER );

        setLeftJoinForSelectByFilter( acteFilter, sbSQL );

        sbSQL.append( ( ( acteFilter.containsCategorieDeliberationCriteria(  ) ) ? SQL_FILTER_CATEGORIE
                                                                                 : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( acteFilter.containsStatutCriteria(  ) ) ? SQL_FILTER_STATUT : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        setQueryParametersForSelectByFilter( acteFilter, sbSQL );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        int nIndex = 1;

        if ( acteFilter.containsCategorieDeliberationCriteria(  ) )
        {
            daoUtil.setInt( nIndex, acteFilter.getCategorieDeliberation(  ) );
            nIndex++;
        }

        if ( acteFilter.containsStatutCriteria(  ) )
        {
            daoUtil.setInt( nIndex, acteFilter.getStatut(  ) );
            nIndex++;
        }

        nIndex = setFiltersForSelectByFilter( acteFilter, nIndex, daoUtil );

        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            listIdsActes.add( daoUtil.getInt( "oa.id_acte" ) );
        }

        daoUtil.free(  );

        return listIdsActes;
    }

    /**
     * Met  jour l'acte pass en paramtre dans la base de donnes
     * @param acte l'acte  mettre  jour
     * @param plugin Plugin
     */
    public void store( Acte<GSeance, GFichier> acte, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_UPDATE, plugin );

        setInsertUpdateQueryParameters( daoUtil, acte );

        daoUtil.setBoolean( 17, acte.getEnLigne(  ) );

        daoUtil.setInt( 18, acte.getId(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Dfinit les paramtres pour les requetes d'insert ou update
     * @param daoUtil DAOUtil
     * @param acte l'acte
     */
    private void setInsertUpdateQueryParameters( DAOUtil daoUtil, Acte<GSeance, GFichier> acte )
    {
        daoUtil.setInt( 1, acte.getId(  ) );
        daoUtil.setString( 2, acte.getReference(  ) );
        daoUtil.setInt( 3, acte.getNumeroDelibere(  ) );
        daoUtil.setInt( 4, acte.getSeance(  ).getIdSeance(  ) );
        daoUtil.setInt( 5, acte.getFormationConseil(  ).getIdFormationConseil(  ) );
        daoUtil.setInt( 6, acte.getIdEntite(  ) );
        daoUtil.setInt( 7, acte.getIdTypeEntite(  ) );
        daoUtil.setInt( 8, acte.getNatureDocument(  ).getIdNatureDocument(  ) );
        daoUtil.setInt( 9, acte.getStatutActe(  ).getIdStatut(  ) );
        daoUtil.setInt( 10, acte.getCategorie(  ).getIdCategorie(  ) );

        if ( acte.getDirection(  ) != null )
        {
            daoUtil.setInt( 11, acte.getDirection(  ).getIdDirection(  ) );
        }
        else
        {
            daoUtil.setIntNull( 11 );
        }

        if ( acte.getDateRetourControleLegalite(  ) != null )
        {
            daoUtil.setTimestamp( 12, acte.getDateRetourControleLegalite(  ) );
        }
        else
        {
            daoUtil.setTimestamp( 12, null );
        }

        if ( acte.getDeliberationFinale(  ) != null )
        {
            daoUtil.setInt( 13, acte.getDeliberationFinale(  ).getId(  ) );
        }
        else
        {
            daoUtil.setIntNull( 13 );
        }

        daoUtil.setInt( 14, acte.getPoids(  ) );
        daoUtil.setInt( 15, acte.getIdTransmission(  ) );

        if ( acte.getIdGroupe(  ) != 0 )
        {
            daoUtil.setInt( 16, acte.getIdGroupe(  ) );
        }
        else
        {
            daoUtil.setIntNull( 16 );
        }
    }

    /**
     * Rcupre la liste des actes
     * @param plugin Plugin
     * @return la liste des actes
     */
    public List<Acte<GSeance, GFichier>> selectActeList( Plugin plugin )
    {
        List<Acte<GSeance, GFichier>> listActe = new ArrayList<Acte<GSeance, GFichier>>(  );
        Acte<GSeance, GFichier> acte;

        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_ACTE_LIST, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            acte = new Acte<GSeance, GFichier>(  );
            acte.setId( daoUtil.getInt( "oa.id_acte" ) );

            FormationConseil f = new FormationConseil(  );
            f.setIdFormationConseil( daoUtil.getInt( "oa.id_formation_conseil" ) );
            f.setCode( daoUtil.getString( "C.code_alpaca" ) );
            acte.setFormationConseil( f );

            StatutActe sa = new StatutActe(  );
            sa.setIdStatut( daoUtil.getInt( "oa.id_statut" ) );
            sa.setLibelle( daoUtil.getString( "S.libelle" ) );
            acte.setStatutActe( sa );

            acte.setIdTransmission( daoUtil.getInt( "oa.transmission_automatique" ) );
            acte.setReference( daoUtil.getString( "oa.reference" ) );
            listActe.add( acte );
        }

        daoUtil.free(  );

        return listActe;
    }

    /**
     * Update l'acte pour prendre en compte le commentaire d'annulation
     * @param acte l'acte concern
     * @param plugin le plugin
     */
    public void insertCommentaireAnnulation( Acte<GSeance, GFichier> acte, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT_COM_ANNULATION, plugin );

        daoUtil.setString( 1, acte.getComAnnulation(  ) );
        daoUtil.setInt( 2, acte.getId(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Permet de publier ou de dpublier un acte
         * Si isPublie gale  TRUE, alors la mthode <b>publie</b> l'acte
         * Sinon la mthode change le statut enLigne de l'acte a <b>false</b>
     * @param nKey l'identifiant de l'acte
     * @param isPublie le statut de l'arrete
     * @param plugin le plugin
     */
    public void publication( int nKey, boolean isPublie, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_PUBLIER, plugin );

        daoUtil.setBoolean( 1, isPublie );
        daoUtil.setInt( 2, StatutActeEnum.PUBLIE.getId(  ) );
        daoUtil.setInt( 3, nKey );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Dfinis les filtres pour selectByFilter et SelectIdsForFilter
     * @param acteFilter le filtre
     * @param nIndex l'index de dpart
     * @param daoUtil DAOUtil
     * @return l'index de sortie
     *
     */
    private int setFiltersForSelectByFilter( GActeFilter acteFilter, int nIndex, DAOUtil daoUtil )
    {
        int nIndexCopy = nIndex;

        if ( acteFilter.containsTypeEntiteCriteria(  ) )
        {
            daoUtil.setInt( nIndexCopy, acteFilter.getIdTypeEntite(  ) );
            nIndexCopy++;
        }

        if ( acteFilter.containsIdEntiteCriteria(  ) )
        {
            daoUtil.setInt( nIndexCopy, acteFilter.getIdEntite(  ) );
            nIndexCopy++;
        }

        if ( acteFilter.containsReferenceCriteria(  ) )
        {
            daoUtil.setString( nIndexCopy, acteFilter.getReference(  ) );
            nIndexCopy++;
        }

        if ( acteFilter.containsSeanceCriteria(  ) )
        {
            daoUtil.setInt( nIndexCopy, acteFilter.getIdSeance(  ) );
            nIndexCopy++;
        }

        if ( acteFilter.containsFormationConseilCriteria(  ) )
        {
            daoUtil.setInt( nIndexCopy, acteFilter.getIdFormationConseil(  ) );
            nIndexCopy++;
        }

        nIndexCopy = setFiltersForSelectByFilterSpec( acteFilter, nIndexCopy, daoUtil );

        return nIndexCopy;
    }

    /**
     * Ajoute les paramtres de la requte
     * @param acteFilter le filtre
     * @param sbSQL le buffer
     */
    private void setQueryParametersForSelectByFilter( GActeFilter acteFilter, StringBuffer sbSQL )
    {
        sbSQL.append( ( ( acteFilter.containsTypeEntiteCriteria(  ) ) ? SQL_FILTER_TYPE_ENTITE
                                                                      : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( acteFilter.containsIdEntiteCriteria(  ) ) ? SQL_FILTER_ENTITE
                                                                    : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( acteFilter.containsReferenceCriteria(  ) ) ? SQL_FILTER_REFERENCE
                                                                     : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( acteFilter.containsSeanceCriteria(  ) ) ? SQL_FILTER_SEANCE : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( acteFilter.containsFormationConseilCriteria(  ) ) ? SQL_FILTER_FORMATION_CONSEIL
                                                                            : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( acteFilter.containsDeliberationFinaleCriteria(  ) ) ? SQL_FILTER_DELIB_FINALE
                                                                              : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( acteFilter.containsDateRetourControleDeLegaliteCriteria(  ) )
            ? SQL_FILTER_DATE_RETOUR_CTRL_LEGALITE : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        setQueryParametersForSelectByFilterSpec( acteFilter, sbSQL );

        sbSQL.append( ( ( acteFilter.getOrderByFormationConseil(  ) ) ? SQL_ORDER_BY_FORMATION_CONSEIL
                                                                      : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( acteFilter.getOrderByStatut(  ) ) ? SQL_ORDER_BY_STATUS : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( acteFilter.getOrderByTransmission(  ) ) ? SQL_ORDER_BY_TRANSMISSION
                                                                  : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
    }
}
