/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.acte;

import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.DocumentActe;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;

import java.util.ArrayList;
import java.util.List;


/**
 * DocumentActeDAO
 * @param <GSeance> seance
 * @param <GFichier> fichier
 */
public class DocumentActeDAO<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
    implements IDocumentActeDAO<GSeance, GFichier>
{
    /*
     * Requetes SQL
     */
    private static final String SQL_QUERY_INSERT = "INSERT INTO ods_acte_documents " + " ( id_acte, id_document )" +
        "VALUES (?,?) ";
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = "SELECT " +
        " id_acte, id_document FROM ods_acte_documents WHERE id_document = ? ";
    private static final String SQL_QUERY_FIND_BY_PAIR = "SELECT " +
        " id_acte, id_document FROM ods_acte_documents WHERE id_document = ? AND id_acte = ? ";
    private static final String SQL_QUERY_DOCUMENT_LIST = "SELECT id_acte, id_document FROM ods_acte_documents ";
    private static final String SQL_QUERY_DELETE = " DELETE FROM ods_acte_documents WHERE id_document = ? AND id_acte = ? ";
    private static final String SQL_QUERY_DELETE_BY_ID_ACTE = " DELETE FROM ods_acte_documents WHERE id_acte = ? ";
    private static final String SQL_QUERY_DELETE_BY_ID_DOCUMENT = " DELETE FROM ods_acte_documents WHERE id_document = ? ";
    private static final String SQL_QUERY_UPDATE = "UPDATE ods_acte_documents SET id_acte = ?" +
        " WHERE id_document = ? ";
    private static final String SQL_QUERY_FIND_BY_ACTE = "SELECT id_document FROM ods_acte_documents WHERE id_acte = ? ";
    private static final String SQL_QUERY_ORDER_BY_ACTE = " ORDER BY id_acte ASC ";

    /**
     * Supprime le document correspondant  lobjet DocumentActe pass en paramtre
     * @param document le document  supprimer
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    public void delete( DocumentActe document, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, document.getIdDocument(  ) );
        daoUtil.setInt( 2, document.getIdActe(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Supprime le document correspondant  lobjet Acte pass en paramtre
     * @param acte l'acte  supprimer
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    public void deleteByActe( Acte<GSeance, GFichier> acte, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE_BY_ID_ACTE, plugin );
        daoUtil.setInt( 1, acte.getId(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Supprime le document correspondant  l'idDocument pass en paramtre
     * @param nIdDocument l'id du document
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    public void deleteByDocument( int nIdDocument, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE_BY_ID_DOCUMENT, plugin );
        daoUtil.setInt( 1, nIdDocument );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Insre le document correspondant  lobjet document pass en paramtre
     * @param document le document  insrer
     * @param plugin Plugin
     */
    public void insert( DocumentActe document, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT, plugin );

        daoUtil.setInt( 1, document.getIdActe(  ) );
        daoUtil.setInt( 2, document.getIdDocument(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Rcupre le document correspondant  l'id nKey
     * @param nKey l'id du document  rcuprer
     * @param plugin Plugin
     * @return le document correspondant  l'id nKey
     */
    public DocumentActe load( int nKey, Plugin plugin )
    {
        DocumentActe document = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            document = new DocumentActe(  );
            document.setIdDocument( daoUtil.getInt( 2 ) );
            document.setIdActe( daoUtil.getInt( 1 ) );
        }

        daoUtil.free(  );

        return document;
    }

    /**
     * Rcupre la liste des documents
     * @param plugin Plugin
     * @return la liste des documents
     */
    public List<DocumentActe> selectDocumentActeList( Plugin plugin )
    {
        List<DocumentActe> listDocuments = new ArrayList<DocumentActe>(  );
        DocumentActe document;

        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DOCUMENT_LIST + SQL_QUERY_ORDER_BY_ACTE, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            document = new DocumentActe(  );
            document.setIdDocument( daoUtil.getInt( 1 ) );
            document.setIdActe( daoUtil.getInt( 2 ) );

            listDocuments.add( document );
        }

        daoUtil.free(  );

        return listDocuments;
    }

    /**
     * Met  jour le document pass en paramtre dans la base de donnes
     * @param document le document  mettre  jour
     * @param plugin Plugin
     */
    public void store( DocumentActe document, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_UPDATE, plugin );

        daoUtil.setInt( 1, document.getIdActe(  ) );
        daoUtil.setInt( 2, document.getIdDocument(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Rcupre la paire document/acte correspondant au document
     * @param document le document
     * @param acte l'acte
     * @param plugin Plugin
     * @return boolean
     */
    public boolean isPairExist( DocumentActe document, Acte<GSeance, GFichier> acte, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PAIR, plugin );
        boolean bPairExist = false;

        daoUtil.setInt( 1, document.getIdDocument(  ) );
        daoUtil.setInt( 2, acte.getId(  ) );
        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            bPairExist = true;
        }

        daoUtil.free(  );

        return bPairExist;
    }

    /**
     * Rcupre les documents correspondant  l'acte
     * @param acte l'acte
     * @param plugin le plugin
     * @return List<Fichier> la liste des documents
     */
    public List<Integer> selectDocumentByActe( Acte<GSeance, GFichier> acte, Plugin plugin )
    {
        List<Integer> listIdFichiers = new ArrayList<Integer>(  );

        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_ACTE, plugin );
        daoUtil.setInt( 1, acte.getId(  ) );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            listIdFichiers.add( daoUtil.getInt( 1 ) );
        }

        daoUtil.free(  );

        return listIdFichiers;
    }
}
