/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.arrete;

import fr.paris.lutece.plugins.ods.dto.arrete.Arrete;
import fr.paris.lutece.plugins.ods.dto.arrete.ArreteFilter;
import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.List;


/**
 * Cette classe permet d'accder/grer la table ods_arrete
 *
 * @param <GSeance> seance
 * @param <GFichier> fichier
 */
public abstract class AbstractArreteDAO<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
    implements IArreteDAO<GSeance, GFichier>
{
    private static final String SQL_QUERY_ORDER_BY_DATE_MODIFICATION = " ORDER BY ar.date_publication DESC ";
    private static final String SQL_QUERY_SELECT_ARRETE = "SELECT ar.id_arrete,ar.id_seance, ar.id_formation_conseil, ar.numero, " +
        " ar.intitule, ar.id_fichier_or, ar.id_fichier_sc, ar.id_categorie, ar.date_publication, ar.date_retour_ctrl_legalite, " +
        "ar.en_ligne, ar.version FROM ods_arrete ar ";
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = SQL_QUERY_SELECT_ARRETE + "WHERE ar.id_arrete = ? ";
    private static final String SQL_QUERY_ARRETES_LIST = SQL_QUERY_SELECT_ARRETE + "WHERE 1 = 1 ";
    private static final String SQL_QUERY_FIND_ID_BY_FICHIER = " SELECT id_arrete, numero, date_publication, version, en_ligne" +
        " FROM ods_arrete WHERE id_fichier_sc = ? ";
    private static final String SQL_FILTER_SEANCE = " AND ar.id_seance = ? ";
    private static final String SQL_FILTER_FORMATION_CONSEIL = " AND ar.id_formation_conseil = ? ";
    private static final String SQL_QUERY_NEW_PK = " SELECT max( id_arrete ) FROM ods_arrete ";
    private static final String SQL_QUERY_INSERT = " INSERT INTO ods_arrete (id_arrete, id_seance, id_formation_conseil, numero," +
        "intitule, id_fichier_or, id_fichier_sc, id_categorie, date_retour_ctrl_legalite,date_publication, en_ligne, version) " +
        "VALUES ( ?,?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? ) ";
    private static final String SQL_QUERY_PUBLIER = " UPDATE ods_arrete SET date_publication = ?, en_ligne = ?, version = ? WHERE id_arrete = ?  ";
    private static final String SQL_QUERY_UPDATE = " UPDATE ods_arrete SET id_seance = ?, id_formation_conseil = ?, numero = ?, " +
        "intitule = ?, id_fichier_or = ?, id_fichier_sc = ?, id_categorie = ? , date_publication = ?, date_retour_ctrl_legalite = ?,en_ligne=?, version=? " +
        "WHERE id_arrete= ?  ";
    private static final String SQL_QUERY_DELETE = " DELETE FROM ods_arrete WHERE id_arrete = ?  ";

    /**
     * Cre une nouvelle instance de fichier
     * @return une nouvelle instance de fichier
     */
    protected abstract GFichier newFichierInstance(  );

    /**
     * Cre une nouvelle instance de sance
     * @return une nouvelle instance de sance
     */
    protected abstract GSeance newSeanceInstance(  );

    /**
    * Cre un nouvel arret  partir de lobjet arrete pass en paramtre
    * @param arrete l'arrete  insrer
    * @param plugin le plugin
    * @return int primaryKey
    */
    public int insert( Arrete<GSeance, GFichier> arrete, Plugin plugin )
    {
        int newPrimaryKey = newPrimaryKey( plugin );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT, plugin );
        arrete.setId( newPrimaryKey );
        daoUtil.setInt( 1, arrete.getId(  ) );

        if ( arrete.getSeance(  ) != null )
        {
            daoUtil.setInt( 2, arrete.getSeance(  ).getIdSeance(  ) );
        }
        else
        {
            daoUtil.setIntNull( 2 );
        }

        if ( arrete.getFormationConseil(  ) == null )
        {
            daoUtil.setIntNull( 3 );
        }
        else
        {
            daoUtil.setInt( 3, arrete.getFormationConseil(  ).getIdFormationConseil(  ) );
        }

        daoUtil.setString( 4, arrete.getNumero(  ) );
        daoUtil.setString( 5, arrete.getIntitule(  ) );

        if ( arrete.getFichierOr(  ) == null )
        {
            daoUtil.setIntNull( 6 );
        }
        else
        {
            daoUtil.setInt( 6, arrete.getFichierOr(  ).getId(  ) );
        }

        if ( arrete.getFichierSc(  ) == null )
        {
            daoUtil.setIntNull( 7 );
        }
        else
        {
            daoUtil.setInt( 7, arrete.getFichierSc(  ).getId(  ) );
        }

        if ( arrete.getCategorie(  ) != null )
        {
            daoUtil.setInt( 8, arrete.getCategorie(  ).getIdCategorie(  ) );
        }
        else
        {
            daoUtil.setIntNull( 8 );
        }

        daoUtil.setTimestamp( 9, arrete.getDateRetourControleLegalite(  ) );
        daoUtil.setTimestamp( 10, new Timestamp( System.currentTimeMillis(  ) ) );
        daoUtil.setBoolean( 11, arrete.getEnLigne(  ) );
        daoUtil.setInt( 12, arrete.getVersion(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );

        return newPrimaryKey;
    }

    /**
     * Modifie l'arrete correspondante  lobjet arrete pass en paramtre
     * @param arrete l'arrete  modifier
     * @param plugin le plugin
     */
    public void store( Arrete<GSeance, GFichier> arrete, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_UPDATE, plugin );

        if ( arrete.getSeance(  ) == null )
        {
            daoUtil.setIntNull( 1 );
        }
        else
        {
            daoUtil.setInt( 1, arrete.getSeance(  ).getIdSeance(  ) );
        }

        if ( arrete.getFormationConseil(  ) == null )
        {
            daoUtil.setIntNull( 2 );
        }
        else
        {
            daoUtil.setInt( 2, arrete.getFormationConseil(  ).getIdFormationConseil(  ) );
        }

        if ( arrete.getNumero(  ) == null )
        {
            daoUtil.setIntNull( 3 );
        }
        else
        {
            daoUtil.setString( 3, arrete.getNumero(  ) );
        }

        if ( arrete.getIntitule(  ) == null )
        {
            daoUtil.setIntNull( 4 );
        }
        else
        {
            daoUtil.setString( 4, arrete.getIntitule(  ) );
        }

        if ( arrete.getFichierOr(  ) == null )
        {
            daoUtil.setIntNull( 5 );
        }
        else
        {
            daoUtil.setInt( 5, arrete.getFichierOr(  ).getId(  ) );
        }

        if ( arrete.getFichierSc(  ) == null )
        {
            daoUtil.setIntNull( 6 );
        }
        else
        {
            daoUtil.setInt( 6, arrete.getFichierSc(  ).getId(  ) );
        }

        if ( arrete.getCategorie(  ) != null )
        {
            daoUtil.setInt( 7, arrete.getCategorie(  ).getIdCategorie(  ) );
        }
        else
        {
            daoUtil.setIntNull( 7 );
        }

        daoUtil.setTimestamp( 8, new Timestamp( System.currentTimeMillis(  ) ) );
        daoUtil.setTimestamp( 9, arrete.getDateRetourControleLegalite(  ) );

        daoUtil.setBoolean( 10, arrete.getEnLigne(  ) );
        daoUtil.setInt( 11, arrete.getVersion(  ) );

        daoUtil.setInt( 12, arrete.getId(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Supprime l'arrete correspondant  lobjet arrete deliberation en paramtre
     * @param arrete l'arrete  supprimer
     * @param plugin le plugin
     * @throws AppException si contrainte de cl trangre
     */
    public void delete( Arrete<GSeance, GFichier> arrete, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, arrete.getId(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Retourne la liste des arretes correspondants aux critres du filtre.
     * @param filter les critres de recherche de la dlibration
     * @param plugin le plugin
     * @return la liste de arretes correspondants aux critres de recherche
     */
    public List<Arrete<GSeance, GFichier>> selectByFilter( ArreteFilter filter, Plugin plugin )
    {
        List<Arrete<GSeance, GFichier>> arretes = new ArrayList<Arrete<GSeance, GFichier>>(  );

        StringBuffer sbSQL = new StringBuffer(  );
        sbSQL.append( SQL_QUERY_ARRETES_LIST );
        sbSQL.append( ( ( filter.containsSeanceCriteria(  ) ) ? SQL_FILTER_SEANCE : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsFormationConseilCriteria(  ) ) ? SQL_FILTER_FORMATION_CONSEIL
                                                                        : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        sbSQL.append( SQL_QUERY_ORDER_BY_DATE_MODIFICATION );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        int nIndex = 1;

        if ( filter.containsSeanceCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdSeance(  ) );
            nIndex++;
        }

        if ( filter.containsFormationConseilCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdFormationConseil(  ) );
            nIndex++;
        }

        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            Arrete<GSeance, GFichier> deliberation = getBasePropertyArrete( daoUtil );
            arretes.add( deliberation );
        }

        daoUtil.free(  );

        return arretes;
    }

    /**
     * renvoie l'arrete ayant comme identifiant de fichier nKey
     * @param nKey idenfiant du fichier
     * @param plugin le Plugin actif
     * @return l'arrete
     */
    public Arrete<GSeance, GFichier> selectByFichier( int nKey, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ID_BY_FICHIER, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        Arrete<GSeance, GFichier> arrete = new Arrete<GSeance, GFichier>(  );

        if ( daoUtil.next(  ) )
        {
            arrete.setId( daoUtil.getInt( "id_arrete" ) );
            arrete.setNumero( daoUtil.getString( "numero" ) );
            arrete.setDatePublication( daoUtil.getTimestamp( "date_publication" ) );
            arrete.setVersion( daoUtil.getInt( "version" ) );
            arrete.setEnLigne( daoUtil.getBoolean( "en_ligne" ) );
        }

        daoUtil.free(  );

        return arrete;
    }

    /**
     * Retourne l'arrete ayant comme identifiant nKey
     * @param nKey identifiant de l'arrete  charger
     * @param plugin le plugin
     * @return la dlibration identifie par nKey
     */
    public Arrete<GSeance, GFichier> load( int nKey, Plugin plugin )
    {
        Arrete<GSeance, GFichier> arrete = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            //arrete = new Arrete(  );
            arrete = getBasePropertyArrete( daoUtil );
        }

        daoUtil.free(  );

        return arrete;
    }

    /**
     * Permet de publier ou de dpublier un arrete
         * Si isPublie gale  TRUE, alors la mthode <b>publie</b> l'arrete et incrmente la version<BR>
         * Sinon la mthode change le statut enLigne de l'arrete a <b>false</b>
     * @param nKey l'identifiant de l'arrete
     * @param tsDatePublication la date de publication
     * @param version la version de l'arrete
     * @param isPublie le statut de l'arrete
     * @param plugin le plugin
     */
    public void publication( int nKey, Timestamp tsDatePublication, int version, boolean isPublie, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_PUBLIER, plugin );
        daoUtil.setTimestamp( 1, tsDatePublication );
        daoUtil.setBoolean( 2, isPublie );

        if ( isPublie )
        {
            daoUtil.setInt( 3, version + 1 );
        }
        else
        {
            daoUtil.setInt( 3, version );
        }

        daoUtil.setInt( 4, nKey );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Permet de creer un objet arrete avec ses proprits de base
     * @param daoUtil contient les informations permettant de construire l'objet
     * @return l'arrete avec les informations de la requte
     */
    private Arrete<GSeance, GFichier> getBasePropertyArrete( DAOUtil daoUtil )
    {
        Arrete<GSeance, GFichier> arrete = new Arrete<GSeance, GFichier>(  );
        arrete.setId( daoUtil.getInt( "ar.id_arrete" ) );

        if ( ( daoUtil.getObject( "ar.version" ) != null ) || ( daoUtil.getInt( "ar.version" ) > 0 ) )
        {
            arrete.setVersion( daoUtil.getInt( "ar.version" ) );
        }
        else
        {
            arrete.setVersion( 0 );
        }

        arrete.setEnLigne( daoUtil.getBoolean( "ar.en_ligne" ) );
        arrete.setDatePublication( daoUtil.getTimestamp( "ar.date_publication" ) );
        arrete.setDateRetourControleLegalite( daoUtil.getTimestamp( "ar.date_retour_ctrl_legalite" ) );
        arrete.setNumero( daoUtil.getString( "ar.numero" ) );
        arrete.setIntitule( daoUtil.getString( "ar.intitule" ) );

        if ( daoUtil.getObject( "ar.id_seance" ) != null )
        {
            GSeance seance = newSeanceInstance(  );
            seance.setIdSeance( daoUtil.getInt( "ar.id_seance" ) );
            arrete.setSeance( seance );
        }

        if ( daoUtil.getObject( "ar.id_categorie" ) != null )
        {
            CategorieDeliberation categorie = new CategorieDeliberation(  );
            categorie.setIdCategorie( daoUtil.getInt( "ar.id_categorie" ) );
            arrete.setCategorie( categorie );
        }

        if ( daoUtil.getObject( "ar.id_formation_conseil" ) != null )
        {
            FormationConseil formationConseil = new FormationConseil(  );
            formationConseil.setIdFormationConseil( daoUtil.getInt( "ar.id_formation_conseil" ) );
            arrete.setFormationConseil( formationConseil );
        }

        if ( daoUtil.getObject( "ar.id_fichier_or" ) != null )
        {
            GFichier fichier = newFichierInstance(  );
            fichier.setId( daoUtil.getInt( "ar.id_fichier_or" ) );
            arrete.setFichierOr( fichier );
        }

        if ( daoUtil.getObject( "ar.id_fichier_sc" ) != null )
        {
            GFichier fichier = newFichierInstance(  );
            fichier.setId( daoUtil.getInt( "ar.id_fichier_sc" ) );
            arrete.setFichierSc( fichier );
        }

        return arrete;
    }

    /**
     * Gnre un nouvel identifiant
     * @param plugin le plugin
     * @return Retourne lidentifiant gnr
     */
    int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey = 1;

        if ( daoUtil.next(  ) )
        {
            nKey = daoUtil.getInt( 1 ) + 1;
        }

        daoUtil.free(  );

        return nKey;
    }
}
