/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.arrete;

import fr.paris.lutece.plugins.ods.business.IBusinessHome;
import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.business.formationconseil.FormationConseilHome;
import fr.paris.lutece.plugins.ods.dto.arrete.Arrete;
import fr.paris.lutece.plugins.ods.dto.arrete.ArreteFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.portal.service.plugin.Plugin;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.Timestamp;

import java.util.List;


/**
 * ArreteHome
 *
 * @param <GSeance> seance
 * @param <GFichier> fichier
 */
public final class ArreteHome<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
    implements IBusinessHome
{
    @Autowired
    private IArreteDAO<GSeance, GFichier> _arreteDAO;
    @Autowired
    private IFichierHome<GFichier, IFichierFilter, ITypeDocument, GSeance> _fichierHome;
    @Autowired
    private FormationConseilHome _formationConseilHome;

    /**
     * Constructeur vide
     */
    private ArreteHome(  )
    {
    }

    /**
    * Cre un nouvel arrete  partir de lobjet arrete pass en paramtre
    * @param arrete l'arrete  insrer
    * @param plugin le Plugin actif
    * @return int primaryKey
    */
    public int create( Arrete<GSeance, GFichier> arrete, Plugin plugin )
    {
        arrete.setId( _arreteDAO.insert( arrete, plugin ) );

        return arrete.getId(  );
    }

    /**
     * Modifie l'arrete correspondant  lobjet arrete pass en paramtre
     * @param arrete l'arrete  modifier
     * @param plugin le Plugin actif
     */
    public void update( Arrete<GSeance, GFichier> arrete, Plugin plugin )
    {
        _arreteDAO.store( arrete, plugin );
    }

    /**
     * Supprime l'arrete correspondant  lobjet arrete pass en paramtre
     * @param arrete l'arrete  supprimer
     * @param plugin le Plugin actif
     */
    public void remove( Arrete<GSeance, GFichier> arrete, Plugin plugin )
    {
        _arreteDAO.delete( arrete, plugin );
    }

    /**
     * renvoie l'arrete ayant comme identifiant nKey
     * @param nKey idenfiant de l'arrete  supprimer
     * @param plugin le Plugin actif
     * @return un arrete
     */
    public Arrete<GSeance, GFichier> findByPrimaryKey( int nKey, Plugin plugin )
    {
        Arrete<GSeance, GFichier> arrete = _arreteDAO.load( nKey, plugin );

        if ( arrete != null )
        {
            setOtherPropertyArrete( arrete, plugin );
        }

        return arrete;
    }

    /**
     * rcupre la liste des arretes correspondant au filtre donn
     * @param filter le filtre
     * @param plugin le Plugin actif
     * @return uen liste des arretes
     */
    public List<Arrete<GSeance, GFichier>> findByFilter( ArreteFilter filter, Plugin plugin )
    {
        List<Arrete<GSeance, GFichier>> listArretes = _arreteDAO.selectByFilter( filter, plugin );

        for ( Arrete<GSeance, GFichier> arrete : listArretes )
        {
            if ( ( arrete != null ) && ( arrete.getFormationConseil(  ) != null ) )
            {
                FormationConseil formationConseil = _formationConseilHome.findByPrimaryKey( arrete.getFormationConseil(  )
                                                                                                  .getIdFormationConseil(  ),
                        plugin );
                arrete.setFormationConseil( formationConseil );
            }
        }

        return listArretes;
    }

    /**
     * renvoie l'arrete ayant comme identifiant de fichier nKey
     * @param nKey idenfiant du fichier
     * @param plugin le Plugin actif
     * @return l'arrete
     */
    public Arrete<GSeance, GFichier> findByFichier( int nKey, Plugin plugin )
    {
        return _arreteDAO.selectByFichier( nKey, plugin );
    }

    /**
     * Permet de publier ou de dpublier un arrete
     * Publication d'un arrete
     * Si isPublier gale  TRUE, alors la mthode <b>publie</b> l'arrete et incrmente la version<BR>
     * Sinon la mthode change le statut enLigne d'un arrete a FALSE
     * @param nKey id de l'arrete
     * @param datePublication TimeStamp date de publication
     * @param version version
     * @param isPublier enLigne
     * @param plugin le Plugin actif
     */
    public void publication( int nKey, Timestamp datePublication, int version, boolean isPublier, Plugin plugin )
    {
        _arreteDAO.publication( nKey, datePublication, version, isPublier, plugin );
    }

    /**
     * Permet de complter l'arrete avec les proprits qui ont des contraintes sur d'autres tables
     * @param arrete arrete  complter
     * @param plugin Plugin
     */
    private void setOtherPropertyArrete( Arrete<GSeance, GFichier> arrete, Plugin plugin )
    {
        if ( arrete.getFichierOr(  ) != null )
        {
            GFichier fichier = _fichierHome.findByPrimaryKey( arrete.getFichierOr(  ).getId(  ), plugin );
            arrete.setFichierOr( fichier );
        }

        if ( arrete.getFichierSc(  ) != null )
        {
            GFichier fichier = _fichierHome.findByPrimaryKey( arrete.getFichierSc(  ).getId(  ), plugin );
            arrete.setFichierSc( fichier );
        }
    }
}
