/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.autremodele;

import fr.paris.lutece.plugins.ods.dto.autremodele.AutreModeleFilter;
import fr.paris.lutece.plugins.ods.dto.autremodele.IAutreModele;
import fr.paris.lutece.plugins.ods.dto.autremodele.TypeAutreModele;
import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;
import fr.paris.lutece.util.sql.Transaction;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;

import java.util.ArrayList;
import java.util.List;


/**
 *
 * AbstractAutreModeleDAOr
 *
 * @param <GAutreModele> autre modele
 */
public abstract class AbstractAutreModeleDAO<GAutreModele extends IAutreModele> implements IAutreModeleDAO<GAutreModele>
{
    private static final String SQL_QUERY_SELECT = "SELECT am.id_autre_modele, am.titre, am.id_type_autre_modele, am.id_fichier_entete," +
        " am.id_fichier_pied, tam.libelle_type ";
    private static final String SQL_QUERY_FROM = " FROM ods_autre_modele am JOIN ods_type_autre_modele tam ON am.id_type_autre_modele = tam.id_type_autre_modele ";
    private static final String SQL_QUERY_WHERE = " WHERE 1 ";
    private static final String SQL_QUERY_WHERE_PK = " WHERE  am.id_autre_modele = ? ";
    private static final String SQL_QUERY_UPDATE = "UPDATE ods_autre_modele SET id_autre_modele = ?, titre = ?, id_type_autre_modele = ?, " +
        " id_fichier_entete = ?, id_fichier_pied = ?";
    private static final String SQL_QUERY_UPDATE_WHERE = " WHERE id_autre_modele = ? ";
    private static final String SQL_QUERY_NEW_PK = " SELECT max( am.id_autre_modele ) FROM ods_autre_modele am ";
    private static final String SQL_QUERY_DELETE = " DELETE FROM ods_autre_modele WHERE id_autre_modele = ?  ";
    private static final String SQL_QUERY_INSERT = " INSERT INTO ods_autre_modele (id_autre_modele, titre, id_type_autre_modele, id_fichier_entete,id_fichier_pied ";
    private static final String SQL_QUERY_INSERT_VALUES = ") VALUES (?, ?, ?, ?, ? ";
    private static final String SQL_QUERY_ALREADY_EXIST = "SELECT COUNT(*) FROM  ods_autre_modele am ";
    private static final String SQL_QUERY_ALREADY_EXIST_WHERE = "WHERE am.id_type_autre_modele = ? AND am.id_autre_modele <> ?";
    private static final String SQL_FILTER_TYPE = " AND am.id_type_autre_modele= ? ";

    /**
     * Insertions spcifique (autre table...)
     * @param autreModele le modele
     * @param transaction la transaction
     * @throws SQLException en cas d'erreur SQL
     */
    protected abstract void doInsertSpec( GAutreModele autreModele, Transaction transaction )
        throws SQLException;

    /**
     * Met  jour la/les table(s) spcifiques
     * @param autreModele le modele
     * @param transaction la transaction
     * @throws SQLException en cas d'erreur SQL
     */
    protected abstract void doUpdateSpec( GAutreModele autreModele, Transaction transaction )
        throws SQLException;

    /**
     * Construit le filtre spcifique
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLFiltersSpec( StringBuilder sbSQL );

    /**
     * Remplit les valeurs des filtres
     * @param nIndex l'index de dpart
     * @param daoUtil DAOUtil
     * @return l'index d'arriver
     */
    protected abstract int setFiltersSpec( int nIndex, DAOUtil daoUtil );

    /**
     * Ajoute les informations spcifique  rcuprer dans le select
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLSelectSpec( StringBuilder sbSQL );

    /**
     * Ajoute les informations spcifique dans le from (jointure..)
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLFromSpec( StringBuilder sbSQL );

    /**
     * Ajoute les valeurs spcifique  l'objet modele
     * @param autreModele le modele
     * @param daoUtil DAOUtil
     */
    protected abstract void setAutreModeleValuesSpec( GAutreModele autreModele, DAOUtil daoUtil );

    /**
     * Supprime un modele de la base
     * @param autreModele le modele
     * @param plugin le plugin
     * @throws AppException si une erreur SQL survient
     */
    public void delete( GAutreModele autreModele, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, autreModele.getIdAutreModele(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Insre une nouvelle entre en base
     * @param autreModele le modele
     * @param plugin le plugin
     * @return l'identifiant en base de l'objet cr
     */
    public int insert( GAutreModele autreModele, Plugin plugin )
    {
        int nKey = newPrimaryKey( plugin );
        autreModele.setIdAutreModele( nKey );

        Transaction transaction = new Transaction( plugin );

        try
        {
            StringBuilder sbSQL = new StringBuilder(  );
            sbSQL.append( SQL_QUERY_INSERT );
            sbSQL.append( SQL_QUERY_INSERT_VALUES );
            sbSQL.append( OdsConstants.CONSTANTE_FERMETURE_ACCOLADE );
            transaction.prepareStatement( sbSQL.toString(  ) );
            setInsertOrUpdateValues( autreModele, transaction.getStatement(  ) );

            transaction.executeStatement(  );

            doInsertSpec( autreModele, transaction );

            transaction.commit(  );

            return nKey;
        }
        catch ( SQLException e )
        {
            transaction.rollback( e );
        }

        return -1;
    }

    /**
     * Dtermine si l'objet existe dj
     * @param idtypeAutreModele le type
     * @param idAutreModele l'identifiant
     * @param plugin le plugin
     * @return <b>true</b> si l'objet existe dj, <b>false</b> sinon.
     */
    public boolean isAlreadyExist( int idtypeAutreModele, int idAutreModele, Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_ALREADY_EXIST );
        buildSQLFromSpec( sbSQL );
        sbSQL.append( SQL_QUERY_ALREADY_EXIST_WHERE );
        buildSQLFiltersSpec( sbSQL );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        int nIndex = 1;
        daoUtil.setInt( nIndex, idtypeAutreModele );
        nIndex++;
        daoUtil.setInt( nIndex, idAutreModele );
        nIndex++;
        setFiltersSpec( nIndex, daoUtil );

        daoUtil.executeQuery(  );

        boolean bExists = daoUtil.next(  ) && ( daoUtil.getInt( 1 ) != 0 );

        daoUtil.free(  );

        return bExists;
    }

    /**
     * Rcupre un autre modle  partir de la base
     * @param nKey l'identifiant
     * @param plugin le plugin
     * @return le modele
     */
    public GAutreModele load( int nKey, Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLSelectSpec( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLFromSpec( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE_PK );
        buildSQLFiltersSpec( sbSQL );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        int nIndex = 1;
        daoUtil.setInt( nIndex, nKey );
        nIndex++;
        setFiltersSpec( nIndex, daoUtil );

        daoUtil.executeQuery(  );

        GAutreModele autreModele = null;

        if ( daoUtil.next(  ) )
        {
            autreModele = newAutreModeleInstance(  );
            setAutreModeleValues( autreModele, daoUtil );
            setAutreModeleValuesSpec( autreModele, daoUtil );
        }

        daoUtil.free(  );

        return autreModele;
    }

    /**
     * Selectionne les autres modles correspondant au filtre
     * @param filter le filtre
     * @param plugin le plugin
     * @return la liste des modele correspondant aux critres du filtre
     */
    public List<GAutreModele> selectListByFilter( AutreModeleFilter filter, Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLSelectSpec( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLFromSpec( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE );

        if ( filter.containsIdTypeCriteria(  ) )
        {
            sbSQL.append( SQL_FILTER_TYPE );
        }

        buildSQLFiltersSpec( sbSQL );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;

        if ( filter.containsIdTypeCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdType(  ) );
            nIndex++;
        }

        setFiltersSpec( nIndex, daoUtil );
        daoUtil.executeQuery(  );

        List<GAutreModele> autresModelesList = new ArrayList<GAutreModele>(  );

        while ( daoUtil.next(  ) )
        {
            GAutreModele autreModele = newAutreModeleInstance(  );
            setAutreModeleValues( autreModele, daoUtil );
            setAutreModeleValuesSpec( autreModele, daoUtil );

            autresModelesList.add( autreModele );
        }

        daoUtil.free(  );

        return autresModelesList;
    }

    /**
     * Met  jour les information du modele
     * @param autreModele le modele
     * @param plugin le plugin
     */
    public void store( GAutreModele autreModele, Plugin plugin )
    {
        Transaction transaction = new Transaction( plugin );

        try
        {
            StringBuilder sbSQL = new StringBuilder(  );
            sbSQL.append( SQL_QUERY_UPDATE );
            sbSQL.append( SQL_QUERY_UPDATE_WHERE );

            transaction.prepareStatement( sbSQL.toString(  ) );

            int nIndex = setInsertOrUpdateValues( autreModele, transaction.getStatement(  ) );
            transaction.getStatement(  ).setInt( nIndex, autreModele.getIdAutreModele(  ) );
            transaction.executeStatement(  );

            doUpdateSpec( autreModele, transaction );

            transaction.commit(  );
        }
        catch ( SQLException e )
        {
            transaction.rollback( e );
        }
    }

    /**
    * Gnre un nouvel identifiant
    * @param plugin le plugin
    * @return Retourne lidentifiant gnr
    */
    protected int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }
        else
        {
            nKey = daoUtil.getInt( 1 ) + 1;
        }

        daoUtil.free(  );

        return nKey;
    }

    /**
     * Met  jour les valeurs des objets internes du modele
     * @param autreModele le modele
     * @param daoUtil daoUtil
     */
    private void setAutreModeleValues( GAutreModele autreModele, DAOUtil daoUtil )
    {
        autreModele.setIdAutreModele( daoUtil.getInt( "am.id_autre_modele" ) );
        autreModele.setTitre( daoUtil.getString( "am.titre" ) );

        if ( daoUtil.getObject( "am.id_type_autre_modele" ) != null )
        {
            TypeAutreModele typeAutreModele = new TypeAutreModele(  );
            typeAutreModele.setIdTypeAutreModele( daoUtil.getInt( "am.id_type_autre_modele" ) );

            if ( daoUtil.getObject( "tam.libelle_type" ) != null )
            {
                typeAutreModele.setLibelle( daoUtil.getString( "tam.libelle_type" ) );
            }

            autreModele.setTypeAutreModele( typeAutreModele );
        }

        if ( daoUtil.getObject( "am.id_fichier_entete" ) != null )
        {
            FichierPhysique fichierEnTete = new FichierPhysique(  );
            fichierEnTete.setIdFichier( daoUtil.getInt( "am.id_fichier_entete" ) );
            autreModele.setEnteteDocument( fichierEnTete );
        }

        if ( daoUtil.getObject( "am.id_fichier_pied" ) != null )
        {
            FichierPhysique fichierPiedDocument = new FichierPhysique(  );
            fichierPiedDocument.setIdFichier( daoUtil.getInt( "am.id_fichier_pied" ) );
            autreModele.setPiedDocument( fichierPiedDocument );
        }
    }

    /**
     * Fixe les valeurs dans le requte
     * @param autreModele le modele
     * @param statement le statement
     * @return l'index (position) de sortie
     * @throws SQLException en cas d'erreur SQL
     */
    private int setInsertOrUpdateValues( GAutreModele autreModele, PreparedStatement statement )
        throws SQLException
    {
        int nIndex = 1;
        statement.setInt( nIndex, autreModele.getIdAutreModele(  ) );
        nIndex++;
        statement.setString( nIndex, autreModele.getTitre(  ) );
        nIndex++;
        statement.setInt( nIndex, autreModele.getTypeAutreModele(  ).getIdTypeAutreModele(  ) );
        nIndex++;

        if ( autreModele.getEnteteDocument(  ) == null )
        {
            statement.setNull( nIndex, Types.INTEGER );
        }
        else
        {
            statement.setInt( nIndex, autreModele.getEnteteDocument(  ).getIdFichier(  ) );
        }

        nIndex++;

        if ( autreModele.getPiedDocument(  ) == null )
        {
            statement.setNull( nIndex, Types.INTEGER );
        }
        else
        {
            statement.setInt( nIndex, autreModele.getPiedDocument(  ).getIdFichier(  ) );
        }

        nIndex++;

        return nIndex;
    }

    /**
     * Cre une nouvelle instance d'IAutreModele
     * @return une nouvelle instance d'IAutreModele
     */
    protected abstract GAutreModele newAutreModeleInstance(  );
}
