/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.categoriedeliberation;

import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;
import fr.paris.lutece.util.sql.Transaction;

import java.sql.SQLException;
import java.sql.Types;

import java.util.ArrayList;
import java.util.List;


/**
 *
 * CategorieDeliberationDAO
 *
 */
public class CategorieDeliberationDAO implements ICategorieDeliberationDAO
{
    private static final String SQL_QUERY_NEW_PK = "SELECT max( id_categorie ) " + "FROM ods_categorie_deliberation ";
    private static final String SQL_QUERY_INSERT = "INSERT INTO ods_categorie_deliberation(" +
        "	id_categorie,code_categorie,libelle_categorie,actif,export_paris_fr,id_categorie_ctrl_legalite) " +
        "VALUES (?,?,?,?,?,?) ";
    private static final String SQL_QUERY_DELETE = "DELETE FROM ods_categorie_deliberation " +
        "WHERE id_categorie = ?  ";
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = "SELECT id_categorie,code_categorie,libelle_categorie,actif,export_paris_fr,id_categorie_ctrl_legalite " +
        "FROM ods_categorie_deliberation " + "WHERE id_categorie = ? ";
    private static final String SQL_QUERY_UPDATE = "UPDATE ods_categorie_deliberation " +
        "SET code_categorie=?,libelle_categorie=?,actif=?,export_paris_fr=?,id_categorie_ctrl_legalite=? " +
        "WHERE  id_categorie=?";
    private static final String SQL_QUERY_FIND_ALL = "SELECT id_categorie,code_categorie,libelle_categorie,actif,export_paris_fr,id_categorie_ctrl_legalite " +
        "FROM ods_categorie_deliberation ";
    private static final String SQL_QUERY_FIND_BY_CODE = "SELECT id_categorie,code_categorie " +
        "FROM ods_categorie_deliberation ";
    private static final String SQL_QUERY_FIND_ID_CATEGORIE_BY_CODE = "SELECT id_categorie FROM ods_categorie_deliberation WHERE code_categorie = ? ";
    private static final String SQL_QUERY_FIND_CATEGORIE_BY_CODE = "SELECT id_categorie,code_categorie,libelle_categorie,actif,export_paris_fr,id_categorie_ctrl_legalite " +
        " FROM ods_categorie_deliberation WHERE code_categorie = ? ";
    private static final String SQL_QUERY_ORDER_BY_CODE_CATEGORIE = " ORDER BY code_categorie ASC ";
    private static final String SQL_QUERY_ORDER_BY_LIBELLE = " ORDER BY libelle_categorie ASC ";
    private static final String SQL_QUERY_FIND_ACTIVES = "SELECT id_categorie,code_categorie,libelle_categorie,actif,export_paris_fr,id_categorie_ctrl_legalite " +
        "FROM ods_categorie_deliberation " + "WHERE actif=true ";
    private static final String SQL_QUERY_FIND_EXPORT_PARIS_FR = "SELECT code_categorie " +
        "FROM ods_categorie_deliberation " + "WHERE export_paris_fr=true ";

    /**
     * Retourne une nouvelle cl primaire unique sur la table ods_categorie_deliberation.
     * @param plugin spcifie le plugin actif
     * @return nouvelle cl primaire
     */
    private int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Cre une nouvelle catgorie de dlibration dans la table ods_categorie_deliberation
     * @param categorieDeliberation la catgorie de dlibration  insrer
     * @param plugin spcifie le plugin actif
     */
    public void insert( CategorieDeliberation categorieDeliberation, Plugin plugin )
    {
        categorieDeliberation.setIdCategorie( newPrimaryKey( plugin ) );

        Transaction transaction = new Transaction( plugin );

        try
        {
            StringBuilder sbSQL = new StringBuilder(  );
            sbSQL.append( SQL_QUERY_INSERT );
            transaction.prepareStatement( sbSQL.toString(  ) );
            transaction.getStatement(  ).setInt( 1, categorieDeliberation.getIdCategorie(  ) );
            transaction.getStatement(  ).setInt( 2, categorieDeliberation.getCode(  ) );
            transaction.getStatement(  ).setString( 3, categorieDeliberation.getLibelle(  ) );
            transaction.getStatement(  ).setBoolean( 4, categorieDeliberation.isActif(  ) );
            transaction.getStatement(  ).setBoolean( 5, categorieDeliberation.isExportParisFr(  ) );

            if ( categorieDeliberation.getCategorieCtrlLegalite(  ) > 0 )
            {
                transaction.getStatement(  ).setInt( 6, categorieDeliberation.getCategorieCtrlLegalite(  ) );
            }
            else
            {
                transaction.getStatement(  ).setNull( 6, Types.INTEGER );
            }

            transaction.executeStatement(  );

            // Commit
            transaction.commit(  );
        }
        catch ( SQLException e )
        {
            // Rollback
            transaction.rollback( e );
        }
    }

    /**
     * Supprime la catgorie passe en paramtre de la table ods_categorie_deliberation
     * @param categorieDeliberation la catgorie de dlibration  supprimer
     * @param plugin spcifie le plugin actif
     * @throws AppException si la catgorie de dlibration est rfrence par un PDD
     */
    public void delete( CategorieDeliberation categorieDeliberation, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, categorieDeliberation.getIdCategorie(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Retourne la catgorie de dlibration associe  la cl passe en paramtre
     * @param nKey l'id unique de la catgorie de dlibration dans la table ods_categorie_deliberation
     * @param plugin spcifie le plugin actif
     * @return la catgorie de dlibration associe  l'identifiant nKey
     */
    public CategorieDeliberation load( int nKey, Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_FIND_BY_PRIMARY_KEY );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;
        daoUtil.setInt( nIndex, nKey );

        daoUtil.executeQuery(  );

        CategorieDeliberation categorieDeliberation = null;

        if ( daoUtil.next(  ) )
        {
            //categorieDeliberation = new CategorieDeliberation(  );
            categorieDeliberation = setCategorieDelibartionValues( daoUtil );
        }

        daoUtil.free(  );

        return categorieDeliberation;
    }

    /**
     * Met  jour les informations d'une catgorie de dlibration dans la table ods_categorie_deliberation
     * @param categorieDeliberation contient les nouvelles valeurs de la catgorie de dlibration
     * @param plugin spcifie le plugin actif
     */
    public void store( CategorieDeliberation categorieDeliberation, Plugin plugin )
    {
        Transaction transaction = new Transaction( plugin );

        try
        {
            StringBuilder sbSQL = new StringBuilder(  );
            sbSQL.append( SQL_QUERY_UPDATE );
            transaction.prepareStatement( sbSQL.toString(  ) );
            transaction.getStatement(  ).setInt( 1, categorieDeliberation.getCode(  ) );
            transaction.getStatement(  ).setString( 2, categorieDeliberation.getLibelle(  ) );
            transaction.getStatement(  ).setBoolean( 3, categorieDeliberation.isActif(  ) );
            transaction.getStatement(  ).setBoolean( 4, categorieDeliberation.isExportParisFr(  ) );

            if ( categorieDeliberation.getCategorieCtrlLegalite(  ) > 0 )
            {
                transaction.getStatement(  ).setInt( 5, categorieDeliberation.getCategorieCtrlLegalite(  ) );
            }
            else
            {
                transaction.getStatement(  ).setNull( 5, Types.INTEGER );
            }

            transaction.getStatement(  ).setInt( 6, categorieDeliberation.getIdCategorie(  ) );

            transaction.executeStatement(  );

            // Commit
            transaction.commit(  );
        }
        catch ( SQLException e )
        {
            // Rollback
            transaction.rollback( e );
        }
    }

    /**
     * Retourne la liste des catgories de dlibrations
     * @param plugin spcifie le plugin actif
     * @return une liste de catgorie de dlibrations
     */
    public List<CategorieDeliberation> loadListeCategorieDeliberation( Plugin plugin )
    {
        List<CategorieDeliberation> listCategorie = new ArrayList<CategorieDeliberation>(  );
        CategorieDeliberation categorie;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ALL + SQL_QUERY_ORDER_BY_CODE_CATEGORIE, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            categorie = setCategorieDelibartionValues( daoUtil );
            listCategorie.add( categorie );
        }

        daoUtil.free(  );

        return listCategorie;
    }

    /**
     * Retourne la liste des codes de la table ods_categorie_deliberation;
     * Exclut de la requte la catgorie de dlibration associe  l'identifiant exceptIdcatgorie de dlibration.
     * @param exceptCategorie la catgorie de dlibration  exclure de la liste ( =null si aucune catgorie de dlibration  exclure)
     * @param plugin spcifie le plugin actif
     * @return la liste des codes de catgorie de dlibration
     */
    public List<String> listCodes( CategorieDeliberation exceptCategorie, Plugin plugin )
    {
        int nIdExceptCategorie;

        if ( exceptCategorie != null )
        {
            nIdExceptCategorie = exceptCategorie.getIdCategorie(  );
        }
        else
        {
            nIdExceptCategorie = -1;
        }

        List<String> listCodes = new ArrayList<String>(  );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_CODE + SQL_QUERY_ORDER_BY_CODE_CATEGORIE, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            if ( daoUtil.getInt( "id_categorie" ) != nIdExceptCategorie )
            {
                listCodes.add( daoUtil.getString( "code_categorie" ) );
            }
        }

        daoUtil.free(  );

        return listCodes;
    }

    /**
     * Retourne l'id de la catgorie correspondant au code pass en paramtre
     * @param nCodeCategorie le code de la ctagorie
     * @param plugin le plugin
     * @return l'id de la catgorie
     */
    public int listIdCategorieByCode( int nCodeCategorie, Plugin plugin )
    {
        int nIdCategorie = -1;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ID_CATEGORIE_BY_CODE, plugin );
        daoUtil.setInt( 1, nCodeCategorie );
        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            nIdCategorie = daoUtil.getInt( "id_categorie" );
        }

        daoUtil.free(  );

        return nIdCategorie;
    }

    /**
     * Retourne la catgorie correspondant au code pass en paramtre
     * @param nCodeCategorie le code de la ctagorie
     * @param plugin le plugin
     * @return la catgorie
     */
    public CategorieDeliberation listCategorieByCode( int nCodeCategorie, Plugin plugin )
    {
        CategorieDeliberation categorie = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_CATEGORIE_BY_CODE, plugin );
        daoUtil.setInt( 1, nCodeCategorie );
        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            categorie = setCategorieDelibartionValues( daoUtil );
        }

        daoUtil.free(  );

        return categorie;
    }

    /**
    * Retourne la liste des catgories de dlibration actives.
    * @param plugin le plugin
    * @return liste des catgories de dlibration actives
    */
    public List<CategorieDeliberation> listCategoriesActives( Plugin plugin )
    {
        List<CategorieDeliberation> listCategorie = new ArrayList<CategorieDeliberation>(  );
        CategorieDeliberation categorie;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ACTIVES + SQL_QUERY_ORDER_BY_CODE_CATEGORIE, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            categorie = setCategorieDelibartionValues( daoUtil );
            listCategorie.add( categorie );
        }

        daoUtil.free(  );

        return listCategorie;
    }

    /**
     * Retourne la liste des catgories de dlibration ordonnes par ordre alphabtiqque des libelles
     * @param plugin le plugin
     * @return la liste des catgories de dlibration ordonnes
     */
    public List<CategorieDeliberation> listCategoriesOrderByLibelle( Plugin plugin )
    {
        List<CategorieDeliberation> listCategorie = new ArrayList<CategorieDeliberation>(  );
        CategorieDeliberation categorie;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ACTIVES + SQL_QUERY_ORDER_BY_LIBELLE, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            categorie = setCategorieDelibartionValues( daoUtil );
            listCategorie.add( categorie );
        }

        daoUtil.free(  );

        return listCategorie;
    }

    /**
     * Retourne la liste des codes des catgories de type march.
     * @param plugin spcifie le plugin actif
     * @return liste des codes des catgories de type march
     */
    public List<String> listCodesCategoriesExportParisFr( Plugin plugin )
    {
        List<String> listCodes = new ArrayList<String>(  );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_EXPORT_PARIS_FR + SQL_QUERY_ORDER_BY_CODE_CATEGORIE, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            listCodes.add( daoUtil.getString( "code_categorie" ) );
        }

        daoUtil.free(  );

        return listCodes;
    }

    /**
     * Cre et remplit un objet CategorieDeliberation
     * @param daoUtil daoUtil
     * @return CategorieDeliberation
     */
    private CategorieDeliberation setCategorieDelibartionValues( DAOUtil daoUtil )
    {
        CategorieDeliberation categorie = new CategorieDeliberation(  );
        categorie.setIdCategorie( daoUtil.getInt( "id_categorie" ) );
        categorie.setCode( daoUtil.getInt( "code_categorie" ) );
        categorie.setLibelle( daoUtil.getString( "libelle_categorie" ) );
        categorie.setActif( daoUtil.getBoolean( "actif" ) );
        categorie.setExportParisFr( daoUtil.getBoolean( "export_paris_fr" ) );
        categorie.setCategorieCtrlLegalite( daoUtil.getInt( "id_categorie_ctrl_legalite" ) );

        return categorie;
    }
}
