/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.deliberationdesignation;

import fr.paris.lutece.plugins.ods.dto.acte.StatutActeEnum;
import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.DeliberationDesignationFilter;
import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.List;


/**
 * Cette classe permet d'accder/grer la table ods_designation
 *
 * @param <GSeance> seance
 * @param <GFichier> fichier
 * @param <GDeliberatioDesignation> DeliberationDesignation
 */
public abstract class AbstractDeliberationDesignationDAO<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>, GDeliberatioDesignation extends IDeliberationDesignation<GSeance, GFichier>>
    implements IDeliberationDesignationDAO<GSeance, GFichier, GDeliberatioDesignation>
{
    private static final String SQL_QUERY_ORDER_BY_DATE_MODIFICATION = " ORDER BY ds.date_publication DESC ";
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = "SELECT ds.id_designation,ds.id_seance, ds.id_formation_conseil, ds.numero," +
        " ds.intitule, ds.id_fichier_or, ds.id_fichier_sc, ds.id_categorie, ds.date_vote, ds.date_publication, " +
        "ds.en_ligne, ds.version FROM ods_designation ds WHERE ds.id_designation = ? ";
    private static final String SQL_QUERY_DESIGNATIONS_LIST = "SELECT ds.id_designation, ds.id_seance, ds.id_formation_conseil, ds.numero," +
        " ds.intitule, ds.id_fichier_or, ds.id_fichier_sc, ds.id_categorie, ds.date_vote, ds.date_publication, " +
        " ds.en_ligne, ds.version FROM ods_designation ds WHERE 1 = 1 ";
    private static final String SQL_QUERY_FIND_ID_BY_FICHIER = " SELECT id_designation, numero, date_publication, version, en_ligne" +
        " FROM ods_designation WHERE id_fichier_sc = ? ";
    private static final String SQL_FILTER_SEANCE = " AND ds.id_seance = ? ";
    private static final String SQL_FILTER_FORMATION_CONSEIL = " AND ds.id_formation_conseil = ? ";
    private static final String SQL_QUERY_NEW_PK = " SELECT max( id_designation ) FROM ods_designation ";
    private static final String SQL_QUERY_INSERT = " INSERT INTO ods_designation (id_designation, id_seance, id_formation_conseil, numero," +
        "intitule, id_fichier_sc, id_categorie, date_vote, date_publication, en_ligne, version) " +
        "VALUES ( ?,?, ?, ?, ?, ?, ?, ?, ?, ?, ? ) ";
    private static final String SQL_QUERY_PUBLIER = " UPDATE ods_designation SET date_publication = ?, en_ligne = ?, version = ? WHERE id_designation = ?  ";
    private static final String SQL_QUERY_UPDATE = " UPDATE ods_designation SET id_seance = ?, id_formation_conseil = ?, numero = ?, " +
        "intitule = ?, id_fichier_sc = ?, id_categorie = ? , date_vote = ?, date_publication = ?, en_ligne=?, version=? " +
        "WHERE id_designation= ?  ";
    private static final String SQL_QUERY_DELETE = " DELETE FROM ods_designation WHERE id_designation = ?  ";
    private static final String SQL_FILTER_STATUT_ACTE = "AND ( SELECT count(*) FROM ods_acte WHERE id_type_entite = ? AND id_entite=ds.id_designation AND id_statut<>? ) ";
    private static final String SQL_FILTER_STATUT_A_PREPARER = " = 0";
    private static final String SQL_FILTER_STATUT_UN_ACTE_AU_MOINS = " >= 1";
    private static final String SQL_FILTER_DATE_VOTE_RENSEIGNEE = " AND date_vote IS NOT NULL ";

    /**
     * Cre une nouvelle instance de ISeance
     * @return une nouvelle instance de ISeance
     */
    protected abstract GSeance newSeanceInstance(  );

    /**
     * Cre une nouvelle instance de IFichier
     * @return une nouvelle instance de IFichier
     */
    protected abstract GFichier newFichierInstance(  );

    /**
    * Cre une nouvelle dlibration de dsignation  partir de lobjet DeliberationDesignation pass en paramtre
    * @param deliberation ladlibration de dsignation  insrer
    * @param plugin le plugin
    * @return int primaryKey
    */
    public int insert( GDeliberatioDesignation deliberation, Plugin plugin )
    {
        int newPrimaryKey = newPrimaryKey( plugin );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT, plugin );
        deliberation.setId( newPrimaryKey );
        daoUtil.setInt( 1, deliberation.getId(  ) );

        if ( deliberation.getSeance(  ) != null )
        {
            daoUtil.setInt( 2, deliberation.getSeance(  ).getIdSeance(  ) );
        }
        else
        {
            daoUtil.setIntNull( 2 );
        }

        if ( deliberation.getFormationConseil(  ) == null )
        {
            daoUtil.setIntNull( 3 );
        }
        else
        {
            daoUtil.setInt( 3, deliberation.getFormationConseil(  ).getIdFormationConseil(  ) );
        }

        daoUtil.setString( 4, deliberation.getNumero(  ) );
        daoUtil.setString( 5, deliberation.getIntitule(  ) );

        if ( deliberation.getFichierSc(  ) == null )
        {
            daoUtil.setIntNull( 6 );
        }
        else
        {
            daoUtil.setInt( 6, deliberation.getFichierSc(  ).getId(  ) );
        }

        if ( deliberation.getCategorie(  ) != null )
        {
            daoUtil.setInt( 7, deliberation.getCategorie(  ).getIdCategorie(  ) );
        }
        else
        {
            daoUtil.setIntNull( 7 );
        }

        daoUtil.setTimestamp( 8, deliberation.getDateVote(  ) );
        daoUtil.setTimestamp( 9, new Timestamp( System.currentTimeMillis(  ) ) );
        daoUtil.setBoolean( 10, deliberation.getEnLigne(  ) );
        daoUtil.setInt( 11, deliberation.getVersion(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );

        return newPrimaryKey;
    }

    /**
     * Modifie la dlibration correspondante  lobjet deliberation pass en paramtre
     * @param deliberation la dlibration  modifier
     * @param plugin le plugin
     */
    public void store( GDeliberatioDesignation deliberation, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_UPDATE, plugin );

        if ( deliberation.getSeance(  ) == null )
        {
            daoUtil.setIntNull( 1 );
        }
        else
        {
            daoUtil.setInt( 1, deliberation.getSeance(  ).getIdSeance(  ) );
        }

        if ( deliberation.getFormationConseil(  ) == null )
        {
            daoUtil.setIntNull( 2 );
        }
        else
        {
            daoUtil.setInt( 2, deliberation.getFormationConseil(  ).getIdFormationConseil(  ) );
        }

        if ( deliberation.getNumero(  ) == null )
        {
            daoUtil.setIntNull( 3 );
        }
        else
        {
            daoUtil.setString( 3, deliberation.getNumero(  ) );
        }

        if ( deliberation.getIntitule(  ) == null )
        {
            daoUtil.setIntNull( 4 );
        }
        else
        {
            daoUtil.setString( 4, deliberation.getIntitule(  ) );
        }

        if ( deliberation.getFichierSc(  ) == null )
        {
            daoUtil.setIntNull( 5 );
        }
        else
        {
            daoUtil.setInt( 5, deliberation.getFichierSc(  ).getId(  ) );
        }

        if ( deliberation.getCategorie(  ) != null )
        {
            daoUtil.setInt( 6, deliberation.getCategorie(  ).getIdCategorie(  ) );
        }
        else
        {
            daoUtil.setIntNull( 6 );
        }

        daoUtil.setTimestamp( 7, deliberation.getDateVote(  ) );
        daoUtil.setTimestamp( 8, new Timestamp( System.currentTimeMillis(  ) ) );
        daoUtil.setBoolean( 9, deliberation.getEnLigne(  ) );
        daoUtil.setInt( 10, deliberation.getVersion(  ) );

        daoUtil.setInt( 11, deliberation.getId(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Supprime la dlibration correspondant  lobjet fichier deliberation en paramtre
     * @param deliberation la dlibration  supprimer
     * @param plugin le plugin
     * @throws AppException si contrainte de cl trangre
     */
    public void delete( GDeliberatioDesignation deliberation, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, deliberation.getId(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Retourne la liste des dlibrations correspondants aux critres du filtre.
     * @param filter les critres de recherche de la dlibration
     * @param plugin le plugin
     * @return la liste de delibrations correspondants aux critres de recherche
     */
    public List<GDeliberatioDesignation> selectByFilter( DeliberationDesignationFilter filter, Plugin plugin )
    {
        List<GDeliberatioDesignation> deliberations = new ArrayList<GDeliberatioDesignation>(  );

        StringBuffer sbSQL = new StringBuffer(  );
        sbSQL.append( SQL_QUERY_DESIGNATIONS_LIST );
        sbSQL.append( ( ( filter.containsSeanceCriteria(  ) ) ? SQL_FILTER_SEANCE : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsFormationConseilCriteria(  ) ) ? SQL_FILTER_FORMATION_CONSEIL
                                                                        : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsStatutActeCriteria(  ) ) ? SQL_FILTER_STATUT_ACTE
                                                                  : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        if ( filter.getStatutActe(  ) == DeliberationDesignationFilter.A_PREPARER )
        {
            sbSQL.append( SQL_FILTER_STATUT_A_PREPARER );
        }
        else if ( filter.getStatutActe(  ) == DeliberationDesignationFilter.UN_ACTE_AU_MOINS )
        {
            sbSQL.append( SQL_FILTER_STATUT_UN_ACTE_AU_MOINS );
        }

        sbSQL.append( ( ( filter.containsStatutActeCriteria(  ) ) ? SQL_FILTER_DATE_VOTE_RENSEIGNEE
                                                                  : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        sbSQL.append( SQL_QUERY_ORDER_BY_DATE_MODIFICATION );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        int nIndex = 1;

        if ( filter.containsSeanceCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdSeance(  ) );
            nIndex++;
        }

        if ( filter.containsFormationConseilCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdFormationConseil(  ) );
            nIndex++;
        }

        if ( filter.containsStatutActeCriteria(  ) )
        {
            daoUtil.setInt( nIndex, ITypeEntite.DESIGNATIONS_DE_REPRESENTANTS );
            nIndex++;
            daoUtil.setInt( nIndex, StatutActeEnum.ANNULE.getId(  ) );
            nIndex++;
        }

        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            GDeliberatioDesignation deliberation = getBasePropertyDeliberation( daoUtil );
            deliberations.add( deliberation );
        }

        daoUtil.free(  );

        return deliberations;
    }

    /**
    * renvoie la dlibration de dsignation ayant comme identifiant de fichier nKey
    * @param nKey idenfiant du fichier
    * @param plugin le Plugin actif
    * @return la dlibration de dsignation
    */
    public GDeliberatioDesignation selectByFichier( int nKey, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ID_BY_FICHIER, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        GDeliberatioDesignation deliberation = newDeliberationDesignationInstance(  );

        if ( daoUtil.next(  ) )
        {
            deliberation.setId( daoUtil.getInt( "id_designation" ) );
            deliberation.setNumero( daoUtil.getString( "numero" ) );
            deliberation.setDatePublication( daoUtil.getTimestamp( "date_publication" ) );
            deliberation.setVersion( daoUtil.getInt( "version" ) );
            deliberation.setEnLigne( daoUtil.getBoolean( "en_ligne" ) );
        }

        daoUtil.free(  );

        return deliberation;
    }

    /**
     * Retourne la dlibration ayant comme identifiant nKey
     * @param nKey identifiant de la dlibration  charger
     * @param plugin le plugin
     * @return la dlibration identifie par nKey
     */
    public GDeliberatioDesignation load( int nKey, Plugin plugin )
    {
        GDeliberatioDesignation deliberation = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            deliberation = getBasePropertyDeliberation( daoUtil );
        }

        daoUtil.free(  );

        return deliberation;
    }

    /**
     * Permet de publier ou de dpublier une dlibration de dsignation
         * Si isPublie gale  TRUE, alors la mthode <b>publie</b> la dlibration et incrmente la version<BR>
         * Sinon la mthode change le statut enLigne de la dlibration a <b>false</b>
     * @param nKey l'identifiant de la dlibration
     * @param tsDatePublication la date de publication
     * @param version la version de la dlibration
     * @param isPublie le statut de la dlibrations
     * @param plugin le plugin
     */
    public void publication( int nKey, Timestamp tsDatePublication, int version, boolean isPublie, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_PUBLIER, plugin );
        daoUtil.setTimestamp( 1, tsDatePublication );
        daoUtil.setBoolean( 2, isPublie );

        if ( isPublie )
        {
            daoUtil.setInt( 3, version + 1 );
        }
        else
        {
            daoUtil.setInt( 3, version );
        }

        daoUtil.setInt( 4, nKey );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Permet de creer un objet dlibration avec ses proprits de base
     * @param daoUtil contient les informations permettant de construire l'objet
     * @return la dlibration avec les informations de la requte
     */
    private GDeliberatioDesignation getBasePropertyDeliberation( DAOUtil daoUtil )
    {
        GDeliberatioDesignation deliberation = newDeliberationDesignationInstance(  );
        deliberation.setId( daoUtil.getInt( "ds.id_designation" ) );

        if ( ( daoUtil.getObject( "ds.version" ) != null ) || ( daoUtil.getInt( "ds.version" ) > 0 ) )
        {
            deliberation.setVersion( daoUtil.getInt( "ds.version" ) );
        }
        else
        {
            deliberation.setVersion( 0 );
        }

        deliberation.setEnLigne( daoUtil.getBoolean( "ds.en_ligne" ) );
        deliberation.setDatePublication( daoUtil.getTimestamp( "ds.date_publication" ) );
        deliberation.setDateVote( daoUtil.getTimestamp( "ds.date_vote" ) );
        deliberation.setNumero( daoUtil.getString( "ds.numero" ) );
        deliberation.setIntitule( daoUtil.getString( "ds.intitule" ) );

        if ( daoUtil.getObject( "ds.id_seance" ) != null )
        {
            GSeance seance = newSeanceInstance(  );
            seance.setIdSeance( daoUtil.getInt( "ds.id_seance" ) );
            deliberation.setSeance( seance );
        }

        if ( daoUtil.getObject( "ds.id_categorie" ) != null )
        {
            CategorieDeliberation categorie = new CategorieDeliberation(  );
            categorie.setIdCategorie( daoUtil.getInt( "ds.id_categorie" ) );
            deliberation.setCategorie( categorie );
        }

        if ( daoUtil.getObject( "ds.id_formation_conseil" ) != null )
        {
            FormationConseil formationConseil = new FormationConseil(  );
            formationConseil.setIdFormationConseil( daoUtil.getInt( "ds.id_formation_conseil" ) );
            deliberation.setFormationConseil( formationConseil );
        }

        if ( daoUtil.getObject( "ds.id_fichier_or" ) != null )
        {
            GFichier fichier = newFichierInstance(  );
            fichier.setId( daoUtil.getInt( "ds.id_fichier_or" ) );
            deliberation.setFichierOr( fichier );
        }

        if ( daoUtil.getObject( "ds.id_fichier_sc" ) != null )
        {
            GFichier fichier = newFichierInstance(  );
            fichier.setId( daoUtil.getInt( "ds.id_fichier_sc" ) );
            deliberation.setFichierSc( fichier );
        }

        return deliberation;
    }

    /**
     * Gnre un nouvel identifiant
     * @param plugin le plugin
     * @return Retourne lidentifiant gnr
     */
    int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey = 1;

        if ( daoUtil.next(  ) )
        {
            nKey = daoUtil.getInt( 1 ) + 1;
        }

        daoUtil.free(  );

        return nKey;
    }

    /**
     * Cre une nouvelle instance de IDeliberationDesignation
     * @return une nouvelle instance de IDeliberationDesignation
     */
    protected abstract GDeliberatioDesignation newDeliberationDesignationInstance(  );
}
