/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.direction;

import fr.paris.lutece.plugins.ods.dto.direction.Direction;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;
import fr.paris.lutece.util.sql.Transaction;

import java.sql.SQLException;

import java.util.ArrayList;
import java.util.List;


/**
 *
 * DirectionDAO
 *
 */
public class DirectionDAO implements IDirectionDAO
{
    private static final String SQL_QUERY_NEW_PK = "SELECT max( id_direction ) " + "FROM ods_direction ";
    private static final String SQL_QUERY_DELETE = "DELETE FROM ods_direction WHERE id_direction = ?  ";
    private static final String SQL_QUERY_INSERT = "INSERT INTO ods_direction(" +
        "	id_direction,code_direction,libelle_court,libelle_long,actif) " + "VALUES (?,?,?,?,?) ";
    private static final String SQL_QUERY_SELECT = "SELECT id_direction,code_direction,libelle_court,libelle_long,actif ";
    private static final String SQL_QUERY_UPDATE = "UPDATE ods_direction " +
        "SET code_direction=?,libelle_court=?,libelle_long=?,actif=? " + "WHERE  id_direction=?";
    private static final String SQL_QUERY_FIND_ALL = SQL_QUERY_SELECT + "FROM ods_direction ";
    private static final String SQL_QUERY_FROM = "FROM ods_direction ";
    private static final String SQL_QUERY_WHERE_ID_DIRECTION = "WHERE id_direction = ? ";
    private static final String SQL_QUERY_FILTER_ACTIF = "WHERE actif=true ";
    private static final String SQL_QUERY_ORDER_BY_CODE_DIRECTION = " ORDER BY code_direction ASC ";

    /**
     * Retourne une nouvelle cl primaire unique sur la table ods_direction.
     * @param plugin le plugin actif
     * @return nouvelle cl primaire
     */
    private int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Supprime la direction passe en paramtre de la table ods_direction
     * @param direction la direction  supprimer
     * @param plugin le plugin actif
     * @throws AppException si la direction est rfrence par un PDD
     */
    public void delete( Direction direction, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, direction.getIdDirection(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Cre une nouvelle direction dans la table ods_direction
     * @param direction la direction  insrer
     * @param plugin le plugin actif
     */
    public void insert( Direction direction, Plugin plugin )
    {
        direction.setIdDirection( newPrimaryKey( plugin ) );

        Transaction transaction = new Transaction( plugin );

        try
        {
            StringBuilder sbSQL = new StringBuilder(  );
            sbSQL.append( SQL_QUERY_INSERT );
            transaction.prepareStatement( sbSQL.toString(  ) );
            transaction.getStatement(  ).setInt( 1, direction.getIdDirection(  ) );
            transaction.getStatement(  ).setString( 2, direction.getCode(  ) );
            transaction.getStatement(  ).setString( 3, direction.getLibelleCourt(  ) );
            transaction.getStatement(  ).setString( 4, direction.getLibelleLong(  ) );
            transaction.getStatement(  ).setBoolean( 5, direction.isActif(  ) );

            transaction.executeStatement(  );

            // Commit
            transaction.commit(  );
        }
        catch ( SQLException e )
        {
            // Rollback
            transaction.rollback( e );
        }
    }

    /**
     * Retourne la direction associe  la cl passe en paramtre
     * @param nKey l'id unique de la direction dans la table ods_direction
     * @param plugin le plugin actif
     * @return la direction associe  l'identifiant nKey
     */
    public Direction load( int nKey, Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        sbSQL.append( SQL_QUERY_FROM );
        sbSQL.append( SQL_QUERY_WHERE_ID_DIRECTION );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;
        daoUtil.setInt( nIndex, nKey );

        daoUtil.executeQuery(  );

        Direction direction = null;

        if ( daoUtil.next(  ) )
        {
            //direction = new Direction(  );
            direction = setDirectionValues( daoUtil );
        }

        daoUtil.free(  );

        return direction;
    }

    /**
     * Met  jour les informations d'une direction dans la table ods_direction
     * @param direction contient les nouvelles valeurs de la direction
     * @param plugin le plugin actif
     */
    public void store( Direction direction, Plugin plugin )
    {
        Transaction transaction = new Transaction( plugin );

        try
        {
            StringBuilder sbSQL = new StringBuilder(  );
            sbSQL.append( SQL_QUERY_UPDATE );
            transaction.prepareStatement( sbSQL.toString(  ) );
            transaction.getStatement(  ).setString( 1, direction.getCode(  ) );
            transaction.getStatement(  ).setString( 2, direction.getLibelleCourt(  ) );
            transaction.getStatement(  ).setString( 3, direction.getLibelleLong(  ) );
            transaction.getStatement(  ).setBoolean( 4, direction.isActif(  ) );
            transaction.getStatement(  ).setInt( 5, direction.getIdDirection(  ) );

            transaction.executeStatement(  );

            // Commit
            transaction.commit(  );
        }
        catch ( SQLException e )
        {
            // Rollback
            transaction.rollback( e );
        }
    }

    /**
     * Retourne la liste des directions
     * @param plugin le plugin actif
     * @return une liste de directions
     */
    public List<Direction> loadListDirection( Plugin plugin )
    {
        List<Direction> listDirection = new ArrayList<Direction>(  );
        Direction direction;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ALL + SQL_QUERY_ORDER_BY_CODE_DIRECTION, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            direction = setDirectionValues( daoUtil );
            listDirection.add( direction );
        }

        daoUtil.free(  );

        return listDirection;
    }

    /**
     * Retourne la liste des codes de direction except celui de la direction associe  exceptIdDirection
     * @param exceptDirection direction  exclure de la liste (aucune exclusion si = null)
     * @param plugin le plugin actif
     * @return la liste des codes de direction
     */
    public List<String> listCodes( Direction exceptDirection, Plugin plugin )
    {
        int nIdExceptDirection;

        if ( exceptDirection != null )
        {
            nIdExceptDirection = exceptDirection.getIdDirection(  );
        }
        else
        {
            nIdExceptDirection = -1;
        }

        List<String> listcodes = new ArrayList<String>(  );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ALL + SQL_QUERY_ORDER_BY_CODE_DIRECTION, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            if ( daoUtil.getInt( 1 ) != nIdExceptDirection )
            {
                listcodes.add( daoUtil.getString( 2 ) );
            }
        }

        daoUtil.free(  );

        return listcodes;
    }

    /**
     * Retourne la liste des directions actives.
     * @param plugin le plugin
     * @return la liste des directions actives
     */
    public List<Direction> loadListDirectionsActives( Plugin plugin )
    {
        List<Direction> listDirection = new ArrayList<Direction>(  );
        Direction direction;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ALL + SQL_QUERY_FILTER_ACTIF + SQL_QUERY_ORDER_BY_CODE_DIRECTION,
                plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            direction = setDirectionValues( daoUtil );
            listDirection.add( direction );
        }

        daoUtil.free(  );

        return listDirection;
    }

    /**
     * Cre un nouvel objet Direction et lui attribut les valeurs contenues dans l'objet DAOUtil
     * @param daoUtil DAOUtil
     * @return un nouvel objet Direction avec les valeurs contenues dans l'objet DAOUtil
     */
    private Direction setDirectionValues( DAOUtil daoUtil )
    {
        Direction direction = new Direction(  );
        direction.setIdDirection( daoUtil.getInt( "id_direction" ) );
        direction.setCode( daoUtil.getString( "code_direction" ) );
        direction.setLibelleCourt( daoUtil.getString( "libelle_court" ) );
        direction.setLibelleLong( daoUtil.getString( "libelle_long" ) );
        direction.setActif( daoUtil.getBoolean( "actif" ) );

        return direction;
    }
}
