/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.elu;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.groupepolitique.GroupePolitique;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;
import fr.paris.lutece.util.sql.Transaction;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;

import java.util.ArrayList;
import java.util.List;


/**
 *
 * AbstractEluDAO
 *
 * @param <GElu> l'IElu
 */
public abstract class AbstractEluDAO<GElu extends IElu> implements IEluDAO<GElu>
{
    protected static final String SQL_QUERY_SELECT = "SELECT " +
        " elu.id_elu,elu.civilite, elu.nom_elu,elu.prenom_elu,elu.actif, elu.ods_id_elu, " +
        " grou.id_groupe,grou.nom_groupe,grou.nom_complet ";
    protected static final String SQL_QUERY_SELECT_FROM = " FROM ods_elu elu " +
        " LEFT JOIN ods_groupe grou on (elu.id_groupe=grou.id_groupe) ";
    protected static final String SQL_QUERY_ORDER_BY_NOM_ELU = " ORDER BY elu.nom_elu ASC ";
    protected static final String SQL_QUERY_WHERE_ACTIF = " WHERE elu.actif=true";
    private static final String SQL_QUERY_DELETE = " DELETE FROM ods_elu WHERE id_elu = ?  ";
    private static final String SQL_QUERY_INSERT = "INSERT INTO ods_elu( " + " id_elu, id_groupe, " +
        " civilite, nom_elu, prenom_elu, actif, ods_id_elu ";
    private static final String SQL_QUERY_INSERT_VALUES = ") " + "VALUES (?,?,?,?,?,?,? ";
    private static final String SQL_QUERY_UPDATE = "UPDATE ods_elu SET id_groupe=?, " +
        "civilite=?, nom_elu=?, prenom_elu=?, actif=?, ods_id_elu=? ";
    private static final String SQL_QUERY_UPDATE_WHERE = "WHERE id_elu=? ";
    private static final String SQL_QUERY_WHERE_ID = " WHERE elu.id_elu = ? ";
    private static final String SQL_QUERY_WHERE = " WHERE 1 ";
    private static final String SQL_QUERY_WHERE_ID_GROUPE = " WHERE grou.id_groupe = ? ";
    private static final String SQL_QUERY_FIND_ALL_ID_REMPLACE = " AND elu.id_elu IN ( SELECT ods_id_elu FROM ods_elu GROUP BY ods_id_elu )";
    private static final String SQL_QUERY_NEW_PK = "SELECT max( id_elu ) FROM ods_elu ";

    // Specifics
    /**
     * Supprime un elu des tables spcifiques
     * @param elu l'elu
     * @param transaction la transaction
     * @throws SQLException lors d'une erreur SQL
     */
    protected abstract void deleteSpec( GElu elu, Transaction transaction )
        throws SQLException;

    /**
     * Insert l'elu dans la table spcifique
     * @param elu l'elu
     * @param transaction la transaction
     * @param plugin le plugin
     * @throws SQLException lors d'une erreur SQL
     */
    protected abstract void insertSpec( GElu elu, Transaction transaction, Plugin plugin )
        throws SQLException;

    /**
     * Met  jour l'elu dans la table spcifique
     * @param elu l'elu
     * @param transaction la transaction
     * @throws SQLException lors d'une erreur SQL
     */
    protected abstract void updateSpec( GElu elu, Transaction transaction )
        throws SQLException;

    /**
     * Ajoute les valeurs spcifiques pour les paramtre spcifiques
     * @param elu l'elu
     * @param nIndex l'index d'entre
     * @param statement le statement
     * @return l'index de sortie
     * @throws SQLException lors d'une erreur SQL
     */
    protected abstract int setInsertOrUpdateValuesSpec( GElu elu, int nIndex, PreparedStatement statement )
        throws SQLException;

    /**
     * Remplit l'objet elu  partir des valeurs contenues dans le DAOUtil
     * @param elu l'elu
     * @param daoUtil DAOUtil
     * @param plugin le plugin
     */
    protected abstract void setEluValuesSpec( GElu elu, DAOUtil daoUtil, Plugin plugin );

    // SQL
    /**
     * Construit les champs spcifiques  rcuprer dans le select
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLSelectSpec( StringBuilder sbSQL );

    /**
     * Construit les jointures ou tables  ajouter dans la clause "FROM"
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLFromSpec( StringBuilder sbSQL );

    /**
     * Ajoute les filtres spcifiques
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLFiltersSpec( StringBuilder sbSQL );

    /**
     * Ajoute les champs spcifiques  insrer
     * @param sbSQL le buffer
     * @return le nombre de champs ajouts
     */
    protected abstract int buildSQLInsertSpec( StringBuilder sbSQL );

    /**
     * Ajoute les champs spcifiques  mettre  jour
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLStoreSpec( StringBuilder sbSQL );

    /**
     * Fixe les valeurs des filtres spcifiques
     * @param nIndex l'index de dpart
     * @param daoUtil DAOUtil
     */
    protected abstract void setFiltersSpec( int nIndex, DAOUtil daoUtil );

    /**
     * Cre une nouvelle instance de GElu
     * @return une nouvelle instance de GElu
     */
    protected abstract GElu newEluInstance(  );

    /**
    * Gnre un nouvel identifiant
    * @param plugin plugin
    * @return Retourne lidentifiant gnr
    */
    protected int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Supprime un lu de la base . Supprime galement dans les tables spcifiques.
     * @param elu l'elu
     * @param plugin le plugin
     * @throws AppException lors d'une erreur de transaction
     */
    public void delete( GElu elu, Plugin plugin ) throws AppException
    {
        Transaction transaction = new Transaction( plugin );

        try
        {
            deleteSpec( elu, transaction );

            transaction.prepareStatement( SQL_QUERY_DELETE );
            transaction.getStatement(  ).setInt( 1, elu.getIdElu(  ) );
            transaction.executeStatement(  );

            transaction.commit(  );
        }
        catch ( SQLException sqlEx )
        {
            transaction.rollback( sqlEx );
            throw new AppException( sqlEx.getMessage(  ), sqlEx );
        }
    }

    /**
     * Insre un lu en base (et galement dans les tables spcifiques)
     * @param elu l'elu
     * @param plugin le plugin
     * @return l'identifiant gnr
     */
    public int insert( GElu elu, Plugin plugin )
    {
        Transaction transaction = new Transaction( plugin );

        try
        {
            StringBuilder sbSQL = new StringBuilder(  );

            int nIdElu = newPrimaryKey( plugin );
            elu.setIdElu( nIdElu );
            sbSQL.append( SQL_QUERY_INSERT );

            int nbParameters = buildSQLInsertSpec( sbSQL );
            sbSQL.append( SQL_QUERY_INSERT_VALUES );

            for ( int nParameter = 0; nParameter < nbParameters; nParameter++ )
            {
                sbSQL.append( OdsConstants.CONSTANTE_VIRGULE );
                sbSQL.append( OdsConstants.CONSTANTE_POINT_INTERROGATION );
            }

            sbSQL.append( OdsConstants.CONSTANTE_FERMETURE_ACCOLADE );

            transaction.prepareStatement( sbSQL.toString(  ) );

            int nIndex = 1;
            transaction.getStatement(  ).setInt( nIndex, elu.getIdElu(  ) );
            nIndex++;
            nIndex = setInsertOrUpdateValues( elu, nIndex, transaction.getStatement(  ) );
            nIndex = setInsertOrUpdateValuesSpec( elu, nIndex, transaction.getStatement(  ) );

            transaction.executeStatement(  );

            insertSpec( elu, transaction, plugin );

            transaction.commit(  );
        }
        catch ( SQLException sqlEx )
        {
            transaction.rollback(  );
            throw new AppException( sqlEx.getMessage(  ), sqlEx );
        }

        return elu.getIdElu(  );
    }

    /**
     * Charge un lu  partir de la cl.
     * @param nKey la cl
     * @param plugin le plugin
     * @return l'elu
     */
    public GElu load( int nKey, Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLSelectSpec( sbSQL );
        sbSQL.append( SQL_QUERY_SELECT_FROM );
        buildSQLFromSpec( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE_ID );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        int nIndex = 1;
        daoUtil.setInt( nIndex, nKey );

        daoUtil.executeQuery(  );

        GElu elu = null;

        if ( daoUtil.next(  ) )
        {
            elu = newEluInstance(  );
            setEluValues( elu, daoUtil );
            setEluValuesSpec( elu, daoUtil, plugin );
        }

        daoUtil.free(  );

        return elu;
    }

    /**
     * Retourne la liste des lus actifs
     * @param plugin le plugin
     * @return la liste des lus actifs
     */
    public List<GElu> selectEluActifs( Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLSelectSpec( sbSQL );
        sbSQL.append( SQL_QUERY_SELECT_FROM );
        buildSQLFromSpec( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE_ACTIF );
        buildSQLFiltersSpec( sbSQL );
        sbSQL.append( SQL_QUERY_ORDER_BY_NOM_ELU );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        setFiltersSpec( 1, daoUtil );
        daoUtil.executeQuery(  );

        List<GElu> elusList = new ArrayList<GElu>(  );
        GElu elu = null;

        while ( daoUtil.next(  ) )
        {
            elu = newEluInstance(  );
            setEluValues( elu, daoUtil );
            setEluValuesSpec( elu, daoUtil, plugin );
            elusList.add( elu );
        }

        daoUtil.free(  );

        return elusList;
    }

    /**
     * Retourne la liste des lus
     * @param plugin le plugin
     * @return la liste des lus
     */
    public List<GElu> selectEluList( Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLSelectSpec( sbSQL );
        sbSQL.append( SQL_QUERY_SELECT_FROM );
        buildSQLFromSpec( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE );
        buildSQLFiltersSpec( sbSQL );
        sbSQL.append( SQL_QUERY_ORDER_BY_NOM_ELU );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        setFiltersSpec( 1, daoUtil );
        daoUtil.executeQuery(  );

        List<GElu> elusList = new ArrayList<GElu>(  );
        GElu elu = null;

        while ( daoUtil.next(  ) )
        {
            elu = newEluInstance(  );
            setEluValues( elu, daoUtil );
            setEluValuesSpec( elu, daoUtil, plugin );
            elusList.add( elu );
        }

        daoUtil.free(  );

        return elusList;
    }

    /**
     * Retourne la liste des lus correpondant au groupe dont la cls est passe en paramtre.
     * @param nKey la cl du groupe
     * @param plugin le plugin
     * @return la liste des lus correspondant au groupe
     */
    public List<GElu> selectEluListbyIdGroupe( int nKey, Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLSelectSpec( sbSQL );
        sbSQL.append( SQL_QUERY_SELECT_FROM );
        buildSQLFromSpec( sbSQL );
        buildSQLFiltersSpec( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE_ID_GROUPE );
        sbSQL.append( SQL_QUERY_ORDER_BY_NOM_ELU );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        setFiltersSpec( 1, daoUtil );
        daoUtil.setInt( 2, nKey );
        daoUtil.executeQuery(  );

        List<GElu> elusList = new ArrayList<GElu>(  );
        GElu elu = null;

        while ( daoUtil.next(  ) )
        {
            elu = newEluInstance(  );
            setEluValues( elu, daoUtil );
            setEluValuesSpec( elu, daoUtil, plugin );
            elusList.add( elu );
        }

        daoUtil.free(  );

        return elusList;
    }

    /**
     * Ajoute les tables et jointures spcifiques pour la slection des rapporteurs
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLFromRapporteursSpec( StringBuilder sbSQL );

    /**
     * Ajoute les champs spcifiques pour la slection des rapporteurs
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLSelectRapporteursSpec( StringBuilder sbSQL );

    /**
     * Ajoute les filtres spcifiques pour la slection des rapporteurs
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLFilterRapporteursSpec( StringBuilder sbSQL );

    /**
     * Retourne la liste des rapporteurs
     * @param plugin le plugin
     * @return la liste des lus
     */
    public List<GElu> selectElusRapporteurs( Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLSelectSpec( sbSQL );
        buildSQLSelectRapporteursSpec( sbSQL );
        sbSQL.append( SQL_QUERY_SELECT_FROM );
        buildSQLFromSpec( sbSQL );
        buildSQLFromRapporteursSpec( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE );
        buildSQLFiltersSpec( sbSQL );
        buildSQLFilterRapporteursSpec( sbSQL );
        sbSQL.append( SQL_QUERY_ORDER_BY_NOM_ELU );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        setFiltersSpec( 1, daoUtil );
        daoUtil.executeQuery(  );

        GElu elu;
        List<GElu> elusList = new ArrayList<GElu>(  );

        while ( daoUtil.next(  ) )
        {
            elu = newEluInstance(  );
            setEluValues( elu, daoUtil );
            setEluValuesSpec( elu, daoUtil, plugin );

            elusList.add( elu );
        }

        daoUtil.free(  );

        return elusList;
    }

    /**
     * Renvoie la liste des Elus rapporteurs actifs filtres par formation conseil
     * @param plugin le plugin
     * @return la liste des Elus rapporteurs actifs filtres par formation conseil
     */
    public List<GElu> selectElusRapporteursActifs( Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLSelectSpec( sbSQL );
        buildSQLSelectRapporteursSpec( sbSQL );
        sbSQL.append( SQL_QUERY_SELECT_FROM );
        buildSQLFromSpec( sbSQL );
        buildSQLFromRapporteursSpec( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE_ACTIF );
        buildSQLFiltersSpec( sbSQL );
        buildSQLFilterRapporteursSpec( sbSQL );
        sbSQL.append( SQL_QUERY_ORDER_BY_NOM_ELU );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        setFiltersSpec( 1, daoUtil );
        daoUtil.executeQuery(  );

        GElu elu;
        List<GElu> elusList = new ArrayList<GElu>(  );

        while ( daoUtil.next(  ) )
        {
            elu = newEluInstance(  );
            setEluValues( elu, daoUtil );
            setEluValuesSpec( elu, daoUtil, plugin );

            elusList.add( elu );
        }

        daoUtil.free(  );

        return elusList;
    }

    /**
    * Retourne la liste des lus actifs remplaants d'autres lus
    * @param plugin le plugin
    * @return la liste des lus actifs remplaants d'autres lus
    */
    public List<GElu> selectEluRemplacantsActifs( Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLSelectSpec( sbSQL );
        sbSQL.append( SQL_QUERY_SELECT_FROM );
        buildSQLFromSpec( sbSQL );
        buildSQLFromRapporteursSpec( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE_ACTIF );
        buildSQLFiltersSpec( sbSQL );
        sbSQL.append( SQL_QUERY_FIND_ALL_ID_REMPLACE );
        sbSQL.append( SQL_QUERY_ORDER_BY_NOM_ELU );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        setFiltersSpec( 1, daoUtil );
        daoUtil.executeQuery(  );

        GElu elu;
        List<GElu> elusList = new ArrayList<GElu>(  );

        while ( daoUtil.next(  ) )
        {
            elu = newEluInstance(  );
            setEluValues( elu, daoUtil );
            setEluValuesSpec( elu, daoUtil, plugin );

            elusList.add( elu );
        }

        daoUtil.free(  );

        return elusList;
    }

    /**
     * Met  jour l'lu en base (et dans les tables spcifiques)
     * @param elu l'elu
     * @param plugin le plugin
     */
    public void store( GElu elu, Plugin plugin )
    {
        Transaction transaction = new Transaction( plugin );

        try
        {
            StringBuilder sbSQL = new StringBuilder(  );

            sbSQL.append( SQL_QUERY_UPDATE );
            buildSQLStoreSpec( sbSQL );
            sbSQL.append( SQL_QUERY_UPDATE_WHERE );

            transaction.prepareStatement( sbSQL.toString(  ) );

            int nIndex = 1;
            nIndex = setInsertOrUpdateValues( elu, nIndex, transaction.getStatement(  ) );
            nIndex = setInsertOrUpdateValuesSpec( elu, nIndex, transaction.getStatement(  ) );
            transaction.getStatement(  ).setInt( nIndex, elu.getIdElu(  ) );
            nIndex++;

            transaction.executeStatement(  );

            updateSpec( elu, transaction );

            transaction.commit(  );
        }
        catch ( SQLException sqlEx )
        {
            transaction.rollback(  );
            throw new AppException( sqlEx.getMessage(  ), sqlEx );
        }
    }

    /**
     * Fixe les valeurs pour l'insertion ou la mise  jour d'un lu dans le requte
     * @param elu l'elu
     * @param nIndex l'index de dpart
     * @param statement le statement
     * @return l'index de sortie
     * @throws SQLException lors d'une erreur SQL
     */
    protected int setInsertOrUpdateValues( GElu elu, int nIndex, PreparedStatement statement )
        throws SQLException
    {
        int nIndexCopy = nIndex;
        statement.setInt( nIndexCopy, elu.getGroupe(  ).getIdGroupe(  ) );
        nIndexCopy++;
        statement.setString( nIndexCopy, elu.getCivilite(  ) );
        nIndexCopy++;
        statement.setString( nIndexCopy, elu.getNomElu(  ) );
        nIndexCopy++;
        statement.setString( nIndexCopy, elu.getPrenomElu(  ) );
        nIndexCopy++;
        statement.setBoolean( nIndexCopy, elu.isActif(  ) );
        nIndexCopy++;

        if ( elu.getEluRemplace(  ) != null )
        {
            statement.setInt( nIndexCopy, elu.getEluRemplace(  ).getIdElu(  ) );
        }
        else
        {
            statement.setNull( nIndexCopy, Types.INTEGER );
        }

        nIndexCopy++;

        return nIndexCopy;
    }

    /**
     * REmplit l'objet Elu  partir de DAOUtil
     * @param elu l'elu
     * @param daoUtil daoUtil
     */
    protected void setEluValues( GElu elu, DAOUtil daoUtil )
    {
        elu.setIdElu( daoUtil.getInt( "elu.id_elu" ) );
        elu.setCivilite( daoUtil.getString( "elu.civilite" ) );
        elu.setNomElu( daoUtil.getString( "elu.nom_elu" ) );
        elu.setPrenomElu( daoUtil.getString( "elu.prenom_elu" ) );
        elu.setActif( daoUtil.getBoolean( "elu.actif" ) );

        // elu remplace
        if ( daoUtil.getObject( "elu.ods_id_elu" ) != null )
        {
            GElu eluRemplace = newEluInstance(  );
            eluRemplace.setIdElu( daoUtil.getInt( "elu.ods_id_elu" ) );
            elu.setEluRemplace( eluRemplace );
        }

        //on recupere le groupe politique de l'lu en crant un objet GroupePolitique et on l'affecte  l'objet elu
        GroupePolitique groupePolitique = new GroupePolitique(  );
        groupePolitique.setIdGroupe( daoUtil.getInt( "grou.id_groupe" ) );
        groupePolitique.setNomGroupe( daoUtil.getString( "grou.nom_groupe" ) );
        groupePolitique.setNomComplet( daoUtil.getString( "grou.nom_complet" ) );
        elu.setGroupe( groupePolitique );
    }
}
