/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.expression;

import fr.paris.lutece.plugins.ods.dto.expression.IExpressionUsuelle;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;
import fr.paris.lutece.util.sql.Transaction;

import java.sql.SQLException;

import java.util.ArrayList;
import java.util.List;


/**
 *
 * AbstractExpressionUsuelleDAO
 *
 * @param <GExpressionUsuelle>
 */
public abstract class AbstractExpressionUsuelleDAO<GExpressionUsuelle extends IExpressionUsuelle>
    implements IExpressionUsuelleDAO<GExpressionUsuelle>
{
    private static final String SQL_QUERY_NEW_PK = "SELECT max( id_expression ) FROM ods_expression_usuelle ";
    private static final String SQL_QUERY_INSERT = "INSERT INTO ods_expression_usuelle(" +
        " id_expression, expression) VALUES (?, ?) ";
    private static final String SQL_QUERY_UPDATE = "UPDATE ods_expression_usuelle " + "SET expression = ? " +
        " WHERE id_expression=? ";
    private static final String SQL_QUERY_DELETE = "DELETE FROM ods_expression_usuelle WHERE id_expression = ? ";
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = "SELECT id_expression, expression " +
        " FROM ods_expression_usuelle WHERE id_expression = ? ";
    private static final String SQL_QUERY_FIND_ALL = "SELECT exp.id_expression, exp.expression " +
        " FROM ods_expression_usuelle exp";
    private static final String SQL_QUERY_ORDER_BY_EXPRESSION = " ORDER BY exp.expression ASC ";

    /**
     * Insertions spcifiques
     * @param expression l'expression
     * @param transaction la transaction
     * @throws SQLException en cas d'erreur SQL
     */
    protected abstract void doInsertSpec( GExpressionUsuelle expression, Transaction transaction )
        throws SQLException;

    /**
     * Mise  jour spcifique
     * @param expression l'expression
     * @param transaction la transaction
     * @throws SQLException en cas d'erreur SQL
     */
    protected abstract void doUpdateSpec( GExpressionUsuelle expression, Transaction transaction )
        throws SQLException;

    /**
     * Retourne la compltion du select pour la liste des expression
     * @return la compltion du select pour la liste des expression
     */
    protected abstract String loadListExpressionsQuerySpec(  );

    /**
     * Remplit les paramtres du DAOUtil pour la liste des expression
     * @param daoUtil le daoUtil
     */
    protected abstract void loadListExpressionsDaoutilSpec( DAOUtil daoUtil );

    /**
     * Gnre une nouvelle instance d'expression usuelle
     * @return  une nouvelle instance d'expression usuelle
     */
    protected abstract GExpressionUsuelle newExpressionUsuelleInstance(  );

    /**
     * Retourne une nouvelle cl primaire unique sur la table ods_expression_usuelle.
     * @param plugin le plugin actif
     * @return nouvelle cl primaire
     */
    private int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;

        daoUtil.free(  );

        return nKey;
    }

    /**
     * Cre une nouvelle expression usuelle dans la table ods_expression_usuelle
     * @param expression l'expression  insrer
     * @param plugin le Plugin
     */
    public void insert( GExpressionUsuelle expression, Plugin plugin )
    {
        expression.setIdExpression( newPrimaryKey( plugin ) );

        Transaction transaction = new Transaction( plugin );

        try
        {
            StringBuilder sbSQL = new StringBuilder(  );
            sbSQL.append( SQL_QUERY_INSERT );
            transaction.prepareStatement( sbSQL.toString(  ) );
            transaction.getStatement(  ).setInt( 1, expression.getIdExpression(  ) );
            transaction.getStatement(  ).setString( 2, expression.getExpression(  ) );

            transaction.executeStatement(  );

            doInsertSpec( expression, transaction );

            // Commit
            transaction.commit(  );
        }
        catch ( SQLException e )
        {
            // Rollback
            transaction.rollback( e );
        }
    }

    /**
     * Met  jour les informations d'une expression usuelle dans la table ods_expression_usuelle
     * @param expression la nouvelle expression
     * @param plugin le Plugin
     */
    public void store( GExpressionUsuelle expression, Plugin plugin )
    {
        Transaction transaction = new Transaction( plugin );

        try
        {
            StringBuilder sbSQL = new StringBuilder(  );
            sbSQL.append( SQL_QUERY_UPDATE );
            transaction.prepareStatement( sbSQL.toString(  ) );
            transaction.getStatement(  ).setInt( 2, expression.getIdExpression(  ) );
            transaction.getStatement(  ).setString( 1, expression.getExpression(  ) );

            transaction.executeStatement(  );

            doUpdateSpec( expression, transaction );

            // Commit
            transaction.commit(  );
        }
        catch ( SQLException e )
        {
            // Rollback
            transaction.rollback( e );
        }
    }

    /**
     * Supprime une expression usuelle de la table ods_expression_usuelle
     * @param expression l'expression  supprimer
     * @param plugin le Plugin
     * @throws AppException Exception Sql si l'expression ne peut tre supprime
     */
    public void delete( GExpressionUsuelle expression, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, expression.getIdExpression(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Retourne l'expression usuelle associe  la cl primaire passe en argument de la mthode.
     * @param nKey l'identifiant de l'expression dans la table ods_expression_usuelle
     * @param plugin le Plugin
     * @return l'expression usuelle associe  la cl primaire passe en argument de la mthode
     */
    public GExpressionUsuelle load( int nKey, Plugin plugin )
    {
        GExpressionUsuelle expression = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            expression = newExpressionUsuelleInstance(  );
            expression.setIdExpression( daoUtil.getInt( 1 ) );
            expression.setExpression( daoUtil.getString( 2 ) );
        }

        daoUtil.free(  );

        return expression;
    }

    /**
     * Retourne la liste des expressions usuelles.
     * @param plugin le Plugin
     * @return la liste des expressions usuelles
     */
    public List<GExpressionUsuelle> loadListExpressions( Plugin plugin )
    {
        List<GExpressionUsuelle> listExpression = new ArrayList<GExpressionUsuelle>(  );

        String strSQL = SQL_QUERY_FIND_ALL;
        strSQL += loadListExpressionsQuerySpec(  );
        strSQL += SQL_QUERY_ORDER_BY_EXPRESSION;

        DAOUtil daoUtil = new DAOUtil( strSQL, plugin );
        loadListExpressionsDaoutilSpec( daoUtil );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            GExpressionUsuelle expression = newExpressionUsuelleInstance(  );
            expression.setIdExpression( daoUtil.getInt( 1 ) );
            expression.setExpression( daoUtil.getString( 2 ) );
            listExpression.add( expression );
        }

        daoUtil.free(  );

        return listExpression;
    }
}
