/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.fichier;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.List;


/**
 * 
 * @author rFlachaire
 *
 * @param <GFichier> IFichier
 * @param <GFichierFilter> IFichierFilter
 * @param <GTypeDocument> ITypeDocument
 * @param <GSeance> ISeance
 * @param <GElu> IELu
 * @param <GVoeuAmendement> IVoeuAmendement
 * @param <GPDD> IPDD
 */
public abstract class AbstractFichierDAO<GFichier extends IFichier<GSeance, GFichier>, GFichierFilter extends IFichierFilter, GTypeDocument extends ITypeDocument, GSeance extends ISeance, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>>
    implements IFichierDAO<GFichier, GFichierFilter, GTypeDocument, GSeance>
{
    protected static final String SQL_QUERY_ORDER_BY_LIBELLE = " ORDER BY libelle_type ";
    private static final String SQL_QUERY_SELECT = "SELECT fc.id_document, fc.texte, fc.version, fc.extension, " +
        " fc.taille, fc.nom, fc.en_ligne, fc.date_publication, fc.intitule, fc.id_fichier, fc.id_type_document, " +
        " fc.id_seance, fc.id_pdd, fc.id_formation_conseil, est_creation_prochaine_seance, est_creation_aval, " +
        " td.libelle_type, " +
        " pdd.reference, pdd.type_pdd, pdd.delegations_services, pdd.mode_introduction, pdd.objet, pdd.pieces_manuelles, pdd.date_vote, " +
        " pdd.en_ligne, pdd.date_publication, pdd.version, pdd.id_formation_conseil ";
    private static final String SQL_QUERY_FROM = "FROM ods_fichier fc " +
        "LEFT JOIN ods_type_document td on (fc.id_type_document = td.id_type_document) " +
        "LEFT JOIN ods_seance seance on (fc.id_seance = seance.id_seance) " +
        "LEFT JOIN ods_pdd pdd on (fc.id_pdd = pdd.id_pdd) ";
    private static final String SQL_QUERY_WHERE_ID_DOCUMENT = "WHERE fc.id_document = ? ";
    private static final String SQL_QUERY_WHERE = " WHERE 1 ";
    private static final String SQL_QUERY_ORDER_BY_INTITULE = " ORDER BY fc.intitule ";
    private static final String SQL_FILTER_SEANCE = " AND fc.id_seance = ? ";
    private static final String SQL_FILTER_TYPE_DOCUMENT = " AND fc.id_type_document = ? ";
    private static final String SQL_FILTER_PUBLICATION = " AND fc.en_ligne = ? ";
    private static final String SQL_FILTER_PDD = " AND fc.id_pdd = ? ";
    private static final String SQL_FILTER_INTITULE = " AND fc.intitule = ? ";
    private static final String SQL_FILTER_FORMATION_CONSEIL = " AND fc.id_formation_conseil = ? ";
    private static final String SQL_FILTER_DATE_PUBLICATION = " AND fc.date_publication > ? ";
    private static final String SQL_FILTER_DATE_PUBLICATION_BEFORE = " AND fc.date_publication < ? ";
    private static final String SQL_FILTER_OR_ID_TYPE_DOC = " OR fc.id_type_document = ? ";
    private static final String SQL_FILTER_AND_ID_TYPE_DOC = " AND ( fc.id_type_document = ? ";
    private static final String ORDER_BY_LIST = " ORDER BY ";
    private static final String SQL_QUERY_TYPE_DOCUMENT_BY_PRIMARY_KEY = " SELECT id_type_document, libelle_type, est_creation_prochaine_seance, est_creation_aval FROM ods_type_document WHERE id_type_document = ? ";
    private static final String SQL_QUERY_TYPE_DOCUMENT_LIST_PROCHAINE_SEANCE = " SELECT id_type_document, libelle_type, est_creation_prochaine_seance, est_creation_aval " +
        "FROM ods_type_document WHERE est_creation_prochaine_seance = 1 " + SQL_QUERY_ORDER_BY_LIBELLE;
    private static final String SQL_QUERY_TYPE_DOCUMENT_LIST_AVAL = " SELECT id_type_document, libelle_type, est_creation_prochaine_seance, est_creation_aval" +
        " FROM ods_type_document WHERE est_creation_aval=1 AND id_type_document NOT IN (" +
        ITypeDocument.DELIBERATION_DESIGNATION + ", " + ITypeDocument.ARRETE + ", " +
        ITypeDocument.LISTE_MEMBRE_PRESENT + ") " + SQL_QUERY_ORDER_BY_LIBELLE;
    private static final String SQL_QUERY_NEW_PK = " SELECT max( id_document ) FROM ods_fichier ";
    private static final String SQL_QUERY_INSERT = " INSERT INTO ods_fichier (id_document, id_fichier, id_pdd, id_type_document, id_seance, intitule, texte, en_ligne, extension, taille, nom, version,id_formation_conseil ";
    private static final String SQL_QUERY_INSERT_VALUES = " VALUES ( ?,?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?";
    private static final String SQL_QUERY_PUBLIER = " UPDATE ods_fichier SET date_publication = ?, en_ligne = ?, version = ? WHERE id_document= ?  ";
    private static final String SQL_QUERY_UPDATE = " UPDATE ods_fichier SET date_publication = ?, id_type_document = ?, id_seance = ?, intitule = ?, texte = ?, en_ligne=?, extension = ?, taille = ?, nom = ? , version = ?, id_formation_conseil=? ";
    private static final String SQL_QUERY_UPDATE_WHERE = " WHERE id_document= ?";
    private static final String SQL_QUERY_DELETE = " DELETE FROM ods_fichier WHERE id_document = ?  ";
    private static final String EST_CREATION_AVAL = "est_creation_aval";
    private static final String EST_CREATION_PROCHAINE_SEANCE = "est_creation_prochaine_seance";
    private static final String LIBELLE_TYPE = "libelle_type";
    private static final String ID_TYPE_DOCUMENT = "id_type_document";

    /**
     * Cre une novuelle instance de sance
     * @return une novuelle instance de sance
     */
    protected abstract GSeance newSeanceInstance(  );

    /**
     * Cre une nouvelle instance de PDD
     * @return une nouvelle instance de PDD
     */
    protected abstract GPDD newPDDInstance(  );

    /**
     * Cre une nouvelle instance de TypeDeDocument
     * @return une nouvelle instance de TypeDeDocument
     */
    protected abstract GTypeDocument getTypeDocument(  );

    /**
     * Renseigne les valeurs spcifiques dans le DAOUtil pour l'insertion ou la mise  jour
     * @param nStartIndex l'index de dpart
     * @param fichier le fichier
     * @param daoUtil daoutil
     * @return l'index de sortie
     */
    protected abstract int setInsertOrUpdateValuesSpec( int nStartIndex, GFichier fichier, DAOUtil daoUtil );

    /**
     * Renseigne les valeurs du fichier  partir de DAOUtil
     * @param fichier le fichier
     * @param daoUtil daoUtil
     */
    protected abstract void setFichierValuesSpec( GFichier fichier, DAOUtil daoUtil );

    /**
     * Ajoute les champs spcifique  la requte d'insertion
     * @param sbSQL le buffer
     * @return le nombre de champs rajouts
     */
    protected abstract int buildSQLInsertSpec( StringBuilder sbSQL );

    /**
     * Ajoute les champs spcifique  la requte de mise  jour
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLUpdateSpec( StringBuilder sbSQL );

    /**
     * Ajoute les filters spcifiques  la requte
     * @param filter le filtre
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLFiltersSpec( GFichierFilter filter, StringBuilder sbSQL );

    /**
     * Renseignes les valeurs spcifique dans le DAOUtil
     * @param nStartIndex l'index de dpart
     * @param filter le filtre
     * @param daoUtil le daoUtil
     * @return l'index de sortie
     */
    protected abstract int setFilterValuesSpec( int nStartIndex, GFichierFilter filter, DAOUtil daoUtil );

    /**
     * Ajoute les champs spcifiques  rcuprer dans le select
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLSelectSpec( StringBuilder sbSQL );

    /**
     * Ajoute les tables et/ou jointures  rajouter dans le select
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLFromSpec( StringBuilder sbSQL );

    /**
     * Retourne le filtre sur les autres documents de sance
     * @return le filtre sur les autres documents de sance
     */
    protected abstract String getFilterAutresDocumentsSeances(  );

    /**
     * Retourne le filtre sur les autres documents de sance passe
     * @return le filtre sur les autres documents de sance passe
     */
    protected abstract String getFilterAutresDocumentsGestionAval(  );

    /**
     * Permet de complter le fichier avec les proprits qui ont des contraintes sur d'autres tables
     * @param daoUtil contient les informations complmentaires du fichier
     * @param fichier fichier  complter  partir des informations du daoUtil
     */
    protected abstract void setOtherPropertyFichierSpec( DAOUtil daoUtil, GFichier fichier );

    /**
     * Renseigne les valeurs du fichier dans le daoUtil
     * @param nStartIndex l'index de dpart
     * @param fichier le fichier
     * @param daoUtil daoUtil
     * @return l'index de sortie
     */
    protected int setInsertOrUpdateValues( int nStartIndex, GFichier fichier, DAOUtil daoUtil )
    {
        int nIndex = nStartIndex;

        if ( fichier.getTypdeDocument(  ) == null )
        {
            daoUtil.setIntNull( nIndex );
        }
        else
        {
            daoUtil.setInt( nIndex, fichier.getTypdeDocument(  ).getId(  ) );
        }

        nIndex++;

        if ( fichier.getSeance(  ) == null )
        {
            daoUtil.setIntNull( nIndex );
        }
        else
        {
            daoUtil.setInt( nIndex, fichier.getSeance(  ).getIdSeance(  ) );
        }

        nIndex++;

        daoUtil.setString( nIndex, fichier.getIntitule(  ) );
        nIndex++;
        daoUtil.setString( nIndex, fichier.getTexte(  ) );
        nIndex++;
        daoUtil.setBoolean( nIndex, fichier.getEnLigne(  ) );
        nIndex++;
        daoUtil.setString( nIndex, fichier.getExtension(  ) );
        nIndex++;
        daoUtil.setInt( nIndex, fichier.getTaille(  ) );
        nIndex++;
        daoUtil.setString( nIndex, fichier.getNom(  ) );
        nIndex++;

        if ( fichier.getVersion(  ) > 0 )
        {
            daoUtil.setInt( nIndex, fichier.getVersion(  ) );
        }
        else
        {
            daoUtil.setIntNull( nIndex );
        }

        nIndex++;

        return nIndex;
    }

    /**
    * Cre un nouveau fichier  partir de lobjet fichier pass en paramtre
    * @param fichier le fichier  insrer
    * @param plugin le plugin
    * @return int primaryKey
    */
    public int insert( GFichier fichier, Plugin plugin )
    {
        int newPrimaryKey = newPrimaryKey( plugin );

        StringBuilder sbSQL = new StringBuilder( SQL_QUERY_INSERT );
        int nNbParameters = buildSQLInsertSpec( sbSQL );
        sbSQL.append( OdsConstants.CONSTANTE_FERMETURE_ACCOLADE );
        sbSQL.append( SQL_QUERY_INSERT_VALUES );

        for ( int nParameter = 0; nParameter < nNbParameters; nParameter++ )
        {
            sbSQL.append( OdsConstants.CONSTANTE_VIRGULE );
            sbSQL.append( OdsConstants.CONSTANTE_POINT_INTERROGATION );
        }

        sbSQL.append( OdsConstants.CONSTANTE_FERMETURE_ACCOLADE );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        fichier.setId( newPrimaryKey );

        int nIndex = 1;

        daoUtil.setInt( nIndex, fichier.getId(  ) );
        nIndex++;

        if ( fichier.getFichier(  ) == null )
        {
            daoUtil.setIntNull( nIndex );
        }
        else
        {
            daoUtil.setInt( nIndex, fichier.getFichier(  ).getIdFichier(  ) );
        }

        nIndex++;

        if ( fichier.getPDD(  ) == null )
        {
            daoUtil.setIntNull( nIndex );
        }
        else
        {
            daoUtil.setInt( nIndex, fichier.getPDD(  ).getId(  ) );
        }

        nIndex++;

        nIndex = setInsertOrUpdateValues( nIndex, fichier, daoUtil );
        nIndex = setInsertOrUpdateValuesSpec( nIndex, fichier, daoUtil );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );

        return newPrimaryKey;
    }

    /**
     * Modifie le fichier correspondant  lobjet fichier pass en paramtre
     * @param fichier le fichier  modifier
     * @param plugin le plugin
     */
    public void store( GFichier fichier, Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder( SQL_QUERY_UPDATE );
        buildSQLUpdateSpec( sbSQL );
        sbSQL.append( SQL_QUERY_UPDATE_WHERE );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;
        daoUtil.setTimestamp( nIndex, fichier.getDatePublication(  ) );
        nIndex++;

        nIndex = setInsertOrUpdateValues( nIndex, fichier, daoUtil );
        nIndex = setInsertOrUpdateValuesSpec( nIndex, fichier, daoUtil );

        daoUtil.setInt( nIndex, fichier.getId(  ) );
        nIndex++;

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Supprime le fichier correspondant  lobjet fichier pass en paramtre
     * @param fichier le fichier  supprimer
     * @param plugin le plugin
     * @throws AppException si contrainte de cl trangre
     */
    public void delete( GFichier fichier, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, fichier.getId(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Retourne la liste des fichiers correspondants aux critres du filtre.
     * @param filter les critres de recherche du fichier
     * @param plugin le plugin
     * @return la liste de fichiers correspondants aux critres de recherche
     */
    public List<GFichier> selectByFilter( GFichierFilter filter, Plugin plugin )
    {
        List<GFichier> fichiers = new ArrayList<GFichier>(  );

        StringBuilder sbSQL = new StringBuilder( SQL_QUERY_SELECT );
        buildSQLSelectSpec( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLFromSpec( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE );

        sbSQL.append( ( ( filter.containsSeanceCriteria(  ) ) ? SQL_FILTER_SEANCE : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsTypeDocumentCriteria(  ) ) ? SQL_FILTER_TYPE_DOCUMENT
                                                                    : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsPublicationCriteria(  ) ) ? SQL_FILTER_PUBLICATION
                                                                   : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsPDDCriteria(  ) ) ? SQL_FILTER_PDD : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        sbSQL.append( ( ( filter.containsIntituleCriteria(  ) ) ? SQL_FILTER_INTITULE : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsAutreDocumentsSeanceCriteria(  ) ) ? getFilterAutresDocumentsSeances(  )
                                                                            : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsIdFormationConseilCriteria(  ) ) ? SQL_FILTER_FORMATION_CONSEIL
                                                                          : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        if ( filter.containsDatePublicationCriteria(  ) )
        {
            if ( filter.containsFilterTypeDocumentCriteria(  ) && ( filter.getFilterTypeDocument(  ) == 1 ) )
            {
                sbSQL.append( SQL_FILTER_DATE_PUBLICATION_BEFORE );
            }
            else if ( filter.containsFilterTypeDocumentCriteria(  ) && ( filter.getFilterTypeDocument(  ) == 2 ) )
            {
                sbSQL.append( SQL_FILTER_DATE_PUBLICATION );
            }
        }
        else
        {
            sbSQL.append( OdsConstants.CONSTANTE_CHAINE_VIDE );
        }

        if ( filter.containsFilterTypeDocumentCriteria(  ) )
        {
            if ( filter.getFilterTypeDocument(  ) == 1 )
            {
                sbSQL.append( getFilterAutresDocumentsSeances(  ) );
            }
            else if ( filter.getFilterTypeDocument(  ) == 2 )
            {
                sbSQL.append( getFilterAutresDocumentsGestionAval(  ) );
            }
        }

        if ( filter.containsListeTypesDocumentsCriteria(  ) )
        {
            List<ITypeDocument> listeTypesDoc = filter.getTypesDocList(  );
            int nNombreFiltresTypeDoc = listeTypesDoc.size(  );
            sbSQL.append( SQL_FILTER_AND_ID_TYPE_DOC );

            for ( int n = 1; n < nNombreFiltresTypeDoc; n++ )
            {
                sbSQL.append( SQL_FILTER_OR_ID_TYPE_DOC );
            }

            sbSQL.append( " ) " );
        }

        buildSQLFiltersSpec( filter, sbSQL );

        // order By
        if ( filter.containsOrderByCriteria(  ) )
        {
            sbSQL.append( buildOrderBy( filter ) );
        }
        else
        {
            sbSQL.append( SQL_QUERY_ORDER_BY_INTITULE );
        }

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        int nIndex = 1;

        if ( filter.containsSeanceCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdSeance(  ) );
            nIndex++;
        }

        if ( filter.containsTypeDocumentCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdTypeDocument(  ) );
            nIndex++;
        }

        if ( filter.containsPublicationCriteria(  ) )
        {
            daoUtil.setBoolean( nIndex, filter.isPublication(  ) );
            nIndex++;
        }

        if ( filter.containsPDDCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdPDD(  ) );
            nIndex++;
        }

        if ( filter.containsIntituleCriteria(  ) )
        {
            daoUtil.setString( nIndex, filter.getIntitule(  ) );
            nIndex++;
        }

        if ( filter.containsIdFormationConseilCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdFormationConseil(  ) );
            nIndex++;
        }

        if ( filter.containsDatePublicationCriteria(  ) )
        {
            daoUtil.setTimestamp( nIndex, filter.getDatePublication(  ) );
            nIndex++;
        }

        if ( filter.containsListeTypesDocumentsCriteria(  ) )
        {
            for ( ITypeDocument typeDoc : filter.getTypesDocList(  ) )
            {
                daoUtil.setInt( nIndex, typeDoc.getId(  ) );
                nIndex++;
            }
        }

        nIndex = setFilterValuesSpec( nIndex, filter, daoUtil );

        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            GFichier fichier = getBasePropertyFichier( daoUtil );
            setOtherPropertyFichier( daoUtil, fichier );
            setOtherPropertyFichierSpec( daoUtil, fichier );

            fichiers.add( fichier );
        }

        daoUtil.free(  );

        return fichiers;
    }

    /**
     * Retourne le Fichier ayant comme identifiant nKey
     * @param nKey identifiant du fichier  charger
     * @param plugin le plugin
     * @return le fichier identifi par nKey
     */
    public GFichier load( int nKey, Plugin plugin )
    {
        GFichier fichier = null;
        StringBuilder sbSQL = new StringBuilder( SQL_QUERY_SELECT );
        buildSQLSelectSpec( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLFromSpec( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE_ID_DOCUMENT );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            fichier = newFichierInstance(  );
            fichier = getBasePropertyFichier( daoUtil );
            setOtherPropertyFichier( daoUtil, fichier );
        }

        daoUtil.free(  );

        return fichier;
    }

    /**
     * Retourne la liste des types de documents
     * @param plugin le plugin
     * @return la liste des objets TypeDocument
     */
    public List<GTypeDocument> loadTypeDocumentList( Plugin plugin )
    {
        List<GTypeDocument> typeDocuments = new ArrayList<GTypeDocument>(  );

        StringBuilder sbSQL = new StringBuilder( SQL_QUERY_SELECT );
        buildSQLSelectSpec( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLFromSpec( sbSQL );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            GTypeDocument typeDocument = setTypeDocumentValues( daoUtil );

            typeDocuments.add( typeDocument );
        }

        daoUtil.free(  );

        return typeDocuments;
    }

    /**
     * Retourne la liste des types de documents pour la prochaine sance
     * @param plugin le plugin
     * @return la liste des objets TypeDocument
     */
    public List<GTypeDocument> loadTypeDocumentProchaineSeanceList( Plugin plugin )
    {
        List<GTypeDocument> typeDocuments = new ArrayList<GTypeDocument>(  );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_TYPE_DOCUMENT_LIST_PROCHAINE_SEANCE, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            GTypeDocument typeDocument = setTypeDocumentValues( daoUtil );

            typeDocuments.add( typeDocument );
        }

        daoUtil.free(  );

        return typeDocuments;
    }

    /**
     * Retourne la liste des types de documents pour la gestion aval
     * @param plugin le plugin
     * @return la liste des objets TypeDocument
     */
    public List<GTypeDocument> loadTypeDocumentAvalList( Plugin plugin )
    {
        List<GTypeDocument> typeDocuments = new ArrayList<GTypeDocument>(  );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_TYPE_DOCUMENT_LIST_AVAL, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            GTypeDocument typeDocument = setTypeDocumentValues( daoUtil );

            typeDocuments.add( typeDocument );
        }

        daoUtil.free(  );

        return typeDocuments;
    }

    /**
     * Retourne le type de document en fonction de l'id pass en parametre
     * @param nKey identifiant du type de document que l'on souhaite
     * @param plugin le plugin
     * @return le type de document identifi par nKey
     */
    public GTypeDocument loadTypeDocumentsById( int nKey, Plugin plugin )
    {
        GTypeDocument typeDocument = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_TYPE_DOCUMENT_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            typeDocument = getTypeDocument(  );
            typeDocument.setId( daoUtil.getInt( ID_TYPE_DOCUMENT ) );
            typeDocument.setLibelle( daoUtil.getString( LIBELLE_TYPE ) );

            if ( daoUtil.getObject( EST_CREATION_PROCHAINE_SEANCE ) != null )
            {
                typeDocument.setEstPourProchaineSeance( daoUtil.getBoolean( EST_CREATION_PROCHAINE_SEANCE ) );
            }

            if ( daoUtil.getObject( EST_CREATION_AVAL ) != null )
            {
                typeDocument.setEstPourGestionAval( daoUtil.getBoolean( EST_CREATION_AVAL ) );
            }
        }

        daoUtil.free(  );

        return typeDocument;
    }

    /**
     * Permet de publier ou de dpublier un fichier
         * Si isPublie gale  TRUE, alors la mthode <b>publie</b> le fichier et incrmente la version<BR>
         * Sinon la mthode change le statut enLigne d'un fichier a <b>false</b>
     * @param nKey l'identifiant du fichier
     * @param tsDatePublication la date de publication
     * @param version la version du fichier
     * @param isPublie le statut du fichier
     * @param plugin le plugin
     */
    public void publication( int nKey, Timestamp tsDatePublication, int version, boolean isPublie, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_PUBLIER, plugin );

        if ( isPublie )
        {
            daoUtil.setTimestamp( 1, tsDatePublication );
            daoUtil.setInt( 3, version + 1 );
        }
        else
        {
            daoUtil.setTimestamp( 1, null );
            daoUtil.setInt( 3, version );
        }

        daoUtil.setBoolean( 2, isPublie );
        daoUtil.setInt( 4, nKey );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Renseigne les valeurs du TypeDocument  partir du DAOUtil
     * @param daoUtil daoUtil
     * @return l'objet GTypeDocument
     */
    protected GTypeDocument setTypeDocumentValues( DAOUtil daoUtil )
    {
        GTypeDocument typeDocument = getTypeDocument(  );
        typeDocument.setId( daoUtil.getInt( ID_TYPE_DOCUMENT ) );
        typeDocument.setLibelle( daoUtil.getString( LIBELLE_TYPE ) );

        if ( daoUtil.getObject( EST_CREATION_PROCHAINE_SEANCE ) != null )
        {
            typeDocument.setEstPourProchaineSeance( daoUtil.getBoolean( EST_CREATION_PROCHAINE_SEANCE ) );
        }

        if ( daoUtil.getObject( EST_CREATION_AVAL ) != null )
        {
            typeDocument.setEstPourGestionAval( daoUtil.getBoolean( EST_CREATION_AVAL ) );
        }

        return typeDocument;
    }

    /**
     * Permet de complter le fichier avec les proprits qui ont des contraintes sur d'autres tables
     * @param daoUtil contient les informations complmentaires du fichier
     * @param fichier fichier  complter  partir des informations du daoUtil
     */
    private void setOtherPropertyFichier( DAOUtil daoUtil, GFichier fichier )
    {
        // si le fichier a un fichier physique on le recupere en creant un objet FichierPhysique et on l'affecte  l'objet Fichier
        if ( daoUtil.getObject( "fc.id_fichier" ) != null )
        {
            FichierPhysique fichierPhysique = new FichierPhysique(  );
            fichierPhysique.setIdFichier( daoUtil.getInt( "fc.id_fichier" ) );
            fichier.setFichier( fichierPhysique );
        }

        // si le fichier a un Type De Document on le recupere en creant un objet Type de document et on l'affecte  l'objet Fichier
        if ( daoUtil.getObject( "fc.id_type_document" ) != null )
        {
            GTypeDocument typeDocument = getTypeDocument(  );
            typeDocument.setId( daoUtil.getInt( "fc.id_type_document" ) );
            typeDocument.setLibelle( daoUtil.getString( "td.libelle_type" ) );
            fichier.setTypdeDocument( typeDocument );
        }

        // si le fichier a une seance on le recupere en creant un objet Seance et on l'affecte  l'objet Fichier
        if ( daoUtil.getObject( "fc.id_seance" ) != null )
        {
            GSeance seance = newSeanceInstance(  );
            seance.setIdSeance( daoUtil.getInt( "fc.id_seance" ) );
            fichier.setSeance( seance );
        }

        if ( daoUtil.getObject( "fc.id_pdd" ) != null )
        {
            GPDD pdd = newPDDInstance(  );
            pdd.setId( daoUtil.getInt( "fc.id_pdd" ) );
            pdd.setReference( daoUtil.getString( "pdd.reference" ) );
            pdd.setTypePdd( daoUtil.getString( "pdd.type_pdd" ) );
            pdd.setDelegationsServices( daoUtil.getBoolean( "pdd.delegations_services" ) );
            pdd.setModeIntroduction( daoUtil.getString( "pdd.mode_introduction" ) );
            pdd.setObjet( daoUtil.getString( "pdd.objet" ) );
            pdd.setPiecesManuelles( daoUtil.getBoolean( "pdd.pieces_manuelles" ) );
            pdd.setDateVote( daoUtil.getTimestamp( "pdd.date_vote" ) );
            /* *pdd.setDateRetourCtrlLegalite( daoUtil.getTimestamp( "pdd.date_retour_ctrl_legalite" ) );*/
            pdd.setEnLigne( daoUtil.getBoolean( "pdd.en_ligne" ) );
            pdd.setDatePublication( daoUtil.getTimestamp( "pdd.date_publication" ) );
            pdd.setVersion( daoUtil.getInt( "pdd.version" ) );
            fichier.setPDD( pdd );

            if ( daoUtil.getObject( "pdd.id_formation_conseil" ) != null )
            {
                FormationConseil formationConseil = new FormationConseil(  );
                formationConseil.setIdFormationConseil( daoUtil.getInt( "pdd.id_formation_conseil" ) );
                fichier.getPDD(  ).setFormationConseil( formationConseil );
            }
        }

        if ( daoUtil.getObject( "fc.id_formation_conseil" ) != null )
        {
            FormationConseil formationConseil = new FormationConseil(  );
            formationConseil.setIdFormationConseil( daoUtil.getInt( "fc.id_formation_conseil" ) );
            fichier.setFormationConseil( formationConseil );
        }
    }

    /**
     * Permet de creer un objet fichier avec ses proprits de base
     * @param daoUtil contient les informations permettant de construire l'objet
     * @return le fichier avec les informations de la requte
     */
    private GFichier getBasePropertyFichier( DAOUtil daoUtil )
    {
        GFichier fichier = newFichierInstance(  );
        fichier.setId( daoUtil.getInt( "fc.id_document" ) );
        fichier.setTexte( daoUtil.getString( "fc.texte" ) );

        if ( ( daoUtil.getObject( "fc.version" ) != null ) || ( daoUtil.getInt( "fc.version" ) > 0 ) )
        {
            fichier.setVersion( daoUtil.getInt( "fc.version" ) );
        }
        else
        {
            fichier.setVersion( 0 );
        }

        fichier.setExtension( daoUtil.getString( "fc.extension" ) );
        fichier.setTaille( daoUtil.getInt( "fc.taille" ) );
        fichier.setNom( daoUtil.getString( "fc.nom" ) );
        fichier.setEnLigne( daoUtil.getBoolean( "fc.en_ligne" ) );
        fichier.setDatePublication( daoUtil.getTimestamp( "fc.date_publication" ) );
        fichier.setIntitule( daoUtil.getString( "fc.intitule" ) );

        return fichier;
    }

    /**
     * Gnre un nouvel identifiant
     * @param plugin le plugin
     * @return Retourne lidentifiant gnr
     */
    int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey = 1;

        if ( daoUtil.next(  ) )
        {
            nKey = daoUtil.getInt( 1 ) + 1;
        }

        daoUtil.free(  );

        return nKey;
    }

    /**
     * retourne la clause order by de la requette en fonction du filter
     * @param filter le filtre de selection
     * @return  retourne l'order by de la requette
     */
    private String buildOrderBy( GFichierFilter filter )
    {
        StringBuffer strOrderBy = new StringBuffer(  );
        strOrderBy.append( ORDER_BY_LIST );

        for ( String order : filter.getOrderByList(  ) )
        {
            strOrderBy.append( order );
            strOrderBy.append( OdsConstants.CONSTANTE_VIRGULE );
        }

        return strOrderBy.substring( 0, strOrderBy.length(  ) - 1 );
    }

    /**
     * Cre une nouvelle instance de fichier
     * @return une nouvelle instance de fichier
     */
    protected abstract GFichier newFichierInstance(  );
}
