/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.modeleordredujour;

import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter;
import fr.paris.lutece.plugins.ods.dto.ordredujour.TypeOrdreDuJour;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;

import java.util.ArrayList;
import java.util.List;


/**
 *
 * AbstractModeleOrdreDuJourDAO
 *
 * @param <GOrdreDuJourFilter> GOrdreDuJourFilter
 * @param <GModeleOrdreDuJour> GModeleOrdreDuJour
 */
public abstract class AbstractModeleOrdreDuJourDAO<GOrdreDuJourFilter extends IOrdreDuJourFilter, GModeleOrdreDuJour extends IModeleOrdreDuJour>
    implements IModeleOrdreDuJourDAO<GOrdreDuJourFilter, GModeleOrdreDuJour>
{
    private static final String SQL_QUERY_SELECT_BY_PK = "SELECT mo.id_modele, mo.id_fichier, " +
        "mo.id_type_odj,mo.ods_id_fichier,mo.id_formation_conseil,mo.titre";
    private static final String SQL_QUERY_SELECT_FROM = " FROM  ods_modele_ordre_jour mo ";
    private static final String SQL_QUERY_SELECT_WHERE_ID_MODELE = "WHERE mo.id_modele=? ";
    private static final String SQL_QUERY_SELECT = "SELECT mo.id_modele, mo.id_fichier," +
        "ty.id_type_odj,ty.libelle_type_odj,mo.ods_id_fichier,fc.id_formation_conseil," +
        "fc.libelle_formation_conseil,mo.titre";
    private static final String SQL_QUERY_JOIN_FORMATION_CONSEIL = " LEFT JOIN ods_formation_conseil fc on (mo.id_formation_conseil=fc.id_formation_conseil) ";
    private static final String SQL_QUERY_JOIN_TYPE_ODJ = " LEFT JOIN ods_type_ordre_jour ty on (mo.id_type_odj=ty.id_type_odj) ";
    private static final String SQL_QUERY_ORDER_BY = " ORDER BY fc.id_formation_conseil,ty.id_type_odj";
    private static final String SQL_QUERY_NEW_PK = " SELECT max( id_modele ) FROM ods_modele_ordre_jour ";
    private static final String SQL_QUERY_INSERT = " INSERT INTO ods_modele_ordre_jour (id_modele, id_fichier, id_type_odj,ods_id_fichier,id_formation_conseil,titre";
    private static final String SQL_QUERY_INSERT_VALUES = ") VALUES ( ?, ?, ?, ?, ?, ? ";
    private static final String SQL_QUERY_UPDATE = " UPDATE ods_modele_ordre_jour SET id_modele=?, id_fichier=?, id_type_odj=?,ods_id_fichier=?,id_formation_conseil=?,titre=?";
    private static final String SQL_QUERY_UPDATE_WHERE = " WHERE id_modele = ? ";
    private static final String SQL_QUERY_DELETE = " DELETE FROM ods_modele_ordre_jour WHERE id_modele = ?  ";
    private static final String SQL_FILTER_FORMATION_CONSEIL = " AND mo.id_formation_conseil = ? ";
    private static final String SQL_FILTER_TYPE = " AND mo.id_type_odj= ? ";
    private static final String SQL_QUERY_WHERE = " WHERE 1 ";

    /**
     * Ajoute les valeurs du filtre dans DAOutil
     * @param modeleOrdreDuJour le modele
     * @param nIndex l'index de dpart
     * @param daoUtil daoUtil
     * @return l'index de sortie
     */
    private int setFiltersForUpdate( GModeleOrdreDuJour modeleOrdreDuJour, int nIndex, DAOUtil daoUtil )
    {
        int nIndexCopy = nIndex;
        daoUtil.setInt( nIndexCopy, modeleOrdreDuJour.getIdModele(  ) );
        nIndexCopy++;

        if ( modeleOrdreDuJour.getEnteteDocument(  ) == null )
        {
            daoUtil.setIntNull( nIndexCopy );
        }
        else
        {
            daoUtil.setInt( nIndexCopy, modeleOrdreDuJour.getEnteteDocument(  ).getIdFichier(  ) );
        }

        nIndexCopy++;

        daoUtil.setInt( nIndexCopy, modeleOrdreDuJour.getTypeOrdreDuJour(  ).getIdTypeOrdreDuJour(  ) );
        nIndexCopy++;

        if ( modeleOrdreDuJour.getPiedDocument(  ) == null )
        {
            daoUtil.setIntNull( nIndexCopy );
        }
        else
        {
            daoUtil.setInt( nIndexCopy, modeleOrdreDuJour.getPiedDocument(  ).getIdFichier(  ) );
        }

        nIndexCopy++;

        daoUtil.setInt( nIndexCopy, modeleOrdreDuJour.getFormationConseil(  ).getIdFormationConseil(  ) );
        nIndexCopy++;
        daoUtil.setString( nIndexCopy, modeleOrdreDuJour.getTitre(  ) );
        nIndexCopy++;

        return nIndexCopy;
    }

    /**
     * Rajoute les champs  insrer
     * @param sbSQL le buffer
     * @return le nombre de champs ajouts
     */
    protected abstract int buildSQLInsertSpec( StringBuilder sbSQL );

    /**
     * Rajoute les champs  mettre  jour
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLStoreSpec( StringBuilder sbSQL );

    /**
     * Renseigne les valeurs dans le daoUtil pour l'insertion ou la modificatio
     * @param modeleOrdreDuJour le modele
     * @param nIndex l'index de dpart
     * @param daoUtil daoUtil
     * @return l'index de sortie
     */
    protected abstract int setInsertOrUpdateValuesSpec( GModeleOrdreDuJour modeleOrdreDuJour, int nIndex,
        DAOUtil daoUtil );

    /**
     * Ajoute les champs  rcuprer dans le select
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLSelectSpec( StringBuilder sbSQL );

    /**
     * Selection d'une cl primaire
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLSelectPKSpec( StringBuilder sbSQL );

    /**
     * Ajoute les jointures spcifique
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLFromSpec( StringBuilder sbSQL );

    /**
     * Ajoute la clause " ORDER BY " spcifique
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLOrderBySpec( StringBuilder sbSQL );

    /**
     * AJoute les filtre sspcifique
     * @param filter le filtre
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLFiltersSpec( GOrdreDuJourFilter filter, StringBuilder sbSQL );

    /**
     * Ajoute les filtre spcifique (indpendants du Filter)
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLFiltersSpec( StringBuilder sbSQL );

    /**
     * Renseigne les valeurs des filtres
     * @param nIndex l'index de dpart
     * @param filter le filtre
     * @param daoUtil daoUtil
     */
    protected abstract void setFiltersSpec( int nIndex, GOrdreDuJourFilter filter, DAOUtil daoUtil );

    /**
     * Met  jour les valeurs du modele  partir de daoutil ainsi que de ses dpendances
     * @param modeleOrdreDuJour le modele
     * @param daoUtil daoUtil
     */
    protected abstract void setModeleOrdreDuJourValuesWithDependanciesSpec( GModeleOrdreDuJour modeleOrdreDuJour,
        DAOUtil daoUtil );

    /**
     * Met  jour les valeurs du modele
     * @param modeleOrdreDuJour le modele
     * @param daoUtil le daoUtil
     */
    protected abstract void setModeleOrdreDuJourValuesSpec( GModeleOrdreDuJour modeleOrdreDuJour, DAOUtil daoUtil );

    /**
     * Cre  une nouvelle isntance de ModeleOrdreDuJour
     * @return une nouvelle isntance de ModeleOrdreDuJour
     */
    protected abstract GModeleOrdreDuJour newModeleOrdreDuJourInstance(  );

    /**
     * Cre un nouveau MODJ  partir de lobjet MODJ pass en paramtre
     * @param modeleOrdreDuJour le MODJ  insrer
     * @param plugin le plugin
     * @return int primaryKey
     */
    public int insert( GModeleOrdreDuJour modeleOrdreDuJour, Plugin plugin )
    {
        int newPrimaryKey = newPrimaryKey( plugin );
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_INSERT );

        int nbParameters = buildSQLInsertSpec( sbSQL );
        sbSQL.append( SQL_QUERY_INSERT_VALUES );

        for ( int nParameter = 0; nParameter < nbParameters; nParameter++ )
        {
            sbSQL.append( OdsConstants.CONSTANTE_VIRGULE );
            sbSQL.append( OdsConstants.CONSTANTE_POINT_INTERROGATION );
        }

        sbSQL.append( OdsConstants.CONSTANTE_FERMETURE_ACCOLADE );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        modeleOrdreDuJour.setIdModele( newPrimaryKey );

        int nIndex = 1;
        nIndex = setFiltersForUpdate( modeleOrdreDuJour, nIndex, daoUtil );
        setInsertOrUpdateValuesSpec( modeleOrdreDuJour, nIndex, daoUtil );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );

        return newPrimaryKey;
    }

    /**
     * Modifie le MODJ correspondant  lobjet MODJ pass en paramtre
     * @param modeleOrdreDuJour le MODJ  modifier
     * @param plugin le plugin
     */
    public void store( GModeleOrdreDuJour modeleOrdreDuJour, Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_UPDATE );
        buildSQLStoreSpec( sbSQL );
        sbSQL.append( SQL_QUERY_UPDATE_WHERE );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;
        nIndex = setFiltersForUpdate( modeleOrdreDuJour, nIndex, daoUtil );
        nIndex = setInsertOrUpdateValuesSpec( modeleOrdreDuJour, nIndex, daoUtil );

        daoUtil.setInt( nIndex, modeleOrdreDuJour.getIdModele(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Supprime le MODJ correspondant  lobjet pass en paramtre
     * @param modeleOrdreDuJour  le  MODJ  supprimer
     * @param plugin le plugin
     * @throws AppException si contrainte de cl trangre
     */
    public void delete( GModeleOrdreDuJour modeleOrdreDuJour, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, modeleOrdreDuJour.getIdModele(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Retourne la liste des ModeleOrdreDuJour disponibles
     * @param plugin le plugin
     * @return la liste des ModeleOrdreDuJour disponibles
     */
    public List<GModeleOrdreDuJour> selectListModeleOrdreDuJourList( Plugin plugin )
    {
        List<GModeleOrdreDuJour> modeleOrdreDuJours = new ArrayList<GModeleOrdreDuJour>(  );
        GModeleOrdreDuJour modeleOrdreDuJour = null;

        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLSelectSpec( sbSQL );
        sbSQL.append( SQL_QUERY_SELECT_FROM );
        sbSQL.append( SQL_QUERY_JOIN_FORMATION_CONSEIL );
        sbSQL.append( SQL_QUERY_JOIN_TYPE_ODJ );
        buildSQLFromSpec( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE );
        buildSQLFiltersSpec( sbSQL );
        sbSQL.append( SQL_QUERY_ORDER_BY );
        buildSQLOrderBySpec( sbSQL );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            modeleOrdreDuJour = setModeleOrdreDuJourValuesWithDependancies( daoUtil );
            setModeleOrdreDuJourValuesWithDependanciesSpec( modeleOrdreDuJour, daoUtil );

            modeleOrdreDuJours.add( modeleOrdreDuJour );
        }

        daoUtil.free(  );

        return modeleOrdreDuJours;
    }

    /**
     * Cre et met les valeurs dans le GModeleOrdreDuJour
     * @param daoUtil le daoUtil
     * @return le GModeleOrdreDuJour
     */
    private GModeleOrdreDuJour setModeleOrdreDuJourValues( DAOUtil daoUtil )
    {
        FichierPhysique fichierEnTete = null;
        TypeOrdreDuJour typeOrdreDuJour = null;
        FichierPhysique fichierPiedDocument = null;
        FormationConseil formationConseil = null;

        GModeleOrdreDuJour modeleOrdreDuJour = newModeleOrdreDuJourInstance(  );

        modeleOrdreDuJour.setIdModele( daoUtil.getInt( "mo.id_modele" ) );

        if ( daoUtil.getObject( "mo.id_fichier" ) != null )
        {
            fichierEnTete = new FichierPhysique(  );
            fichierEnTete.setIdFichier( daoUtil.getInt( "mo.id_fichier" ) );
            modeleOrdreDuJour.setEnteteDocument( fichierEnTete );
        }

        if ( daoUtil.getObject( "mo.id_type_odj" ) != null )
        {
            typeOrdreDuJour = new TypeOrdreDuJour(  );
            typeOrdreDuJour.setIdTypeOrdreDuJour( daoUtil.getInt( "mo.id_type_odj" ) );
            modeleOrdreDuJour.setTypeOrdreDuJour( typeOrdreDuJour );
        }

        if ( daoUtil.getObject( "mo.ods_id_fichier" ) != null )
        {
            fichierPiedDocument = new FichierPhysique(  );
            fichierPiedDocument.setIdFichier( daoUtil.getInt( "mo.ods_id_fichier" ) );
            modeleOrdreDuJour.setPiedDocument( fichierPiedDocument );
        }

        if ( daoUtil.getObject( "mo.id_formation_conseil" ) != null )
        {
            formationConseil = new FormationConseil(  );
            formationConseil.setIdFormationConseil( daoUtil.getInt( "mo.id_formation_conseil" ) );
            modeleOrdreDuJour.setFormationConseil( formationConseil );
        }

        modeleOrdreDuJour.setTitre( daoUtil.getString( "mo.titre" ) );

        return modeleOrdreDuJour;
    }

    /**
     * Cre et met  jour les valeurs du GModeleOrdreDuJour ainsi que ses dpendances
     * @param daoUtil daOUtil
     * @return GModeleOrdreDuJour
     */
    private GModeleOrdreDuJour setModeleOrdreDuJourValuesWithDependancies( DAOUtil daoUtil )
    {
        FichierPhysique fichierEnTete = null;
        TypeOrdreDuJour typeOrdreDuJour = null;
        FichierPhysique fichierPiedDocument = null;
        FormationConseil formationConseil = null;

        GModeleOrdreDuJour modeleOrdreDuJour = newModeleOrdreDuJourInstance(  );

        modeleOrdreDuJour.setIdModele( daoUtil.getInt( "mo.id_modele" ) );

        if ( daoUtil.getObject( "mo.id_fichier" ) != null )
        {
            fichierEnTete = new FichierPhysique(  );
            fichierEnTete.setIdFichier( daoUtil.getInt( "mo.id_fichier" ) );
            modeleOrdreDuJour.setEnteteDocument( fichierEnTete );
        }

        typeOrdreDuJour = new TypeOrdreDuJour(  );

        if ( daoUtil.getObject( "ty.id_type_odj" ) != null )
        {
            typeOrdreDuJour.setIdTypeOrdreDuJour( daoUtil.getInt( "ty.id_type_odj" ) );
            typeOrdreDuJour.setLibelle( daoUtil.getString( "ty.libelle_type_odj" ) );
        }

        modeleOrdreDuJour.setTypeOrdreDuJour( typeOrdreDuJour );

        if ( daoUtil.getObject( "mo.ods_id_fichier" ) != null )
        {
            fichierPiedDocument = new FichierPhysique(  );
            fichierPiedDocument.setIdFichier( daoUtil.getInt( "mo.ods_id_fichier" ) );
            modeleOrdreDuJour.setPiedDocument( fichierPiedDocument );
        }

        formationConseil = new FormationConseil(  );

        if ( daoUtil.getObject( "fc.id_formation_conseil" ) != null )
        {
            formationConseil.setIdFormationConseil( daoUtil.getInt( "fc.id_formation_conseil" ) );
            formationConseil.setLibelle( daoUtil.getString( "fc.libelle_formation_conseil" ) );
        }

        modeleOrdreDuJour.setFormationConseil( formationConseil );

        modeleOrdreDuJour.setTitre( daoUtil.getString( "mo.titre" ) );

        return modeleOrdreDuJour;
    }

    /**
     * Retourne le ModeleOrdreDuJour ayant comme identifiant nKey
     * @param nKey identifiant du ModeleOrdreDuJour  charger
     * @param plugin le plugin
     * @return le fichier identifi par nKey
     */
    public GModeleOrdreDuJour load( int nKey, Plugin plugin )
    {
        GModeleOrdreDuJour modeleOrdreDuJour = null;

        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT_BY_PK );
        // TODO vrifier pertinence
        buildSQLSelectPKSpec( sbSQL );
        sbSQL.append( SQL_QUERY_SELECT_FROM );
        sbSQL.append( SQL_QUERY_SELECT_WHERE_ID_MODELE );
        buildSQLFiltersSpec( sbSQL );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            modeleOrdreDuJour = setModeleOrdreDuJourValues( daoUtil );
            setModeleOrdreDuJourValuesSpec( modeleOrdreDuJour, daoUtil );
        }

        daoUtil.free(  );

        return modeleOrdreDuJour;
    }

    /**
     * Retourne le ModeleOrdreDuJour rpondant au filtre ordreDuJourFilter
     * @param filter le filrte d'ordre du jour
     * @param plugin le plugin
     * @return le ModeleOrdreDuJour rpondant au filtre ordreDuJourFilter
     */
    public GModeleOrdreDuJour load( GOrdreDuJourFilter filter, Plugin plugin )
    {
        GModeleOrdreDuJour modeleOrdreDuJour = null;

        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLSelectSpec( sbSQL );
        sbSQL.append( SQL_QUERY_SELECT_FROM );
        sbSQL.append( SQL_QUERY_JOIN_FORMATION_CONSEIL );
        sbSQL.append( SQL_QUERY_JOIN_TYPE_ODJ );
        buildSQLFromSpec( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE );
        buildSQLFiltersSpec( sbSQL );

        sbSQL.append( ( ( filter.containsIdFormationConseilCriteria(  ) ) ? SQL_FILTER_FORMATION_CONSEIL
                                                                          : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsIdTypeCriteria(  ) ) ? SQL_FILTER_TYPE : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        buildSQLFiltersSpec( filter, sbSQL );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;

        if ( filter.containsIdFormationConseilCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdFormationConseil(  ) );
            nIndex++;
        }

        if ( filter.containsIdTypeCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdType(  ) );
            nIndex++;
        }

        setFiltersSpec( nIndex, filter, daoUtil );

        sbSQL.append( SQL_QUERY_ORDER_BY );
        buildSQLOrderBySpec( sbSQL );

        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            modeleOrdreDuJour = setModeleOrdreDuJourValuesWithDependancies( daoUtil );
            setModeleOrdreDuJourValuesWithDependanciesSpec( modeleOrdreDuJour, daoUtil );
        }

        daoUtil.free(  );

        return modeleOrdreDuJour;
    }

    /**
     * Gnre un nouvel identifiant
     * @param plugin le plugin
     * @return Retourne lidentifiant gnr
     */
    int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }
}
