/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.ordredujour;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter;
import fr.paris.lutece.plugins.ods.dto.ordredujour.TypeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;
import fr.paris.lutece.util.sql.Transaction;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.sql.Types;

import java.util.ArrayList;
import java.util.List;


/**
 *
 * AbstractOrdreDuJourDAO
 *
 * @param <GEntreeOrdreDuJour> {@link IEntreeOrdreDuJour}
 * @param <GOrdreDuJour> {@link IOrdreDuJourFilter}
 * @param <GOrdreDuJourFilter> {@link IOrdreDuJourFilter}
 * @param <GSeance> {@link ISeance}
 * @param <GElu> {@link IElu}
 * @param <GFichier> {@link IFichier}
 */
public abstract class AbstractOrdreDuJourDAO<GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>, GOrdreDuJourFilter extends IOrdreDuJourFilter, GSeance extends ISeance, GElu extends IElu, GFichier extends IFichier<GSeance, GFichier>>
    implements IOrdreDuJourDAO<GOrdreDuJour, GEntreeOrdreDuJour, GOrdreDuJourFilter, GSeance, GElu, GFichier>
{
    protected static final String SQL_QUERY_SELECT = "SELECT odj.id_odj,odj.id_seance, fc.id_formation_conseil, " +
        "fc.libelle_formation_conseil,ty.id_type_odj,ty.libelle_type_odj, " +
        "odj.ods_id_odj,odj.intitule, odj.mode_classement, odj.publie,odj.date_publication, " +
        "odj.est_sauvegarde, odj.xml_entete, odj.xml_pied_de_page, odj.id_fichier, " + "odj.date_debut_publication";
    protected static final String SQL_QUERY_FROM = "FROM ods_odj odj " +
        "LEFT JOIN ods_formation_conseil fc on(odj.id_formation_conseil=fc.id_formation_conseil) " +
        "LEFT JOIN ods_type_ordre_jour ty on(odj.id_type_odj=ty.id_type_odj) ";
    protected static final String SQL_QUERY_WHERE = " WHERE 1 ";
    protected static final String SQL_QUERY_NEW_PK = "SELECT max( id_odj ) FROM ods_odj ";
    protected static final String SQL_QUERY_DELETE = "DELETE FROM ods_odj WHERE id_odj=?";
    protected static final String SQL_QUERY_SELECT_ODJ_BY_RAPPORTEUR_FOR_PDD = "SELECT odj.id_odj FROM ods_odj odj " +
        " LEFT JOIN ods_entree_ordre_jour in_odj ON(odj.id_odj = in_odj.id_odj AND id_pdd IS NOT NULL) " +
        " LEFT JOIN ods_entree_elus in_elu ON(in_odj.id_entree = in_elu.id_entree ) " +
        " WHERE in_elu.id_elu = ? AND odj.est_sauvegarde = 0 AND odj.publie = 1 ";
    protected static final String SQL_QUERY_SELECT_ODJ_BY_RAPPORTEUR_FOR_VNR = "SELECT odj.id_odj FROM ods_odj odj " +
        " LEFT JOIN ods_entree_ordre_jour in_odj ON(odj.id_odj = in_odj.id_odj AND id_va IS NOT NULL) " +
        " LEFT JOIN ods_entree_elus in_elu ON(in_odj.id_entree = in_elu.id_entree ) " +
        " WHERE in_elu.id_elu = ? AND odj.est_sauvegarde = 0 AND odj.publie = 1 ";
    protected static final String SQL_QUERY_UPDATE_DATE_PUBLICATION = "UPDATE ods_odj SET " +
        " date_debut_publication=? WHERE id_odj=?";
    protected static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = " WHERE odj.id_odj=? " +
        "AND odj.id_type_odj=ty.id_type_odj ";
    protected static final String SQL_QUERY_INSERT = "INSERT INTO ods_odj( " +
        " id_odj, id_seance,id_formation_conseil,id_type_odj, ods_id_odj, intitule ," +
        " mode_classement,publie,date_publication,est_sauvegarde,xml_entete,xml_pied_de_page ";
    protected static final String SQL_QUERY_INSERT_VALUES = " VALUES (?,?,?,?,?,?,?,?,?,?,?,?";
    protected static final String SQL_QUERY_UPDATE = "UPDATE ods_odj SET " +
        " id_odj=?, id_seance=?,id_formation_conseil=?,id_type_odj=?, ods_id_odj=?," +
        " intitule =? ,mode_classement=?, publie=?,date_publication=?," +
        " est_sauvegarde=?, xml_entete = ?, xml_pied_de_page = ?, id_fichier=? ";
    protected static final String SQL_QUERY_UPDATE_WHERE_CLAUSE = " WHERE id_odj=?";
    protected static final String SQL_QUERY_FIND_BY_REFERENCE = " AND odj.id_type_odj IN (0, 1, 2) ";

    // FILTERS
    protected static final String SQL_FILTER_FORMATION_CONSEIL = " AND odj.id_formation_conseil = ? ";
    protected static final String SQL_FILTER_PUBLIE = " AND odj.publie = ?  ";
    protected static final String SQL_FILTER_SEANCE = " AND odj.id_seance= ? ";
    protected static final String SQL_FILTER_TYPE = " AND odj.id_type_odj= ? ";
    protected static final String SQL_FILTER_EST_SAUVEGARDE = " AND odj.est_sauvegarde = ? ";
    protected static final String SQL_FILTER_COMMISSION_NULL = " AND odj.id_commission is NULL ";
    protected static final String SQL_QUERY_ALREADY_EXIST = "SELECT COUNT(*) FROM  ods_odj odj where 1=1 ";
    protected static final String SQL_QUERY_ORDER_LIST_ODJ_BY_DATE_PUBLICATION = " ORDER BY odj.date_publication  DESC ";
    protected static final String SQL_FILTER_DATE_PUBLICATION = " AND odj.date_publication > ? ";
    protected static final String SQL_FILTER_TYPE_ODJ = " AND odj.id_type_odj = ? ";
    protected static final String SQL_QUERY_LIMIT_1 = " LIMIT 1 ";

    /**
     * Cre une nouvelle instance de fichier
     * @return une nouvelle instance de fichier
     */
    protected abstract GFichier newFichierInstance(  );

    /**
    * Gnre un nouvel identifiant
    * @param plugin le Plugin
    * @return Retourne lidentifiant gnr
    */
    protected int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Ajoute un ordre du jour.
     * Gnre une cl primaire, ajoute l'ordre du jour et effectue les actions spcifiques.
     * Un rollback de la transaction est ralis si une erreur se produit lors de l'ajout de l'ordre du jour ou lors des actions spcifiques.
     * @param odj l'ordre du jour
     * @param plugin le Plugin
     * @return la cl primaire
     */
    public int insert( GOrdreDuJour odj, Plugin plugin )
    {
        int newPrimaryKey = newPrimaryKey( plugin );

        Transaction transaction = new Transaction( plugin );

        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_INSERT );

        int nbParameters = buildInsertSpec( sbSQL );
        sbSQL.append( OdsConstants.CONSTANTE_FERMETURE_ACCOLADE );
        sbSQL.append( SQL_QUERY_INSERT_VALUES );

        for ( int nIndexParameter = 0; nIndexParameter < nbParameters; nIndexParameter++ )
        {
            sbSQL.append( OdsConstants.CONSTANTE_VIRGULE );
            sbSQL.append( OdsConstants.CONSTANTE_POINT_INTERROGATION );
        }

        sbSQL.append( OdsConstants.CONSTANTE_FERMETURE_ACCOLADE );

        try
        {
            transaction.prepareStatement( sbSQL.toString(  ) );

            PreparedStatement preparedStatement = transaction.getStatement(  );
            int nIndex = 1;
            preparedStatement.setInt( nIndex, newPrimaryKey );
            nIndex++;
            nIndex = setInsertUpdateQueryValues( nIndex, odj, preparedStatement );
            nIndex = setInsertOrUpdateValuesSpec( nIndex, odj, preparedStatement );
            transaction.executeStatement(  );

            odj.setIdOrdreDuJour( newPrimaryKey );

            doInsertSpec( odj, transaction );

            transaction.commit(  );
        }
        catch ( SQLException e )
        {
            transaction.rollback( e );
            throw new AppException( "Insertion ordre du jour", e );
        }

        return newPrimaryKey;
    }

    /**
     * Update l'ordre du jour transmis en parametre, effectue ensuite les actions spcifiques.
     * Un rollback de la transaction est ralis si une erreur se produit lors de la mise  jour de l'ordre du jour ou lors des actions spcifiques
     * @param odj OrdreDuJour
     * @param plugin Plugin
     */
    public void store( GOrdreDuJour odj, Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_UPDATE );
        buildUpdateSpec( sbSQL );
        sbSQL.append( SQL_QUERY_UPDATE_WHERE_CLAUSE );

        Transaction transaction = new Transaction( plugin );

        try
        {
            transaction.prepareStatement( sbSQL.toString(  ) );

            PreparedStatement preparedStatement = transaction.getStatement(  );
            int nIndex = 1;
            preparedStatement.setInt( nIndex, odj.getIdOrdreDuJour(  ) );
            nIndex++;
            nIndex = setInsertUpdateQueryValues( nIndex, odj, preparedStatement );

            if ( odj.getFichier(  ) != null )
            {
                preparedStatement.setInt( nIndex, odj.getFichier(  ).getId(  ) );
            }
            else
            {
                preparedStatement.setNull( nIndex, Types.INTEGER );
            }

            nIndex++;

            nIndex = setInsertOrUpdateValuesSpec( nIndex, odj, preparedStatement );

            preparedStatement.setInt( nIndex, odj.getIdOrdreDuJour(  ) );
            nIndex++;

            transaction.executeStatement(  );

            doUpdateSpec( odj, transaction );

            transaction.commit(  );
        }
        catch ( SQLException e )
        {
            transaction.rollback( e );
            throw new AppException( "Update ordre du jour", e );
        }
    }

    /**
     *supprime l'ordre du jour pass en parametre
     * @param nIdOrdreDuJour identifiant de l'odj
     * @param plugin Plugin
     */
    public void delete( int nIdOrdreDuJour, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, nIdOrdreDuJour );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     *retourne l'ordre du jour d'id nKey
     * @param nKey int
     * @param plugin Plugin
     * @return OrdreDuJour ordreDuJour
     */
    public GOrdreDuJour load( int nKey, Plugin plugin )
    {
        GOrdreDuJour ordreDuJour = null;

        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLSelect( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLFrom( sbSQL );
        sbSQL.append( SQL_QUERY_FIND_BY_PRIMARY_KEY );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            ordreDuJour = setOrdreDuJourValues( daoUtil );

            if ( daoUtil.getObject( "odj.id_fichier" ) != null )
            {
                GFichier fichier = newFichierInstance(  );
                fichier.setId( daoUtil.getInt( "odj.id_fichier" ) );
                ordreDuJour.setFichier( fichier );
            }

            setOrdreDuJourValuesSpec( ordreDuJour, daoUtil );
        }

        daoUtil.free(  );

        return ordreDuJour;
    }

    /**
     * Trouve un odj avec le filtre
     * @param filter le filtre
     * @param plugin le plugin
     * @return l'ordre du Jour
     */
    public GOrdreDuJour findByOdjFilter( GOrdreDuJourFilter filter, Plugin plugin )
    {
        GOrdreDuJour ordreDuJour = null;

        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLSelect( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLFrom( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE );

        setFilters( sbSQL, filter );
        buildFiltersSpec( sbSQL, filter );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;

        nIndex = setFiltersParameters( nIndex, filter, daoUtil );
        setFiltersParametersSpec( nIndex, filter, daoUtil );

        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            ordreDuJour = setOrdreDuJourValues( daoUtil );
        }

        daoUtil.free(  );

        return ordreDuJour;
    }

    /**
     * Trouver un odj par le filtre
     * @param filter le filtre
     * @param plugin le plugin
     * @return l'ordre du jour
     */
    public GOrdreDuJour findOdjReferenceByFilter( GOrdreDuJourFilter filter, Plugin plugin )
    {
        GOrdreDuJour ordreDuJour = null;

        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLSelect( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLFrom( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE );

        sbSQL.append( SQL_FILTER_COMMISSION_NULL );
        sbSQL.append( SQL_QUERY_FIND_BY_REFERENCE );

        setFilters( sbSQL, filter );
        buildFiltersSpec( sbSQL, filter );

        sbSQL.append( SQL_QUERY_ORDER_LIST_ODJ_BY_DATE_PUBLICATION );
        sbSQL.append( SQL_QUERY_LIMIT_1 );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;

        nIndex = setFiltersParameters( nIndex, filter, daoUtil );
        setFiltersParametersSpec( nIndex, filter, daoUtil );

        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            ordreDuJour = setOrdreDuJourValues( daoUtil );
        }

        daoUtil.free(  );

        return ordreDuJour;
    }

    /**
     * Retourne la liste des odj correspondant au filtre
     * @param plugin le plugin
     * @param filter le filtre
     * @param bFrontOffice <b>true</b> si pour le FO, <b>false</b> sinon.
     * @return la liste d'ODJ
     */
    public List<GOrdreDuJour> selectOrdreDuJourList( Plugin plugin, GOrdreDuJourFilter filter, boolean bFrontOffice )
    {
        List<GOrdreDuJour> listOrdreDuJour = new ArrayList<GOrdreDuJour>(  );
        GOrdreDuJour ordreDuJour = null;

        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLSelect( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLFrom( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE );
        setFilters( sbSQL, filter );
        buildFiltersSpec( sbSQL, filter );

        if ( bFrontOffice )
        {
            sbSQL.append( SQL_QUERY_ORDER_LIST_ODJ_BY_DATE_PUBLICATION );
        }
        else
        {
            sbSQL.append( getSQLOrderByForBack(  ) );
        }

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;

        nIndex = setFiltersParameters( nIndex, filter, daoUtil );
        setFiltersParametersSpec( nIndex, filter, daoUtil );

        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            ordreDuJour = setOrdreDuJourValues( daoUtil );
            setOrdreDuJourValuesSpec( ordreDuJour, daoUtil );
            listOrdreDuJour.add( ordreDuJour );
        }

        daoUtil.free(  );

        return listOrdreDuJour;
    }

    /**
     * Retourne les ODJ dont l'une des entres a cet lu dans ses rapporteurs
     * @param nIdRapporteur l'id de l'u rapporteur
     * @param bPourPdd true si les rapporteurs doivent etre rattachs  un pdd,
     * false si les rapporteurs doivent etre rattachs  un voeu non rattach
     * @param plugin le plugin
     * @return les ODJ filtrs par rapporteur
     */
    public List<GOrdreDuJour> selectByRapporteur( int nIdRapporteur, boolean bPourPdd, Plugin plugin )
    {
        List<GOrdreDuJour> listOdjs = new ArrayList<GOrdreDuJour>(  );
        String strSqlQuery;

        if ( bPourPdd )
        {
            strSqlQuery = SQL_QUERY_SELECT_ODJ_BY_RAPPORTEUR_FOR_PDD;
        }
        else
        {
            strSqlQuery = SQL_QUERY_SELECT_ODJ_BY_RAPPORTEUR_FOR_VNR;
        }

        DAOUtil daoUtil = new DAOUtil( strSqlQuery, plugin );
        daoUtil.setInt( 1, nIdRapporteur );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            GOrdreDuJour odj = newODJInstance(  );
            odj.setIdOrdreDuJour( daoUtil.getInt( 1 ) );
            listOdjs.add( odj );
        }

        daoUtil.free(  );

        return listOdjs;
    }

    /**
     * Enregistre la date de dbut du processus de publication
     * @param nIdOdj l'id de l'ordre du jour  publier
     * @param dateDebutPublication la date de dbut de publication
     * @param plugin Plugin
     */
    public void storeDateDebutPublication( int nIdOdj, Timestamp dateDebutPublication, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_UPDATE_DATE_PUBLICATION, plugin );
        daoUtil.setTimestamp( 1, dateDebutPublication );
        daoUtil.setInt( 2, nIdOdj );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Rajoute les filtre
     * @param sbSQL le buffer
     * @param filter le filtre
     */
    private void setFilters( StringBuilder sbSQL, GOrdreDuJourFilter filter )
    {
        sbSQL.append( ( ( filter.containsIdFormationConseilCriteria(  ) ) ? SQL_FILTER_FORMATION_CONSEIL
                                                                          : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsIdPublieCriteria(  ) ) ? SQL_FILTER_PUBLIE : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsIdSeanceCriteria(  ) ) ? SQL_FILTER_SEANCE : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsIdTypeCriteria(  ) ) ? SQL_FILTER_TYPE : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsIdSauvegardeCriteria(  ) ) ? SQL_FILTER_EST_SAUVEGARDE
                                                                    : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsDatePublicationCriteria(  ) ) ? SQL_FILTER_DATE_PUBLICATION
                                                                       : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
    }

    /**
     * Dfinit les paramtres de base pour la requete
     * @param filter le filtre
     * @param daoUtil la requete prpare
     * @param nIndex l'index courant
     * @return nIndex l'index ractualis
     */
    private int setFiltersParameters( int nIndex, GOrdreDuJourFilter filter, DAOUtil daoUtil )
    {
        int nIndexCopy = nIndex;

        if ( filter.containsIdFormationConseilCriteria(  ) )
        {
            daoUtil.setInt( nIndexCopy, filter.getIdFormationConseil(  ) );
            nIndexCopy++;
        }

        if ( filter.containsIdPublieCriteria(  ) )
        {
            daoUtil.setInt( nIndexCopy, filter.getIdPublie(  ) );
            nIndexCopy++;
        }

        if ( filter.containsIdSeanceCriteria(  ) )
        {
            daoUtil.setInt( nIndexCopy, filter.getIdSeance(  ) );
            nIndexCopy++;
        }

        if ( filter.containsIdTypeCriteria(  ) )
        {
            daoUtil.setInt( nIndexCopy, filter.getIdType(  ) );
            nIndexCopy++;
        }

        if ( filter.containsIdSauvegardeCriteria(  ) )
        {
            if ( filter.getIdSauvegarde(  ) == 1 )
            {
                daoUtil.setBoolean( nIndexCopy, true );
            }
            else
            {
                daoUtil.setBoolean( nIndexCopy, false );
            }

            nIndexCopy++;
        }

        if ( filter.containsDatePublicationCriteria(  ) )
        {
            daoUtil.setTimestamp( nIndexCopy, filter.getDatePublication(  ) );
            nIndexCopy++;
        }

        return nIndexCopy;
    }

    /**
     * Test si un odj rpondant au filtre 'filter' existe en base.
     * @param filter le filtre appliqu pour la recherche
     * @param plugin  plugin
     * @return true si un odj rpondant au filtre 'filter' existe en base, false sinon
     */
    public boolean isAlreadyExist( GOrdreDuJourFilter filter, Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );

        sbSQL.append( SQL_QUERY_ALREADY_EXIST );

        setFilters( sbSQL, filter );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;
        setFiltersParameters( nIndex, filter, daoUtil );

        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) && ( daoUtil.getInt( 1 ) != 0 ) )
        {
            daoUtil.free(  );

            return true;
        }

        daoUtil.free(  );

        return false;
    }

    /**
     * initialise les valeurs ordredujour formationconseil typeordredujour seance  partir du rsultat de la requete sql
     * @param daoUtil le daoUtil
     * @return l'ordre du jour
     */
    private GOrdreDuJour setOrdreDuJourValues( DAOUtil daoUtil )
    {
        GOrdreDuJour ordreDuJour = newODJInstance(  );

        ordreDuJour.setIdOrdreDuJour( daoUtil.getInt( "odj.id_odj" ) );

        GSeance seance = newSeanceInstance(  );
        seance.setIdSeance( daoUtil.getInt( "odj.id_seance" ) );
        ordreDuJour.setSeance( seance );

        FormationConseil formationConseil = new FormationConseil(  );
        formationConseil.setIdFormationConseil( daoUtil.getInt( "fc.id_formation_conseil" ) );
        formationConseil.setLibelle( daoUtil.getString( "fc.libelle_formation_conseil" ) );
        ordreDuJour.setFormationConseil( formationConseil );

        TypeOrdreDuJour typeOrdreDuJour = new TypeOrdreDuJour(  );
        typeOrdreDuJour.setIdTypeOrdreDuJour( daoUtil.getInt( "ty.id_type_odj" ) );
        typeOrdreDuJour.setLibelle( daoUtil.getString( "ty.libelle_type_odj" ) );
        ordreDuJour.setTypeOrdreDuJour( typeOrdreDuJour );

        if ( daoUtil.getObject( "odj.id_fichier" ) != null )
        {
            GFichier fichier = newFichierInstance(  );
            fichier.setId( daoUtil.getInt( "odj.id_fichier" ) );
            ordreDuJour.setFichier( fichier );
        }

        if ( null != daoUtil.getObject( "odj.ods_id_odj" ) )
        {
            GOrdreDuJour ordreDuJourSauvegarde = newODJInstance(  );
            ordreDuJourSauvegarde.setIdOrdreDuJour( daoUtil.getInt( "odj.ods_id_odj" ) );
            ordreDuJour.setOrdreDuJourSauveGarde( ordreDuJourSauvegarde );
        }

        ordreDuJour.setIntitule( daoUtil.getString( "odj.intitule" ) );
        ordreDuJour.setModeClassement( daoUtil.getString( "odj.mode_classement" ) );
        ordreDuJour.setPublie( daoUtil.getBoolean( "odj.publie" ) );
        ordreDuJour.setDatePublication( daoUtil.getTimestamp( "odj.date_publication" ) );
        ordreDuJour.setEstSauvegarde( daoUtil.getBoolean( "odj.est_sauvegarde" ) );
        ordreDuJour.setXmlEntete( daoUtil.getString( "odj.xml_entete" ) );
        ordreDuJour.setXmlPiedDePage( daoUtil.getString( "odj.xml_pied_de_page" ) );

        return ordreDuJour;
    }

    /**
     * Renseigne les valeurs dans la requete
     * @param nIndex l'index de dpart
     * @param odj l'ordre du jour
     * @param statement le statement
     * @return l'index de sortie
     * @throws SQLException en cas d'erreur SQL
     */
    private int setInsertUpdateQueryValues( int nIndex, GOrdreDuJour odj, PreparedStatement statement )
        throws SQLException
    {
        int nIndexCopy = nIndex;
        statement.setInt( nIndexCopy, odj.getSeance(  ).getIdSeance(  ) );
        nIndexCopy++;
        statement.setInt( nIndexCopy, odj.getFormationConseil(  ).getIdFormationConseil(  ) );
        nIndexCopy++;
        statement.setInt( nIndexCopy, odj.getTypeOrdreDuJour(  ).getIdTypeOrdreDuJour(  ) );
        nIndexCopy++;

        if ( null != odj.getOrdreDuJourSauveGarde(  ) )
        {
            statement.setInt( nIndexCopy, odj.getOrdreDuJourSauveGarde(  ).getIdOrdreDuJour(  ) );
        }
        else
        {
            statement.setNull( nIndexCopy, Types.INTEGER );
        }

        nIndexCopy++;

        statement.setString( nIndexCopy, odj.getIntitule(  ) );
        nIndexCopy++;
        statement.setString( nIndexCopy, odj.getModeClassement(  ) );
        nIndexCopy++;
        statement.setBoolean( nIndexCopy, odj.getPublie(  ) );
        nIndexCopy++;
        statement.setTimestamp( nIndexCopy, odj.getDatePublication(  ) );
        nIndexCopy++;
        statement.setBoolean( nIndexCopy, odj.getEstSauvegarde(  ) );
        nIndexCopy++;
        statement.setString( nIndexCopy, odj.getXmlEntete(  ) );
        nIndexCopy++;
        statement.setString( nIndexCopy, odj.getXmlPiedDePage(  ) );
        nIndexCopy++;

        return nIndexCopy;
    }

    /**
     * Returns a new ODJ instance
     * @return a new ODJ instance
     */
    protected abstract GOrdreDuJour newODJInstance(  );

    /**
     * Returns a ISeance instance
     * @return a ISeance instance
     */
    protected abstract GSeance newSeanceInstance(  );

    /**
     * Remplit les valeurs spcifiques de l'objet ordreDuJour avec les informations contenues dans le daoUtil
     * @param ordreDuJour l'ordre du jour
     * @param daoUtil DAOUtil
     */
    protected abstract void setOrdreDuJourValuesSpec( GOrdreDuJour ordreDuJour, DAOUtil daoUtil );

    /**
     * Permet de remplir les valeurs pour les filtres spcifiques
     * @param nIndex l'index de dpart
     * @param filter le filtre
     * @param daoUtil DaoUtil
     * @return l'index de sortie
     */
    protected abstract int setFiltersParametersSpec( int nIndex, GOrdreDuJourFilter filter, DAOUtil daoUtil );

    /**
     * Ajoute les filtres spcifiques
     * @param sbSQL la requete
     * @param filter le filtre
     */
    protected abstract void buildFiltersSpec( StringBuilder sbSQL, GOrdreDuJourFilter filter );

    /**
     * Ajoute les jointures ou tables spcifiques utiliss lors de la requete
     * @param sbSQL la requete
     */
    protected abstract void buildSQLFrom( StringBuilder sbSQL );

    /**
     * Ajoute les colonnes spcifiques
     * @param sbSQL la requete
     */
    protected abstract void buildSQLSelect( StringBuilder sbSQL );

    /**
     * La clause Order by pour le back office
     * @return La clause Order by pour le back office
     */
    protected abstract String getSQLOrderByForBack(  );

    /**
     * Permet de remplir les valeurs spcifiques pour l'insertion ou mise  jour de l'odj
     * @param nIndex l'index de dpart
     * @param ordreDuJour l'ordre du jour
     * @param statement le statement sur lequel spcifier les valeurs
     * @return l'index en sortie
     * @throws SQLException dans le cas d'une erreur SQL
     */
    protected abstract int setInsertOrUpdateValuesSpec( int nIndex, GOrdreDuJour ordreDuJour,
        PreparedStatement statement ) throws SQLException;

    /**
     * Permet d'effectuer des actions spcifiques aprs l'insertion de l'odj
     * @param odj l'ordre du jour
     * @param transaction la transaction
     * @throws SQLException provoquera le rollback (odj non insr)
     */
    protected abstract void doInsertSpec( GOrdreDuJour odj, Transaction transaction )
        throws SQLException;

    /**
     * Permet d'effectuer des actions spcifiques aprs la mise  jour de l'odj
     * @param odj l'ordre du jour
     * @param transaction la transaction
     * @throws SQLException provoquera le rollback (odj non mis  jour)
     */
    protected abstract void doUpdateSpec( GOrdreDuJour odj, Transaction transaction )
        throws SQLException;

    /**
     * Permet d'ajouter les colonnes spcifiques lors de l'insert
     * @param sbSQL la requete sql
     * @return le nombre de paramtre ajouts
     */
    protected abstract int buildInsertSpec( StringBuilder sbSQL );

    /**
     * Permet d'ajouter les colonnes spcifiques lors de l'update
     * @param sbSQL la requete SQL
     */
    protected abstract void buildUpdateSpec( StringBuilder sbSQL );
}
