/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.requete;

import fr.paris.lutece.plugins.ods.dto.requete.Critere;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.requete.TypeCritereEnum;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.utils.commons.DateUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.util.sql.DAOUtil;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;


/**
 * RequeteUtilisateurDAO : pour grer les requtes sauvegardes par les utilisateurs
 * @param <GSeance> {@link ISeance}
 * @param <GSeanceFilter> {@link ISeanceFilter}
 * @param <GRequeteUtilisateur> {@link IRequeteUtilisateur}
 */
public abstract class AbstractRequeteUtilisateurDAO<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>>
    implements IRequeteUtilisateurDAO<GSeance, GSeanceFilter, GRequeteUtilisateur>
{
    protected static final String SQL_QUERY_LOAD = "SELECT id_requete, id_utilisateur, type_requete, intitule, " +
    	"is_archive, is_notifie, date_creation FROM ods_requete ";
    
    protected static final String SQL_FILTER_BY_USER = " WHERE id_utilisateur = ?";
    protected static final String SQL_ORDER_BY_DATE = " ORDER BY date_creation DESC";
    
    private static final String SQL_QUERY_INSERT_REQUETE = "INSERT INTO ods_requete(id_requete, id_utilisateur, " +
        "type_requete, intitule, is_notifie, is_archive, date_creation) VALUES(?,?,?,?,?,?,?)";
    private static final String SQL_QUERY_UPDATE_REQUETE = "UPDATE ods_requete SET is_notifie = ? ";
    private static final String SQL_QUERY_DELETE_REQUETE = "DELETE FROM ods_requete";
    private static final String SQL_QUERY_NEW_REQUETE_PK = "SELECT MAX(id_requete) FROM ods_requete";
    private static final String SQL_QUERY_INSERT_CRITERE = "INSERT INTO ods_critere(id_critere, id_requete, " +
        "id_type_critere, valeur_critere) VALUES (?,?,?,?)";
    private static final String SQL_QUERY_DELETE_CRITERE = "DELETE FROM ods_critere";
    private static final String SQL_QUERY_FIND_CRITERES = "SELECT id_critere, valeur_critere FROM ods_critere";
    private static final String SQL_QUERY_NEW_CRITERE_PK = "SELECT MAX(id_critere) FROM ods_critere";
    private static final String ALL_TYPES_DOC_NEXT = "proposition:projet:amendement:voeu";
    private static final String ALL_TYPES_DOC_ARCHIVE = "proposition:projet:amendement:voeu:seance:autres";
    private static final String PDD_ONLY = "proposition:projet";
    private static final String SQL_FILTER_BY_REQUETE = " WHERE id_requete = ?";       
    private static final String SQL_FILTER_BY_CRITERE = " AND id_type_critere = ?";

    /**
     * Cre une nouvelle instance de RequeteUtilisateur
     * @return une nouvelle instance de RequeteUtilisateur
     */
    protected abstract GRequeteUtilisateur newRequeteUtilisateurInstance(  );

    protected abstract void deleteSpec( GRequeteUtilisateur requete, Plugin plugin );

    /**
     * Insert les criteres spcifiques
     * @param requete la requete
     * @param critere les criteres
     * @param plugin le plugin
     */
    protected abstract void insertAllCriteresSpec( GRequeteUtilisateur requete, Critere critere, Plugin plugin );

    /**
     * Charge les informations spcifiques pour la requete
     * @param requete la requete
     * @param plugin le plugin
     */
    protected abstract void loadAllCriteresSpec( GRequeteUtilisateur requete, Plugin plugin );

    /**
     * Sauvegarde une nouvelle requte utilisateur dans la base
     * @param requete la requte utilisateur
     * @param plugin le plugin
     */
    public void insert( GRequeteUtilisateur requete, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT_REQUETE, plugin );
        int nPrimaryKey = newPrimaryKeyOnRequete( plugin );
        daoUtil.setInt( 1, nPrimaryKey );
        daoUtil.setString( 2, requete.getUserName(  ) );
        daoUtil.setString( 3, requete.getTypeRequete(  ) );
        daoUtil.setString( 4, requete.getNomRequete(  ) );
        daoUtil.setBoolean( 5, requete.isNotifie(  ) );
        daoUtil.setBoolean( 6, requete.isRechercheArchive(  ) );
        daoUtil.setTimestamp( 7, new Timestamp( System.currentTimeMillis(  ) ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );

        requete.setIdRequete( nPrimaryKey );
        insertAllCriteres( requete, plugin );
    }

    /**
     * Supprime une requte utilisateur sauvegarde
     * @param requete la requte utilisateur  supprimer
     * @param plugin le plugin
     */
    public void delete( GRequeteUtilisateur requete, Plugin plugin )
    {
        deleteAllCriteres( requete, plugin );
        deleteSpec( requete, plugin );

        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE_REQUETE + SQL_FILTER_BY_REQUETE, plugin );
        daoUtil.setInt( 1, requete.getIdRequete(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Met  jour les informations de la requte utilisateur<br/>
     * Attention cette mthode <u><b>ne modifie pas le nombre ou la valeur des critres</b></u><br/>
     * Le seul attribut que l'on peut changer est la notification par mail
     * @param requete la requete  modifier
     * @param plugin le plugin
     */
    public void update( GRequeteUtilisateur requete, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_UPDATE_REQUETE + SQL_FILTER_BY_REQUETE, plugin );
        daoUtil.setBoolean( 1, requete.isNotifie(  ) );
        daoUtil.setInt( 2, requete.getIdRequete(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Retourne la requte utilisateur identifie par nKey
     * @param nKey l'identifiant de la requte
     * @param plugin le plugin
     * @return la requte utilisateur identifie par nKey
     */
    public GRequeteUtilisateur load( int nKey, Plugin plugin )
    {
        GRequeteUtilisateur requete = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_LOAD + SQL_FILTER_BY_REQUETE, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            requete = newRequeteUtilisateurInstance(  );
            requete.setIdRequete( nKey );
            requete.setUserName( daoUtil.getString( 2 ) );
            requete.setTypeRequete( daoUtil.getString( 3 ) );
            requete.setNomRequete( daoUtil.getString( 4 ) );
            requete.setRechercheArchive( daoUtil.getBoolean( 5 ) );
            requete.setNotifie( daoUtil.getBoolean( 6 ) );
            requete.setDateCreation( daoUtil.getTimestamp( 7 ) );
            loadAllCriteres( requete, plugin );
        }

        daoUtil.free(  );

        return requete;
    }

    /**
     * Retourne la liste des requtes sauvegardes par un utilisateur
     * @param strUserName le 'nom' de l'utilisateur connect
     * @param plugin le plugin
     * @return la liste des requtes de l'utilisateur
     */
    public List<GRequeteUtilisateur> selectRequetesByUser( String strUserName, Plugin plugin )
    {
        List<GRequeteUtilisateur> listeRequetesByUser = new ArrayList<GRequeteUtilisateur>(  );
        GRequeteUtilisateur requete = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_LOAD + SQL_FILTER_BY_USER + SQL_ORDER_BY_DATE, plugin );
        daoUtil.setString( 1, strUserName );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            requete = newRequeteUtilisateurInstance(  );
            requete.setIdRequete( daoUtil.getInt( 1 ) );
            requete.setUserName( strUserName );
            requete.setTypeRequete( daoUtil.getString( 3 ) );
            requete.setNomRequete( daoUtil.getString( 4 ) );
            requete.setRechercheArchive( daoUtil.getBoolean( 5 ) );
            requete.setNotifie( daoUtil.getBoolean( 6 ) );
            requete.setDateCreation( daoUtil.getTimestamp( 7 ) );
            loadAllCriteres( requete, plugin );
            listeRequetesByUser.add( requete );
        }

        daoUtil.free(  );

        return listeRequetesByUser;
    }

    /**
     * Enregistre les critres de la recherche dans la base
     * @param requete la requte
     * @param plugin le plugin
     */
    private void insertAllCriteres( GRequeteUtilisateur requete, Plugin plugin )
    {
        Critere critere = new Critere(  );
        critere.setIdRequete( requete.getIdRequete(  ) );

        String strChampRecherche = requete.getChampRecherche(  );

        if ( ( strChampRecherche != null ) && !strChampRecherche.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            critere.setTypeCritere( TypeCritereEnum.CHAMP_RECHERCHE.getIdTypeCritere(  ) );
            critere.setValeurCritere( strChampRecherche );
            insertCritere( critere, plugin );
        }

        String strTypesDocument;

        if ( requete.getTypeRequete(  ).equals( "simple" ) )
        {
            if ( requete.isRechercheArchive(  ) )
            {
                strTypesDocument = ALL_TYPES_DOC_ARCHIVE;
            }
            else
            {
                strTypesDocument = ALL_TYPES_DOC_NEXT;
            }
        }
        else if ( requete.getTypeRequete(  ).equals( "reference" ) )
        {
            strTypesDocument = PDD_ONLY;
        }
        else
        {
            strTypesDocument = requete.getListeTypesDocument(  );
        }

        critere.setTypeCritere( TypeCritereEnum.TYPES_DOCUMENT.getIdTypeCritere(  ) );
        critere.setValeurCritere( strTypesDocument );
        insertCritere( critere, plugin );

        Timestamp tsDate1 = requete.getPremiereDate(  );

        if ( tsDate1 != null )
        {
            Calendar calendar = new GregorianCalendar(  );
            calendar.setTimeInMillis( tsDate1.getTime(  ) );

            int nJDate1 = calendar.get( Calendar.DATE );
            String strJDate1 = ( ( nJDate1 < 10 ) ? ( "0" + nJDate1 ) : nJDate1 ) + "/";
            int nMDate1 = calendar.get( Calendar.MONTH ) + 1;
            String strMDate1 = ( ( nMDate1 < 10 ) ? ( "0" + nMDate1 ) : nMDate1 ) + "/";
            String strDate1 = strJDate1 + strMDate1 + calendar.get( Calendar.YEAR );
            critere.setTypeCritere( TypeCritereEnum.DATE_1.getIdTypeCritere(  ) );
            critere.setValeurCritere( strDate1 );
            insertCritere( critere, plugin );
        }

        Timestamp tsDate2 = requete.getDeuxiemeDate(  );

        if ( tsDate2 != null )
        {
            Calendar calendar = new GregorianCalendar(  );
            calendar.setTimeInMillis( tsDate2.getTime(  ) );

            int nJDate2 = calendar.get( Calendar.DATE );
            String strJDate2 = ( ( nJDate2 < 10 ) ? ( "0" + nJDate2 ) : nJDate2 ) + "/";
            int nMDate2 = calendar.get( Calendar.MONTH ) + 1;
            String strMDate2 = ( ( nMDate2 < 10 ) ? ( "0" + nMDate2 ) : nMDate2 ) + "/";
            String strDate2 = strJDate2 + strMDate2 + calendar.get( Calendar.YEAR );
            critere.setTypeCritere( TypeCritereEnum.DATE_2.getIdTypeCritere(  ) );
            critere.setValeurCritere( strDate2 );
            insertCritere( critere, plugin );
        }

        String strRapporteurs = requete.getListeRapporteurs(  );

        if ( ( strRapporteurs != null ) && !strRapporteurs.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            critere.setTypeCritere( TypeCritereEnum.RAPPORTEURS.getIdTypeCritere(  ) );
            critere.setValeurCritere( strRapporteurs );
            insertCritere( critere, plugin );
        }

        String strArrondissements = requete.getListeArrondissements(  );

        if ( ( strArrondissements != null ) && !strArrondissements.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            critere.setTypeCritere( TypeCritereEnum.ARRONDISSEMENTS.getIdTypeCritere(  ) );
            critere.setValeurCritere( strArrondissements );
            insertCritere( critere, plugin );
        }

        String strDirections = requete.getListeDirections(  );

        if ( ( strDirections != null ) && !strDirections.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            critere.setTypeCritere( TypeCritereEnum.DIRECTIONS.getIdTypeCritere(  ) );
            critere.setValeurCritere( strDirections );
            insertCritere( critere, plugin );
        }

        String strCategoriesDeliberation = requete.getListeCategoriesDeliberation(  );

        if ( ( strCategoriesDeliberation != null ) &&
                !strCategoriesDeliberation.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            critere.setTypeCritere( TypeCritereEnum.CATEGORIES_DELIBERATION.getIdTypeCritere(  ) );
            critere.setValeurCritere( strCategoriesDeliberation );
            insertCritere( critere, plugin );
        }

        String strGroupesDepositaires = requete.getListeGroupesDepositaires(  );

        if ( ( strGroupesDepositaires != null ) &&
                !strGroupesDepositaires.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            critere.setTypeCritere( TypeCritereEnum.GROUPES_DEPOSITAIRES.getIdTypeCritere(  ) );
            critere.setValeurCritere( strGroupesDepositaires );
            insertCritere( critere, plugin );
        }

        if ( requete.isDeposeExecutif(  ) )
        {
            critere.setTypeCritere( TypeCritereEnum.DEPOSE_EXECUTIF.getIdTypeCritere(  ) );
            critere.setValeurCritere( OdsConstants.INTEGER_TRUE );
            insertCritere( critere, plugin );
        }

        String strElusDepositaires = requete.getListeElusDepositaires(  );

        if ( ( strElusDepositaires != null ) && !strElusDepositaires.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            critere.setTypeCritere( TypeCritereEnum.ELUS_DEPOSITAIRES.getIdTypeCritere(  ) );
            critere.setValeurCritere( strElusDepositaires );
            insertCritere( critere, plugin );
        }

        String strStatutsProjets = requete.getListeStatutsProjets(  );

        if ( ( strStatutsProjets != null ) && !strStatutsProjets.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            critere.setTypeCritere( TypeCritereEnum.STATUTS_PROJETS.getIdTypeCritere(  ) );
            critere.setValeurCritere( strStatutsProjets );
            insertCritere( critere, plugin );
        }

        String strStatutsPropositions = requete.getListeStatutsPropositions(  );

        if ( ( strStatutsPropositions != null ) &&
                !strStatutsPropositions.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            critere.setTypeCritere( TypeCritereEnum.STATUTS_PROPOSITIONS.getIdTypeCritere(  ) );
            critere.setValeurCritere( strStatutsPropositions );
            insertCritere( critere, plugin );
        }

        String strStatutsVoeuxAmendements = requete.getListeStatutsVoeuxAmendements(  );

        if ( ( strStatutsVoeuxAmendements != null ) &&
                !strStatutsVoeuxAmendements.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            critere.setTypeCritere( TypeCritereEnum.STATUTS_VOEUX_AMENDEMENTS.getIdTypeCritere(  ) );
            critere.setValeurCritere( strStatutsVoeuxAmendements );
            insertCritere( critere, plugin );
        }

        insertAllCriteresSpec( requete, critere, plugin );
    }

    /**
     * Supprime tous les critres de recherche de la requte
     * @param requete la requte
     * @param plugin le plugin
     */
    private void deleteAllCriteres( GRequeteUtilisateur requete, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE_CRITERE + SQL_FILTER_BY_REQUETE, plugin );
        daoUtil.setInt( 1, requete.getIdRequete(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Rcupre la valeur des critres de recherche d'une requte enregistre
     * @param requete la requte
     * @param plugin le plugin
     */
    protected void loadAllCriteres( GRequeteUtilisateur requete, Plugin plugin )
    {
        requete.setChampRecherche( loadCritere( requete, TypeCritereEnum.CHAMP_RECHERCHE.getIdTypeCritere(  ), plugin )
                                       .getValeurCritere(  ) );
        requete.setListeTypesDocument( ( loadCritere( requete, TypeCritereEnum.TYPES_DOCUMENT.getIdTypeCritere(  ),
                plugin ) ).getValeurCritere(  ) );
        requete.setPremiereDate( DateUtils.getDate( loadCritere( requete, TypeCritereEnum.DATE_1.getIdTypeCritere(  ),
                    plugin ).getValeurCritere(  ), true ) );
        requete.setDeuxiemeDate( DateUtils.getDate( loadCritere( requete, TypeCritereEnum.DATE_2.getIdTypeCritere(  ),
                    plugin ).getValeurCritere(  ), true ) );
        requete.setListeFormationsConseil( loadCritere( requete,
                TypeCritereEnum.FORMATIONS_CONSEIL.getIdTypeCritere(  ), plugin ).getValeurCritere(  ) );
        requete.setListeRapporteurs( loadCritere( requete, TypeCritereEnum.RAPPORTEURS.getIdTypeCritere(  ), plugin )
                                         .getValeurCritere(  ) );
        requete.setListeArrondissements( loadCritere( requete, TypeCritereEnum.ARRONDISSEMENTS.getIdTypeCritere(  ),
                plugin ).getValeurCritere(  ) );
        requete.setListeDirections( loadCritere( requete, TypeCritereEnum.DIRECTIONS.getIdTypeCritere(  ), plugin )
                                        .getValeurCritere(  ) );
        requete.setListeCategoriesDeliberation( loadCritere( requete,
                TypeCritereEnum.CATEGORIES_DELIBERATION.getIdTypeCritere(  ), plugin ).getValeurCritere(  ) );
        requete.setListeGroupesDepositaires( loadCritere( requete,
                TypeCritereEnum.GROUPES_DEPOSITAIRES.getIdTypeCritere(  ), plugin ).getValeurCritere(  ) );
        requete.setListeElusDepositaires( loadCritere( requete, TypeCritereEnum.ELUS_DEPOSITAIRES.getIdTypeCritere(  ),
                plugin ).getValeurCritere(  ) );

        if ( loadCritere( requete, TypeCritereEnum.DEPOSE_EXECUTIF.getIdTypeCritere(  ), plugin ).getValeurCritere(  )
                     .equals( OdsConstants.INTEGER_TRUE ) )
        {
            requete.setDeposeExecutif( true );
        }

        requete.setListeStatutsProjets( loadCritere( requete, TypeCritereEnum.STATUTS_PROJETS.getIdTypeCritere(  ),
                plugin ).getValeurCritere(  ) );
        requete.setListeStatutsPropositions( loadCritere( requete,
                TypeCritereEnum.STATUTS_PROPOSITIONS.getIdTypeCritere(  ), plugin ).getValeurCritere(  ) );
        requete.setListeStatutsVoeuxAmendements( loadCritere( requete,
                TypeCritereEnum.STATUTS_VOEUX_AMENDEMENTS.getIdTypeCritere(  ), plugin ).getValeurCritere(  ) );
        loadAllCriteresSpec( requete, plugin );
    }

    /**
     * Enregistre un critre de recherche
     * @param critere le critre  sauvegarder
     * @param plugin le plugin
     */
    protected void insertCritere( Critere critere, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT_CRITERE, plugin );
        daoUtil.setInt( 1, newPrimaryKeyOnCritere( plugin ) );
        daoUtil.setInt( 2, critere.getIdRequete(  ) );
        daoUtil.setInt( 3, critere.getTypeCritere(  ) );
        daoUtil.setString( 4, critere.getValeurCritere(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Rcupre le critre voulu pour une requte donne
     * @param requete la requte
     * @param idTypeCritere le type de critre
     * @param plugin le plugin
     * @return le critre voulu pour une requte donne
     */
    protected Critere loadCritere( GRequeteUtilisateur requete, int idTypeCritere, Plugin plugin )
    {
        Critere critere = new Critere(  );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_CRITERES + SQL_FILTER_BY_REQUETE + SQL_FILTER_BY_CRITERE, plugin );
        daoUtil.setInt( 1, requete.getIdRequete(  ) );
        daoUtil.setInt( 2, idTypeCritere );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            critere.setIdCritere( daoUtil.getInt( 1 ) );
            critere.setIdRequete( requete.getIdRequete(  ) );
            critere.setTypeCritere( idTypeCritere );
            critere.setValeurCritere( daoUtil.getString( 2 ) );
        }

        daoUtil.free(  );

        return critere;
    }

    /**
    * Retourne une nouvelle cl primaire unique sur la table ods_requete
    * @param plugin plugin
    * @return un identifiant unique
    */
    private int newPrimaryKeyOnRequete( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_REQUETE_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Retourne une nouvelle cl primaire unique sur la table ods_critere
     * @param plugin plugin
     * @return un identifiant unique
     */
    private int newPrimaryKeyOnCritere( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_CRITERE_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }
}
