/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.seance;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.utils.commons.DateUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;
import fr.paris.lutece.util.sql.Transaction;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.List;


/**
 * 
 * AbstractSeanceDAO
 *
 * @param <GSeance> {@link ISeance}
 * @param <GSeanceFilter> {@link ISeanceFilter}
 * @param <GFichier> {@link IFichier}
 * @param <GElu> {@link IElu}
 * @param <GVoeuAmendement> {@link IVoeuAmendement}
 * @param <GPDD> {@link IPDD}
 */
public abstract class AbstractSeanceDAO
<
	GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, 
	GFichier extends IFichier<GSeance, GFichier>, 
	GElu extends IElu, 
	GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, 
	GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>
>
implements ISeanceDAO<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD>
{
    protected static final String SQL_QUERY_NEW_PK = " SELECT max( se.id_seance ) FROM ods_seance se ";
    protected static final String SQL_QUERY_SELECT = "SELECT se.id_seance, se.date_seance, se.date_publication, se.taille_archive ";
    protected static final String SQL_QUERY_FROM = " FROM ods_seance se ";
    protected static final String SQL_QUERY_FROM_PDD = " FROM ods_seance se, ods_odj odj, ods_entree_ordre_jour entree ";
    protected static final String SQL_QUERY_WHERE_ID_SEANCE = " WHERE se.id_seance = ? ";
    protected static final String SQL_QUERY_SEANCE_AFTER = " WHERE se.id_seance >= ? AND se.date_seance < ? ";
    protected static final String SQL_QUERY_SEANCE_NEXT = " WHERE se.date_seance >= ?  ";
    protected static final String SQL_QUERY_SEANCE_EN_COURS = " WHERE date_seance <= ? AND date_fin_seance >= ? ";
    protected static final String SQL_QUERY_SEANCE_WIDTH_FILTER_LIST = " WHERE date_seance >= ? AND date_seance <= ? ";
    protected static final String SQL_KEYWORK_WHERE = " WHERE 1 ";
    protected static final String SQL_QUERY_ORDER_BY_DATE_SEANCE = " ORDER BY se.date_seance ASC ";
    protected static final String SQL_ORDER_BY_ID_SEANCE = " ORDER BY id_seance ASC ";
    protected static final String SQL_QUERY_LIMIT_1 = " LIMIT 1";
    protected static final String SQL_QUERY_SEANCE_OLD = " and se.date_seance < ? ";
    protected static final String SQL_QUERY_SEANCE_LIST = " WHERE date_seance >= ? ";
    protected static final String SQL_QUERY_BY_PDD = " WHERE se.id_seance = odj.id_seance AND odj.id_type_odj = 2 AND entree.id_odj = odj.id_odj AND entree.id_entite = ? AND entree.id_type_entite = " +
        ITypeEntite.PDD;
    protected static final String SQL_QUERY_DELETE = " DELETE FROM ods_seance WHERE id_seance = ?  ";
    protected static final String SQL_QUERY_INSERT = " INSERT INTO ods_seance(id_seance, date_seance, date_publication, taille_archive ";
    protected static final String SQL_QUERY_UPDATE = " UPDATE ods_seance SET date_seance = ?, date_publication = ?, taille_archive = ? ";
    protected static final String SQL_QUERY_UPDATE_WHERE_CLAUSE = " WHERE id_seance = ? ";
    protected static final String SQL_QUERY_INSERT_VALUES = " ) VALUES ( ?, ?, ?, ? ";
    private static final String SQL_QUERY_SEANCE_PDD_SPEC = "SELECT se.id_seance, se.date_seance, se.date_publication, se.taille_archive , sext.type_seance, sext.id_formation_conseil " +
        " FROM ods_seance se, ods_odj odj, ods_entree_ordre_jour entree " +
        " LEFT JOIN ods_seance_extension sext on se.id_seance = sext.id_seance " +
        " WHERE se.id_seance = odj.id_seance " + " AND odj.id_type_odj = 2 " + " AND entree.id_odj = odj.id_odj " +
        " AND entree.id_entite = ? " + " AND entree.id_type_entite = " + ITypeEntite.PDD +
        " AND sext.id_formation_conseil = ? " + " AND sext.type_seance = ? ";

    /**
     * Insere les informations dans les tables spcifiques
     * @param seance la seance
     * @param transaction la transaction
     * @throws SQLException en cas d'erreur SQL
     */
    protected abstract void doInsertSpec( GSeance seance, Transaction transaction )
        throws SQLException;

    /**
     * Met  jour les informations dans les table spcifiques
     * @param seance la seance
     * @param transaction la transaction
     * @throws SQLException en cas d'erreur SQL
     */
    protected abstract void doUpdateSpec( GSeance seance, Transaction transaction )
        throws SQLException;

    /**
     * Renseigne les valeurs pour l'insertion ou la mise  jour
     * @param nIndex l'index de dpart
     * @param seance la sance
     * @param preparedStatement le statement
     * @return l'index de sortie
     * @throws SQLException en cas d'erreur SQL
     */
    protected abstract int setInsertUpdateQueryValuesSpec( int nIndex, GSeance seance,
        PreparedStatement preparedStatement ) throws SQLException;

    /**
     * Ajoute les champs  mettre  jour
     * @param sbSQL le buffer
     */
    protected abstract void buildSQLQueryUpdateSpec( StringBuilder sbSQL );

    /**
     * Ajoute les champs  insrer
     * @param sbSQL le buffer
     * @return le nombre de champs ajouts
     */
    protected abstract int buildSQLQueryInsertSpec( StringBuilder sbSQL );

    /**
     * Ajoute le champs  rcuprer dans le select
     * @param buffer le buffer
     */
    protected abstract void buildSQLQuerySelect( StringBuilder buffer );

    /**
     * Ajoute les jointures
     * @param buffer le buffer
     */
    protected abstract void buildSQLQueryFrom( StringBuilder buffer );

    /**
     * Ajoute les filtres
     * @param seanceFilter le filtre
     * @param buffer le buffer
     */
    protected abstract void buildSQLQueryFilter( GSeanceFilter seanceFilter, StringBuilder buffer );

    /**
     * Ajoute la clause "ORDER BY"
     * @param buffer le buffer
     */
    protected abstract void buildSQLQueryOrderBySeance( StringBuilder buffer );

    /***
     * Ajoute la clause "ORDER BY" pour les sances passes
     * @param buffer le buffer
     */
    protected abstract void buildSQLQueryOrdreBySeanceForOldSeance( StringBuilder buffer );

    /**
     * Renseigne les paramtres
     * @param nStartIndex l'index de dpart
     * @param seanceFilter le filtre
     * @param daoUtil daoUtil
     * @return l'index de sortie
     */
    protected abstract int fillParameters( int nStartIndex, GSeanceFilter seanceFilter, DAOUtil daoUtil );

    /**
     * Renseigne les informations spcifiques de la sance
     * @param seance la sance
     * @param daoUtil daoutil
     */
    protected abstract void fillSeanceFromQuerySpec( GSeance seance, DAOUtil daoUtil );

    /**
     * Retourne la date
     * @return la date
     */
    protected abstract Timestamp getCurrentDateSpec(  );

    /**
     * Cre une nouvelle instance de sance
     * @return une nouvelle instance de sance
     */
    protected abstract GSeance newSeanceInstance(  );

    /**
     * Cre une nouvelle instance de SeanceFilter
     * @return une nouvelle instance de SeanceFilter
     */
    protected abstract GSeanceFilter newSeanceFilterInstance(  );

    /**
    * Cre une nouvelle sance  partir de lobjet seance pass en paramtre
    * @param seance la sance  insrer
    * @param plugin Plugin
    */
    public void insert( GSeance seance, Plugin plugin )
    {
        seance.setIdSeance( newPrimaryKey( plugin ) );

        Transaction transaction = new Transaction( plugin );

        try
        {
            // Insert 'ods_seance'
            StringBuilder sbSQL = new StringBuilder(  );
            sbSQL.append( SQL_QUERY_INSERT );

            int nbParameters = buildSQLQueryInsertSpec( sbSQL );
            sbSQL.append( SQL_QUERY_INSERT_VALUES );

            for ( int nParameter = 0; nParameter < nbParameters; nParameter++ )
            {
                sbSQL.append( OdsConstants.CONSTANTE_VIRGULE );
                sbSQL.append( OdsConstants.CONSTANTE_POINT_INTERROGATION );
            }

            sbSQL.append( OdsConstants.CONSTANTE_FERMETURE_ACCOLADE );
            transaction.prepareStatement( sbSQL.toString(  ) );

            int nIndex = 1;
            transaction.getStatement(  ).setInt( nIndex, seance.getIdSeance(  ) );
            nIndex++;
            transaction.getStatement(  ).setTimestamp( nIndex, seance.getDateSeance(  ) );
            nIndex++;
            transaction.getStatement(  ).setTimestamp( nIndex, seance.getDatePublicationArchive(  ) );
            nIndex++;
            transaction.getStatement(  ).setLong( nIndex, seance.getTailleArchive(  ) );
            nIndex++;
            setInsertUpdateQueryValuesSpec( nIndex, seance, transaction.getStatement(  ) );

            transaction.executeStatement(  );

            doInsertSpec( seance, transaction );

            // Commit
            transaction.commit(  );
        }
        catch ( SQLException e )
        {
            // Rollback
            transaction.rollback( e );
        }
    }

    /**
     * Met  jour la sance en base
     * @param seance la seance
     * @param plugin le plugin
     */
    public void store( GSeance seance, Plugin plugin )
    {
        Transaction transaction = new Transaction( plugin );

        try
        {
            // Update 'ods_seance'
            StringBuilder sbSQL = new StringBuilder(  );
            sbSQL.append( SQL_QUERY_UPDATE );
            buildSQLQueryUpdateSpec( sbSQL );
            sbSQL.append( SQL_QUERY_UPDATE_WHERE_CLAUSE );
            transaction.prepareStatement( sbSQL.toString(  ) );

            int nIndex = 1;
            transaction.getStatement(  ).setTimestamp( nIndex, seance.getDateSeance(  ) );
            nIndex++;
            transaction.getStatement(  ).setTimestamp( nIndex, seance.getDatePublicationArchive(  ) );
            nIndex++;
            transaction.getStatement(  ).setLong( nIndex, seance.getTailleArchive(  ) );
            nIndex++;
            nIndex = setInsertUpdateQueryValuesSpec( nIndex, seance, transaction.getStatement(  ) );

            transaction.getStatement(  ).setInt( nIndex, seance.getIdSeance(  ) );
            nIndex++;
            transaction.executeStatement(  );

            doUpdateSpec( seance, transaction );

            // Commit
            transaction.commit(  );
        }
        catch ( SQLException e )
        {
            // Rollback
            transaction.rollback( e );
        }
    }

    /**
    * Supprime la sance correspondant   lobjet seance pass en paramtre
    * @param nIdSeance la sance  supprimer
    * @param plugin Plugin
    * @throws AppException exception lev si une dependance existe
    */
    public void delete( int nIdSeance, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );

        daoUtil.setInt( 1, nIdSeance );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Trouve la dernire sance passe
     * @param plugin le plugin
     * @return la seance
     */
    public GSeance findLastSeance( Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLQuerySelect( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLQueryFrom( sbSQL );
        sbSQL.append( SQL_KEYWORK_WHERE );
        sbSQL.append( SQL_QUERY_SEANCE_OLD );

        GSeanceFilter seanceFilter = newSeanceFilterInstance(  );
        buildSQLQueryFilter( seanceFilter, sbSQL );
        buildSQLQueryOrdreBySeanceForOldSeance( sbSQL );
        sbSQL.append( SQL_QUERY_LIMIT_1 );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;
        daoUtil.setTimestamp( nIndex, getCurrentDateSpec(  ) );
        nIndex++;

        fillParameters( nIndex, seanceFilter, daoUtil );

        daoUtil.executeQuery(  );

        GSeance seance = null;

        if ( daoUtil.next(  ) )
        {
            seance = newSeanceInstance(  );
            fillSeanceFromQuery( seance, daoUtil );
            fillSeanceFromQuerySpec( seance, daoUtil );
        }

        daoUtil.free(  );

        return seance;
    }

    /**
     * Trouve la prochaine sance
     * @param plugin le plugin
     * @return la seance
     */
    public GSeance findNextSeance( Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLQuerySelect( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLQueryFrom( sbSQL );
        sbSQL.append( SQL_QUERY_SEANCE_NEXT );

        GSeanceFilter seanceFilter = newSeanceFilterInstance(  );
        buildSQLQueryFilter( seanceFilter, sbSQL );
        sbSQL.append( SQL_QUERY_ORDER_BY_DATE_SEANCE );
        sbSQL.append( SQL_QUERY_LIMIT_1 );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;
        Timestamp tsCurrentDate = getCurrentDateSpec(  );
        daoUtil.setTimestamp( nIndex, tsCurrentDate );
        nIndex++;
        fillParameters( nIndex, seanceFilter, daoUtil );

        daoUtil.executeQuery(  );

        GSeance seance = null;

        if ( daoUtil.next(  ) )
        {
            seance = newSeanceInstance(  );
            fillSeanceFromQuery( seance, daoUtil );
            fillSeanceFromQuerySpec( seance, daoUtil );
        }

        daoUtil.free(  );

        return seance;
    }

    /**
     * Trouve la liste des sances passes
     * @param plugin le plugin
     * @return la liste des sances passes
     */
    public List<GSeance> findOldSeance( Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLQuerySelect( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLQueryFrom( sbSQL );
        sbSQL.append( SQL_KEYWORK_WHERE );
        sbSQL.append( SQL_QUERY_SEANCE_OLD );

        GSeanceFilter seanceFilter = newSeanceFilterInstance(  );
        buildSQLQueryFilter( seanceFilter, sbSQL );
        buildSQLQueryOrdreBySeanceForOldSeance( sbSQL );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;
        daoUtil.setTimestamp( nIndex, getCurrentDateSpec(  ) );
        nIndex++;

        fillParameters( nIndex, seanceFilter, daoUtil );

        daoUtil.executeQuery(  );

        List<GSeance> seanceList = new ArrayList<GSeance>(  );

        while ( daoUtil.next(  ) )
        {
            GSeance seance = newSeanceInstance(  );
            fillSeanceFromQuery( seance, daoUtil );
            fillSeanceFromQuerySpec( seance, daoUtil );
            seanceList.add( seance );
        }

        daoUtil.free(  );

        return seanceList;
    }

    /**
     * Trouve la liste des sances aprs la sance passe en paramtre
     * @param idSeance l'identifiant de la sance
     * @param plugin le plugin
     * @return la liste des sances
     */
    public List<GSeance> findSeanceAfter( int idSeance, Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLQuerySelect( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLQueryFrom( sbSQL );
        sbSQL.append( SQL_QUERY_SEANCE_AFTER );

        GSeanceFilter seanceFilter = newSeanceFilterInstance(  );
        buildSQLQueryFilter( seanceFilter, sbSQL );
        buildSQLQueryOrdreBySeanceForOldSeance( sbSQL );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        int nIndex = 1;
        daoUtil.setInt( nIndex, idSeance );
        nIndex++;
        daoUtil.setTimestamp( nIndex, DateUtils.getCurrentDate(  ) );
        nIndex++;

        nIndex = fillParameters( nIndex, seanceFilter, daoUtil );

        daoUtil.executeQuery(  );

        List<GSeance> seanceList = new ArrayList<GSeance>(  );

        while ( daoUtil.next(  ) )
        {
            GSeance seance = newSeanceInstance(  );
            fillSeanceFromQuery( seance, daoUtil );
            fillSeanceFromQuerySpec( seance, daoUtil );
            seanceList.add( seance );
        }

        daoUtil.free(  );

        return seanceList;
    }

    /**
     * Trouve la sance associe  un PDD
     * @param idPdd l'identifiant du PDD
     * @param plugin le plugin
     * @return la seance
     */
    public GSeance findSeanceByPdd( int idPdd, Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLQuerySelect( sbSQL );
        sbSQL.append( SQL_QUERY_FROM_PDD );
        buildSQLQueryFrom( sbSQL );
        sbSQL.append( SQL_QUERY_BY_PDD );

        GSeanceFilter seanceFilter = newSeanceFilterInstance(  );
        buildSQLQueryFilter( seanceFilter, sbSQL );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;
        daoUtil.setInt( nIndex, idPdd );
        nIndex++;
        fillParameters( nIndex, seanceFilter, daoUtil );

        daoUtil.executeQuery(  );

        GSeance seance = null;

        if ( daoUtil.next(  ) )
        {
            seance = newSeanceInstance(  );
            fillSeanceFromQuery( seance, daoUtil );
            fillSeanceFromQuerySpec( seance, daoUtil );
        }

        daoUtil.free(  );

        return seance;
    }

    /**
     * Trouve la sance associe  un PDD
     * @param pdd le PDD
     * @param plugin le plugin
     * @return la seance
     */
    public GSeance findSeanceByPddSpec( GPDD pdd, Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SEANCE_PDD_SPEC );

        GSeanceFilter seanceFilter = newSeanceFilterInstance(  );

        //buildSQLQueryFilter( seanceFilter, sbSQL );
        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;
        daoUtil.setInt( nIndex, pdd.getId(  ) );
        nIndex++;
        daoUtil.setInt( nIndex, pdd.getFormationConseil(  ).getIdFormationConseil(  ) );
        nIndex++;
        daoUtil.setInt( nIndex, seanceFilter.getTypeSeance(  ) );
        nIndex++;

        daoUtil.executeQuery(  );

        GSeance seance = null;

        if ( daoUtil.next(  ) )
        {
            seance = newSeanceInstance(  );
            fillSeanceFromQuery( seance, daoUtil );
            fillSeanceFromQuerySpec( seance, daoUtil );
        }

        daoUtil.free(  );

        return seance;
    }

    /**
     * Trouve la sance en cours
     * @param plugin le plugin
     * @return la seance
     */
    public GSeance findSeanceEnCours( Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLQuerySelect( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLQueryFrom( sbSQL );
        sbSQL.append( SQL_QUERY_SEANCE_EN_COURS );

        GSeanceFilter seanceFilter = newSeanceFilterInstance(  );
        buildSQLQueryFilter( seanceFilter, sbSQL );
        sbSQL.append( SQL_ORDER_BY_ID_SEANCE );
        sbSQL.append( SQL_QUERY_LIMIT_1 );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;
        Timestamp tsCurrentDate = DateUtils.getCurrentDate(  );
        daoUtil.setTimestamp( nIndex, tsCurrentDate );
        nIndex++;
        daoUtil.setTimestamp( nIndex, tsCurrentDate );
        nIndex++;
        nIndex = fillParameters( nIndex, seanceFilter, daoUtil );

        daoUtil.executeQuery(  );

        GSeance seance = null;

        if ( daoUtil.next(  ) )
        {
            seance = newSeanceInstance(  );
            fillSeanceFromQuery( seance, daoUtil );
            fillSeanceFromQuerySpec( seance, daoUtil );
        }

        daoUtil.free(  );

        return seance;
    }

    /**
     * Charge une sance  partir de la cl
     * @param nKey la cl
     * @param plugin le plugin
     * @return la sance
     */
    public GSeance load( int nKey, Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLQuerySelect( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLQueryFrom( sbSQL );
        sbSQL.append( SQL_QUERY_WHERE_ID_SEANCE );

        GSeanceFilter seanceFilter = newSeanceFilterInstance(  );
        seanceFilter.setTypeSeance( Integer.parseInt( OdsConstants.FILTER_ALL ) );
        buildSQLQueryFilter( seanceFilter, sbSQL );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;
        daoUtil.setInt( nIndex, nKey );
        nIndex++;
        fillParameters( nIndex, seanceFilter, daoUtil );

        daoUtil.executeQuery(  );

        GSeance seance = null;

        if ( daoUtil.next(  ) )
        {
            seance = newSeanceInstance(  );
            fillSeanceFromQuery( seance, daoUtil );
            fillSeanceFromQuerySpec( seance, daoUtil );
        }

        daoUtil.free(  );

        return seance;
    }

    /**
     * renvoie la liste des sances dont la date de dbut est comprise entre startDateFilter et endDateFilter
     *
     * @param startDateFilter date de dbut
     * @param endDateFilter date de fin
     * @param plugin Plugin
     * @return liste d'objets Seance
     */
    public List<GSeance> loadWithFilter( Timestamp startDateFilter, Timestamp endDateFilter, Plugin plugin )
    {
        GSeanceFilter seanceFilter = newSeanceFilterInstance(  );

        return loadWithFilter( startDateFilter, endDateFilter, seanceFilter, plugin );
    }

    /**
     * renvoie la liste des sances dont la date de dbut est comprise entre startDateFilter et endDateFilter
     *
     * @param startDateFilter date de dbut
     * @param endDateFilter date de fin
     * @param seanceFilter filtre de seance
     * @param plugin Plugin
     * @return liste d'objets Seance
     */
    public List<GSeance> loadWithFilter( Timestamp startDateFilter, Timestamp endDateFilter,
        GSeanceFilter seanceFilter, Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLQuerySelect( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLQueryFrom( sbSQL );
        sbSQL.append( SQL_QUERY_SEANCE_WIDTH_FILTER_LIST );

        buildSQLQueryFilter( seanceFilter, sbSQL );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;
        daoUtil.setTimestamp( nIndex, startDateFilter );
        nIndex++;
        daoUtil.setTimestamp( nIndex, endDateFilter );
        nIndex++;
        nIndex = fillParameters( nIndex, seanceFilter, daoUtil );

        daoUtil.executeQuery(  );

        List<GSeance> seanceList = new ArrayList<GSeance>(  );

        while ( daoUtil.next(  ) )
        {
            GSeance seance = newSeanceInstance(  );
            fillSeanceFromQuery( seance, daoUtil );
            fillSeanceFromQuerySpec( seance, daoUtil );

            seanceList.add( seance );
        }

        daoUtil.free(  );

        return seanceList;
    }

    /**
     * renvoie la liste des sances dont la date de fin est suprieur ou gale a la date du jour
     * @param plugin Plugin
     * @return liste d'objets Seance
     */
    public List<GSeance> seanceList( Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLQuerySelect( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLQueryFrom( sbSQL );

        StringBuilder sbFilter = new StringBuilder(  );
        GSeanceFilter seanceFilter = newSeanceFilterInstance(  );
        seanceFilter.setTypeSeance( Integer.parseInt( OdsConstants.FILTER_ALL ) );
        sbSQL.append( SQL_KEYWORK_WHERE );
        buildSQLQueryFilter( seanceFilter, sbFilter );

        if ( sbFilter.length(  ) > 0 )
        {
            sbSQL.append( sbFilter );
        }

        buildSQLQueryOrderBySeance( sbSQL );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;
        fillParameters( nIndex, seanceFilter, daoUtil );

        daoUtil.executeQuery(  );

        List<GSeance> seanceList = new ArrayList<GSeance>(  );

        while ( daoUtil.next(  ) )
        {
            GSeance seance = newSeanceInstance(  );
            fillSeanceFromQuery( seance, daoUtil );
            fillSeanceFromQuerySpec( seance, daoUtil );
            seanceList.add( seance );
        }

        daoUtil.free(  );

        return seanceList;
    }

    /**
     * Retourne la liste des prochaines sances
     * @param plugin le plugin
     * @return la liste des prochaines sances
     */
    public List<GSeance> findNextSeancesList( Plugin plugin )
    {
        StringBuilder sbSQL = new StringBuilder(  );
        sbSQL.append( SQL_QUERY_SELECT );
        buildSQLQuerySelect( sbSQL );
        sbSQL.append( SQL_QUERY_FROM );
        buildSQLQueryFrom( sbSQL );

        StringBuilder sbFilter = new StringBuilder(  );
        GSeanceFilter seanceFilter = newSeanceFilterInstance(  );
        seanceFilter.setTypeSeance( Integer.parseInt( OdsConstants.FILTER_ALL ) );
        sbSQL.append( SQL_QUERY_SEANCE_LIST );
        buildSQLQueryFilter( seanceFilter, sbFilter );

        if ( sbFilter.length(  ) > 0 )
        {
            sbSQL.append( sbFilter );
        }

        buildSQLQueryOrderBySeance( sbSQL );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;

        daoUtil.setTimestamp( nIndex, DateUtils.getCurrentDate(  ) );
        nIndex++;
        fillParameters( nIndex, seanceFilter, daoUtil );

        daoUtil.executeQuery(  );

        List<GSeance> seanceList = new ArrayList<GSeance>(  );

        while ( daoUtil.next(  ) )
        {
            GSeance seance = newSeanceInstance(  );
            fillSeanceFromQuery( seance, daoUtil );
            fillSeanceFromQuerySpec( seance, daoUtil );
            seanceList.add( seance );
        }

        daoUtil.free(  );

        return seanceList;
    }

    /**
    * Gnre un nouvel identifiant
    * @param plugin Plugin
    * @return Retourne lidentifiant gnr
    */
    protected int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Remplit l'objet sance
     * @param seance la sance
     * @param daoUtil daoUtil
     */
    protected void fillSeanceFromQuery( GSeance seance, DAOUtil daoUtil )
    {
        seance.setIdSeance( daoUtil.getInt( "se.id_seance" ) );
        seance.setDateSeance( daoUtil.getTimestamp( "se.date_seance" ) );

        if ( daoUtil.getObject( "se.taille_archive" ) != null )
        {
            seance.setDatePublicationArchive( daoUtil.getTimestamp( "se.date_publication" ) );
            seance.setTailleArchive( (long) daoUtil.getDouble( "se.taille_archive" ) );
        }
    }
}
