/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.seance;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.Timestamp;

import java.util.List;


/**
 *
 * AbstractSeanceHome
 *
 * @param <GSeance> {@link ISeance}
 * @param <GSeanceFilter> {@link ISeanceFilter}
 * @param <GFichier> {@link IFichier}
 * @param <GElu> {@link IElu}
 * @param <GVoeuAmendement> {@link IVoeuAmendement}
 * @param <GPDD> {@link IPDD}
 * @param <GSeanceDAO> {@link ISeanceDAO}
 */
public abstract class AbstractSeanceHome
<
	GSeance extends ISeance, 
	GSeanceFilter extends ISeanceFilter, 
	GSeanceDAO extends ISeanceDAO<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD>, 
	GFichier extends IFichier<GSeance, GFichier>, 
	GElu extends IElu, 
	GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, 
	GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>
>
implements ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD>
{
    @Autowired
    protected GSeanceDAO _seanceDAO;

    /**
    * Cre une nouvelle sance  partir de lobjet seance pass en paramtre
    * @param seance la sance  insrer
    * @param plugin Plugin
    */
    public void create( GSeance seance, Plugin plugin )
    {
        _seanceDAO.insert( seance, plugin );
    }

    /**
     * Modifie la sance correspondant  lobjet seance pass en paramtre
     * @param seance la sance  modifier
     * @param plugin Plugin
     */
    public void update( GSeance seance, Plugin plugin )
    {
        _seanceDAO.store( seance, plugin );
    }

    /**
     * Supprime la sance correspondant   lobjet seance pass en paramtre
     * @param nIdSeance la sance  supprimer
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    public void remove( int nIdSeance, Plugin plugin )
        throws AppException
    {
        _seanceDAO.delete( nIdSeance, plugin );
    }

    /**
     * renvoie la sance ayant comme identifiant nKey
     * @param nKey idenfiant de la sance  supprimer
     * @param plugin Plugin
     * @return objet Seance
     */
    public GSeance findByPrimaryKey( int nKey, Plugin plugin )
    {
        return _seanceDAO.load( nKey, plugin );
    }

    /**
     * renvoie la liste des sances
     * @param plugin Plugin
     * @return liste d'objets Seance
     */
    public List<GSeance> findSeanceList( Plugin plugin )
    {
        return _seanceDAO.seanceList( plugin );
    }

    /**
     * renvoie la liste des sances dont la date de dbut est comprise entre startDateFilter et endDateFilter
     *
     * @param startDateFilter date de dbut
     * @param endDateFilter date de fin
     * @param plugin Plugin
     * @return liste d'objets Seance
     */
    public List<GSeance> findSeanceWithFilterList( Timestamp startDateFilter, Timestamp endDateFilter, Plugin plugin )
    {
        return _seanceDAO.loadWithFilter( startDateFilter, endDateFilter, plugin );
    }

    /**
     * renvoie la liste des sances dont la date de dbut est comprise entre startDateFilter et endDateFilter
     *
     * @param startDateFilter date de dbut
     * @param endDateFilter date de fin
     * @param plugin Plugin
     * @param seanceFilter seance filter
     * @return liste d'objets Seance
     */
    public List<GSeance> findSeanceWithFilterList( Timestamp startDateFilter, Timestamp endDateFilter,
        GSeanceFilter seanceFilter, Plugin plugin )
    {
        return _seanceDAO.loadWithFilter( startDateFilter, endDateFilter, seanceFilter, plugin );
    }

    /**
     * Retourne la prochaine sance dont la date de fin est suprieur ou gale a la date du jour
     * @param plugin Plugin
     * @return objet Seance
     */
    public GSeance getProchaineSeance( Plugin plugin )
    {
        return _seanceDAO.findNextSeance( plugin );
    }

    /**
     * Retourne la liste des sances prcdentes
     * @param plugin Plugin
     * @return la liste des Seance prcdentes
     */
    public List<GSeance> findOldSeance( Plugin plugin )
    {
        return _seanceDAO.findOldSeance( plugin );
    }

    /**
     * Retourne la dernire sance dont la date de fin est infrieure  la date du jour
     * @param plugin Plugin
     * @return objet Seance
     */
    public GSeance getDerniereSeance( Plugin plugin )
    {
        return _seanceDAO.findLastSeance( plugin );
    }

    /**
     * Retourne la sance en cours dont la date de dbut est infrieure  la date du jour
     * et la date de fin est suprieure  la date du jour
     * @param plugin Plugin
     * @return objet Seance
     */
    public GSeance getSeanceEnCours( Plugin plugin )
    {
        return _seanceDAO.findSeanceEnCours( plugin );
    }

    /**
    * Retourne la sance  laquelle est rattache un pdd
    * @param nIdPdd l'identifiant du Pdd
    * @param plugin le Plugin
    * @return la sance  laquelle est rattache un pdd
    */
    public GSeance findSeanceByPdd( int nIdPdd, Plugin plugin )
    {
        return _seanceDAO.findSeanceByPdd( nIdPdd, plugin );
    }

    /**
     * Retourne la sance  laquelle est rattache un pdd
     * @param pdd le Pdd
     * @param plugin le Plugin
     * @return la sance  laquelle est rattache un pdd
     */
    public GSeance findSeanceByPddSpec( GPDD pdd, Plugin plugin )
    {
        return _seanceDAO.findSeanceByPddSpec( pdd, plugin );
    }

    /**
     * Retourne la liste des sances qui sont aprs la sance dont son ID est pass en paramtre
     * @param nIdSeance l'identifiant de la sance
     * @param plugin le Plugin
     * @return la liste des sances
     */
    public List<GSeance> findSeanceAfter( int nIdSeance, Plugin plugin )
    {
        return _seanceDAO.findSeanceAfter( nIdSeance, plugin );
    }

    /**
     * Retourne la liste des seances prochaines
     * @param plugin le Plugin
     * @return la liste des prochaines sances
     */
    public List<GSeance> findNextSeancesList( Plugin plugin )
    {
        return _seanceDAO.findNextSeancesList( plugin );
    }
}
