/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.search;

import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.arrete.Arrete;
import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.DeliberationDesignation;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.statut.Statut;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.util.sql.DAOUtil;

import java.util.ArrayList;
import java.util.List;


/**
 * 
 * SearchDAO
 * 
 * @param <GFichier> {@link IFichier}
 * @param <GSeance> {@link ISeance}
 * @param <GSeanceFilter> {@link ISeanceFilter}
 * @param <GRequeteUtilisateur> {@link IRequeteUtilisateur}
 * @param <GElu> {@link IElu}
 * @param <GVoeuAmendement> {@link IVoeuAmendement}
 * @param <GPDD> {@link IPDD}
 */
public abstract class AbstractSearchDAO
<
	GFichier extends IFichier<GSeance, GFichier>, 
	GSeance extends ISeance, 
	GSeanceFilter extends ISeanceFilter, 
	GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, 
	GElu extends IElu, 
	GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, 
	GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>
>
implements ISearchDAO<GFichier, GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GVoeuAmendement, GPDD>
{
    /* JOINTURES PDD */
    protected static final String SQL_JOIN_CATEGORIE_ON_PDD = "LEFT JOIN ods_categorie_deliberation cd ON (pdd.id_categorie = cd.id_categorie) ";
    protected static final String SQL_JOIN_DIRECTION_ON_PDD = "LEFT JOIN ods_direction dir ON (pdd.id_direction = dir.id_direction) " +
        "LEFT JOIN ods_co_emettrice co ON (pdd.id_pdd=co.id_pdd) ";
    protected static final String SQL_JOIN_FORMATION_CONSEIL_ON_PDD = "LEFT JOIN ods_formation_conseil fc ON (pdd.id_formation_conseil = fc.id_formation_conseil) ";
    protected static final String SQL_JOIN_ARRONDISSEMENT_ON_PDD = "LEFT JOIN ods_arrondissement_du_pdd arr_du_pdd ON (pdd.id_pdd = arr_du_pdd.id_pdd)";
    protected static final String SQL_JOIN_STATUT_ON_PDD = "LEFT JOIN ods_statut st ON (pdd.id_statut = st.id_statut) ";

    /* JOINTURES VOEUX / AMENDEMENTS */
    protected static final String SQL_JOIN_STATUT_ON_VA = " LEFT JOIN ods_statut st ON ( st.id_statut = va.id_statut ) ";
    protected static final String SQL_JOIN_SEANCE_ON_VA = " LEFT JOIN ods_fascicule fa ON ( va.id_fascicule = fa.id_fascicule ) " +
        " LEFT JOIN ods_seance se ON ( fa.id_seance = se.id_seance ) ";
    protected static final String SQL_JOIN_FICHIER_ON_VA = " LEFT JOIN ods_fichier fic ON ( fic.id_document = va.id_texte_initial ) ";

    /* INITIALISATION DES FILTRES */
    protected static final String SQL_INIT_WHERE_VA_EN_LIGNE = " WHERE va.en_ligne = 1 ";
    protected static final String SQL_INIT_WHERE_PDD_EN_LIGNE = " WHERE pdd.en_ligne = 1 ";
    protected static final String SQL_INIT_WHERE = " WHERE 1 = 1 ";

    /* SELECTION DES DONNEES A RECUPERER */
    protected static final String SQL_QUERY_PDD_LIST = "SELECT DISTINCT pdd.id_pdd, pdd.reference, pdd.version, pdd.date_publication, " +
        "pdd.objet, pdd.type_pdd, pdd.id_formation_conseil, pdd.id_statut, st.id_statut, st.libelle_statut FROM ods_pdd pdd ";
    protected static final String SQL_QUERY_FICHIERS = "SELECT DISTINCT fic.id_document , fic.intitule , fic.extension , fic.taille , " +
        " fic.date_publication, fic.version, fic.texte, fic.id_fichier, se.id_seance, se.date_seance, se.date_fin_seance " +
        " FROM ods_fichier fic LEFT JOIN ods_seance se ON ( fic.id_seance = se.id_seance ) WHERE fic.en_ligne = 1 ";
    protected static final String SQL_QUERY_VA_LIST = "SELECT DISTINCT va.id_va, va.reference_va_front, va.objet, va.date_publication, va.version, va.id_statut, " +
        " va.type, va.id_fascicule, fa.code_fascicule, st.id_statut, st.libelle_statut, fic.id_document, fic.extension, fic.taille,fic.intitule, fic.date_publication, " +
        " fic.version, se.id_seance, se.date_seance, se.date_fin_seance FROM ods_voeu_amendement va ";

    /* FILTRES PDD */
    protected static final String SQL_FILTER_FORMATION_CONSEIL_ON_PDD = " AND pdd.id_formation_conseil = ? ";
    protected static final String SQL_FILTER_TYPE_PDD = " AND pdd.type_pdd = ? ";
    protected static final String SQL_FILTER_REFERENCE_PDD = " AND pdd.reference LIKE ? ";

    /* FILTRES VOEUX / AMENDEMENTS */
    protected static final String SQL_FILTER_PROCHAINE_SEANCE_ON_VA = " AND se.id_seance = ? ";

    /* FILTRES SUR LES FICHIERS */
    protected static final String SQL_FILTER_ID_FICHIER = " AND fic.id_document IN ( ";
    protected static final String SQL_FILTER_TYPE_FICHIER = " AND fic.id_type_document IN ( ";
    protected static final String SQL_FILTER_SEANCE_ON_FICHIER = " AND fic.id_seance = ? ";
    protected static final String SQL_FILTER_LISTE_SEANCES_ON_FICHIER = " AND fic.id_seance IN (  ";

    /* TRI DES REQUETES */
    protected static final String SQL_QUERY_ORDER_BY_REFERENCE = " ORDER BY pdd.reference ";

    /* SELECTION DES DONNEES A RECUPERER */
    private static final String SQL_QUERY_VA_LIST_ARCHIVES = "SELECT DISTINCT va.id_va, va.reference_va_front, va.objet, va.date_publication, va.version, va.id_statut, " +
        " va.type, va.id_fascicule, fa.code_fascicule, st.id_statut, st.libelle_statut, fic.id_document, fic.extension, fic.taille, fic.intitule, fic.date_publication, " +
        " fic.version, se.id_seance, se.date_seance, se.date_fin_seance, fic2.id_document, fic2.extension, fic2.taille, fic2.intitule, fic2.date_publication, " +
        " fic2.version FROM ods_voeu_amendement va ";
    private static final String SQL_QUERY_PDD_DU_VA = "SELECT DISTINCT pdd.id_pdd, pdd.reference, pdd.id_formation_conseil FROM ods_pdd pdd ,ods_va_rattache_pdd vrp ";

    /* SELECTION DES DONNEES A RECUPERER */
    private static final String SQL_QUERY_ARRETES = "SELECT ar.id_arrete, ar.id_seance, ar.id_formation_conseil, ar.numero, " +
        " ar.intitule, ar.id_fichier_or, ar.id_fichier_sc, ar.id_categorie, ar.date_publication, ar.en_ligne, ar.version " +
        " FROM ods_arrete ar, ods_seance se WHERE ar.id_seance = se.id_seance AND ar.en_ligne = 1";
    private static final String SQL_QUERY_DELIBERATIONS_DESIGNATION = "SELECT de.id_designation, de.id_seance, de.id_formation_conseil, de.numero, " +
        " de.intitule, de.id_fichier_or, de.id_fichier_sc, de.id_categorie, de.date_publication, de.en_ligne, de.version, ac.id_acte " +
        " FROM ods_designation de, ods_seance se, ods_acte ac WHERE de.id_seance = se.id_seance AND de.en_ligne = 1 " +
        " AND ac.id_entite = de.id_designation AND ac.id_type_entite = 3 AND ac.en_ligne = 1";

    /* JOINTURES VOEUX / AMENDEMENTS */
    private static final String SQL_JOIN_VA_DEPOSE_ON_VA = " LEFT JOIN ods_va_depose_par vadp ON ( va.id_va = vadp.id_va ) ";
    private static final String SQL_JOIN_FICHIER_ON_VA_ARCHIVES = " LEFT JOIN ods_fichier fic ON ( fic.id_document = va.id_texte_initial ) " +
        " LEFT JOIN ods_fichier fic2 ON ( fic2.id_document = va.id_delib )";

    /* INITIALISATION DES FILTRES */
    private static final String SQL_INIT_WHERE_PDD_DU_VA = " WHERE vrp.id_pdd= pdd.id_pdd AND pdd.en_ligne = 1 AND vrp.id_va = ?";

    /* FILTRES PDD */
    private static final String SQL_FILTER_CATEGORIE_ON_PDD = " AND pdd.id_categorie IN (";
    private static final String SQL_FILTER_ID_PDD = " AND pdd.id_pdd IN ( ";
    private static final String SQL_FILTER_DIRECTION_CO_EM_ON_PDD = " AND ( ( pdd.id_pdd=co.id_pdd AND co.id_direction IN ( ";
    private static final String SQL_FILTER_DIRECTION_ON_PDD = " ) ) OR pdd.id_direction IN ( ";
    private static final String SQL_FILTER_ARRONDISSEMENT_ON_PDD = " AND arr_du_pdd.arrondissement IN ( ";
    private static final String SQL_FILTER_GROUPE_ON_PDD = " AND pdd.id_groupe IN ( ";
    private static final String SQL_FILTER_LISTE_SEANCES_ON_PDD = " AND pdd.id_pdd IN ( SELECT pdd.id_pdd FROM ods_pdd pdd " +
        " LEFT JOIN ods_entree_ordre_jour entree ON ( entree.id_entite = pdd.id_pdd )" +
        " LEFT JOIN ods_odj odj ON ( entree.id_odj = odj.id_odj ) " +
        " WHERE odj.id_type_odj = 2 AND entree.id_type_entite = " + ITypeEntite.PDD + " AND odj.id_seance IN ( ";

    /* FILTRES VOEUX / AMENDEMENTS */
    private static final String SQL_FILTER_FORMATION_CONSEIL_ON_VA = " AND va.id_formation_conseil = ? ";
    private static final String SQL_FILTER_RAPPORTEUR_ON_VOEU = " AND va.id_va in ( SELECT va.id_va FROM ods_voeu_amendement va " +
        " LEFT JOIN ods_entree_ordre_jour entree ON ( entree.id_va = va.id_va ) " +
        " LEFT JOIN ods_entree_elus elu ON ( entree.id_entree= elu.id_entree ) " +
        " LEFT JOIN ods_odj odj ON ( entree.id_odj = odj.id_odj ) " + " WHERE elu.id_elu IN ( ";
    private static final String SQL_FILTER_DEPOSITAIRE_ON_VA = " vadp.id_va = va.id_va AND ( ";
    private static final String SQL_FILTER_VA_DEPOSE_EXECUTIF = " va.depose_executif=true ";
    private static final String SQL_FILTER_GROUPE_ELU_ON_VA = " vadp.id_elu IN ( SELECT elu.id_elu FROM ods_elu elu " +
        "LEFT JOIN ods_groupe groupe ON ( groupe.id_groupe = elu.id_groupe ) " + "WHERE groupe.id_groupe IN ( ";
    private static final String SQL_FILTER_GROUPE_ON_VA = " vadp.id_groupe IN ( ";
    private static final String SQL_FILTER_ELU_ON_VA = " vadp.id_elu IN ( ";
    private static final String SQL_FILTER_SEANCE_ON_VA = " AND se.id_seance = ? ";
    private static final String SQL_FILTER_LISTE_SEANCES_ON_VA = " AND se.id_seance IN ( ";
    private static final String SQL_FILTER_ID_VA = " AND va.id_va IN ( ";
    private static final String SQL_FILTER_TYPE_AMENDEMENT = " AND ( va.type = 'A' OR va.type = 'LR' ) ";
    private static final String SQL_FILTER_TYPE_VOEU = " AND ( va.type = 'V' OR va.type = 'VNR' ) ";
    private static final String SQL_FILTER_SATUT_PDD = " AND  pdd.id_statut in (  ";
    private static final String SQL_FILTER_SATUT_VOEU_AMENDEMENT = " AND  va.id_statut in (  ";

    /* FILTRES SUR LES FICHIERS */
    private static final String SQL_FILTER_ID_PDD_ON_FICHIER = " AND fic.id_pdd = ? ";
    private static final String SQL_FILTER_TYPE_FICHIER_DELIBERATION_ON_FICHIER = " AND fic.id_type_document NOT IN ( " +
        ITypeDocument.DELIBERATION + " ) ";

    /* FILTRES SUR LES ARRETES */
    private static final String SQL_FILTER_ID_ARRETE = " AND ar.id_arrete IN ( ";
    private static final String SQL_FILTER_LISTE_SEANCES_ON_ARRETE = " AND ar.id_seance IN (  ";
    private static final String SQL_FILTER_FORMATION_CONSEIL_ON_ARRETE = " AND ar.id_formation_conseil = ? ";

    /* FILTRES SUR LES DELIBERATIONS DE DESIGNATION */
    private static final String SQL_FILTER_ID_DELIBERATION_DESIGNATION = " AND de.id_designation IN ( ";
    private static final String SQL_FILTER_LISTE_SEANCES_ON_DELIBERATION_DESIGNATION = " AND de.id_seance IN (  ";
    private static final String SQL_FILTER_FORMATION_CONSEIL_ON_DELIBERATION_DESIGNATION = " AND de.id_formation_conseil = ? ";

    /* TRI DES REQUETES */
    private static final String SQL_QUERY_ORDER_PDD_DU_VA = " ORDER BY vrp.numero_ordre";
    private static final String SQL_QUERY_ORDER_FICHIERS_DU_PDD = " ORDER BY fic.id_type_document, fic.id_document";

    /**
     * Cre une nouvelle instance de requete utilisateur
     * @return une nouvelle instance de requete utilisateur
     */
    protected abstract GRequeteUtilisateur getRequeteUtilisateur(  );

    protected abstract void exploiterDaoVASpec( DAOUtil daoVA, GVoeuAmendement va );

    /**
     * Prpare la requete SQL pour les projets
     * @param requete la requete
     * @param listIdsProjets la liste des ids de projet
     * @param plugin le plugin
     * @return la requete SQL pour les projets
     */
    protected abstract String preparerSqlProjetsSpec( GRequeteUtilisateur requete, List<String> listIdsProjets,
        Plugin plugin );

    /**
     * Prpare la requete SQL pour les propositions
     * @param requete la requete
     * @param listIdsProjets la liste des ids de proposition
     * @param plugin le plugin
     * @return la requete SQL pour les propositions
     */
    protected abstract String preparerSqlPropositionsSpec( GRequeteUtilisateur requete, List<String> listIdsProjets,
        Plugin plugin );

    /**
     * Prpare le SQL pour les amendements
     * @param requete la requete
     * @param listIdsAmendements la liste des ids amendements
     * @param plugin le plugin
     * @return le SQL pour les amendements
     */
    protected abstract String preparerSqlAmendementsSpec( GRequeteUtilisateur requete, List<String> listIdsAmendements,
        Plugin plugin );

    /**
     * Prpare le SQL pour les voeux
     * @param requete la requete
     * @param listIdsVoeux la liste des ids des voeux
     * @param plugin le plugin
     * @return le SQL pour les voeux
     */
    protected abstract String preparerSqlVoeuxSpec( GRequeteUtilisateur requete, List<String> listIdsVoeux,
        Plugin plugin );

    /**
     * Cre une nouvelle instance de sance
     * @return une nouvelle instance de sance
     */
    protected abstract GSeance newSeanceInstance(  );

    /**
     * Cre une nouvelle  instance de fichier
     * @return une nouvelle instance de fichier
     */
    protected abstract GFichier newFichierInstance(  );

    /**
     * Cre une nouvelle isntance de voeu amendement
     * @return une nouvelle instance de voeu amendement
     */
    protected abstract GVoeuAmendement newVoeuAmendementInstance(  );

    /**
     * Cre une nouvelle instance de PDD
     * @return une nouvelle instance de PDD
     */
    protected abstract GPDD newPDDInstance(  );

    /**
     * Retourne le filtre de sance pour un PDD
     * @return le filtre de ssance pour un PDD
     */
    protected abstract String getSqlFilterSeanceOnPdd(  );

    /**
     * Retourne le filtre pour les rapporteurs d'un pdd
     * @return le filtre pour les rapporteurs d'un pdd
     */
    protected abstract String getSqlFilterRapporteurOnPdd(  );

    /**
     * Retourne le filtre de prochaine sance pour les PDD
     * @return le filtre de prochaine sance pour les PDD
     */
    protected abstract String getSqlFilterProchaineSeanceOnPdd(  );

    /**
     * Retourne la liste des PDD  partir du PDD
     * @param daoPddsAIndexer le ADO
     * @return la liset des PDD
     */
    protected abstract List<GPDD> exploiterDaoPDDspec( DAOUtil daoPddsAIndexer );

    /**
     * Met les valeurs du filtre dans le DAOUtil
     * @param archive <b>true</b> si archive, <b>false</b> sinon
     * @param prochaineSeance la prochaine sance
     * @param plugin le plugin
     * @param daoUtil daoUtil
     * @param nIndex l'index de dpart
     * @return l'index de sortie
     */
    protected abstract int selectPDDsAIndexerSpec( boolean archive, GSeance prochaineSeance, Plugin plugin,
        DAOUtil daoUtil, int nIndex );

    /**
     * Retourne la requte SQL de slection de tous les PDDs  indexer
     * @param bArchive <code>true</code> ==> tous les PDDs inscrits  un ordre du jour dfinitif d'une sance passe,<br>
     *                                    <code>false</code> ==> tous les PDDs de la prochaine sance, ie qui ne sont pas inscrits  un OdJ dfinitif d'une sance passe
     * @param plugin le plugin
     * @return la requte de slection des PDDs
     */
    protected abstract String preparerSqlPddsAIndexerSpec( boolean bArchive, Plugin plugin );

    /**
     * Prpare la requte de slection de tous les VA  indexer
     * @param bArchive si la recherche concerne les archives
     * @param plugin le plugin ODS
     * @return la requte de slection des VAs
     */
    protected abstract String preparerSqlVAAIndexerSpec( boolean bArchive, Plugin plugin );

    /**
     * Exploite les rsultats d'une requte de slection de fichiers
     * @param daoFichiers le rsultat de la requte
     * @return une liste de fichiers
     */
    protected abstract List<GFichier> exploiterDaoFichier( DAOUtil daoFichiers );

    /**
     * Retourne le filtre la requete de la liste des sance pour un pdd
     * @return le filtre  la requete de la liste des sance pour un pdd
     */
    protected String getFilterListSeancePDD(  )
    {
        return SQL_FILTER_LISTE_SEANCES_ON_PDD;
    }

    /* ************************************* */
    /*              PROJETS                  */
    /* ************************************* */

    /**
     * Retourne la liste des projets filtrs par les critres de recherche
     * @param requete la requte utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsProjets la liste des PDDs retourns par la recherche dans l'index
     *                         (<code>null</code> si pas de mot-cl renseign)
     * @param plugin le plugin
     * @return la liste des {@link PDD} de type projet recherchs
     */
    public List<GPDD> selectProjetsDeliberation( GRequeteUtilisateur requete, GSeance prochaineSeance,
        List<String> listIdsProjets, Plugin plugin )
    {
        String strSql;

        if ( requete.getTypeRequete(  ).equals( OdsParameters.REFERENCE ) )
        {
            strSql = preparerSqlPddReference( requete, plugin );
        }
        else
        {
            strSql = preparerSqlProjets( requete, listIdsProjets, plugin );
            strSql += preparerSqlProjetsSpec( requete, listIdsProjets, plugin );
            strSql += SQL_QUERY_ORDER_BY_REFERENCE;
        }

        DAOUtil daoProjets = new DAOUtil( strSql, plugin );

        daoProjets.setString( 1, IPDD.CONSTANTE_TYPE_PROJET );

        int nIndex = 2;

        if ( requete.isFiltreFormationsConseilActif( plugin ) )
        {
            daoProjets.setString( nIndex, requete.getListeFormationsConseil(  ) );
            nIndex++;
        }

        if ( !requete.isRechercheArchive(  ) )
        {
            daoProjets.setInt( nIndex, prochaineSeance.getIdSeance(  ) );
            nIndex++;
        }

        if ( requete.getTypeRequete(  ).equals( OdsParameters.REFERENCE ) )
        {
            daoProjets.setString( nIndex, "%" + requete.getChampRecherche(  ) + "%" );
        }

        daoProjets.executeQuery(  );

        return exploiterDaoPDD( daoProjets );
    }

    /* ****************************************** */
    /*              PROPOSITIONS                  */
    /* ****************************************** */

    /**
    * Retourne la liste des propositions filtrs par les critres de recherche
    * @param requete la requte utilisateur
    * @param prochaineSeance la prochaine sance
    * @param listIdsPropositions la liste des PDDs retourns par la recherche dans l'index
    *                         (<code>null</code> si pas de mot-cl renseign)
    * @param plugin le plugin
    * @return la liste des {@link PDD} de type proposition recherchs
    */
    public List<GPDD> selectPropositionsDeliberation( GRequeteUtilisateur requete, GSeance prochaineSeance,
        List<String> listIdsPropositions, Plugin plugin )
    {
        String strSql;

        if ( requete.getTypeRequete(  ).equals( OdsParameters.REFERENCE ) )
        {
            strSql = preparerSqlPddReference( requete, plugin );
        }
        else
        {
            strSql = preparerSqlPropositions( requete, listIdsPropositions, plugin );
            strSql += preparerSqlPropositionsSpec( requete, listIdsPropositions, plugin );
            strSql += SQL_QUERY_ORDER_BY_REFERENCE;
        }

        DAOUtil daoPropositions = new DAOUtil( strSql, plugin );

        daoPropositions.setString( 1, IPDD.CONSTANTE_TYPE_PROP );

        int nIndex = 2;

        if ( requete.isFiltreFormationsConseilActif( plugin ) )
        {
            daoPropositions.setString( nIndex, requete.getListeFormationsConseil(  ) );
            nIndex++;
        }

        if ( !requete.isRechercheArchive(  ) )
        {
            daoPropositions.setInt( nIndex, prochaineSeance.getIdSeance(  ) );
            nIndex++;
        }

        if ( requete.getTypeRequete(  ).equals( OdsParameters.REFERENCE ) )
        {
            daoPropositions.setString( nIndex, "%" + requete.getChampRecherche(  ) + "%" );
        }

        daoPropositions.executeQuery(  );

        return exploiterDaoPDD( daoPropositions );
    }

    /* ************************************************* */
    /*                FICHIERS D'UN PDD                  */
    /* ************************************************* */

    /**
     * Retourne la liste des fichiers du PDD correspondant aux critres de recherche
     * @param listIdsFichiers la liste des ids de fichier correspondants aux critres de recherche (<code>null</code> si pas de mot-cl)
     * @param nIdPdd l'id du pdd
     * @param plugin le plugin
     * @return une liste de fichiers
     */
    public List<GFichier> selectFichiersDuPdd( List<String> listIdsFichiers, int nIdPdd, Plugin plugin )
    {
        String strSql = preparerSqlFichiersDuPdd( listIdsFichiers );

        DAOUtil daoFichiersDuPdd = new DAOUtil( strSql, plugin );
        daoFichiersDuPdd.setInt( 1, nIdPdd );
        daoFichiersDuPdd.executeQuery(  );

        return exploiterDaoFichier( daoFichiersDuPdd );
    }

    /* ******************************************** */
    /*              PDDS A INDEXER                  */
    /* ******************************************** */

    /**
     * Retourne la liste des Pdds  indexer
     * @param bArchive indique si l'on veut les Pdds archivs ou de la prochaine sance
     * @param prochaineSeance la prochaine sance (<code>null</code> si pas de prochaine sance)
     * @param plugin le plugin
     * @return une liste de Pdds
     */
    public List<GPDD> selectPDDsAIndexer( boolean bArchive, GSeance prochaineSeance, Plugin plugin )
    {
        String strSql = preparerSqlPddsAIndexerSpec( bArchive, plugin );

        DAOUtil daoPddsAIndexer = new DAOUtil( strSql, plugin );

        int nIndex = 0;

        if ( !bArchive )
        {
            nIndex++;
            daoPddsAIndexer.setInt( nIndex, prochaineSeance.getIdSeance(  ) );
        }

        selectPDDsAIndexerSpec( bArchive, prochaineSeance, plugin, daoPddsAIndexer, nIndex );

        daoPddsAIndexer.executeQuery(  );

        return exploiterDaoPDDspec( daoPddsAIndexer );
    }

    /**
    * Retourne la liste des Pdds  ajouter dans l'index des archives
    * @param nIdAncienneSeance l'id de la dernire sance coule
    * @param plugin le plugin
    * @return une liste de Pdds
    */
    public List<GPDD> selectPDDsChangementSeance( int nIdAncienneSeance, Plugin plugin )
    {
        String strSql = preparerSqlPDDChangementSeance(  );

        DAOUtil daoPddsAIndexer = new DAOUtil( strSql, plugin );
        daoPddsAIndexer.setInt( 1, nIdAncienneSeance );

        daoPddsAIndexer.executeQuery(  );

        return exploiterDaoPDD( daoPddsAIndexer );
    }

    /* ************************************ */
    /*              AMENDEMENTS             */
    /* ************************************ */

    /**
     * Retourne la liste des amendements filtrs par les critres de recherche
     * @param requete la requte utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsAmendements la liste des amendements retourns par la recherche dans l'index
     *                         (<code>null</code> si pas de mot-cl renseign)
     * @param plugin le plugin
     * @return la liste des {@link VoeuAmendement} de type Amendement ou Lettre Rectificative recherchs
     */
    public List<GVoeuAmendement> selectAmendements( GRequeteUtilisateur requete, GSeance prochaineSeance,
        List<String> listIdsAmendements, Plugin plugin )
    {
        String strSql = preparerSqlAmendements( requete, listIdsAmendements, plugin );
        strSql += preparerSqlAmendementsSpec( requete, listIdsAmendements, plugin );

        DAOUtil daoAmendements = new DAOUtil( strSql, plugin );

        int nIndex = 1;

        if ( requete.isFiltreFormationsConseilActif( plugin ) )
        {
            daoAmendements.setString( nIndex, requete.getListeFormationsConseil(  ) );
            nIndex++;
        }

        if ( !requete.isRechercheArchive(  ) )
        {
            daoAmendements.setInt( nIndex, prochaineSeance.getIdSeance(  ) );
            nIndex++;
        }

        daoAmendements.executeQuery(  );

        List<GVoeuAmendement> listAmendements = exploiterDaoVA( daoAmendements, requete.isRechercheArchive(  ) );

        for ( GVoeuAmendement amendement : listAmendements )
        {
            amendement.setPdds( getListePddDuVa( amendement.getId(  ), plugin ) );
        }

        return listAmendements;
    }

    /* ********************************************** */
    /*                    VOEUX                       */
    /* ********************************************** */

    /**
     * Retourne la liste des voeux filtrs par les critres de recherche
     * @param requete la requte utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsVoeux la liste des voeux retourns par la recherche dans l'index
     *                         (<code>null</code> si pas de mot-cl renseign)
     * @param plugin le plugin
     * @return la liste des {@link VoeuAmendement} de type Voeu ou Voeu Non Rattach recherchs
     */
    public List<GVoeuAmendement> selectVoeux( GRequeteUtilisateur requete, GSeance prochaineSeance,
        List<String> listIdsVoeux, Plugin plugin )
    {
        String strSql = preparerSqlVoeux( requete, listIdsVoeux, plugin );
        strSql += preparerSqlVoeuxSpec( requete, listIdsVoeux, plugin );

        DAOUtil daoVoeux = new DAOUtil( strSql, plugin );

        int nIndex = 1;

        if ( requete.isFiltreFormationsConseilActif( plugin ) )
        {
            daoVoeux.setString( nIndex, requete.getListeFormationsConseil(  ) );
            nIndex++;
        }

        if ( !requete.isRechercheArchive(  ) )
        {
            daoVoeux.setInt( nIndex, prochaineSeance.getIdSeance(  ) );
            nIndex++;
        }

        daoVoeux.executeQuery(  );

        List<GVoeuAmendement> listVoeux = exploiterDaoVA( daoVoeux, requete.isRechercheArchive(  ) );

        for ( GVoeuAmendement voeu : listVoeux )
        {
            voeu.setPdds( getListePddDuVa( voeu.getId(  ), plugin ) );
        }

        return listVoeux;
    }

    /* ******************************************** */
    /*                PDDS D'UN VA                  */
    /* ******************************************** */

    /**
     * Retourne la liste des PDDs auxquels le {@link VoeuAmendement} est rattach
     * @param nIdVa l'id du voeu ou de l'amendement
     * @param plugin le plugin
     * @return une liste de PDD rattachs au VA
     */
    protected List<GPDD> getListePddDuVa( int nIdVa, Plugin plugin )
    {
        List<GPDD> listPddsDuVa = new ArrayList<GPDD>(  );

        String strSqlPddsDuVa = SQL_QUERY_PDD_DU_VA + SQL_INIT_WHERE_PDD_DU_VA + SQL_QUERY_ORDER_PDD_DU_VA;

        DAOUtil daoPddsDuVa = new DAOUtil( strSqlPddsDuVa, plugin );
        daoPddsDuVa.setInt( 1, nIdVa );
        daoPddsDuVa.executeQuery(  );

        while ( daoPddsDuVa.next(  ) )
        {
            GPDD pdd = newPDDInstance(  );
            pdd.setId( daoPddsDuVa.getInt( 1 ) );
            pdd.setReference( daoPddsDuVa.getString( 2 ) );

            FormationConseil fc = new FormationConseil(  );
            fc.setIdFormationConseil( daoPddsDuVa.getInt( 3 ) );
            pdd.setFormationConseil( fc );

            listPddsDuVa.add( pdd );
        }

        daoPddsDuVa.free(  );

        return listPddsDuVa;
    }

    /* ********************************************* */
    /*                VAS A INDEXER                  */
    /* ********************************************* */

    /**
     * Retourne la liste des voeux et amendements  indexer
     * @param bArchive indique si l'on veut les VAs archivs ou de la prochaine sance
     * @param prochaineSeance la prochaine sance (<code>null</code> si pas de prochaine sance)
     * @param plugin le plugin
     * @return une liste de VoeuAmendement
     */
    public List<GVoeuAmendement> selectVAAIndexer( boolean bArchive, GSeance prochaineSeance, Plugin plugin )
    {
        String strSql = preparerSqlVAAIndexerSpec( bArchive, plugin );

        DAOUtil daoVAAIndexer = new DAOUtil( strSql, plugin );

        if ( !bArchive )
        {
            daoVAAIndexer.setInt( 1, prochaineSeance.getIdSeance(  ) );
        }

        daoVAAIndexer.executeQuery(  );

        return exploiterDaoVA( daoVAAIndexer, false );
    }

    /**
     * Retourne la liste des Voeux et Amendements  indexer dans les archives aprs un changement de sance
     * @param nIdAncienneSeance la dernire sance passe
     * @param plugin le plugin
     * @return une liste de VoeuAmendment
     */
    public List<GVoeuAmendement> selectVAChangementSeance( int nIdAncienneSeance, Plugin plugin )
    {
        String strSql = preparerSqlVAChangementSeance(  );

        DAOUtil daoVAAIndexer = new DAOUtil( strSql, plugin );
        daoVAAIndexer.setInt( 1, nIdAncienneSeance );

        daoVAAIndexer.executeQuery(  );

        return exploiterDaoVA( daoVAAIndexer, false );
    }

    /* ************************************************** */
    /*                FICHIERS A INDEXER                  */
    /* ************************************************** */

    /**
     * Retourne la liste des fichiers  indexer dans les archives
     * @param plugin le plugin
     * @return une liste de fichiers
     */
    public List<GFichier> selectFichiersAIndexer( Plugin plugin )
    {
        String strSql = preparerSqlFichiersAIndexer( plugin );

        DAOUtil daoFichiersAIndexer = new DAOUtil( strSql, plugin );
        daoFichiersAIndexer.executeQuery(  );

        return exploiterDaoFichier( daoFichiersAIndexer );
    }

    /**
     * Retourne la liste des fichiers  indexer lors du changement de sance
     * @param nIdDerniereSeance l'id de la sance qui se termine
     * @param plugin le plugin
     * @return une liste de fichiers
     */
    public List<GFichier> selectFichiersChangementSeance( int nIdDerniereSeance, Plugin plugin )
    {
        String strSql = preparerSqlFichiersChangementSeance(  );

        DAOUtil daoFichiers = new DAOUtil( strSql, plugin );
        daoFichiers.setInt( 1, nIdDerniereSeance );
        daoFichiers.executeQuery(  );

        return exploiterDaoFichier( daoFichiers );
    }

    /* ***************************************** */
    /*                FICHIERS                   */
    /* ***************************************** */

    /**
     * Retourne la liste des documents de sance filtrs par les critres de recherche
     * @param requete la requte utilisateur
     * @param listIdsFichiers la liste des id de fichier retourns par la recherche dans l'index
     * @param plugin le plugin
     * @return la liste des {@link Fichier} recherchs
     */
    public List<GFichier> selectAutresDocuments( GRequeteUtilisateur requete, List<String> listIdsFichiers,
        Plugin plugin )
    {
        String strSql = preparerSqlAutresDocuments( requete, listIdsFichiers, plugin );

        DAOUtil daoAutresDocs = new DAOUtil( strSql, plugin );
        daoAutresDocs.executeQuery(  );

        return exploiterDaoFichier( daoAutresDocs );
    }

    /**
     * Retourne la liste arretes filtrs par les critres de recherche
     * @param requete la requte utilisateur
     * @param listIdsArretes la liste des id d'arrtes retourns par la recherche dans l'index
     * @param plugin le plugin
     * @return la liste des arretes recherchs
     */
    public List<Arrete<GSeance, GFichier>> selectArretes( GRequeteUtilisateur requete, List<String> listIdsArretes,
        Plugin plugin )
    {
        String strSql = preparerSqlArretes( requete, listIdsArretes, plugin );

        DAOUtil daoArretes = new DAOUtil( strSql, plugin );

        if ( requete.isFiltreFormationsConseilActif( plugin ) )
        {
            daoArretes.setString( 1, requete.getListeFormationsConseil(  ) );
        }

        daoArretes.executeQuery(  );

        return exploiterDaoArrete( daoArretes );
    }

    /**
     * Retourne la liste des dlibration de dsignation  indexer
     * @param plugin le plugin
     * @return une liste de dlibrations de dsignation
     */
    public List<DeliberationDesignation<GSeance, GFichier>> selectDeliberationDesignationAIndexer( Plugin plugin )
    {
        String strSql = preparerSqlDeliberationDesignationAIndexer(  );

        DAOUtil daoDeliberationDesignationAIndexer = new DAOUtil( strSql, plugin );

        daoDeliberationDesignationAIndexer.executeQuery(  );

        return exploiterDaoDeliberationDesignation( daoDeliberationDesignationAIndexer );
    }

    /**
     * Retourne la liste des dlibration de dsignation filtres par les critres de recherche
     * @param requete la requte utilisateur
     * @param listIdsDeliberationsDesignation la liste des identifiants des dlibration de dsignation
     * @param plugin le plugin ODS
     * @return la liste des dlibration de dsignation recherches
     */
    public List<DeliberationDesignation<GSeance, GFichier>> selectDeliberationsDesignation( 
        GRequeteUtilisateur requete, List<String> listIdsDeliberationsDesignation, Plugin plugin )
    {
        String strSql = preparerSqlDeliberationsDesignation( requete, listIdsDeliberationsDesignation, plugin );

        DAOUtil daoDeliberationsDesignation = new DAOUtil( strSql, plugin );

        if ( requete.isFiltreFormationsConseilActif( plugin ) )
        {
            daoDeliberationsDesignation.setString( 1, requete.getListeFormationsConseil(  ) );
        }

        daoDeliberationsDesignation.executeQuery(  );

        return exploiterDaoDeliberationDesignation( daoDeliberationsDesignation );
    }

    /* ******************************************************* */
    /*                  PREPARATIONS DES SQLS                  */
    /* ******************************************************* */

    /**
     * Prpare la requte correspondant aux critres de recherche de l'utilisateur
     * @param requete la requte utilisateur
     * @param listIdsProjets la liste des id de pdd retourns par la recherche dans l'index
     * @param plugin le plugin
     * @return la requte SQL correspondant aux critres de recherche
     */
    protected String preparerSqlProjets( GRequeteUtilisateur requete, List<String> listIdsProjets, Plugin plugin )
    {
        StringBuffer sbSqlProjets = new StringBuffer(  );

        sbSqlProjets.append( SQL_QUERY_PDD_LIST + SQL_JOIN_STATUT_ON_PDD );

        sbSqlProjets.append( ( requete.isFiltreFormationsConseilActif( plugin ) ? SQL_JOIN_FORMATION_CONSEIL_ON_PDD
                                                                                : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSqlProjets.append( ( requete.isFiltreDirectionsActif(  ) ? SQL_JOIN_DIRECTION_ON_PDD
                                                                   : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSqlProjets.append( ( requete.isFiltreCategoriesDeliberationActif(  ) ? SQL_JOIN_CATEGORIE_ON_PDD
                                                                               : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSqlProjets.append( ( requete.isFiltreArrondissementsActif(  ) ? SQL_JOIN_ARRONDISSEMENT_ON_PDD
                                                                        : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        sbSqlProjets.append( ( SQL_INIT_WHERE_PDD_EN_LIGNE + SQL_FILTER_TYPE_PDD ) );

        sbSqlProjets.append( ( requete.isFiltreRapporteursActif(  ) ? getFiltreRapporteurPDD( requete )
                                                                    : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSqlProjets.append( ( requete.isFiltreFormationsConseilActif( plugin ) ? SQL_FILTER_FORMATION_CONSEIL_ON_PDD
                                                                                : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSqlProjets.append( ( requete.isFiltreDirectionsActif(  ) ? getFiltreDirectionsProjet( requete )
                                                                   : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSqlProjets.append( ( requete.isFiltreCategoriesDeliberationActif(  ) ? getFiltreCategoriesPDD( requete )
                                                                               : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSqlProjets.append( ( requete.isFiltreArrondissementsActif(  ) ? getFiltreArrondissementsPDD( requete )
                                                                        : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        /* if ( requete.isRechercheArchive(  ) )
         {
             sbSqlProjets.append( getFiltreSeancesPDD( requete, plugin ) );
         }
         else
         {
             sbSqlProjets.append( SQL_FILTER_PROCHAINE_SEANCE_ON_PDD );
         }*/
        if ( ( listIdsProjets != null ) && ( listIdsProjets.size(  ) != 0 ) )
        {
            sbSqlProjets.append( getFiltreIdsPdds( listIdsProjets ) );
        }

        sbSqlProjets.append( ( requete.isFiltreStatutsProjets(  ) ? getFiltreStatutsProjets( requete )
                                                                  : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        return sbSqlProjets.toString(  );
    }

    /**
     * Prpare la requte de slection des propositions selon les critres de recherche
     * @param requete la requte utilisateur
     * @param listIdsPdd la liste des pdds retourns par la recherche dans l'index
     * @param plugin le plugin
     * @return la requte SQL gnre avec les donnes utilisateurs
     */
    private String preparerSqlPropositions( GRequeteUtilisateur requete, List<String> listIdsPdd, Plugin plugin )
    {
        // prparation de la requte: tables jointes
        StringBuffer sbSqlPropositions = new StringBuffer(  );

        sbSqlPropositions.append( SQL_QUERY_PDD_LIST + SQL_JOIN_STATUT_ON_PDD );

        sbSqlPropositions.append( ( requete.isFiltreFormationsConseilActif( plugin )
            ? SQL_JOIN_FORMATION_CONSEIL_ON_PDD : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSqlPropositions.append( ( requete.isFiltreCategoriesDeliberationActif(  ) ? SQL_JOIN_CATEGORIE_ON_PDD
                                                                                    : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSqlPropositions.append( ( requete.isFiltreArrondissementsActif(  ) ? SQL_JOIN_ARRONDISSEMENT_ON_PDD
                                                                             : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        sbSqlPropositions.append( ( SQL_INIT_WHERE_PDD_EN_LIGNE + SQL_FILTER_TYPE_PDD ) );

        // prparation de la requte: filtres
        sbSqlPropositions.append( ( requete.isFiltreFormationsConseilActif( plugin )
            ? SQL_FILTER_FORMATION_CONSEIL_ON_PDD : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSqlPropositions.append( ( requete.isFiltreRapporteursActif(  ) ? getFiltreRapporteurPDD( requete )
                                                                         : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSqlPropositions.append( ( requete.isFiltreGroupesDepositairesActif(  )
            ? getFiltreGroupesDepositairesProposition( requete ) : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSqlPropositions.append( ( requete.isFiltreCategoriesDeliberationActif(  ) ? getFiltreCategoriesPDD( requete )
                                                                                    : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSqlPropositions.append( ( requete.isFiltreArrondissementsActif(  ) ? getFiltreArrondissementsPDD( requete )
                                                                             : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        /*
                if ( requete.isRechercheArchive(  ) )
                {
                    sbSqlPropositions.append( getFiltreSeancesPDD( requete, plugin ) );
                }
                else
                {
                    sbSqlPropositions.append( SQL_FILTER_PROCHAINE_SEANCE_ON_PDD );
                }*/
        if ( ( listIdsPdd != null ) && ( listIdsPdd.size(  ) != 0 ) )
        {
            sbSqlPropositions.append( getFiltreIdsPdds( listIdsPdd ) );
        }

        sbSqlPropositions.append( ( requete.isFiltreStatutsPropositions(  ) ? getFiltreStatutsPropositions( requete )
                                                                            : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        return sbSqlPropositions.toString(  );
    }

    /**
     * Prpare la requte SQL de slection pour une recherche de PDD par rfrence
     * @param requete la requte de l'utilisateur
     * @param plugin le plugin
     * @return la requte de slection des PDDs
     */
    protected String preparerSqlPddReference( GRequeteUtilisateur requete, Plugin plugin )
    {
        StringBuffer sbSqlPdds = new StringBuffer(  );

        sbSqlPdds.append( SQL_QUERY_PDD_LIST + SQL_JOIN_STATUT_ON_PDD );

        sbSqlPdds.append( SQL_INIT_WHERE_PDD_EN_LIGNE + SQL_FILTER_TYPE_PDD );

        if ( requete.isRechercheArchive(  ) )
        {
            sbSqlPdds.append( getFiltreSeancesPDD( requete, plugin ) );
        }
        else
        {
            sbSqlPdds.append( getSqlFilterProchaineSeanceOnPdd(  ) );
        }

        sbSqlPdds.append( SQL_FILTER_REFERENCE_PDD );
        sbSqlPdds.append( SQL_QUERY_ORDER_BY_REFERENCE );

        return sbSqlPdds.toString(  );
    }

    /**
     * Retourne la requte SQL de slection de toutes les dlibrations de dsignation  indexer
     * @return la requte de slection des dlibrations
     */
    private String preparerSqlDeliberationDesignationAIndexer(  )
    {
        StringBuffer sbSqlPdds = new StringBuffer(  );
        sbSqlPdds.append( SQL_QUERY_DELIBERATIONS_DESIGNATION );

        return sbSqlPdds.toString(  );
    }

    /**
     * Prpare la requte de slection des PDDs  ajouter  l'index des archives
     * @return une requte de slection de PDDs
     */
    private String preparerSqlPDDChangementSeance(  )
    {
        StringBuffer sbSqlPdds = new StringBuffer(  );

        sbSqlPdds.append( SQL_QUERY_PDD_LIST + SQL_JOIN_STATUT_ON_PDD );
        sbSqlPdds.append( SQL_INIT_WHERE );
        sbSqlPdds.append( getSqlFilterSeanceOnPdd(  ) );

        return sbSqlPdds.toString(  );
    }

    /**
     * Prpare la requte SQL de slection des amendements selon les critres de recherche
     * @param requete la requte de l'utilisateur
     * @param listIdsAmendements la liste des ids de VA retrouvs dans l'index et contenant le mot-cl recherch
     * @param plugin le plugin
     * @return la liste des amendements ou lettres rectificatives recherchs
     */
    private String preparerSqlAmendements( GRequeteUtilisateur requete, List<String> listIdsAmendements, Plugin plugin )
    {
        StringBuffer sbSqlAmendements = new StringBuffer(  );

        if ( requete.isRechercheArchive(  ) )
        {
            sbSqlAmendements.append( SQL_QUERY_VA_LIST_ARCHIVES );
        }
        else
        {
            sbSqlAmendements.append( SQL_QUERY_VA_LIST );
        }

        sbSqlAmendements.append( SQL_JOIN_SEANCE_ON_VA );
        sbSqlAmendements.append( SQL_JOIN_STATUT_ON_VA );

        if ( requete.isRechercheArchive(  ) )
        {
            sbSqlAmendements.append( SQL_JOIN_FICHIER_ON_VA_ARCHIVES );
        }
        else
        {
            sbSqlAmendements.append( SQL_JOIN_FICHIER_ON_VA );
        }

        sbSqlAmendements.append( ( ( requete.isFiltreGroupesDepositairesActif(  ) ||
            requete.isFiltreElusDepositairesActif(  ) ) ? SQL_JOIN_VA_DEPOSE_ON_VA : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        sbSqlAmendements.append( SQL_INIT_WHERE_VA_EN_LIGNE + SQL_FILTER_TYPE_AMENDEMENT );

        sbSqlAmendements.append( ( requete.isFiltreFormationsConseilActif( plugin )
            ? SQL_FILTER_FORMATION_CONSEIL_ON_VA : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSqlAmendements.append( ( ( ( requete.isFiltreGroupesDepositairesActif(  ) ||
            requete.isFiltreElusDepositairesActif(  ) ) || requete.isDeposeExecutif(  ) )
            ? getFiltreDepositaires( requete ) : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        if ( requete.isRechercheArchive(  ) )
        {
            sbSqlAmendements.append( getFiltreListeSeancesVA( requete, plugin ) );
        }
        else
        {
            sbSqlAmendements.append( SQL_FILTER_PROCHAINE_SEANCE_ON_VA );
        }

        if ( ( listIdsAmendements != null ) && ( listIdsAmendements.size(  ) != 0 ) )
        {
            sbSqlAmendements.append( getFiltreIdsVA( listIdsAmendements ) );
        }

        sbSqlAmendements.append( ( requete.isFiltreStatutsVoeuxAmendements(  )
            ? getFiltreStatutsVoeuxAmendements( requete ) : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        return sbSqlAmendements.toString(  );
    }

    /**
     * Prpare la requte Sql de slection des voeux selon les critres de recherche
     * @param requete la requte de l'utilisateur
     * @param listIdsVoeux la list des ids de voeux indexs contenant le mot-cl recherch
     * @param plugin le plugin
     * @return la requte de slection des voeux
     */
    private String preparerSqlVoeux( GRequeteUtilisateur requete, List<String> listIdsVoeux, Plugin plugin )
    {
        StringBuffer sbSqlVoeux = new StringBuffer(  );

        if ( requete.isRechercheArchive(  ) )
        {
            sbSqlVoeux.append( SQL_QUERY_VA_LIST_ARCHIVES );
        }
        else
        {
            sbSqlVoeux.append( SQL_QUERY_VA_LIST );
        }

        sbSqlVoeux.append( SQL_JOIN_SEANCE_ON_VA );
        sbSqlVoeux.append( SQL_JOIN_STATUT_ON_VA );

        if ( requete.isRechercheArchive(  ) )
        {
            sbSqlVoeux.append( SQL_JOIN_FICHIER_ON_VA_ARCHIVES );
        }
        else
        {
            sbSqlVoeux.append( SQL_JOIN_FICHIER_ON_VA );
        }

        sbSqlVoeux.append( ( ( requete.isFiltreGroupesDepositairesActif(  ) ||
            requete.isFiltreElusDepositairesActif(  ) || requete.isDeposeExecutif(  ) ) ? SQL_JOIN_VA_DEPOSE_ON_VA
                                                                                        : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        sbSqlVoeux.append( SQL_INIT_WHERE_VA_EN_LIGNE + SQL_FILTER_TYPE_VOEU );

        sbSqlVoeux.append( ( requete.isFiltreFormationsConseilActif( plugin ) ? SQL_FILTER_FORMATION_CONSEIL_ON_VA
                                                                              : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSqlVoeux.append( ( requete.isFiltreRapporteursActif(  ) ? getFiltreRapporteursVoeu( requete )
                                                                  : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSqlVoeux.append( ( ( ( requete.isFiltreGroupesDepositairesActif(  ) ||
            requete.isFiltreElusDepositairesActif(  ) ) || requete.isDeposeExecutif(  ) )
            ? getFiltreDepositaires( requete ) : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        if ( requete.isRechercheArchive(  ) )
        {
            sbSqlVoeux.append( getFiltreListeSeancesVA( requete, plugin ) );
        }
        else
        {
            sbSqlVoeux.append( SQL_FILTER_PROCHAINE_SEANCE_ON_VA );
        }

        if ( ( listIdsVoeux != null ) && ( listIdsVoeux.size(  ) != 0 ) )
        {
            sbSqlVoeux.append( getFiltreIdsVA( listIdsVoeux ) );
        }

        sbSqlVoeux.append( ( requete.isFiltreStatutsVoeuxAmendements(  ) ? getFiltreStatutsVoeuxAmendements( requete )
                                                                         : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        return sbSqlVoeux.toString(  );
    }

    /**
     * Prpare la requte SQL de slection des fichiers du PDD
     * @param listIdsFichiers la liste des ids de fichier du pdd retourns par la recherche dans l'index
     * @return la requte de slection des fichiers
     */
    private String preparerSqlFichiersDuPdd( List<String> listIdsFichiers )
    {
        String strSql = SQL_QUERY_FICHIERS + SQL_FILTER_ID_PDD_ON_FICHIER;

        if ( listIdsFichiers != null )
        {
            strSql += getFiltreIds( listIdsFichiers, SQL_FILTER_ID_FICHIER );
        }

        strSql += SQL_FILTER_TYPE_FICHIER_DELIBERATION_ON_FICHIER;
        strSql += SQL_QUERY_ORDER_FICHIERS_DU_PDD;

        return strSql;
    }

    /**
     * Prpare la requte SQL de slection des voeux et amendements  ajouter dans l'index des archives
     * @return la requte de slection des VA
     */
    private String preparerSqlVAChangementSeance(  )
    {
        StringBuffer sbSqlVA = new StringBuffer(  );

        sbSqlVA.append( SQL_QUERY_VA_LIST );

        sbSqlVA.append( SQL_JOIN_SEANCE_ON_VA );
        sbSqlVA.append( SQL_JOIN_STATUT_ON_VA );
        sbSqlVA.append( SQL_JOIN_FICHIER_ON_VA );

        sbSqlVA.append( SQL_INIT_WHERE_VA_EN_LIGNE );

        sbSqlVA.append( SQL_FILTER_SEANCE_ON_VA );

        return sbSqlVA.toString(  );
    }

    /**
     * Prpare la requte SQL permettant de rcuprer tous les documents archivs
     * @param plugin le plugin
     * @return la requte de slection des documents  archiver
     */
    protected String preparerSqlFichiersAIndexer( Plugin plugin )
    {
        StringBuffer sbSqlFichiersAIndexer = new StringBuffer(  );

        sbSqlFichiersAIndexer.append( SQL_QUERY_FICHIERS );

        List<Integer> listTypesDocuments = new ArrayList<Integer>(  );
        listTypesDocuments.add( ITypeDocument.CONVOCATION_DU_MAIRE );
        listTypesDocuments.add( ITypeDocument.LISTE_INTEGRALE_DES_QUESTIONS );
        listTypesDocuments.add( ITypeDocument.RELEVE_DE_LA_CONFERENCE_DORGANISATION );
        listTypesDocuments.add( ITypeDocument.RESUME_DES_QUESTIONS );
        listTypesDocuments.add( ITypeDocument.DESIGNATION );
        listTypesDocuments.add( ITypeDocument.DOCUMENT_LIBRE );
        listTypesDocuments.add( ITypeDocument.COMPTE_RENDU );
        listTypesDocuments.add( ITypeDocument.LISTE_DE_DESIGNATION );
        sbSqlFichiersAIndexer.append( getFiltreTypesDocumentsFichier( listTypesDocuments ) );

        GRequeteUtilisateur requete = getRequeteUtilisateur(  );
        requete.setRechercheArchive( true );
        sbSqlFichiersAIndexer.append( getFiltreListeSeance( requete, SQL_FILTER_LISTE_SEANCES_ON_FICHIER, plugin ) );

        return sbSqlFichiersAIndexer.toString(  );
    }

    /**
     * Prpare la requte SQL de slection des fichiers  rajouter dans l'index des archives
     * @return la requte de slection des nouveaux fichiers archivs
     */
    protected String preparerSqlFichiersChangementSeance(  )
    {
        StringBuffer sbSqlFichiersAArchiver = new StringBuffer(  );

        sbSqlFichiersAArchiver.append( SQL_QUERY_FICHIERS );

        List<Integer> listTypesDocuments = new ArrayList<Integer>(  );
        listTypesDocuments.add( ITypeDocument.CONVOCATION_DU_MAIRE );
        listTypesDocuments.add( ITypeDocument.LISTE_INTEGRALE_DES_QUESTIONS );
        listTypesDocuments.add( ITypeDocument.RELEVE_DE_LA_CONFERENCE_DORGANISATION );
        listTypesDocuments.add( ITypeDocument.RESUME_DES_QUESTIONS );
        listTypesDocuments.add( ITypeDocument.DESIGNATION );
        listTypesDocuments.add( ITypeDocument.DOCUMENT_LIBRE );
        sbSqlFichiersAArchiver.append( getFiltreTypesDocumentsFichier( listTypesDocuments ) );

        sbSqlFichiersAArchiver.append( SQL_FILTER_SEANCE_ON_FICHIER );

        return sbSqlFichiersAArchiver.toString(  );
    }

    /**
     * Prpare la requte de slection des autres documents de sance
     * @param requete la requte de l'utilisateur
     * @param listIdsFichiers la liste d'id de fichier qui contiennent le mot cl recherch
     * @param plugin le plugin
     * @return la requte de slection des autres documents
     */
    protected String preparerSqlAutresDocuments( GRequeteUtilisateur requete, List<String> listIdsFichiers,
        Plugin plugin )
    {
        StringBuffer sbSqlFichiers = new StringBuffer(  );

        sbSqlFichiers.append( SQL_QUERY_FICHIERS );

        List<Integer> listTypesDocuments = new ArrayList<Integer>(  );
        listTypesDocuments.add( ITypeDocument.CONVOCATION_DU_MAIRE );
        listTypesDocuments.add( ITypeDocument.LISTE_INTEGRALE_DES_QUESTIONS );
        listTypesDocuments.add( ITypeDocument.RELEVE_DE_LA_CONFERENCE_DORGANISATION );
        listTypesDocuments.add( ITypeDocument.RESUME_DES_QUESTIONS );
        listTypesDocuments.add( ITypeDocument.DESIGNATION );
        listTypesDocuments.add( ITypeDocument.DOCUMENT_LIBRE );
        listTypesDocuments.add( ITypeDocument.COMPTE_RENDU );
        listTypesDocuments.add( ITypeDocument.LISTE_DE_DESIGNATION );
        sbSqlFichiers.append( getFiltreTypesDocumentsFichier( listTypesDocuments ) );

        sbSqlFichiers.append( getFiltreListeSeance( requete, SQL_FILTER_LISTE_SEANCES_ON_FICHIER, plugin ) );

        if ( ( listIdsFichiers != null ) && !listIdsFichiers.isEmpty(  ) )
        {
            sbSqlFichiers.append( getFiltreIds( listIdsFichiers, SQL_FILTER_ID_FICHIER ) );
        }

        return sbSqlFichiers.toString(  );
    }

    /**
     * Prpare la requte de slection des arretes
     * @param requete la requte de l'utilisateur
     * @param listIdsArretes la liste d'id d'arretes qui contiennent le mot cl recherch
     * @param plugin le plugin
     * @return la requte de slection des arretes
     */
    private String preparerSqlArretes( GRequeteUtilisateur requete, List<String> listIdsArretes, Plugin plugin )
    {
        StringBuffer sbSqlArretes = new StringBuffer(  );

        sbSqlArretes.append( SQL_QUERY_ARRETES );

        sbSqlArretes.append( ( requete.isFiltreFormationsConseilActif( plugin )
            ? SQL_FILTER_FORMATION_CONSEIL_ON_ARRETE : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        sbSqlArretes.append( getFiltreListeSeance( requete, SQL_FILTER_LISTE_SEANCES_ON_ARRETE, plugin ) );

        if ( ( listIdsArretes != null ) && !listIdsArretes.isEmpty(  ) )
        {
            sbSqlArretes.append( getFiltreIds( listIdsArretes, SQL_FILTER_ID_ARRETE ) );
        }

        return sbSqlArretes.toString(  );
    }

    /**
     * Prpare la requte de slection des dlibrations de dsignation
     * @param requete la requte de l'utilisateur
     * @param listIdsDeliberationsDesignation la liste d'id de deliberations de dsignation qui contiennent le mot cl recherch
     * @param plugin le plugin
     * @return la requte de slection des dlibrations de dsignation
     */
    private String preparerSqlDeliberationsDesignation( GRequeteUtilisateur requete,
        List<String> listIdsDeliberationsDesignation, Plugin plugin )
    {
        StringBuffer sbSqlDeliberationsDesignation = new StringBuffer(  );

        sbSqlDeliberationsDesignation.append( SQL_QUERY_DELIBERATIONS_DESIGNATION );

        sbSqlDeliberationsDesignation.append( ( requete.isFiltreFormationsConseilActif( plugin )
            ? SQL_FILTER_FORMATION_CONSEIL_ON_DELIBERATION_DESIGNATION : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        sbSqlDeliberationsDesignation.append( getFiltreListeSeance( requete,
                SQL_FILTER_LISTE_SEANCES_ON_DELIBERATION_DESIGNATION, plugin ) );

        if ( ( listIdsDeliberationsDesignation != null ) && !listIdsDeliberationsDesignation.isEmpty(  ) )
        {
            sbSqlDeliberationsDesignation.append( getFiltreIds( listIdsDeliberationsDesignation,
                    SQL_FILTER_ID_DELIBERATION_DESIGNATION ) );
        }

        return sbSqlDeliberationsDesignation.toString(  );
    }

    /* ******************************************************* */
    /*                  EXPLOITATION DES DAOS                  */
    /* ******************************************************* */

    /**
     * Exploite le rsultat de la requte SQL
     * @param daoPDDs le dao contenant le rsultat de la recherche de {@link PDD}
     * @return la liste des {@link PDD}
     */
    protected List<GPDD> exploiterDaoPDD( DAOUtil daoPDDs )
    {
        List<GPDD> listPDDs = new ArrayList<GPDD>(  );

        while ( daoPDDs.next(  ) )
        {
            GPDD pdd = newPDDInstance(  );

            pdd.setId( daoPDDs.getInt( "pdd.id_pdd" ) );
            pdd.setReference( daoPDDs.getString( "pdd.reference" ) );
            pdd.setObjet( daoPDDs.getString( "pdd.objet" ) );
            pdd.setDatePublication( daoPDDs.getTimestamp( "pdd.date_publication" ) );
            pdd.setVersion( daoPDDs.getInt( "pdd.version" ) );
            pdd.setTypePdd( daoPDDs.getString( "pdd.type_pdd" ) );

            if ( daoPDDs.getObject( "pdd.id_statut" ) != null )
            {
                Statut statut = new Statut(  );
                statut.setIdStatut( daoPDDs.getInt( "st.id_statut" ) );
                statut.setLibelle( daoPDDs.getString( "st.libelle_statut" ) );
                pdd.setStatut( statut );
            }

            if ( daoPDDs.getObject( "pdd.id_formation_conseil" ) != null )
            {
                FormationConseil formation = new FormationConseil(  );
                formation.setIdFormationConseil( daoPDDs.getInt( "pdd.id_formation_conseil" ) );
                pdd.setFormationConseil( formation );
            }

            pdd.setFichiers( new ArrayList<GFichier>(  ) );

            listPDDs.add( pdd );
        }

        daoPDDs.free(  );

        return listPDDs;
    }

    /**
     * Exploite le rsultat de la requte de slection des VoeuAmendement selon les critres de recherche
     * @param daoVA le dao contenant le rsultat de la requte
     * @param isArchives true si la recherche est pour les archives, false sinon
     * @return un liste de {@link VoeuAmendement}
     */
    private List<GVoeuAmendement> exploiterDaoVA( DAOUtil daoVA, boolean isArchives )
    {
        List<GVoeuAmendement> listVA = new ArrayList<GVoeuAmendement>(  );
        GVoeuAmendement va;

        while ( daoVA.next(  ) )
        {
            va = newVoeuAmendementInstance(  );

            va.setId( daoVA.getInt( "va.id_va" ) );
            va.setReferenceFront( daoVA.getString( "va.reference_va_front" ) );
            va.setObjet( daoVA.getString( "va.objet" ) );
            va.setDatePublication( daoVA.getTimestamp( "va.date_publication" ) );
            va.setVersion( daoVA.getInt( "va.version" ) );
            va.setType( daoVA.getString( "va.type" ) );

            if ( daoVA.getObject( "va.id_statut" ) != null )
            {
                Statut statut = new Statut(  );
                statut.setIdStatut( daoVA.getInt( "st.id_statut" ) );
                statut.setLibelle( daoVA.getString( "st.libelle_statut" ) );
                va.setStatut( statut );
            }

            GFichier fichier = newFichierInstance(  );
            fichier.setId( daoVA.getInt( "fic.id_document" ) );
            fichier.setTaille( daoVA.getInt( "fic.taille" ) );
            fichier.setExtension( daoVA.getString( "fic.extension" ) );
            fichier.setIntitule( daoVA.getString( "fic.intitule" ) );
            fichier.setDatePublication( daoVA.getTimestamp( "fic.date_publication" ) );
            fichier.setVersion( daoVA.getInt( "fic.version" ) );
            va.setFichier( fichier );

            if ( isArchives )
            {
                GFichier deliberation = newFichierInstance(  );
                deliberation.setId( daoVA.getInt( "fic2.id_document" ) );
                deliberation.setTaille( daoVA.getInt( "fic2.taille" ) );
                deliberation.setExtension( daoVA.getString( "fic2.extension" ) );
                deliberation.setIntitule( daoVA.getString( "fic2.intitule" ) );
                deliberation.setDatePublication( daoVA.getTimestamp( "fic2.date_publication" ) );
                deliberation.setVersion( daoVA.getInt( "fic2.version" ) );
                va.setDeliberation( deliberation );
            }

            exploiterDaoVASpec( daoVA, va );

            listVA.add( va );
        }

        daoVA.free(  );

        return listVA;
    }

    /**
     * Exploite les rsultats d'une requte de slection d'arretes
     * @param daoArretes le rsultat de la requte
     * @return une liste d'arretes
     */
    private List<Arrete<GSeance, GFichier>> exploiterDaoArrete( DAOUtil daoArretes )
    {
        List<Arrete<GSeance, GFichier>> listArretes = new ArrayList<Arrete<GSeance, GFichier>>(  );
        Arrete<GSeance, GFichier> arrete;

        while ( daoArretes.next(  ) )
        {
            arrete = new Arrete<GSeance, GFichier>(  );

            arrete.setId( daoArretes.getInt( "ar.id_arrete" ) );
            arrete.setNumero( daoArretes.getString( "ar.numero" ) );
            arrete.setIntitule( daoArretes.getString( "ar.intitule" ) );
            arrete.setDatePublication( daoArretes.getTimestamp( "ar.date_publication" ) );
            arrete.setEnLigne( daoArretes.getBoolean( "ar.en_ligne" ) );
            arrete.setVersion( daoArretes.getInt( "ar.version" ) );

            FormationConseil conseil = new FormationConseil(  );
            conseil.setIdFormationConseil( daoArretes.getInt( "ar.id_formation_conseil" ) );
            arrete.setFormationConseil( conseil );

            GFichier fichierOr = newFichierInstance(  );
            fichierOr.setId( daoArretes.getInt( "ar.id_fichier_or" ) );
            arrete.setFichierOr( fichierOr );

            GFichier fichierSc = newFichierInstance(  );
            fichierSc.setId( daoArretes.getInt( "ar.id_fichier_sc" ) );
            arrete.setFichierSc( fichierSc );

            CategorieDeliberation categorie = new CategorieDeliberation(  );
            categorie.setIdCategorie( daoArretes.getInt( "ar.id_categorie" ) );
            arrete.setCategorie( categorie );

            GSeance seance = newSeanceInstance(  );
            seance.setIdSeance( daoArretes.getInt( "ar.id_seance" ) );
            arrete.setSeance( seance );

            listArretes.add( arrete );
        }

        daoArretes.free(  );

        return listArretes;
    }

    /**
     * Exploite les rsultats d'une requte de slection de dlibrations de dsignation
     * @param daoDeliberationsDesignation le rsultat de la requte
     * @return une liste de dlibrations de dsignation
     */
    private List<DeliberationDesignation<GSeance, GFichier>> exploiterDaoDeliberationDesignation( 
        DAOUtil daoDeliberationsDesignation )
    {
        List<DeliberationDesignation<GSeance, GFichier>> listDeliberationsDesignation = new ArrayList<DeliberationDesignation<GSeance, GFichier>>(  );
        DeliberationDesignation<GSeance, GFichier> deliberation;

        while ( daoDeliberationsDesignation.next(  ) )
        {
            deliberation = new DeliberationDesignation<GSeance, GFichier>(  );

            deliberation.setId( daoDeliberationsDesignation.getInt( "de.id_designation" ) );
            deliberation.setNumero( daoDeliberationsDesignation.getString( "de.numero" ) );
            deliberation.setIntitule( daoDeliberationsDesignation.getString( "de.intitule" ) );
            deliberation.setDatePublication( daoDeliberationsDesignation.getTimestamp( "de.date_publication" ) );
            deliberation.setEnLigne( daoDeliberationsDesignation.getBoolean( "de.en_ligne" ) );
            deliberation.setVersion( daoDeliberationsDesignation.getInt( "de.version" ) );

            FormationConseil conseil = new FormationConseil(  );
            conseil.setIdFormationConseil( daoDeliberationsDesignation.getInt( "de.id_formation_conseil" ) );
            deliberation.setFormationConseil( conseil );

            if ( daoDeliberationsDesignation.getObject( "de.id_fichier_or" ) != null )
            {
                GFichier fichierOr = newFichierInstance(  );
                fichierOr.setId( daoDeliberationsDesignation.getInt( "de.id_fichier_or" ) );
                deliberation.setFichierOr( fichierOr );
            }

            GFichier fichierSc = newFichierInstance(  );
            fichierSc.setId( daoDeliberationsDesignation.getInt( "de.id_fichier_sc" ) );
            deliberation.setFichierSc( fichierSc );

            CategorieDeliberation categorie = new CategorieDeliberation(  );
            categorie.setIdCategorie( daoDeliberationsDesignation.getInt( "de.id_categorie" ) );
            deliberation.setCategorie( categorie );

            GSeance seance = newSeanceInstance(  );
            seance.setIdSeance( daoDeliberationsDesignation.getInt( "de.id_seance" ) );
            deliberation.setSeance( seance );

            int nIdActe = daoDeliberationsDesignation.getInt( "ac.id_acte" );

            if ( nIdActe != -1 )
            {
                Acte<GSeance, GFichier> acteCourant = new Acte<GSeance, GFichier>(  );
                acteCourant.setId( nIdActe );
                deliberation.setActeCourant( acteCourant );
            }

            listDeliberationsDesignation.add( deliberation );
        }

        daoDeliberationsDesignation.free(  );

        return listDeliberationsDesignation;
    }

    /* ************************************************* */
    /*                  FILTRES SUR PDD                  */
    /* ************************************************* */

    /**
     * Retourne le filtre SQL sur les rapporteurs
     * @param requete la requte utilisateur
     * @return le filtre sur les rapporteurs sur la table ods_pdd
     */
    protected String getFiltreRapporteurPDD( GRequeteUtilisateur requete )
    {
        StringBuffer sbFiltreRapporteurs = new StringBuffer(  );

        sbFiltreRapporteurs.append( getSqlFilterRapporteurOnPdd(  ) );

        sbFiltreRapporteurs.append( requete.getListeRapporteurs(  ) );
        sbFiltreRapporteurs.append( " ) ) " );

        return sbFiltreRapporteurs.toString(  );
    }

    /**
     * Retourne le filtre SQL sur les directions et directions co-emettrices des projets de dlibration
     * @param requete la requte utilisateur
     * @return le filtre sur les directions de la table ods_pdd
     */
    protected String getFiltreDirectionsProjet( GRequeteUtilisateur requete )
    {
        StringBuffer sbFiltreDirections = new StringBuffer(  );

        sbFiltreDirections.append( SQL_FILTER_DIRECTION_CO_EM_ON_PDD );
        sbFiltreDirections.append( requete.getListeDirections(  ) );
        sbFiltreDirections.append( SQL_FILTER_DIRECTION_ON_PDD );
        sbFiltreDirections.append( requete.getListeDirections(  ) );
        sbFiltreDirections.append( " ) ) " );

        return sbFiltreDirections.toString(  );
    }

    /**
     * Retourne le filtre SQL sur les catgories de dlibration des PDDs
     * @param requete la requte utilisateur
     * @return le filtre sur les catgories de dlibration d'un PDD
     */
    protected String getFiltreCategoriesPDD( GRequeteUtilisateur requete )
    {
        StringBuffer sbFiltreCategories = new StringBuffer(  );

        sbFiltreCategories.append( SQL_FILTER_CATEGORIE_ON_PDD );
        sbFiltreCategories.append( requete.getListeCategoriesDeliberation(  ) );
        sbFiltreCategories.append( " ) " );

        return sbFiltreCategories.toString(  );
    }

    /**
     * Retourne le filtre SQL sur les arrondissements du PDD
     * @param requete la requte utilisateur
     * @return le filtre sur les arrondissements sur la table ods_pdd
     */
    protected String getFiltreArrondissementsPDD( GRequeteUtilisateur requete )
    {
        StringBuffer sbFiltreArrondissements = new StringBuffer(  );

        sbFiltreArrondissements.append( SQL_FILTER_ARRONDISSEMENT_ON_PDD );

        sbFiltreArrondissements.append( requete.getListeArrondissements(  ) );
        sbFiltreArrondissements.append( " ) " );

        return sbFiltreArrondissements.toString(  );
    }

    /**
     * Retourne le filtre Sql sur les statuts des projets
     * @param requete la requte utilisateur
     * @return le filtre Sql sur les statuts des projets
     */
    protected String getFiltreStatutsProjets( GRequeteUtilisateur requete )
    {
        StringBuffer sbfiltreStatutsProjets = new StringBuffer(  );

        sbfiltreStatutsProjets.append( SQL_FILTER_SATUT_PDD );

        sbfiltreStatutsProjets.append( requete.getListeStatutsProjets(  ) );
        sbfiltreStatutsProjets.append( " )  " );

        return sbfiltreStatutsProjets.toString(  );
    }

    /**
     * Retourne le filtre Sql sur les statuts des propositions
     * @param requete la requte utilisateur
     * @return le filtre Sql sur les statuts des propositions
     */
    protected String getFiltreStatutsPropositions( GRequeteUtilisateur requete )
    {
        StringBuffer sbfiltreStatutsProjets = new StringBuffer(  );

        sbfiltreStatutsProjets.append( SQL_FILTER_SATUT_PDD );

        sbfiltreStatutsProjets.append( requete.getListeStatutsPropositions(  ) );
        sbfiltreStatutsProjets.append( " )  " );

        return sbfiltreStatutsProjets.toString(  );
    }

    /**
     * Retourne le filtre Sql sur les statuts des voeux amendements
     * @param requete la requte utilisateur
     * @return le filtre Sql sur les statuts des voeux amendements
     */
    protected String getFiltreStatutsVoeuxAmendements( GRequeteUtilisateur requete )
    {
        StringBuffer sbfiltreStatutsProjets = new StringBuffer(  );

        sbfiltreStatutsProjets.append( SQL_FILTER_SATUT_VOEU_AMENDEMENT );

        sbfiltreStatutsProjets.append( requete.getListeStatutsVoeuxAmendements(  ) );
        sbfiltreStatutsProjets.append( " )  " );

        return sbfiltreStatutsProjets.toString(  );
    }

    /**
     * Retourne le filtre SQL par rapport aux sances auxquelles les pdds sont rattachs
     * @param requete la requte utilisateur
     * @param plugin le plugin
     * @return un filtre sance sur la table ods_pdd
     */
    protected String getFiltreSeancesPDD( GRequeteUtilisateur requete, Plugin plugin )
    {
        StringBuffer sbFiltreSeances = new StringBuffer(  );

        sbFiltreSeances.append( getFilterListSeancePDD(  ) );

        boolean bFirst = true;
        List<GSeance> listSeances = requete.getListeSeances( plugin );

        if ( ( listSeances != null ) && ( listSeances.size(  ) != 0 ) )
        {
            for ( GSeance seance : listSeances )
            {
                ISeance newSeance = seance;
                sbFiltreSeances.append( ( bFirst ? OdsConstants.CONSTANTE_CHAINE_VIDE : ", " ) );
                sbFiltreSeances.append( newSeance.getIdSeance(  ) );
                bFirst = false;
            }
        }
        else
        {
            //on ajoute l'id d'une sance qui n'existe pas
            sbFiltreSeances.append( "-1" );
        }

        sbFiltreSeances.append( " ) ) " );

        return sbFiltreSeances.toString(  );
    }

    /**
     * Retourne un filtre SQL sur le groupe dpositaire d'une proposition de dlibration
     * @param requete la requte utilisateur
     * @return un filtre sur les groupes dpositaires de la table ods_pdd
     */
    protected String getFiltreGroupesDepositairesProposition( GRequeteUtilisateur requete )
    {
        StringBuffer sbFiltreGroupe = new StringBuffer(  );

        sbFiltreGroupe.append( SQL_FILTER_GROUPE_ON_PDD );

        sbFiltreGroupe.append( requete.getListeGroupesDepositaires(  ) );
        sbFiltreGroupe.append( " ) " );

        return sbFiltreGroupe.toString(  );
    }

    /**
     * Retourne le filtre SQL limitant  une liste d'ids la recherche de PDD
     * @param listIdsPdds la liste des ids PDD
     * @return le filtre sur les id de la table ods_pdd
     */
    protected String getFiltreIdsPdds( List<String> listIdsPdds )
    {
        StringBuffer sbFiltreIdsPdd = new StringBuffer(  );

        sbFiltreIdsPdd.append( SQL_FILTER_ID_PDD );

        boolean bFirst = true;

        if ( ( listIdsPdds != null ) && ( listIdsPdds.size(  ) != 0 ) )
        {
            for ( String strIdPdd : listIdsPdds )
            {
                sbFiltreIdsPdd.append( ( bFirst ? OdsConstants.CONSTANTE_CHAINE_VIDE : ", " ) );
                sbFiltreIdsPdd.append( strIdPdd );
                bFirst = false;
            }
        }
        else
        {
            // on ajoute l'id d'un pdd qui n'exites pas
            sbFiltreIdsPdd.append( "-1" );
        }

        sbFiltreIdsPdd.append( " ) " );

        return sbFiltreIdsPdd.toString(  );
    }

    /* ************************************************* */
    /*                  FILTRES SUR VA                   */
    /* ************************************************* */

    /**
     * Retourne le filtre SQL sur les rapporteurs lis  un voeu
     * @param requete la requte de l'utilisateur
     * @return un filtre sur les rapporteurs d'un voeu
     */
    protected String getFiltreRapporteursVoeu( GRequeteUtilisateur requete )
    {
        StringBuffer sbFiltreRapporteurs = new StringBuffer(  );

        sbFiltreRapporteurs.append( SQL_FILTER_RAPPORTEUR_ON_VOEU );

        sbFiltreRapporteurs.append( requete.getListeRapporteurs(  ) );
        sbFiltreRapporteurs.append( " ) ) " );

        return sbFiltreRapporteurs.toString(  );
    }

    /**
     * Retourne le filtre SQL sur les dpositaires d'un VA (groupes et lus et dpos par l'excutif)
     * @param requete la requte utilisateur
     * @return un filtre sur les dpositaires d'un {@link VoeuAmendement}
     */
    protected String getFiltreDepositaires( GRequeteUtilisateur requete )
    {
        boolean bFiltreGroupe = requete.isFiltreGroupesDepositairesActif(  );
        boolean bFiltreElu = requete.isFiltreElusDepositairesActif(  );
        boolean bFiltreDeposeExecutif = requete.isDeposeExecutif(  );
        StringBuffer sbFiltreDepositaires = new StringBuffer(  );

        if ( bFiltreDeposeExecutif && !bFiltreGroupe && !bFiltreElu )
        {
            sbFiltreDepositaires.append( " AND " );
            sbFiltreDepositaires.append( SQL_FILTER_VA_DEPOSE_EXECUTIF );
        }
        else
        {
            if ( bFiltreDeposeExecutif )
            {
                sbFiltreDepositaires.append( " AND " );
                sbFiltreDepositaires.append( " ( " );
                sbFiltreDepositaires.append( SQL_FILTER_VA_DEPOSE_EXECUTIF );
                sbFiltreDepositaires.append( " OR " );
            }
            else
            {
                sbFiltreDepositaires.append( " AND" );
            }

            sbFiltreDepositaires.append( SQL_FILTER_DEPOSITAIRE_ON_VA );

            if ( bFiltreGroupe )
            {
                sbFiltreDepositaires.append( " ( " );
                sbFiltreDepositaires.append( SQL_FILTER_GROUPE_ON_VA );
                sbFiltreDepositaires.append( requete.getListeGroupesDepositaires(  ) );
                sbFiltreDepositaires.append( " ) ) " );
                sbFiltreDepositaires.append( " OR " );
                sbFiltreDepositaires.append( " ( " );
                sbFiltreDepositaires.append( SQL_FILTER_GROUPE_ELU_ON_VA );
                sbFiltreDepositaires.append( requete.getListeGroupesDepositaires(  ) );
                sbFiltreDepositaires.append( " ) )" );
                sbFiltreDepositaires.append( " ) " );

                if ( bFiltreElu )
                {
                    sbFiltreDepositaires.append( " OR " );
                }
            }

            if ( bFiltreElu )
            {
                sbFiltreDepositaires.append( SQL_FILTER_ELU_ON_VA );
                sbFiltreDepositaires.append( requete.getListeElusDepositaires(  ) );
                sbFiltreDepositaires.append( " ) " );
            }

            sbFiltreDepositaires.append( " ) " );

            if ( bFiltreDeposeExecutif )
            {
                sbFiltreDepositaires.append( " ) " );
            }
        }

        return sbFiltreDepositaires.toString(  );
    }

    /**
     * Retourne le filtre SQL sur les sances d'un {@link VoeuAmendement}
     * @param requete la requte utilisateur
     * @param plugin le plugin
     * @return un filtre sance sur la table ods_voeu_amendement
     */
    protected String getFiltreListeSeancesVA( GRequeteUtilisateur requete, Plugin plugin )
    {
        StringBuffer sbFiltreSeances = new StringBuffer(  );

        sbFiltreSeances.append( SQL_FILTER_LISTE_SEANCES_ON_VA );

        boolean bFirst = true;
        List<GSeance> listSeances = requete.getListeSeances( plugin );

        if ( ( listSeances != null ) && ( listSeances.size(  ) != 0 ) )
        {
            for ( GSeance seance : listSeances )
            {
                ISeance newSeance = seance;
                sbFiltreSeances.append( ( bFirst ? OdsConstants.CONSTANTE_CHAINE_VIDE : ", " ) );
                sbFiltreSeances.append( newSeance.getIdSeance(  ) );
                bFirst = false;
            }
        }
        else
        {
            //on ajoute l'id d'une seance qui n'existe pas
            sbFiltreSeances.append( "-1" );
        }

        sbFiltreSeances.append( " ) " );

        return sbFiltreSeances.toString(  );
    }

    /**
     * Retourne le filtre SQL limitant  une liste d'ids les {@link VoeuAmendement} recherchs
     * @param listIdsAmendements la liste des ids de {@link VoeuAmendement}  filtrer
     * @return un filtre id_va sur la table ods_voeu_amendement
     */
    protected String getFiltreIdsVA( List<String> listIdsAmendements )
    {
        StringBuffer sbFiltreIdsAmendements = new StringBuffer(  );

        sbFiltreIdsAmendements.append( SQL_FILTER_ID_VA );

        boolean bFirst = true;

        if ( ( listIdsAmendements != null ) && ( listIdsAmendements.size(  ) != 0 ) )
        {
            for ( String strIdAmendement : listIdsAmendements )
            {
                sbFiltreIdsAmendements.append( ( bFirst ? OdsConstants.CONSTANTE_CHAINE_VIDE : ", " ) );
                sbFiltreIdsAmendements.append( strIdAmendement );
                bFirst = false;
            }
        }
        else
        {
            //on ajoute l'id d'un va qui n'existe pas	
            sbFiltreIdsAmendements.append( "-1" );
        }

        sbFiltreIdsAmendements.append( " ) " );

        return sbFiltreIdsAmendements.toString(  );
    }

    /* ************************************************* */
    /*                  FILTRES SUR FICHIERS                   */
    /* ************************************************* */
    /**
     * Retourne le filtre SQL sur les types de documents d'un fichier
     * @param listTypesDoc la liste des types de documents
     * @return un filtre sur les types de document de la table ods_fichier
     */
    protected String getFiltreTypesDocumentsFichier( List<Integer> listTypesDoc )
    {
        StringBuffer sbFiltreTypesDoc = new StringBuffer(  );

        sbFiltreTypesDoc.append( SQL_FILTER_TYPE_FICHIER );

        boolean bFirst = true;

        if ( ( listTypesDoc != null ) && ( listTypesDoc.size(  ) != 0 ) )
        {
            for ( Integer typeDoc : listTypesDoc )
            {
                sbFiltreTypesDoc.append( ( bFirst ? OdsConstants.CONSTANTE_CHAINE_VIDE : ", " ) );
                sbFiltreTypesDoc.append( typeDoc );
                bFirst = false;
            }
        }
        else
        {
            //on ajoute l'id d'un type de document qui n'existe pas
            sbFiltreTypesDoc.append( "-1" );
        }

        sbFiltreTypesDoc.append( " ) " );

        return sbFiltreTypesDoc.toString(  );
    }

    /**
     * Retourne un filtre sur la sance
     * @param requete la requete de recherche
     * @param plugin le plugin ODS
     * @param strQuery la requete SQL correspondant au filtre de la seance
     * @return un filtre sur la sance
     */
    protected String getFiltreListeSeance( GRequeteUtilisateur requete, String strQuery, Plugin plugin )
    {
        StringBuffer sbFiltreSeances = new StringBuffer(  );

        sbFiltreSeances.append( strQuery );

        boolean bFirst = true;
        List<GSeance> listSeances = requete.getListeSeances( plugin );

        if ( ( listSeances != null ) && ( listSeances.size(  ) != 0 ) )
        {
            for ( GSeance seance : listSeances )
            {
                ISeance newSeance = seance;
                sbFiltreSeances.append( ( bFirst ? OdsConstants.CONSTANTE_CHAINE_VIDE : ", " ) );
                sbFiltreSeances.append( newSeance.getIdSeance(  ) );
                bFirst = false;
            }
        }
        else
        {
            //on ajoute l'id d'une seance qui n'existe pas
            sbFiltreSeances.append( "-1" );
        }

        sbFiltreSeances.append( " ) " );

        return sbFiltreSeances.toString(  );
    }

    /**
     * Retourne le filtre sur l'id des fichiers
     * @param listIdsFichiers la liste des identifiants de fichiers
     * @param strQuery la requete SQL
     * @return le filtre sur l'identifiant des fichiers
     */
    protected String getFiltreIds( List<String> listIdsFichiers, String strQuery )
    {
        StringBuffer sbFiltreIdsFichiers = new StringBuffer(  );

        sbFiltreIdsFichiers.append( strQuery );

        boolean bFirst = true;

        if ( ( listIdsFichiers != null ) && ( listIdsFichiers.size(  ) != 0 ) )
        {
            for ( String strIdFichier : listIdsFichiers )
            {
                sbFiltreIdsFichiers.append( ( bFirst ? OdsConstants.CONSTANTE_CHAINE_VIDE : ", " ) );
                sbFiltreIdsFichiers.append( strIdFichier );
                bFirst = false;
            }
        }
        else
        {
            //on ajoute l'id d'un fichier qui n'exites pas
            sbFiltreIdsFichiers.append( "-1" );
        }

        sbFiltreIdsFichiers.append( " ) " );

        return sbFiltreIdsFichiers.toString(  );
    }
}
