/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.statut;

import fr.paris.lutece.plugins.ods.dto.statut.IStatutFilter;
import fr.paris.lutece.plugins.ods.dto.statut.Statut;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;

import java.util.ArrayList;
import java.util.List;


/**
 * AbstractStatutDAO
 * @param <GStatutFilter> {@link IStatutFilter}
 */
public abstract class AbstractStatutDAO<GStatutFilter extends IStatutFilter> implements IStatutDAO<GStatutFilter>
{
    /*
     * Requetes SQL
     */
    private static final String SQL_QUERY_NEW_PK = "SELECT max( id_statut ) FROM ods_statut ";
    private static final String SQL_QUERY_INSERT = "INSERT INTO ods_statut( " +
        "			id_statut, libelle_statut, est_pour_pdd, est_pour_voeu, est_pour_amendement " + "VALUES (?,?,?,?,?) ";
    private static final String SQL_QUERY_SELECT = "SELECT " +
        "	id_statut, libelle_statut, est_pour_pdd, est_pour_voeu, est_pour_amendement " + " FROM ods_statut";
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = SQL_QUERY_SELECT + " WHERE id_statut = ? ";
    private static final String SQL_QUERY_STATUT_LIST = SQL_QUERY_SELECT;
    private static final String SQL_QUERY_DELETE = " DELETE FROM ods_statut WHERE id_statut = ?  ";
    private static final String SQL_QUERY_UPDATE = "UPDATE ods_statut " +
        " SET libelle_statut = ?, est_pour_pdd = ?, est_pour_voeu = ?, est_pour_amendement= ? ";
    private static final String SQL_QUERY_STATUT_LIST_BY_FILTER = SQL_QUERY_SELECT + " WHERE 1=1 ";
    private static final String SQL_QUERY_FILTER_BY_POUR_PDD = " AND est_pour_pdd = true ";
    private static final String SQL_QUERY_FILTER_BY_POUR_VOEU = " AND est_pour_voeu = true ";
    private static final String SQL_QUERY_FILTER_BY_POUR_VOEU_AMENDEMENT = " AND (  est_pour_voeu = true OR est_pour_amendement = true )";
    private static final String SQL_QUERY_FILTER_BY_POUR_AMENDEMENT = " AND est_pour_amendement = true ";
    private static final String SQL_QUERY_ORDER_BY_LIBELLE = " ORDER BY libelle_statut ASC ";

    /**
     * Ajoute les filtes pour la liste des statuts
     * @param filter le filtre
     * @param sbSQL le buffer
     */
    protected abstract void selectStatutListWithFilterSpec( GStatutFilter filter, StringBuffer sbSQL );

    /**
     * Gnre un nouvel identifiant
     * @param plugin plugin
     * @return Retourne lidentifiant gnr
     */
    int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Supprime le statut correspondant  lobjet statut pass en paramtre
     * @param statut le statut  supprimer
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    public void delete( Statut statut, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, statut.getIdStatut(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Insre le statut correspondant  lobjet statut pass en paramtre
     * @param statut le statut  insrer
     * @param plugin Plugin
     */
    public void insert( Statut statut, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT, plugin );
        int nKey = newPrimaryKey( plugin );

        daoUtil.setInt( 1, nKey );
        daoUtil.setString( 2, statut.getLibelle(  ) );
        daoUtil.setBoolean( 3, statut.estPourPDD(  ) );
        daoUtil.setBoolean( 4, statut.estPourVoeu(  ) );
        daoUtil.setBoolean( 5, statut.estPourAmendement(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Rcupre le statut correspondant  l'id nKey
     * @param nKey l'id du statut  rcuprer
     * @param plugin Plugin
     * @return le statut correspondant  l'id nKey
     */
    public Statut load( int nKey, Plugin plugin )
    {
        Statut statut = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            statut = setStatutValues( daoUtil );
        }

        daoUtil.free(  );

        return statut;
    }

    /**
     * Rcupre la liste des statuts correspondants au filtre
     * @param filter le filtre
     * @param plugin Plugin
     * @return la liste des statuts correspondants au filtre
     */
    public List<Statut> selectStatutListWithFilter( GStatutFilter filter, Plugin plugin )
    {
        List<Statut> listStatuts = new ArrayList<Statut>(  );
        Statut statut = null;

        StringBuffer sbSQL = new StringBuffer(  );

        sbSQL.append( SQL_QUERY_STATUT_LIST_BY_FILTER );
        sbSQL.append( ( ( filter.containsPourPDDCriteria(  ) ) ? SQL_QUERY_FILTER_BY_POUR_PDD
                                                               : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsPourVoeuCriteria(  ) ) ? SQL_QUERY_FILTER_BY_POUR_VOEU
                                                                : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsPourAmendementCriteria(  ) ) ? SQL_QUERY_FILTER_BY_POUR_AMENDEMENT
                                                                      : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsPourVoeuAmendementCriteria(  ) ) ? SQL_QUERY_FILTER_BY_POUR_VOEU_AMENDEMENT
                                                                          : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        // Appel au DAO spcifique
        selectStatutListWithFilterSpec( filter, sbSQL );

        sbSQL.append( SQL_QUERY_ORDER_BY_LIBELLE );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            statut = setStatutValues( daoUtil );

            listStatuts.add( statut );
        }

        daoUtil.free(  );

        return listStatuts;
    }

    /**
     * Rcupre la liste des statuts
     * @param plugin Plugin
     * @return la liste des statuts
     */
    public List<Statut> selectStatutList( Plugin plugin )
    {
        List<Statut> listStatuts = new ArrayList<Statut>(  );
        Statut statut;

        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_STATUT_LIST + SQL_QUERY_ORDER_BY_LIBELLE, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            statut = setStatutValues( daoUtil );

            listStatuts.add( statut );
        }

        daoUtil.free(  );

        return listStatuts;
    }

    /**
     * Met  jour le statut pass en paramtre dans la base de donnes
     * @param statut le statut  mettre  jour
     * @param plugin Plugin
     */
    public void store( Statut statut, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_UPDATE, plugin );

        daoUtil.setString( 1, statut.getLibelle(  ) );
        daoUtil.setBoolean( 2, statut.estPourPDD(  ) );
        daoUtil.setBoolean( 3, statut.estPourVoeu(  ) );
        daoUtil.setBoolean( 4, statut.estPourAmendement(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Cre et remplit un objet Statut avec les valeurs contenues dans le DAOUtil
     * @param daoUtil daoUtil
     * @return l'objet Statut
     */
    private Statut setStatutValues( DAOUtil daoUtil )
    {
        Statut statut = new Statut(  );

        statut.setIdStatut( daoUtil.getInt( "id_statut" ) );
        statut.setLibelle( daoUtil.getString( "libelle_statut" ) );
        statut.setPourPDD( daoUtil.getBoolean( "est_pour_pdd" ) );
        statut.setPourVoeu( daoUtil.getBoolean( "est_pour_voeu" ) );
        statut.setPourAmendement( daoUtil.getBoolean( "est_pour_amendement" ) );

        return statut;
    }
}
