/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.search;

import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.business.search.ISearchDAO;
import fr.paris.lutece.plugins.ods.business.search.ISearchHome;
import fr.paris.lutece.plugins.ods.dto.arrete.Arrete;
import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.DeliberationDesignation;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppPropertiesService;

import java.sql.Timestamp;

import java.util.List;


/**
 * DocSearchService: ralise la recherche multi-critres
 */
public abstract class AbstractDocSearchService<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, GIndexedDocSearchService extends IIndexedDocSearchService<GSeance, GSeanceFilter, GRequeteUtilisateur>, GElu extends IElu, GFichier extends IFichier<GSeance, GFichier>, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GSearchDAO extends ISearchDAO<GFichier, GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GVoeuAmendement, GPDD>>
    implements IDocSearchService<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPDD>
{
    protected static final String INDEX_PATH_CURRENT = AppPropertiesService.getProperty( 
            "ods.search.lucene.current.indexPath" );
    protected static final String INDEX_PATH_ARCHIVE = AppPropertiesService.getProperty( 
            "ods.search.lucene.archive.indexPath" );

    protected abstract GSeance newSeanceInstance(  );

    protected abstract ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD> getSeanceHome(  );

    protected abstract ISearchHome<GFichier, GSeance, GSeanceFilter, GRequeteUtilisateur, GSearchDAO, GElu, GVoeuAmendement, GPDD> getSearchHome(  );

    protected abstract GIndexedDocSearchService getIndexedDocSearchService( String strPath );

    /**
     * Retourne la liste des projets correspondant aux critres de recherche
     * @param requete la requte de l'utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsProjets la liste des projets retrouvs dans l'index contenant le(s) mot(s) cl recherchs
     * @param listSeances la liste des sances pour les fiches de sance
     * @param plugin le plugin
     * @return une liste de PDDs de type projet
     */
    public List<GPDD> getProjets( GRequeteUtilisateur requete, GSeance prochaineSeance, List<String> listIdsProjets,
        List<GSeance> listSeances, Plugin plugin )
    {
        List<GPDD> listProjets = getSearchHome(  )
                                     .findProjetsDeliberation( requete, prochaineSeance, listIdsProjets, plugin );

        GIndexedDocSearchService indexedDocSearchService;

        if ( requete.isRechercheArchive(  ) )
        {
            indexedDocSearchService = getIndexedDocSearchService( INDEX_PATH_ARCHIVE );
        }
        else
        {
            indexedDocSearchService = getIndexedDocSearchService( INDEX_PATH_CURRENT );
        }

        boolean bMotCle = !requete.getTypeRequete(  ).equals( OdsParameters.REFERENCE ) &&
            ( requete.getChampRecherche(  ) != null ) &&
            !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( requete.getChampRecherche(  ) );

        for ( GPDD projet : listProjets )
        {
            List<String> listIdsFichiersProjet = null;

            if ( bMotCle )
            {
                listIdsFichiersProjet = indexedDocSearchService.getListeIdsFichiersPdd( requete, projet.getId(  ) );
            }

            if ( ( listIdsFichiersProjet == null ) || !listIdsFichiersProjet.isEmpty(  ) )
            {
                projet.setFichiers( getSearchHome(  ).findFichiersDuPdd( listIdsFichiersProjet, projet.getId(  ), plugin ) );
            }

            if ( listSeances != null )
            {
                GSeance seance = getSeanceHome(  ).findSeanceByPdd( projet.getId(  ), plugin );

                if ( ( seance != null ) && !listSeances.contains( seance ) )
                {
                    listSeances.add( seance );
                }
            }
        }

        return listProjets;
    }

    /**
     * Retourne la liste des propositions correspondant aux critres de recherche
     * @param requete la requte de l'utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsPropositions la liste des propositions retrouves dans l'index contenant le(s) mot(s) cl recherchs
     * @param listSeances la liste des sances pour les fiches de sance
     * @param plugin le plugin
     * @return une liste de PDDs de type proposition
     */
    public List<GPDD> getPropositions( GRequeteUtilisateur requete, GSeance prochaineSeance,
        List<String> listIdsPropositions, List<GSeance> listSeances, Plugin plugin )
    {
        List<GPDD> listPropositions = getSearchHome(  )
                                          .findPropositionsDeliberation( requete, prochaineSeance, listIdsPropositions,
                plugin );

        GIndexedDocSearchService indexedDocSearchService;

        if ( requete.isRechercheArchive(  ) )
        {
            indexedDocSearchService = getIndexedDocSearchService( INDEX_PATH_ARCHIVE );
        }
        else
        {
            indexedDocSearchService = getIndexedDocSearchService( INDEX_PATH_CURRENT );
        }

        boolean bMotCle = !requete.getTypeRequete(  ).equals( OdsParameters.REFERENCE ) &&
            ( requete.getChampRecherche(  ) != null ) &&
            !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( requete.getChampRecherche(  ) );

        for ( GPDD proposition : listPropositions )
        {
            List<String> listIdsFichiersProposition = null;

            if ( bMotCle )
            {
                listIdsFichiersProposition = indexedDocSearchService.getListeIdsFichiersPdd( requete,
                        proposition.getId(  ) );
            }

            if ( ( listIdsFichiersProposition == null ) || !listIdsFichiersProposition.isEmpty(  ) )
            {
                proposition.setFichiers( getSearchHome(  )
                                             .findFichiersDuPdd( listIdsFichiersProposition, proposition.getId(  ),
                        plugin ) );
            }

            if ( listSeances != null )
            {
                GSeance seance = getSeanceHome(  ).findSeanceByPdd( proposition.getId(  ), plugin );

                if ( ( seance != null ) && !listSeances.contains( seance ) )
                {
                    listSeances.add( seance );
                }
            }
        }

        return listPropositions;
    }

    /**
     * La liste des amendements correspondant aux critres de recherche
     * @param requete la requte de l'utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsAmendements la liste des amendements retrouvs dans l'index
     * @param listSeances la liste des sances pour les fiches de sance
     * @param plugin le plugin
     * @return une liste de VA de type Amendement ou Lettre Rectificative
     */
    public List<GVoeuAmendement> getAmendements( GRequeteUtilisateur requete, GSeance prochaineSeance,
        List<String> listIdsAmendements, List<GSeance> listSeances, Plugin plugin )
    {
        List<GVoeuAmendement> listAmendements = getSearchHome(  )
                                                    .findAmendements( requete, prochaineSeance, listIdsAmendements,
                plugin );

        if ( listSeances != null )
        {
            for ( GVoeuAmendement amendement : listAmendements )
            {
                if ( amendement.getSeance(  ) != null )
                {
                    GSeance seance = amendement.getSeance(  );

                    if ( ( seance != null ) && !listSeances.contains( seance ) )
                    {
                        listSeances.add( seance );
                    }
                }
            }
        }

        return listAmendements;
    }

    /**
     * La liste des voeux correspondant aux critres de recherche
     * @param requete la requte de l'utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsVoeux la liste des voeux retrouvs dans l'index
     * @param listSeances la liste des sances pour les fiches de sance
     * @param plugin le plugin
     * @return une liste de VA de type Voeu ou Voeu Non Rattach
     */
    public List<GVoeuAmendement> getVoeux( GRequeteUtilisateur requete, GSeance prochaineSeance,
        List<String> listIdsVoeux, List<GSeance> listSeances, Plugin plugin )
    {
        List<GVoeuAmendement> listVoeux = getSearchHome(  ).findVoeux( requete, prochaineSeance, listIdsVoeux, plugin );

        if ( listSeances != null )
        {
            for ( GVoeuAmendement voeu : listVoeux )
            {
                if ( voeu.getSeance(  ) != null )
                {
                    GSeance seance = voeu.getSeance(  );

                    if ( ( seance != null ) && !listSeances.contains( seance ) )
                    {
                        listSeances.add( seance );
                    }
                }
            }
        }

        return listVoeux;
    }

    /**
     * Retourne la liste des autres documents de sance corrspondant aux critres de recherche
     * @param requete la requte de l'utilisateur
     * @param listIdsFichiers la liste des fichiers indexs correspondant aux critres de recherche
     * @param listSeances la liste des sances pour les fiches de sance
     * @param plugin le plugin
     * @return une liste de fichiers
     */
    public List<GFichier> getAutresDocuments( GRequeteUtilisateur requete, List<String> listIdsFichiers,
        List<GSeance> listSeances, Plugin plugin )
    {
        List<GFichier> listFichiers = getSearchHome(  ).findAutresDocuments( requete, listIdsFichiers, plugin );

        if ( listSeances != null )
        {
            for ( GFichier fichier : listFichiers )
            {
                GSeance seance = fichier.getSeance(  );

                if ( ( seance != null ) && !listSeances.contains( seance ) )
                {
                    listSeances.add( seance );
                }
            }
        }

        return listFichiers;
    }

    /**
     * Retourne la liste des arrets de sance corrspondant aux critres de recherche
     * @param requete la requte de l'utilisateur
     * @param listIdsArretes la liste des arretes indexs correspondant aux critres de recherche
     * @param listSeances la liste des sances pour les fiches de sance
     * @param plugin le plugin
     * @return une liste d'arretes
     */
    public List<Arrete<GSeance, GFichier>> getArretes( GRequeteUtilisateur requete, List<String> listIdsArretes,
        List<GSeance> listSeances, Plugin plugin )
    {
        List<Arrete<GSeance, GFichier>> listArretes = getSearchHome(  ).findArretes( requete, listIdsArretes, plugin );

        if ( listSeances != null )
        {
            for ( Arrete<GSeance, GFichier> arrete : listArretes )
            {
                GSeance seance = arrete.getSeance(  );

                if ( ( seance != null ) && !listSeances.contains( seance ) )
                {
                    listSeances.add( seance );
                }
            }
        }

        return listArretes;
    }

    /**
     * Retourne la liste des dlibration de dsignation de sance corrspondant aux critres de recherche
     * @param requete la requte de l'utilisateur
     * @param listIdsDeliberationsDesignation la liste des dlibration de dsigantion indexes correspondant aux critres de recherche
     * @param listSeances la liste des sances pour les fiches de sance
     * @param plugin le plugin
     * @return une liste de dlibration de dsignation
     */
    public List<DeliberationDesignation<GSeance, GFichier>> getDeliberationsDesignation( GRequeteUtilisateur requete,
        List<String> listIdsDeliberationsDesignation, List<GSeance> listSeances, Plugin plugin )
    {
        List<DeliberationDesignation<GSeance, GFichier>> listDeliberationsDesignation = getSearchHome(  )
                                                                                            .findDeliberationsDesignation( requete,
                listIdsDeliberationsDesignation, plugin );

        if ( listSeances != null )
        {
            for ( DeliberationDesignation<GSeance, GFichier> deliberation : listDeliberationsDesignation )
            {
                GSeance seance = deliberation.getSeance(  );

                if ( ( seance != null ) && !listSeances.contains( seance ) )
                {
                    listSeances.add( seance );
                }
            }
        }

        return listDeliberationsDesignation;
    }

    //////////////////////////////////////////////////////////////////////////////////////////////////
    //																								//
    //											Sances												//
    //																								//
    //////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Retourne la liste des sances comprises entre les 2 dates de la requte
     * @param requete la requete utilisateur
     * @param plugin le plugin
     * @return la liste des sances
     */
    public List<GSeance> getSeances( GRequeteUtilisateur requete, Plugin plugin )
    {
        List<GSeance> listSeances; // = new ArrayList<ISeance>(  );

        Timestamp tsDate1 = requete.getPremiereDate(  );
        Timestamp tsDate2; // = requete.getDeuxiemeDate(  );

        if ( tsDate1 == null )
        {
            listSeances = getSeanceHome(  ).findOldSeance( plugin );
        }
        else
        {
            tsDate2 = requete.getDeuxiemeDate(  );

            if ( ( tsDate2 == null ) ||
                    tsDate2.after( getSeanceHome(  ).getDerniereSeance( plugin ).getDateSeance(  ) ) )
            {
                tsDate2 = getSeanceHome(  ).getDerniereSeance( plugin ).getDateSeance(  );
            }

            listSeances = getSeanceHome(  ).findSeanceWithFilterList( tsDate1, tsDate2, plugin );
        }

        return listSeances;
    }
}
