/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.search;

import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.queryParser.MultiFieldQueryParser;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.search.BooleanClause;
import org.apache.lucene.search.Hit;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;

import java.io.IOException;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.StringTokenizer;


/**
 * Cette classe ralise la recherche dans l'index Lucene
 */
public abstract class AbstractIndexedDocSearchService<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>>
    implements IIndexedDocSearchService<GSeance, GSeanceFilter, GRequeteUtilisateur>
{
    private static final String DEFAULT_INDEX_PATH = "ods.search.lucene.default.indexPath";
    private static final String ERREUR_CHARGEMENT_OBJETS = "ods.odsindexeddocsearchservice.message.erreurChargement";
    private static final String ERREUR_RECHERCHE = "ods.odsindexeddocsearchservice.message.erreurRecherche";
    private static final String ERREUR_LECTURE_INDEX = "ods.odsindexeddocsearchservice.message.erreurLecture";
    private static final String ERREUR_FERMETURE_INDEX = "ods.odsindexeddocsearchservice.message.erreurFermeture";
    private static final String ERREUR_OUVERTURE_INDEX = "ods.odsindexeddocsearchservice.message.erreurOuverture";
    private static final String ERREUR_CHARGEMENT_ANALYSEUR = "ods.odsindexeddocsearchservice.message.erreurChargementAnalyseur";
    private static final String ERREUR_CLASSE_ANALYSEUR_INVALIDE = "ods.odsindexeddocsearchservice.message.erreurAnalyseurInvalide";
    private static final String PROPERTY_ANALYSER_CLASSNAME = "ods.search.lucene.analyser.className";
    private static Analyzer _analyzer;
    private String _strIndex;
    private IndexSearcher _searcher;

    /**
     * Constructeur priv, initialise les valeurs de _strIndex et _analyzer
     * @param strIndex le chemin de l'index
     */
    public AbstractIndexedDocSearchService( String strIndex )
    {
        if ( ( strIndex != null ) && !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( strIndex ) )
        {
            _strIndex = strIndex;
        }
        else
        {
            _strIndex = AppPathService.getPath( DEFAULT_INDEX_PATH );
        }

        if ( ( _strIndex == null ) || ( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( _strIndex ) ) )
        {
            throw new AppException(  );
        }

        String strAnalyserClassName = AppPropertiesService.getProperty( PROPERTY_ANALYSER_CLASSNAME );

        if ( ( strAnalyserClassName == null ) || ( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( strAnalyserClassName ) ) )
        {
            throw new AppException( I18nService.getLocalizedString( ERREUR_CLASSE_ANALYSEUR_INVALIDE,
                    Locale.getDefault(  ) ) );
        }

        try
        {
            _analyzer = (Analyzer) Class.forName( strAnalyserClassName ).newInstance(  );
        }
        catch ( Exception e )
        {
            throw new AppException( I18nService.getLocalizedString( ERREUR_CHARGEMENT_ANALYSEUR, Locale.getDefault(  ) ),
                e );
        }
    }

    /**
     * Force la fermeture de l'index si une opration s'est mal droule
     * @throws AppException si une erreur survient lors de la fermeture de l'index
     */
    private void fermerIndex(  ) throws AppException
    {
        try
        {
            _searcher.close(  );
        }
        catch ( IOException ioe )
        {
            throw new AppException( I18nService.getLocalizedString( ERREUR_FERMETURE_INDEX, Locale.getDefault(  ) ) +
                ioe.getMessage(  ) );
        }
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IIndexedDocSearchService#getListeIdsProjets(GRequeteUtilisateur)
         */
    public List<String> getListeIdsProjets( GRequeteUtilisateur requete )
    {
        return getListIds( requete, OdsConstants.TYPE_PROJET );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IIndexedDocSearchService#getListeIdsPropositions(GRequeteUtilisateur)
         */
    public List<String> getListeIdsPropositions( GRequeteUtilisateur requete )
    {
        return getListIds( requete, OdsConstants.TYPE_PROPOSITION );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IIndexedDocSearchService#getListeIdsAmendements(GRequeteUtilisateur)
         */
    public List<String> getListeIdsAmendements( GRequeteUtilisateur requete )
    {
        return getListIds( requete, OdsConstants.TYPE_AMENDEMENT );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IIndexedDocSearchService#getListeIdsVoeux(GRequeteUtilisateur)
         */
    public List<String> getListeIdsVoeux( GRequeteUtilisateur requete )
    {
        return getListIds( requete, OdsConstants.TYPE_VOEU );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IIndexedDocSearchService#getListeIdsFichiersPdd(GRequeteUtilisateur, int)
         */
    public List<String> getListeIdsFichiersPdd( GRequeteUtilisateur requete, int nIdPdd )
    {
        return getListIds( requete, OdsConstants.TYPE_FICHIER_PDD + nIdPdd );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IIndexedDocSearchService#getListeIdsAutresFichiers(GRequeteUtilisateur)
         */
    public List<String> getListeIdsAutresFichiers( GRequeteUtilisateur requete )
    {
        return getListIds( requete, OdsConstants.TYPE_AUTRE_DOC );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IIndexedDocSearchService#getListeIdsArretes(GRequeteUtilisateur)
         */
    public List<String> getListeIdsArretes( GRequeteUtilisateur requete )
    {
        return getListIds( requete, OdsConstants.TYPE_ARRETE );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IIndexedDocSearchService#getListeIdsDeliberationsDesignation(GRequeteUtilisateur)
         */
    public List<String> getListeIdsDeliberationsDesignation( GRequeteUtilisateur requete )
    {
        return getListIds( requete, OdsConstants.TYPE_DELIBERATION_DESIGNATION );
    }

    /**
     * Retourne la liste des ids retrouvs dans l'index avec le mot-cl recherch
     * @param requete la requte utilisateur
     * @param strTypeObjet le type d'objet recherch
     * @return une liste d'ids de propositions
     */
    protected List<String> getListIds( GRequeteUtilisateur requete, String strTypeObjet )
    {
        List<String> listIds = new ArrayList<String>(  );

        try
        {
            _searcher = new IndexSearcher( _strIndex );
        }
        catch ( IOException ioe )
        {
            AppLogService.error( ioe.getClass(  ).toString(  ) + " : " +
                I18nService.getLocalizedString( ERREUR_OUVERTURE_INDEX, Locale.getDefault(  ) ) + _strIndex );

            return listIds;
        }

        String[] tabFields = { OdsConstants.FIELD_TYPE_OBJET, OdsConstants.FIELD_CONTENT };
        String[] tabRecherches = { strTypeObjet, prepareRequeteLucene( requete.getChampRecherche(  ) ) };
        BooleanClause.Occur[] tabClauses = { BooleanClause.Occur.MUST, BooleanClause.Occur.MUST };

        Query query;

        try
        {
            query = MultiFieldQueryParser.parse( tabRecherches, tabFields, tabClauses, _analyzer );
        }
        catch ( ParseException pe )
        {
            fermerIndex(  );
            throw new AppException( I18nService.getLocalizedString( ERREUR_LECTURE_INDEX, Locale.getDefault(  ) ) +
                _strIndex + pe.getMessage(  ) );
        }

        Hits hits = null;

        try
        {
            hits = _searcher.search( query );
        }
        catch ( IOException ioe )
        {
            fermerIndex(  );
            throw new AppException( I18nService.getLocalizedString( ERREUR_RECHERCHE, Locale.getDefault(  ) ) );
        }
        catch ( Exception e )
        {
            fermerIndex(  );
        }

        if ( hits != null )
        {
            Iterator<Hit> iterator = hits.iterator(  );

            while ( iterator.hasNext(  ) )
            {
                Hit hit = ( iterator.next(  ) );
                Document doc;

                try
                {
                    doc = hit.getDocument(  );
                }
                catch ( IOException ioe )
                {
                    fermerIndex(  );
                    throw new AppException( I18nService.getLocalizedString( ERREUR_CHARGEMENT_OBJETS,
                            Locale.getDefault(  ) ) + ioe.getMessage(  ) );
                }

                listIds.add( doc.get( OdsConstants.FIELD_ODS_OBJECT_ID ) );
            }
        }

        fermerIndex(  );

        return listIds;
    }

    /**
     * Prepare la requte utilisateur pour que les mots cls soient associs avec l'oprateur boolen AND
     * @param requeteUtilisateur la requte de l'utilisateur
     * @return la requte Lucene pour la recherche ODS
     */
    private String prepareRequeteLucene( String requeteUtilisateur )
    {
        StringTokenizer stRequete = new StringTokenizer( requeteUtilisateur, " ", false );
        StringBuffer sbRequeteLucene = new StringBuffer(  );

        while ( stRequete.hasMoreTokens(  ) )
        {
            if ( sbRequeteLucene.length(  ) != 0 )
            {
                sbRequeteLucene.append( " && " );
            }

            sbRequeteLucene.append( stRequete.nextToken(  ) );
        }

        return sbRequeteLucene.toString(  ).toLowerCase(  );
    }
}
