/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.search;

import fr.paris.lutece.plugins.ods.business.IBusinessHome;
import fr.paris.lutece.plugins.ods.business.acte.ActeHome;
import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.business.indexer.IndexerHome;
import fr.paris.lutece.plugins.ods.business.indexer.action.IndexerActionHome;
import fr.paris.lutece.plugins.ods.dto.DesignationComparator;
import fr.paris.lutece.plugins.ods.dto.IBusinessItem;
import fr.paris.lutece.plugins.ods.dto.IDesignation;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.arrete.Arrete;
import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.DeliberationDesignation;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.indexer.Indexer;
import fr.paris.lutece.plugins.ods.dto.indexer.action.IndexerAction;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.pdd.PDDComparator;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.VoeuAmendementFrontComparator;
import fr.paris.lutece.plugins.ods.service.acte.IActeService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppPropertiesService;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;


/**
 * ODSSearchService
 */
public abstract class AbstractSearchService<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, GResultatRequete extends IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPDD>, GIndexedDocSearchService extends IIndexedDocSearchService<GSeance, GSeanceFilter, GRequeteUtilisateur>, GDocSearchService extends IDocSearchService<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPDD>, GFichier extends IFichier<GSeance, GFichier>, GActeFilter extends IActeFilter<GFichier, GSeance>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>>
    implements ISearchService<GSeance, GSeanceFilter, GRequeteUtilisateur, GResultatRequete, GElu, GFichier, GVoeuAmendement, GPDD>
{
    private static final int DEFAUT_MAX_NOMBRE_FICHIERS = 50;
    protected static final String PROPERTY_INDEXER_NAME = "ods.searchindexer.name";
    private static final String PROPERTY_MAX_NOMBRE_FICHIERS = "ods.search.maxNombreFichiers";
    protected static final int MAX_NOMBRE_FICHIERS = AppPropertiesService.getPropertyInt( PROPERTY_MAX_NOMBRE_FICHIERS,
            DEFAUT_MAX_NOMBRE_FICHIERS );
    protected static final String PARAM_PROJET = "projet";
    protected static final String PARAM_PROPOSITION = "proposition";
    private static final String PARAM_AMENDEMENT = "amendement";
    private static final String PARAM_VOEU = "voeu";
    private static final String PARAM_SEANCE = "seance";
    private static final String PARAM_AUTRES = "autres";
    private static final String PARAM_DESIGNATION = "designation";
    protected static final String INDEX_PATH_CURRENT = AppPropertiesService.getProperty( 
            "ods.search.lucene.current.indexPath" );
    protected static final String INDEX_PATH_ARCHIVE = AppPropertiesService.getProperty( 
            "ods.search.lucene.archive.indexPath" );
    @Autowired
    protected IndexerHome _indexerHome;
    @Autowired
    protected ActeHome<GSeance, GFichier, GActeFilter> _acteHome;
    @Autowired
    protected IFichierHome<GFichier, IFichierFilter, ITypeDocument, GSeance> _fichierHome;
    @Autowired
    protected IndexerActionHome _indexerActionHome;
    @Autowired
    protected IActeService<GSeance, GFichier, GActeFilter> _acteService;
    protected Map<Integer, IBusinessHome> _mappingHome;

    protected abstract GDocSearchService getDocSearchService(  );

    protected abstract void getDeletedObjectsSpec( IndexerAction action, List<IBusinessItem> listObjets, Plugin plugin )
        throws AppException;

    protected abstract void setActionValuesSpec( IndexerAction action, IBusinessItem item );

    protected abstract GIndexedDocSearchService getIndexedDocSearchService( String strPath );

    protected abstract Acte<GSeance, GFichier> getActeCourant( List<Acte<GSeance, GFichier>> listActes );

    /**
     * recherche les informations spcifique
     * @param resultatRequete
     * @param requete
     * @param prochaineSeance
     * @param bIsLimited
     * @param plugin
     */
    protected abstract int getAllObjectsByCriteriasSpec( GResultatRequete resultatRequete, GRequeteUtilisateur requete,
        GSeance prochaineSeance, boolean bIsLimited, boolean bMotCle, List<GSeance> listSeances, Plugin plugin );

    /**
     * Gnre un nouvel objet IResultatRequete
     * @return
     */
    protected abstract GResultatRequete newResultatRequeteInstance(  );

    //    private Map<Integer, ISearchServiceManaged> mappingService;
    public Map<Integer, IBusinessHome> getMappingHome(  )
    {
        return _mappingHome;
    }

    public void setMappingHome( Map<Integer, IBusinessHome> mappingHome )
    {
        this._mappingHome = mappingHome;
    }

    //    public Map<Integer, ISearchServiceManaged> getMappingService() 
    //    {
    //        return mappingService;
    //    }
    //        
    //    public void setMappingService(Map<Integer, ISearchServiceManaged> mappingService) 
    //    {
    //        this.mappingService = mappingService;
    //    }

    //////////////////////////////////////////////////////////////////////////////////////////
    //																						//
    //								Fonctions d'indexation									//
    //																						//
    //////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Pour chaque indexer, cette mthode insre une nouvelle action de cration dans l'index
     * @param item l'objet  indexer
     * @param bArchive indique dans quel index il faut ajouter l'objet
     * @param plugin le plugin
     */
    public void addObjectToIndex( IBusinessItem item, boolean bArchive, Plugin plugin )
    {
        // 5 types d'objet possibles: Fichier, VoeuAmendement PDD, Arrete ou Dsignation de dlibration
        List<Indexer> listIndexers = _indexerHome.findAllIndexers( plugin );
        IndexerAction action = new IndexerAction(  );

        action.setType( item.getTypeEntite(  ) );
        action.setIdObjet( item.getId(  ) );
        action.setCodeTask( IndexerAction.ACTION_CREATE );
        action.setArchive( bArchive );

        for ( Indexer indexer : listIndexers )
        {
            action.setIdIndexer( indexer.getIdIndexer(  ) );
            _indexerActionHome.create( action, plugin );
        }
    }

    /**
     * Pour chaque indexer, cette mthode insre une nouvelle action de modification de l'index
     * @param item l'objet  modifier dans l'index
     * @param bArchive indique dans quel index il faut modifier l'objet
     * @param plugin le plugin
     */
    public void updateObjectInIndex( IBusinessItem item, boolean bArchive, Plugin plugin )
    {
        List<Indexer> listIndexers = _indexerHome.findAllIndexers( plugin );
        IndexerAction action = new IndexerAction(  );

        for ( Indexer indexer : listIndexers )
        {
            // si l'action d'ajout a t effectu par l'indexer ( = null),
            // demander la modif; sinon ne rien faire, les infos seront directement les bonnes
            if ( ( _indexerActionHome.findActionOnObject( indexer.getIdIndexer(  ), IndexerAction.ACTION_CREATE, item,
                        plugin ) == null ) &&
                    ( _indexerActionHome.findActionOnObject( indexer.getIdIndexer(  ), IndexerAction.ACTION_MODIFY,
                        item, plugin ) == null ) )
            {
                action.setType( item.getTypeEntite(  ) );
                action.setIdObjet( item.getId(  ) );
                action.setIdIndexer( indexer.getIdIndexer(  ) );
                action.setCodeTask( IndexerAction.ACTION_MODIFY );
                action.setArchive( bArchive );

                _indexerActionHome.create( action, plugin );
            }
        }
    }

    /**
     * Pour chaque indexer, cette mthode insre une nouvelle action de suppression dans l'index
     * @param item l'objet  supprimer de l'index
     * @param bArchive indique dans quel index il faut supprimer l'objet
     * @param plugin plugin
     */
    public void deleteObjectFromIndex( IBusinessItem item, boolean bArchive, Plugin plugin )
    {
        List<Indexer> listIndexers = _indexerHome.findAllIndexers( plugin );
        IndexerAction action = new IndexerAction(  );

        for ( Indexer indexer : listIndexers )
        {
            // si l'action d'ajout a t effectu par l'indexer ( = null),
            // demander la suppression; sinon annuler l'ajout (l'indexation n'tait pas encore passe)
            if ( _indexerActionHome.findActionOnObject( indexer.getIdIndexer(  ), IndexerAction.ACTION_CREATE, item,
                        plugin ) == null )
            {
                setActionValuesSpec( action, item );
                //action.setType( item.getTypeEntite(  ) );
                action.setIdObjet( item.getId(  ) );
                action.setIdIndexer( indexer.getIdIndexer(  ) );
                action.setCodeTask( IndexerAction.ACTION_DELETE );
                action.setArchive( bArchive );

                _indexerActionHome.create( action, plugin );
            }
            else
            {
                _indexerActionHome.remove( _indexerActionHome.findActionOnObject( indexer.getIdIndexer(  ),
                        IndexerAction.ACTION_CREATE, item, plugin ), plugin );
            }
        }
    }

    //////////////////////////////////////////////////////////////////////////////////////////
    //																						//
    //						 Fonctions pour l'indexation incrmentale						//
    //																						//
    //////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Retourne la liste des objets ajouts depuis la dernire indexation
     * @param bArchive indique sur quel index oprer (archive ou prochaine sance)
     * @param plugin le plugin
     * @return la liste de fichiers ajouts depuis la dernire indexation
     * @throws  AppException  AppException
     */
    public List<IBusinessItem> getAddedObjects( boolean bArchive, Plugin plugin )
        throws AppException
    {
        String strNomIndexer = AppPropertiesService.getProperty( PROPERTY_INDEXER_NAME );

        int nIdIndexer = _indexerHome.findByName( strNomIndexer, plugin ).getIdIndexer(  );

        List<IBusinessItem> listObjets = new ArrayList<IBusinessItem>(  );

        List<IndexerAction> indexerActions = _indexerActionHome.findActionsByIndexer( nIdIndexer,
                IndexerAction.ACTION_CREATE, true, bArchive, plugin );

        int nIdObjet;
        int nIdType;

        for ( IndexerAction action : indexerActions )
        {
            nIdObjet = action.getIdObjet(  );
            nIdType = action.getType(  );

            listObjets.add( _mappingHome.get( nIdType ).findByPrimaryKey( nIdObjet, plugin ) );
        }

        return listObjets;
    }

    /**
     * Retourne la liste des objets modifis depuis la dernire indexation
     * @param bDelete indique si l'on doit supprimer l'action aprs l'avoir lue
     * @param bArchive indique sur quel index oprer (archive ou prochaine sance)
     * @param plugin le plugin
     * @return la liste des objets modifis depuis la dernire indexation
     * @throws AppException AppException
     */
    public List<IBusinessItem> getUpdatedObjects( boolean bDelete, boolean bArchive, Plugin plugin )
        throws AppException
    {
        String strNomIndexer = AppPropertiesService.getProperty( PROPERTY_INDEXER_NAME );

        int nIdIndexer = _indexerHome.findByName( strNomIndexer, plugin ).getIdIndexer(  );

        List<IBusinessItem> listObjets = new ArrayList<IBusinessItem>(  );

        List<IndexerAction> indexerActions = _indexerActionHome.findActionsByIndexer( nIdIndexer,
                IndexerAction.ACTION_MODIFY, bDelete, bArchive, plugin );

        int nIdObjet;
        int nIdType;

        for ( IndexerAction action : indexerActions )
        {
            nIdObjet = action.getIdObjet(  );
            nIdType = action.getType(  );

            listObjets.add( _mappingHome.get( nIdType ).findByPrimaryKey( nIdObjet, plugin ) );
        }

        return listObjets;
    }

    /**
     * Retourne la liste des objets supprims depuis la dernire indexation
     * @param bArchive indique sur quel index oprer (archive ou prochaine sance)
     * @param plugin le plugin
     * @return la liste des objets supprims depuis la dernire indexation
     * @throws AppException AppException
     */
    public List<IBusinessItem> getDeletedObjects( boolean bArchive, Plugin plugin )
        throws AppException
    {
        String strNomIndexer = AppPropertiesService.getProperty( PROPERTY_INDEXER_NAME );

        int nIdIndexer = _indexerHome.findByName( strNomIndexer, plugin ).getIdIndexer(  );

        List<IBusinessItem> listObjets = new ArrayList<IBusinessItem>(  );

        List<IndexerAction> indexerActions = _indexerActionHome.findActionsByIndexer( nIdIndexer,
                IndexerAction.ACTION_DELETE, true, bArchive, plugin );

        //int nIdObjet;
        //int nIdType;
        for ( IndexerAction action : indexerActions )
        {
            getDeletedObjectsSpec( action, listObjets, plugin );

            //listObjets.add( mappingHome.get( nIdType ).findByPrimaryKey( nIdObjet, plugin ) );
        }

        return listObjets;
    }

    //////////////////////////////////////////////////////////////////////////////////////////
    //																						//
    //								Fonctions de recherche									//
    //																						//
    //////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Retourne la liste des PDD recherchs par rfrence
     * @param requete la requte utilisateur
     * @param prochaineSeance la prochaine sance
     * @param plugin le plugin
     * @return la liste des PDD dont la rfrence contient la chane passe dans la requte
     */
    public GResultatRequete getPDDByReference( GRequeteUtilisateur requete, GSeance prochaineSeance, Plugin plugin )
    {
        GResultatRequete resultatRequete = newResultatRequeteInstance(  );
        int nNombreFichiers = 0;
        GActeFilter acteFilter = _acteService.getActeFilter(  );

        List<GPDD> listProjets = getDocSearchService(  ).getProjets( requete, prochaineSeance, null, null, plugin );

        if ( listProjets != null )
        {
            acteFilter.setIdTypeEntite( ITypeEntite.PROJETS_DE_DELIBERATIONS );

            for ( GPDD projet : listProjets )
            {
                nNombreFichiers += projet.getFichiers(  ).size(  );

                acteFilter.setIdEntite( projet.getId(  ) );

                List<Acte<GSeance, GFichier>> listActes = _acteHome.findByFilter( acteFilter, plugin );

                for ( Acte<GSeance, GFichier> acte : listActes )
                {
                    GFichier fDeliberationFinale = _fichierHome.findByPrimaryKey( acte.getDeliberationFinale(  ).getId(  ),
                            plugin );
                    acte.setDeliberationFinale( fDeliberationFinale );
                }

                projet.setListeActes( listActes );
            }

            Collections.sort( listProjets, new PDDComparator<GSeance, GFichier, GElu, GVoeuAmendement, GPDD>(  ) );
        }

        List<GPDD> listPropositions = getDocSearchService(  )
                                          .getPropositions( requete, prochaineSeance, null, null, plugin );

        if ( listPropositions != null )
        {
            acteFilter.setIdTypeEntite( ITypeEntite.PROPOSITIONS_DE_DELIBERATIONS );

            for ( GPDD proposition : listPropositions )
            {
                nNombreFichiers += proposition.getFichiers(  ).size(  );

                acteFilter.setIdEntite( proposition.getId(  ) );

                List<Acte<GSeance, GFichier>> listActes = _acteHome.findByFilter( acteFilter, plugin );

                for ( Acte<GSeance, GFichier> acte : listActes )
                {
                    GFichier fDeliberationFinale = _fichierHome.findByPrimaryKey( acte.getDeliberationFinale(  ).getId(  ),
                            plugin );
                    acte.setDeliberationFinale( fDeliberationFinale );
                }

                proposition.setListeActes( listActes );
            }

            Collections.sort( listPropositions, new PDDComparator<GSeance, GFichier, GElu, GVoeuAmendement, GPDD>(  ) );
        }

        resultatRequete.setListeProjets( listProjets );
        resultatRequete.setListePropositions( listPropositions );

        int nbObjetDetailles = 0;

        if ( listProjets != null )
        {
            nbObjetDetailles += listProjets.size(  );
        }

        if ( listPropositions != null )
        {
            nbObjetDetailles += listPropositions.size(  );
        }

        resultatRequete.setNombreObjetsDetailles( nbObjetDetailles );
        resultatRequete.setNombreResultats( nNombreFichiers );
        resultatRequete.setRequeteUtilisateur( requete );

        return resultatRequete;
    }

    /**
     * Retourne le rsultat de la recherche multi-critres
     * @param requete la requte de l'utilisateur
     * @param prochaineSeance la prochaine sance
     * @param bIsLimited true si le nombre est limit, false sinon
     * @param plugin le plugin
     * @return le rsultat de la recherche
     */
    public GResultatRequete getAllObjectsByCriterias( GRequeteUtilisateur requete, GSeance prochaineSeance,
        boolean bIsLimited, Plugin plugin )
    {
        GResultatRequete resultatRequete = newResultatRequeteInstance(  );
        resultatRequete.setRequeteUtilisateur( requete );

        int nNombreResultats = 0;
        List<GSeance> listSeances = null;

        GIndexedDocSearchService indexedDocSearchService;
        boolean bArchive = requete.isRechercheArchive(  );

        if ( bArchive )
        {
            indexedDocSearchService = getIndexedDocSearchService( INDEX_PATH_ARCHIVE );
        }
        else
        {
            indexedDocSearchService = getIndexedDocSearchService( INDEX_PATH_CURRENT );
        }

        boolean bMotCle = ( requete.getChampRecherche(  ) != null ) &&
            ( !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( requete.getChampRecherche(  ).trim(  ) ) );

        if ( bMotCle && requete.getListeTypesDocumentFromRepresentation(  ).contains( PARAM_SEANCE ) )
        {
            listSeances = new ArrayList<GSeance>(  );
        }

        /**
         * Gestion des amendements
         */
        List<GVoeuAmendement> listAmendements = null;

        if ( requete.getListeTypesDocumentFromRepresentation(  ).contains( PARAM_AMENDEMENT ) )
        {
            List<String> listIdsAmendements = null;

            if ( bMotCle )
            {
                listIdsAmendements = indexedDocSearchService.getListeIdsAmendements( requete );
            }

            // si listIdsPropositions est null alors le mot-cl n'a pas t trouv dans l'index 
            //		==> aucune proposition n'est retourne
            if ( ( listIdsAmendements == null ) || !listIdsAmendements.isEmpty(  ) )
            {
                listAmendements = getDocSearchService(  )
                                      .getAmendements( requete, prochaineSeance, listIdsAmendements, listSeances, plugin );
                nNombreResultats += listAmendements.size(  );
            }
        }

        if ( bIsLimited && ( nNombreResultats > MAX_NOMBRE_FICHIERS ) )
        {
            return null;
        }

        if ( listAmendements != null )
        {
            Collections.sort( listAmendements,
                new VoeuAmendementFrontComparator<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>(  ) );
        }

        resultatRequete.setListeAmendements( listAmendements );

        /**
         * Gestion des voeux
         */
        List<GVoeuAmendement> listVoeux = null;

        if ( requete.getListeTypesDocumentFromRepresentation(  ).contains( PARAM_VOEU ) )
        {
            List<String> listIdsVoeux = null;

            if ( bMotCle )
            {
                listIdsVoeux = indexedDocSearchService.getListeIdsVoeux( requete );
            }

            //	si listIdsPropositions est null alors le mot-cl n'a pas t trouv dans l'index 
            //	==> aucune proposition n'est retourne
            if ( ( listIdsVoeux == null ) || !listIdsVoeux.isEmpty(  ) )
            {
                listVoeux = getDocSearchService(  ).getVoeux( requete, prochaineSeance, listIdsVoeux, listSeances,
                        plugin );

                GActeFilter acteFilter = _acteService.getActeFilter(  );
                acteFilter.setIdTypeEntite( ITypeEntite.VOEUX );

                for ( GVoeuAmendement voeu : listVoeux )
                {
                    acteFilter.setIdEntite( voeu.getId(  ) );

                    List<Acte<GSeance, GFichier>> listActes = _acteHome.findByFilter( acteFilter, plugin );

                    if ( !listActes.isEmpty(  ) )
                    {
                        Acte<GSeance, GFichier> acte = getActeCourant( listActes );

                        if ( acte != null )
                        {
                            GFichier fDeliberationFinale = _fichierHome.findByPrimaryKey( acte.getDeliberationFinale(  )
                                                                                              .getId(  ), plugin );
                            acte.setDeliberationFinale( fDeliberationFinale );
                            voeu.setActeCourant( acte );
                        }
                    }
                }

                nNombreResultats += listVoeux.size(  );
            }
        }

        if ( bIsLimited && ( nNombreResultats > MAX_NOMBRE_FICHIERS ) )
        {
            return null;
        }

        if ( listVoeux != null )
        {
            Collections.sort( listVoeux,
                new VoeuAmendementFrontComparator<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>(  ) );
        }

        resultatRequete.setListeVoeux( listVoeux );

        /*
         * Gestion des autres documents de sance
         */
        List<GFichier> listAutresFichiers = null;

        if ( requete.getListeTypesDocumentFromRepresentation(  ).contains( PARAM_AUTRES ) )
        {
            List<String> listIdsFichiers = null;

            if ( bMotCle )
            {
                listIdsFichiers = indexedDocSearchService.getListeIdsAutresFichiers( requete );
            }

            if ( ( listIdsFichiers == null ) || !listIdsFichiers.isEmpty(  ) )
            {
                listAutresFichiers = getDocSearchService(  )
                                         .getAutresDocuments( requete, listIdsFichiers, listSeances, plugin );
                nNombreResultats += listAutresFichiers.size(  );
            }
        }

        if ( bIsLimited && ( nNombreResultats > MAX_NOMBRE_FICHIERS ) )
        {
            return null;
        }

        resultatRequete.setListeAutresDocuments( listAutresFichiers );

        /**
         * Gestion des dsignations (Arrts et dlibrations de dsignation)
         */
        List<Arrete<GSeance, GFichier>> listArretes = null;
        List<DeliberationDesignation<GSeance, GFichier>> listDeliberationsDesignation = null;
        List<IDesignation<GSeance, GFichier>> listDesignations = new ArrayList<IDesignation<GSeance, GFichier>>(  );

        if ( requete.getListeTypesDocumentFromRepresentation(  ).contains( PARAM_DESIGNATION ) )
        {
            List<String> listIdsArretes = null;
            List<String> listIdsDeliberationsDesignation = null;

            if ( bMotCle )
            {
                listIdsArretes = indexedDocSearchService.getListeIdsArretes( requete );
                listIdsDeliberationsDesignation = indexedDocSearchService.getListeIdsDeliberationsDesignation( requete );
            }

            if ( ( listIdsArretes == null ) || !listIdsArretes.isEmpty(  ) )
            {
                listArretes = getDocSearchService(  ).getArretes( requete, listIdsArretes, listSeances, plugin );
                nNombreResultats += listArretes.size(  );
            }

            if ( ( listIdsDeliberationsDesignation == null ) || !listIdsDeliberationsDesignation.isEmpty(  ) )
            {
                listDeliberationsDesignation = getDocSearchService(  )
                                                   .getDeliberationsDesignation( requete,
                        listIdsDeliberationsDesignation, listSeances, plugin );
                nNombreResultats += listDeliberationsDesignation.size(  );
            }
        }

        if ( bIsLimited && ( nNombreResultats > MAX_NOMBRE_FICHIERS ) )
        {
            return null;
        }

        if ( listArretes != null )
        {
            listDesignations.addAll( listArretes );
        }

        if ( listDeliberationsDesignation != null )
        {
            listDesignations.addAll( listDeliberationsDesignation );
        }

        Collections.sort( listDesignations, new DesignationComparator<GSeance, GFichier>(  ) );
        resultatRequete.setListeDesignations( listDesignations );

        int nResultatsSpec = getAllObjectsByCriteriasSpec( resultatRequete, requete, prochaineSeance, bIsLimited,
                bMotCle, listSeances, plugin );

        // trop de resultat
        if ( bIsLimited && ( nResultatsSpec == -1 ) )
        {
            return null;
        }

        nNombreResultats += nResultatsSpec;

        if ( bIsLimited && ( nNombreResultats > MAX_NOMBRE_FICHIERS ) )
        {
            return null;
        }

        /**
         * Gestion des sance
         */
        if ( requete.getListeTypesDocumentFromRepresentation(  ).contains( PARAM_SEANCE ) && !bMotCle )
        {
            listSeances = getDocSearchService(  ).getSeances( requete, plugin );
        }

        resultatRequete.setListeSeances( listSeances );

        resultatRequete.setNombreResultats( nNombreResultats );
        resultatRequete.setNombreObjetsDetailles( nNombreResultats );

        return resultatRequete;
    }
}
