/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.search;

import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.spring.SpringContextService;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexModifier;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.Term;
import org.apache.lucene.store.FSDirectory;

import java.io.IOException;

import java.util.List;
import java.util.Locale;


/**
 * IndexationService
 */
public abstract class IndexationService
{
    public static final boolean IS_ARCHIVE = true;
    private static final String ERREUR_MODIFICATION_INDEX = "ods.odsindexationservice.message.erreur.modifyIndex";
    private static final String ERREUR_FERMETURE_INDEX = "ods.odsindexationservice.message.erreur.closeIndex";
    private static final String ERREUR_CHARGEMENT_ANALYSEUR = "ods.odsindexationservice.message.erreur.loadAnalyser";
    private static final String ERREUR_CLASSE_ANALYSEUR_INVALIDE = "ods.odsindexationservice.message.erreur.invalidAnalyser";
    private static final String ERREUR_OUVERTURE_INDEX = "ods.odsindexationservice.message.erreur.openIndex";

    // variables dfinies dans WEB-INF/conf/plugins/ods.properties
    private static final String PROPERTY_WRITER_MERGE_FACTOR = "ods.search.mergeFactor";
    private static final String PROPERTY_WRITER_MAX_FIELD_LENGTH = "ods.search.maxFieldLength";
    private static final String PROPERTY_ANALYSER_CLASSNAME = "ods.search.lucene.analyser.className";
    private static final String DEFAULT_INDEX_PATH = "ods.search.lucene.default.indexPath";
    private static final int DEFAULT_WRITER_MERGE_FACTOR = 20;
    private static final int DEFAULT_WRITER_MAX_FIELD_LENGTH = 1000000;
    private static Analyzer _analyzer;
    private static int _nWriterMergeFactor = DEFAULT_WRITER_MERGE_FACTOR;
    private static int _nWriterMaxFieldLength = DEFAULT_WRITER_MAX_FIELD_LENGTH;
    private String _strIndex;
    private boolean _bArchiveService;
    private IODSSearchIndexerFactoryService _searchIndexerFactory = ( (IODSSearchIndexerFactoryService) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
            "odsSearchIndexerFactoryService" ) );

    /**
     * Constructeur
     * @param strIndexPath le chemin de l'index
     * @param bArchiveService indique si le service porte sur l'index archive ou prochaine seance
     */
    public IndexationService( String strIndexPath, boolean bArchiveService )
    {
        _bArchiveService = bArchiveService;

        if ( ( strIndexPath != null ) && !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( strIndexPath ) )
        {
            _strIndex = strIndexPath;
        }
        else
        {
            _strIndex = AppPathService.getPath( DEFAULT_INDEX_PATH );
        }

        if ( ( _strIndex == null ) || ( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( _strIndex ) ) )
        {
            throw new AppException(  );
        }

        String strAnalyserClassName = AppPropertiesService.getProperty( PROPERTY_ANALYSER_CLASSNAME );

        if ( ( strAnalyserClassName == null ) || ( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( strAnalyserClassName ) ) )
        {
            throw new AppException( I18nService.getLocalizedString( ERREUR_CLASSE_ANALYSEUR_INVALIDE,
                    Locale.getDefault(  ) ) );
        }

        _nWriterMergeFactor = AppPropertiesService.getPropertyInt( PROPERTY_WRITER_MERGE_FACTOR,
                DEFAULT_WRITER_MERGE_FACTOR );
        _nWriterMaxFieldLength = AppPropertiesService.getPropertyInt( PROPERTY_WRITER_MAX_FIELD_LENGTH,
                DEFAULT_WRITER_MAX_FIELD_LENGTH );

        try
        {
            _analyzer = (Analyzer) Class.forName( strAnalyserClassName ).newInstance(  );
        }
        catch ( Exception e )
        {
            throw new AppException( I18nService.getLocalizedString( ERREUR_CHARGEMENT_ANALYSEUR, Locale.getDefault(  ) ),
                e );
        }
    }

    /**
    * Lance l'indexation
    * @throws AppException en cas d'erreur lors de la manipulation de l'index
    */
    public void processIndexing(  ) throws AppException
    {
        IODSSearchIndexer searchIndexer = _searchIndexerFactory.getSearchIndexer( _bArchiveService );
        IndexWriter indexWriter;

        // On ouvre l'criture sur l'index
        try
        {
            if ( IndexReader.isLocked( _strIndex ) )
            {
                IndexReader.unlock( FSDirectory.getDirectory( _strIndex ) );
            }

            indexWriter = new IndexWriter( _strIndex, _analyzer, false );
        }
        catch ( IOException ioe )
        {
            throw new AppException( ERREUR_OUVERTURE_INDEX, ioe );
        }

        indexWriter.setMergeFactor( _nWriterMergeFactor );
        indexWriter.setMaxFieldLength( _nWriterMaxFieldLength );

        List<Document> listDocs = null;

        // On rcupre puis on supprime les documents  supprimer de l'index
        listDocs = searchIndexer.getDocumentsToRemove(  );

        for ( Document doc : listDocs )
        {
            try
            {
                indexWriter.deleteDocuments( new Term( OdsConstants.FIELD_ID_OBJET,
                        doc.getField( OdsConstants.FIELD_ID_OBJET ).stringValue(  ) ) );
            }
            catch ( IOException ioe )
            {
                AppLogService.error( I18nService.getLocalizedString( ERREUR_MODIFICATION_INDEX, Locale.getDefault(  ) ) );
            }
        }

        // On rcupre puis on supprime les documents  ajouter dans l'index
        searchIndexer.getDocumentsToIndex( indexWriter );

        //      On ferme l'criture sur l'index
        try
        {
            indexWriter.flush(  );
            indexWriter.optimize(  );
        }
        catch ( IOException ioe )
        {
            throw new AppException( ERREUR_FERMETURE_INDEX, ioe );
        }
        finally
        {
            try
            {
                indexWriter.close(  );
            }
            catch ( Exception ioe )
            {
                throw new AppException( ERREUR_FERMETURE_INDEX, ioe );
            }
        }
    }

    /**
     * Initialise l'index
     */
    public void initIndex(  )
    {
        IndexWriter indexWriter = null;

        try
        {
            if ( IndexReader.isLocked( _strIndex ) )
            {
                IndexReader.unlock( FSDirectory.getDirectory( _strIndex ) );
            }

            // param booleen  true ==> cre l'index s'il n'existe pas encore
            indexWriter = new IndexWriter( _strIndex, _analyzer, true );
        }
        catch ( IOException ioe )
        {
            AppLogService.error( ioe );
            throw new AppException( ERREUR_OUVERTURE_INDEX, ioe );
        }

        AppLogService.debug( "START INIT " + ( _bArchiveService ? "ARCHIVE" : "PROCHAINE SEANCE" ) );

        IODSSearchIndexer searchIndexer = _searchIndexerFactory.getSearchIndexer( _bArchiveService );

        try
        {
            searchIndexer.getDocuments( indexWriter );
        }
        catch ( Exception e )
        {
            AppLogService.error( e );
        }

        try
        {
            indexWriter.optimize(  );
        }
        catch ( IOException ioe )
        {
            AppLogService.error( ioe );
            throw new AppException( ERREUR_FERMETURE_INDEX, ioe );
        }
        finally
        {
            try
            {
                indexWriter.close(  );
            }
            catch ( Exception e )
            {
                AppLogService.error( e );
            }
        }

        AppLogService.debug( "END INIT " );
    }

    /**
     * Rajoute les documents de l'ordre du jour dfinitif de la dernire sance  l'index archive
     * @param nIdAncienneSeance l'ancienne sance
     * @throws AppException AppException
     */
    public void changeSeance( int nIdAncienneSeance ) throws AppException
    {
        IODSSearchIndexer searchIndexer = _searchIndexerFactory.getSearchIndexer( IS_ARCHIVE );

        IndexWriter indexWriter = null;

        try
        {
            if ( IndexReader.isLocked( _strIndex ) )
            {
                IndexReader.unlock( FSDirectory.getDirectory( _strIndex ) );
            }

            indexWriter = new IndexWriter( _strIndex, _analyzer );
        }
        catch ( IOException ioe )
        {
            throw new AppException( ERREUR_OUVERTURE_INDEX, ioe );
        }

        searchIndexer.getDocumentsAArchiver( nIdAncienneSeance, indexWriter );

        try
        {
            indexWriter.optimize(  );
        }
        catch ( IOException ioe )
        {
            throw new AppException( ERREUR_FERMETURE_INDEX, ioe );
        }
        finally
        {
            try
            {
                indexWriter.close(  );
            }
            catch ( Exception e )
            {
                AppLogService.error( e );
            }
        }
    }

    /**
     * Fonction qui force la rinitialisation de l'index en supprimant tous ses documents
     */
    public void viderIndex(  )
    {
        IndexModifier indexModifier = null;

        try
        {
            if ( IndexReader.isLocked( _strIndex ) )
            {
                IndexReader.unlock( FSDirectory.getDirectory( _strIndex ) );
            }

            indexModifier = new IndexModifier( _strIndex, _analyzer, false );

            int nNbDocs = indexModifier.docCount(  );

            for ( int i = 0; i < nNbDocs; i++ )
            {
                indexModifier.deleteDocument( i );
            }

            indexModifier.flush(  );
            indexModifier.optimize(  );
        }
        catch ( IOException ioe )
        {
            AppLogService.error( ioe );
        }
        finally
        {
            if ( indexModifier != null )
            {
                try
                {
                    indexModifier.close(  );
                }
                catch ( IOException ioe )
                {
                    AppLogService.error( ioe );
                }
            }
        }
    }
}
