/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.search;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.ISOLatin1AccentFilter;
import org.apache.lucene.analysis.LowerCaseFilter;
import org.apache.lucene.analysis.StopFilter;
import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.analysis.standard.StandardFilter;
import org.apache.lucene.analysis.standard.StandardTokenizer;

import java.io.Reader;

import java.util.HashSet;
import java.util.Set;


/**
 * Analyzer for French language. Supports an external list of stopwords (words that
 * will not be indexed at all) and an external list of exclusions (word that will
 * not be stemmed, but indexed).
 * A default set of stopwords is used unless an alternative list is specified, the
 * exclusion list is empty by default.
 *
 * @author Patrick Talbot (based on Gerhard Schwarz's work for German)
 * @version $Id: FrenchAnalyzer.java 472959 2006-11-09 16:21:50Z yonik $
 */
public final class SearchAnalyser extends Analyzer
{
    /**
     * Extended list of typical French stopwords.
     */
    public static final String[] FRENCH_STOP_WORDS = 
        {
            "a", "afin", "ai", "ainsi", "aprs", "attendu", "au", "aujourd", "auquel", "aussi", "autre", "autres", "aux",
            "auxquelles", "auxquels", "avait", "avant", "avec", "avoir", "c", "car", "ce", "ceci", "cela", "celle",
            "celles", "celui", "cependant", "certain", "certaine", "certaines", "certains", "ces", "cet", "cette",
            "ceux", "chez", "ci", "combien", "comme", "comment", "concernant", "contre", "d", "dans", "de", "debout",
            "dedans", "dehors", "del", "depuis", "derrire", "des", "dsormais", "desquelles", "desquels", "dessous",
            "dessus", "devant", "devers", "devra", "divers", "diverse", "diverses", "doit", "donc", "dont", "du",
            "duquel", "durant", "ds", "elle", "elles", "en", "entre", "environ", "est", "et", "etc", "etre", "eu",
            "eux", "except", "hormis", "hors", "hlas", "hui", "il", "ils", "j", "je", "jusqu", "jusque", "l", "la",
            "laquelle", "le", "lequel", "les", "lesquelles", "lesquels", "leur", "leurs", "lorsque", "lui", "l", "ma",
            "mais", "malgr", "me", "merci", "mes", "mien", "mienne", "miennes", "miens", "moi", "moins", "mon",
            "moyennant", "mme", "mmes", "n", "ne", "ni", "non", "nos", "notre", "nous", "nanmoins", "ntre", "ntres",
            "on", "ont", "ou", "outre", "o", "par", "parmi", "partant", "pas", "pass", "pendant", "plein", "plus",
            "plusieurs", "pour", "pourquoi", "proche", "prs", "puisque", "qu", "quand", "que", "quel", "quelle",
            "quelles", "quels", "qui", "quoi", "quoique", "revoici", "revoil", "s", "sa", "sans", "sauf", "se", "selon",
            "seront", "ses", "si", "sien", "sienne", "siennes", "siens", "sinon", "soi", "soit", "son", "sont", "sous",
            "suivant", "sur", "ta", "te", "tes", "tien", "tienne", "tiennes", "tiens", "toi", "ton", "tous", "tout",
            "toute", "toutes", "tu", "un", "une", "va", "vers", "voici", "voil", "vos", "votre", "vous", "vu", "vtre",
            "vtres", "y", "", "a", "s", "t", "tre", "",
        };

    /**
     * Contains the stopwords used with the StopFilter.
     */
    private Set _stoptable = new HashSet(  );

    /**
     * Builds an analyzer with the default stop words ({@link #FRENCH_STOP_WORDS}).
     */
    public SearchAnalyser(  )
    {
        _stoptable = StopFilter.makeStopSet( FRENCH_STOP_WORDS );
    }

    /**
     * Builds an analyzer with the given stop words.
     * @param stopwords the list of stop words
     */
    public SearchAnalyser( String[] stopwords )
    {
        _stoptable = StopFilter.makeStopSet( stopwords );
    }

    /**
     * Creates a TokenStream which tokenizes all the text in the provided Reader.
     * @param fieldName the field name
     * @param reader the reader
     *
     * @return A TokenStream build from a StandardTokenizer filtered with
     *         StandardFilter, StopFilter, FrenchStemFilter and LowerCaseFilter
     */
    public TokenStream tokenStream( String fieldName, Reader reader )
    {
        if ( fieldName == null )
        {
            throw new IllegalArgumentException( "fieldName must not be null" );
        }

        if ( reader == null )
        {
            throw new IllegalArgumentException( "reader must not be null" );
        }

        TokenStream result = new StandardTokenizer( reader );
        result = new StandardFilter( result );
        result = new StopFilter( result, _stoptable );
        result = new ISOLatin1AccentFilter( result );
        // Convert to lowercase after stemming!
        result = new LowerCaseFilter( result );

        return result;
    }
}
