/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.utilisateur;

import fr.paris.lutece.plugins.ods.business.utilisateur.UtilisateurHome;
import fr.paris.lutece.plugins.ods.dto.utilisateur.Utilisateur;
import fr.paris.lutece.plugins.ods.utils.commons.DateUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.security.LuteceUser;
import fr.paris.lutece.portal.service.security.SecurityService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 * LuteceUserManager
 */
public abstract class AbstractUtilisateurService implements IUtilisateurService
{
    private static final String MESSAGE_CURRENT_USER_NULL = "ods.front.utilisateur.message.userNull";
    private static final String MARK_URL_LOGOUT = "logout_url";
    private static final String PROPERTY_SYNCHRONIZE_UTILISATEUR = "ods.notification.synchronizeUser";
    private static final String PROPERTY_URL_LOGOUT = "mylutece.url.doLogout";
    @Autowired
    private UtilisateurHome _utilisateurHome;

    /**
     * Constructeur vide
     */
    public AbstractUtilisateurService(  )
    {
    }

    protected abstract void gestionRolesUtilisateur( Map<String, Object> model, HttpServletRequest request );

    /**
     * Effectue les traitements pour la gestion des utilisateurs sur le front
     * @param model la table de hashage qui va tre envoy  la template freemarker
     * @param request la requte HTTP
     * @param plugin le plugin
     */
    public void gestionUtilisateur( Map<String, Object> model, HttpServletRequest request, Plugin plugin )
    {
        boolean isNotPremiereConnexion = gestionConnexion( request, plugin );

        Utilisateur utilisateur = (Utilisateur) request.getSession(  ).getAttribute( OdsMarks.MARK_UTILISATEUR );

        model.put( OdsMarks.MARK_USER_GIVEN_NAME, getGivenName( request ) );
        model.put( MARK_URL_LOGOUT,
            AppPathService.getBaseUrl( request ) + AppPropertiesService.getProperty( PROPERTY_URL_LOGOUT ) );

        if ( ( utilisateur != null ) && isNotPremiereConnexion )
        {
            model.put( OdsMarks.MARK_DERNIERE_CONNEXION, utilisateur.getDerniereConnexion(  ) );
        }
        else if ( !isNotPremiereConnexion )
        {
            request.getSession(  ).setAttribute( OdsMarks.MARK_PREMIERE_CONNEXION, true );
        }

        gestionRolesUtilisateur( model, request );
    }

    /**
     * Retourne le nom de l'utilisateur connect
     * @param request la requete
     * @return le nom de l'utilisateur connect
     */
    public String getGivenName( HttpServletRequest request )
    {
        LuteceUser currentUser = SecurityService.getInstance(  ).getRegisteredUser( request );

        if ( currentUser != null )
        {
            return currentUser.getUserInfo( LuteceUser.NAME_GIVEN ) + " " +
            currentUser.getUserInfo( LuteceUser.NAME_FAMILY );
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Gere la derniere connexion d'un utilisateur en base et en session
     * @param request la requpte Http
     * @param plugin la Plugin
     * @return TRUE si l'utilisateur existe, FALSE sinon
     */
    private boolean gestionConnexion( HttpServletRequest request, Plugin plugin )
    {
        Utilisateur utilisateur = getUtilisateurCourant( request, plugin );
        boolean utilisateurExists = false;

        if ( ( utilisateur != null ) && ( utilisateur.getDerniereConnexion(  ) != null ) )
        {
            utilisateurExists = true;
        }

        if ( ( utilisateur != null ) && ( utilisateur.getDerniereIdSession(  ) != null ) &&
                ( request.getSession(  ) != null ) &&
                !utilisateur.getDerniereIdSession(  ).equals( request.getSession(  ).getId(  ) ) )
        {
            request.getSession(  ).setAttribute( OdsMarks.MARK_UTILISATEUR, utilisateur );

            Utilisateur utilisateurMaj = new Utilisateur( utilisateur );
            utilisateurMaj.setDerniereConnexion( DateUtils.getCurrentDate(  ) );
            utilisateurMaj.setDerniereIdSession( request.getSession(  ).getId(  ) );

            _utilisateurHome.update( utilisateurMaj, plugin );
        }

        return utilisateurExists;
    }

    /**
     * Rcupre l'utilisateur courant et le cre si il n'existe pas.
     * @param request la requte Http
     * @param plugin la Plugin
     * @return l'utilisateur courant
     */
    public Utilisateur getUtilisateurCourant( HttpServletRequest request, Plugin plugin )
    {
        String strIdUser = "-1";
        LuteceUser currentUser = SecurityService.getInstance(  ).getRegisteredUser( request );

        if ( currentUser != null )
        {
            strIdUser = currentUser.getName(  );
        }

        if ( strIdUser.equals( "-1" ) )
        {
            AppLogService.error( MESSAGE_CURRENT_USER_NULL );

            return null;
        }

        Utilisateur utilisateur = _utilisateurHome.findByPrimaryKey( strIdUser, plugin );

        if ( utilisateur == null )
        {
            utilisateur = new Utilisateur(  );

            utilisateur.setIdUtilisateur( currentUser.getName(  ) );
            utilisateur.setNom( currentUser.getUserInfo( LuteceUser.NAME_FAMILY ) );
            utilisateur.setPrenom( currentUser.getUserInfo( LuteceUser.NAME_GIVEN ) );
            utilisateur.setMail( currentUser.getUserInfo( LuteceUser.BUSINESS_INFO_ONLINE_EMAIL ) );
            utilisateur.setMailCopie1( null );
            utilisateur.setMailCopie2( null );
            utilisateur.setDerniereConnexion( null );
            utilisateur.setDerniereIdSession( "0" );

            _utilisateurHome.create( utilisateur, plugin );
        }

        return utilisateur;
    }

    /**
     * retourne la liste des utilisateurs du front
     * @param plugin plugin
     * @return la liste des utilisateurs
     */
    public Collection<Utilisateur> getUserFrontList( Plugin plugin )
    {
        Map<String, Utilisateur> hmapUtilisateurs = _utilisateurHome.findUserListHashmap( plugin );
        Map<String, Utilisateur> hmapUtilisateursRetour = new HashMap<String, Utilisateur>(  );

        Collection<LuteceUser> frontUserList = SecurityService.getInstance(  ).getUsers(  );

        for ( LuteceUser obj : frontUserList )
        {
            Utilisateur user = hmapUtilisateurs.get( obj.getName(  ) );

            if ( user == null )
            {
                user = new Utilisateur(  );

                user.setIdUtilisateur( obj.getName(  ) );
                user.setNom( obj.getUserInfo( LuteceUser.NAME_FAMILY ) );
                user.setPrenom( obj.getUserInfo( LuteceUser.NAME_GIVEN ) );
                user.setMail( obj.getUserInfo( LuteceUser.BUSINESS_INFO_ONLINE_EMAIL ) );
                user.setMailCopie1( null );
                user.setMailCopie2( null );

                String strSynchronizeUtilisateur = AppPropertiesService.getProperty( PROPERTY_SYNCHRONIZE_UTILISATEUR );

                if ( ( strSynchronizeUtilisateur != null ) &&
                        strSynchronizeUtilisateur.equals( OdsConstants.INTEGER_TRUE ) )
                {
                    _utilisateurHome.create( user, plugin );
                }
            }

            user.setRoles( obj.getRoles(  ) );

            hmapUtilisateursRetour.put( user.getIdUtilisateur(  ), user );
        }

        return hmapUtilisateursRetour.values(  );
    }
}
