/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.ui.field;

import fr.paris.lutece.plugins.ods.dto.annotations.Settings;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.apache.commons.beanutils.PropertyUtils;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Map.Entry;


/**
 * Classe abstraite representant le comportemement par defaut des champs
 *
 * @param <T> le type d'element manipule dans le champ
 */
public abstract class AbstractField<T> implements Field
{
    protected static final String ATTRIBUTE_MAXLENGTH = "attribute_maxlength";
    protected static final String ATTRIBUTE_SIZE = "attribute_size";
    protected static final String ATTRIBUTE_REQUIRED = "attribute_required";
    protected static final String ATTRIBUTE_ROWS = "attribute_rows";
    protected static final String ATTRIBUTE_COLS = "attribute_cols";
    protected String _id;
    protected T _value;
    protected boolean _bIsReadonly;
    protected Map<JsEventEnum, String> _jsEventMap = new HashMap<JsEventEnum, String>(  );

    /**
     * Map d'attributs communs transmis  chaque template
     */
    protected Map<String, Object> _attributes = new HashMap<String, Object>(  );

    /**
     * Methode utilitaire chargee d'effectuer l'initialisation des attributs communs d'un champ :
     * <ul>
     * <li>identifiant</li>
     * <li>valeur</li>
     * <li>attributs HTML definis par les annotations Settings</li>
     * </ul>
     *
     * La valeur est renseignee a partir de l'objet (<strong>qui doit</strong> repondre a la norme JavaBean) et en fonction
     * de l'identifiant de ce champ dans l'objet.
     *
     * @param id l'identifiant du champ
     * @param o l'objet permettant de recuperer la valeur
     */
    protected void init( String id, Object o )
    {
        this._id = id;

        try
        {
            this._value = (T) PropertyUtils.getProperty( o, id );

            Method m = PropertyUtils.getReadMethod( PropertyUtils.getPropertyDescriptor( o, id ) );

            if ( m != null )
            {
                Settings settingsAnnotation = m.getAnnotation( Settings.class );

                if ( settingsAnnotation != null )
                {
                    if ( settingsAnnotation.length(  ) != -1 )
                    {
                        _attributes.put( ATTRIBUTE_SIZE, settingsAnnotation.length(  ) );
                    }

                    if ( settingsAnnotation.maxSize(  ) != -1 )
                    {
                        _attributes.put( ATTRIBUTE_MAXLENGTH, settingsAnnotation.maxSize(  ) );
                    }

                    _attributes.put( ATTRIBUTE_REQUIRED, settingsAnnotation.required(  ) );
                    _attributes.put( ATTRIBUTE_COLS, settingsAnnotation.cols(  ) );
                    _attributes.put( ATTRIBUTE_ROWS, settingsAnnotation.rows(  ) );
                }
            }
        }
        catch ( IllegalAccessException e )
        {
            AppLogService.debug( e );
        }
        catch ( InvocationTargetException e )
        {
            AppLogService.debug( e );
        }
        catch ( NoSuchMethodException e )
        {
            AppLogService.debug( e );
        }
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.ui.field.Field#getId()
     */
    public String getId(  )
    {
        return this._id;
    }

    /**
     * Methode utilitaire permettant de fusionner le template html avec un model passe en parametre
     *
     * @param model le model a fusionner avec le template
     * @return le resultat de la fusion dans un fragment HTML
     */
    protected String renderHtml( Map<String, Object> model )
    {
        Template templateAnnotation = this.getClass(  ).getAnnotation( Template.class );

        if ( ( templateAnnotation != null ) && !"".equals( templateAnnotation.templatePath(  ) ) )
        {
            model.put( "event_list", jsEventMapToString(  ) );
            model.put( "readonly", isReadonly(  ) );
            model.putAll( getAttributes(  ) );

            try
            {
                HtmlTemplate template = AppTemplateService.getTemplate( templateAnnotation.templatePath(  ),
                        Locale.FRANCE, model );

                return template.getHtml(  );
            }
            catch ( AppException e )
            {
                AppLogService.error( "Error rendering field : " + getId(  ), e );
                throw e;
            }
        }

        return null;
    }

    /**
     * Methode utilitaire permettant de fusionner le template html avec un model passe en parametre
     *
     * @param model le model a fusionner avec le template
     * @return le resultat de la fusion dans un fragment Javascript
     */
    protected String renderJS( Map<String, Object> model )
    {
        // TODO check JSTemplatePath
        HtmlTemplate template = AppTemplateService.getTemplate( getJSTemplatePath(  ), Locale.FRANCE, model );

        return template.getHtml(  );
    }

    /**
     * Definit le chemin vers le template HTML du javascript
     *
     * @return le chemin vers le template, peut tre nul si aucune fontion javascript n'est associee
     * au champ
     */
    protected String getJSTemplatePath(  )
    {
        return null;
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.ui.field.Field#renderJS()
     */
    public String renderJS(  )
    {
        return null;
    }

    /**
     * Setter pour la map evenement javascript.
     *
     * Permet d'associer a un evenement javascript (onclick, onchange,...) un code javascript.
     * @param jsEventMap
     */
    public void setJsEventMap( Map<JsEventEnum, String> jsEventMap )
    {
        this._jsEventMap = jsEventMap;
    }

    /**
     * Methode utiliser pour la transformation vers le template HTML
     * @return un fragament javascript, ou chaine vide s'il n'y a pas de javascript
     */
    private String jsEventMapToString(  )
    {
        StringBuilder sbResult = new StringBuilder(  );

        for ( Entry<JsEventEnum, String> e : _jsEventMap.entrySet(  ) )
        {
            sbResult.append( " " + e.getKey(  ).getEventName(  ) + "='" + e.getValue(  ) + "'" );
        }

        return sbResult.toString(  );
    }

    @Override
    public boolean equals( Object o )
    {
        if ( ( o == null ) || !( o instanceof Field ) )
        {
            return false;
        }
        else
        {
            return this.getId(  ).equals( ( (Field) o ).getId(  ) );
        }
    }

    @Override
    public int hashCode(  )
    {
        if ( getId(  ) == null )
        {
            return super.hashCode(  );
        }

        return getId(  ).hashCode(  );
    }

    public boolean isReadonly(  )
    {
        return _bIsReadonly;
    }

    public void setReadonly( boolean isReadonly )
    {
        this._bIsReadonly = isReadonly;
    }

    public Map<String, Object> getAttributes(  )
    {
        return _attributes;
    }

    public void setRequired( boolean isRequired )
    {
        _attributes.put( ATTRIBUTE_REQUIRED, isRequired );
    }
}
