/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.ui.field;

import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.ReferenceList;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Classe Factory pour la cration des lments de base d'un formulaire.
 *
 * De manire globale, les mthodes prennent en paramtre un id et un bean pour rcuprer la valeur du formulaire.
 * Ensuite, on peut passer en paramtre un label (ou cl i18n) afin de crer un champ et son label associ.
 */
public abstract class AbstractFieldFactory
{
    private static final HashMap<JsEventEnum, String> applyButtonEventMap = new HashMap<JsEventEnum, String>(  );

    static
    {
        applyButtonEventMap.put( JsEventEnum.ONCLICK, "javascript: doAppliquer();" );
    }

    private static final HashMap<JsEventEnum, String> cancelButtonEventMap = new HashMap<JsEventEnum, String>(  );

    static
    {
        cancelButtonEventMap.put( JsEventEnum.ONCLICK, "javascript: annuler();" );
    }

    private static final String NOTICE_TEMPLATE_PATH = "admin/plugins/ods/ui/notice.html";
    private static final String ARRONDISSEMENTS_TEMPLATE_PATH = "admin/plugins/ods/projetdeliberation/arrondissements.html";
    private static final String COEMETRICES_TEMPLATE_PATH = "admin/plugins/ods/projetdeliberation/directionsCoemetrices.html";
    private static final String CONSTANT_BUTTON = "button";

    /**
     * Creation de champ date sans label
     *
     * @param id
     * @param formBean
     * @return
     */
    public static Field getDateField( String id, Object formBean )
    {
        return new DateField( id, formBean );
    }

    /**
     * Creation de champ date avec label
     *
     * @param id
     * @param formBean
     * @param label
     * @return
     */
    public static Field getDateField( String id, Object formBean, String label )
    {
        return new LabelField( label, getDateField( id, formBean ) );
    }

    /**
     * Association de champs sans label
     *
     * @param f1
     * @param f2
     * @return
     */
    public static Field composeFields( Field f1, Field f2 )
    {
        return new CompositeField( f1, f2 );
    }

    /**
     * Association de champs avec label
     *
     * @param f1
     * @param f2
     * @param label
     * @return
     */
    public static Field composeFields( Field f1, Field f2, String label )
    {
        return new LabelField( label, composeFields( f1, f2 ) );
    }

    /**
     * Creation d'un champ texte sans label
     *
     * @param id
     * @param formBean
     * @return
     */
    public static Field getTextField( String id, Object formBean )
    {
        return new TextField( id, formBean );
    }

    /**
     * Creation d'un champ texte avec label
     *
     * @param id
     * @param formBean
     * @param label
     * @return
     */
    public static Field getTextField( String id, Object formBean, String label )
    {
        return new LabelField( label, getTextField( id, formBean ) );
    }

    /**
     * Affichage d'une simple valeur
     *
     * @param value
     * @param label
     * @return
     */
    public static Field getCaptionField( String value, String label )
    {
        return new LabelField( label, new CaptionField( value ) );
    }

    /**
     * Affichage d'une simple valeur
     *
     * @param value
     * @return
     */
    public static Field getCaptionField( String value )
    {
        return new CaptionField( value );
    }

    /**
     * Affichage d'un champ textarea avec label
     *
     * @param id
     * @param formBean
     * @param label
     * @return
     */
    public static Field getTextArea( String id, Object formBean, String label )
    {
        return new LabelField( label, new TextAreaField( id, formBean ) );
    }

    /**
     * Affichage d'une combo avec label
     *
     * @param id
     * @param formBean
     * @param label
     * @param values
     * @return
     */
    public static Field getComboListField( String id, Object formBean, String label, ReferenceList values )
    {
        return getComboListField( id, formBean, label, values, null );
    }

    /**
     * Affichage d'une checkbox simple avec label
     *
     * @param id
     * @param formBean
     * @param label
     * @return
     */
    public static Field getSimpleCheckbox( String id, Object formBean, String label )
    {
        return new LabelField( label, new SimpleCheckbox( id, formBean ) );
    }

    /**
     * Affichage d'un template freemarker personnalise
     *
     * @param id
     * @param templatePath
     * @param model
     * @param templateJSPath
     * @param jsModel
     * @return
     */
    protected static Field getTemplateField( String id, String templatePath, Map<String, Object> model,
        String templateJSPath, Map<String, Object> jsModel )
    {
        return new TemplateField( id, templatePath, model, templateJSPath, jsModel );
    }

    /**
     * Affichage du formulaire de saisie des arrondissements
     *
     * @param id
     * @param model
     * @param jsModel
     * @return
     */
    public static Field getArrondissementsField( String id, List<?extends Object> selectedValues,
        ReferenceList referenceValues )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );
        model.put( "liste_arrondissements_added", selectedValues );
        model.put( "liste_arrondissements", referenceValues );
        model.put( "id_arrondissement_selected", "-1" );

        return getTemplateField( id, ARRONDISSEMENTS_TEMPLATE_PATH, model, null, null );
    }

    /**
     * Affichage du formulaire de saisie des directions coemetrices
     *
     * @param id
     * @param selectedValues
     * @param referenceValues
     * @return
     */
    public static Field getDirectionsCoemetricesField( String id, List<?extends Object> selectedValues,
        ReferenceList referenceValues )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );
        model.put( "liste_direction_co_emetrice_added", selectedValues );
        model.put( "liste_direction_co_emetrice", referenceValues );
        model.put( "id_direction_co_emetrice_selected", "-1" );

        return getTemplateField( id, COEMETRICES_TEMPLATE_PATH, model, null, null );
    }

    /**
     * Ajout d'evenements javascript
     *
     * @param field
     * @param jsEventMap
     */
    public static void addJsEvent( Field field, HashMap<JsEventEnum, String> jsEventMap )
    {
        // TODO Auto-generated method stub
        ( (AbstractField<?>) field ).setJsEventMap( jsEventMap );
    }

    /**
     * Ajout d'une combo avec label et javascript
     *
     * @param id
     * @param formBean
     * @param label
     * @param values
     * @param jsEventMap
     * @return
     */
    public static Field getComboListField( String id, Object formBean, String label, ReferenceList values,
        Map<JsEventEnum, String> jsEventMap )
    {
        if ( values == null )
        {
            throw new AppException( "Empty values for combo field : " + id );
        }

        ComboListField<Integer> comboListField = new ComboListField<Integer>( id, formBean );
        comboListField.fill( values );

        if ( jsEventMap != null )
        {
            comboListField.setJsEventMap( jsEventMap );
        }

        return new LabelField( label, comboListField );
    }

    /**
     * Creation d'un champ hidden sur valeur numerique
     *
     * @param id
     * @param formBean
     * @return
     */
    public static Field getHiddenIntField( String id, Object formBean )
    {
        // TODO Auto-generated method stub
        return new HiddenField<Integer>( id, formBean );
    }

    /**
     * Creation d'un champ hidden sur chaine de caracteres
     *
     * @param id
     * @param formBean
     * @return
     */
    public static Field getHiddenStringField( String id, Object formBean )
    {
        // TODO Auto-generated method stub
        return new HiddenField<String>( id, formBean );
    }

    /**
     * Creation d'un bouton avec libelle par defaut
     *
     * @param buttonType {@link ButtonTypeEnum}
     * @return
     */
    public static Field getButton( ButtonTypeEnum buttonType )
    {
        return getButton( buttonType, null );
    }

    /**
     * Creation d'un bouton avec specification du label
     *
     * @param buttonType {@link ButtonTypeEnum}
     * @param label cle i18n
     * @return
     */
    public static Field getButton( ButtonTypeEnum buttonType, String label )
    {
        Field result;

        switch ( buttonType )
        {
            case APPLY_BUTTON:
                result = new ButtonField( ( label == null ) ? "ods.pdd.button.appliquer" : label, CONSTANT_BUTTON );
                ( (ButtonField) result ).setJsEventMap( applyButtonEventMap );

                break;

            case CANCEL_BUTTON:
                result = new ButtonField( ( label == null ) ? "ods.pdd.button.annuler" : label, CONSTANT_BUTTON );
                ( (ButtonField) result ).setJsEventMap( cancelButtonEventMap );

                break;

            case SUBMIT_BUTTON:
                result = new ButtonField( ( label == null ) ? "ods.pdd.button.enregistrer" : label, "submit" );

                break;

            default:
                result = null;

                break;
        }

        return result;
    }

    /**
     * Creation d'un bouton avec specification du label
     *
     * @param buttonType {@link ButtonTypeEnum}
     * @param label cle i18n
     * @return
     */
    public static Field getButton( ButtonTypeEnum buttonType, String label, String strButtonType )
    {
        Field result;

        switch ( buttonType )
        {
            case APPLY_BUTTON:
                result = new ButtonField( ( label == null ) ? "ods.pdd.button.appliquer" : label,
                        ( strButtonType == null ) ? CONSTANT_BUTTON : strButtonType );
                ( (ButtonField) result ).setJsEventMap( applyButtonEventMap );

                break;

            case CANCEL_BUTTON:
                result = new ButtonField( ( label == null ) ? "ods.pdd.button.annuler" : label,
                        ( strButtonType == null ) ? CONSTANT_BUTTON : strButtonType );
                ( (ButtonField) result ).setJsEventMap( cancelButtonEventMap );

                break;

            case SUBMIT_BUTTON:
                result = new ButtonField( ( label == null ) ? "ods.pdd.button.enregistrer" : label,
                        ( strButtonType == null ) ? "submit" : strButtonType );

                break;

            default:
                result = null;

                break;
        }

        return result;
    }

    /**
     * Creation d'un champ upload avec label
     *
     * @param id l'id du champ de type Fichier dans le bean
     * @param formBean
     * @param label
     * @param idFileItem l'id du champ de type FileItem dans le bean (pour la gestion de l'upload)
     * @return
     */
    public static Field getFichierField( String id, Object formBean, String label, String idFileItem )
    {
        return new LabelField( label, new FichierField( id, formBean, idFileItem ) );
    }

    /**
     * Positionne les champs dont les identifiants sont dans la liste readonlyFields en readonly
     *
     * @param result
     * @param readonlyFields
     */
    public static void setReadonlyFields( List<Field> result, List<String> readonlyFields )
    {
        for ( final String readonlyField : readonlyFields )
        {
            Field f = new AbstractField<Void>(  )
                {
                    public String render(  )
                    {
                        // TODO Auto-generated method stub
                        return null;
                    }

                    @Override
                    public String getId(  )
                    {
                        return readonlyField;
                    }
                };

            int pos = result.indexOf( f );

            if ( pos > -1 )
            {
                result.get( pos ).setReadonly( true );
            }
        }
    }

    /**
     * Creation d'un element de type highlightedBox.
     *
     * @param strStartId identifiant du debut de la box
     * @param strEndId identifiant de la fin de la box
     * @param strTitle cl i18n du titre  afficher dans la box
     * @param fields champs  inclure
     * @return une liste de Field
     */
    public static List<Field> getBox( String strStartId, String strEndId, String strTitle, Field... fields )
    {
        ArrayList<Field> result = new ArrayList<Field>(  );
        result.add( new StartHighlightedBox( strStartId, strTitle ) );
        result.addAll( Arrays.asList( fields ) );
        result.add( new EndHighlightedBox( strEndId ) );

        return result;
    }

    /**
     * Creation d'un layout horizontal.
     *
     * Cet lment est utilis par exemple pour placer les boutons d'un formulaire sur une mme ligne
     *
     * @param id identifiant du layout
     * @param fields champs contenus dans le layout
     * @return
     */
    public static Field getHorizontalLayout( String id, Field... fields )
    {
        return new HorizontalLayout( id, fields );
    }

    /**
     * Creation d'un layout horizontal.
     *
     * Cet lment est utilis par exemple pour placer les boutons d'un formulaire sur une mme ligne
     *
     * @param id identifiant du layout
     * @param label
     * @param fields champs contenus dans le layout
     * @return
     */
    public static Field getHorizontalLayout( String id, String label, Field... fields )
    {
        return new LabelField( label, new HorizontalLayout( id, fields ) );
    }

    /**
     * Rcupration de l'extrait notice prsent sur certains formulaires
     *
     * @param id identifiant du champ
     * @return
     */
    public static Field getNotice( String id )
    {
        return getNotice( id, NOTICE_TEMPLATE_PATH );
    }

    /**
     * Rcupration d'une notice correpondant au template pass en paramtre.
     *
     * @param id identifiant du champ
     * @return
     */
    public static Field getNotice( String id, String strTemplatePath )
    {
        return getTemplateField( id, strTemplatePath, null, null, null );
    }
}
