/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.utils.business;

import fr.paris.lutece.plugins.ods.dto.horodatage.ITrace;
import fr.paris.lutece.plugins.ods.dto.horodatage.TraceNotification;
import fr.paris.lutece.plugins.ods.dto.horodatage.TracePublication;
import fr.paris.lutece.plugins.ods.utils.commons.ShaUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;

import org.apache.commons.codec.binary.Base64;

import java.util.List;


/**
 *
 * HorodatageUtils
 *
 */
public final class HorodatageUtils
{
    private static final String MESSAGE_CSV_ID_TRACE = "id_trace";
    private static final String MESSAGE_CSV_DATE_TRACE = "date_trace";
    private static final String MESSAGE_CSV_SIGNATURE = "signature";
    private static final String MESSAGE_CSV_FAMILLE_DOC = "famille_doc";
    private static final String MESSAGE_CSV_ID_DB_ODS = "id_db_ods";
    private static final String MESSAGE_CSV_REFERENCE = "reference";
    private static final String MESSAGE_CSV_DATE_PUBLICATION = "date_publication";
    private static final String MESSAGE_CSV_INTITULE = "intitule";
    private static final String MESSAGE_CSV_VERSION = "version";
    private static final String MESSAGE_CSV_ACTION = "action";
    private static final String MESSAGE_CSV_OBJET = "objet";
    private static final String MESSAGE_CSV_CONTENU = "contenu";
    private static final String MESSAGE_CSV_ID_USER = "id_utilisateur";
    private static final String MESSAGE_CSV_DATE_ENVOI = "date_envoi";
    private static final String MESSAGE_CSV_DESTINATAIRES = "destinataires";

    /**
     * Constructeur vide
     */
    private HorodatageUtils(  )
    {
        // rien
    }

    /**
     * Retourne la signature numriques SHA224 du contenu
     *
     * @param contenu
     *            Contenu de la notification
     * @return la signature
     */
    public static String getSignature( String contenu )
    {
        String signature = null;

        if ( contenu != null )
        {
            signature = ShaUtils.toSha224String( contenu );

            if ( signature == null )
            {
                signature = "error";
            }
        }

        return signature;
    }

    /**
     * Ecrit les donnes commune  toutes les traces dans le fichier  tlcharger
     *
     * @param listField List<String>
     * @param trace Trace
     */
    public static void writeTrace( List<String> listField, ITrace trace )
    {
        listField.add( Integer.toString( trace.getIdTrace(  ) ) );
        listField.add( trace.getDateTrace(  ).toString(  ) );
        listField.add( trace.getSignatures(  ) );
    }

    /**
     * Ecrit les donnes spcifique aux traces de type Notification dans le fichier  tlcharger
     *
     * @param listField List<String> la liste correspondant  la ligne du CSV
     * @param trace Trace
     */
    public static void writeTraceNotification( List<String> listField, TraceNotification trace )
    {
        listField.add( trace.getObjet(  ) );
        listField.add( new String( Base64.encodeBase64( trace.getContenu(  ).getBytes(  ) ) ) );
        listField.add( trace.getIdUtilisateur(  ) );
        listField.add( trace.getDateEnvoi(  ).toString(  ) );
        listField.add( trace.getDestinataires(  ) );
    }

    /**
     * Ecrit les donnes spcifique aux traces de type Publication dans le fichier  tlcharger
     *
     * @param listField List<String> la liste correspondant  la ligne du CSV
     * @param trace Trace
     */
    public static void writeTracePublication( List<String> listField, TracePublication trace )
    {
        listField.add( trace.getFamilleDoc(  ) );
        listField.add( Integer.toString( trace.getIdDbOds(  ) ) );

        if ( trace.getReference(  ) != null )
        {
            listField.add( trace.getReference(  ) );
        }
        else
        {
            listField.add( OdsConstants.CONSTANTE_CHAINE_VIDE );
        }

        listField.add( trace.getDatePublication(  ).toString(  ) );

        if ( trace.getIntitule(  ) != null )
        {
            listField.add( trace.getIntitule(  ) );
        }
        else
        {
            listField.add( OdsConstants.CONSTANTE_CHAINE_VIDE );
        }

        listField.add( Integer.toString( trace.getVersion(  ) ) );
        listField.add( trace.getAction(  ).toString(  ) );
    }

    /**
     * Ecrit les titres des colonnes pour un type de trace
     *
     * @param listToCSVWriter List<String[]>
     * @param typeHorodatage Type PUBLICATION ou NOTIFICATION
     */
    public static void writeColonnes( List<String[]> listToCSVWriter, String typeHorodatage )
    {
        if ( typeHorodatage.equals( OdsConstants.CONSTANTE_PUBLICATION ) )
        {
            String[] header = 
                {
                    MESSAGE_CSV_ID_TRACE, MESSAGE_CSV_DATE_TRACE, MESSAGE_CSV_SIGNATURE, MESSAGE_CSV_FAMILLE_DOC,
                    MESSAGE_CSV_ID_DB_ODS, MESSAGE_CSV_REFERENCE, MESSAGE_CSV_DATE_PUBLICATION, MESSAGE_CSV_INTITULE,
                    MESSAGE_CSV_VERSION, MESSAGE_CSV_ACTION,
                };
            listToCSVWriter.add( header );
        }
        else if ( typeHorodatage.equals( OdsConstants.CONSTANTE_NOTIFICATION ) )
        {
            String[] header = 
                {
                    MESSAGE_CSV_ID_TRACE, MESSAGE_CSV_DATE_TRACE, MESSAGE_CSV_SIGNATURE, MESSAGE_CSV_OBJET,
                    MESSAGE_CSV_CONTENU, MESSAGE_CSV_ID_USER, MESSAGE_CSV_DATE_ENVOI, MESSAGE_CSV_DESTINATAIRES,
                };
            listToCSVWriter.add( header );
        }
    }
}
