/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.utils.business;

import fr.paris.lutece.plugins.ods.business.ordredujour.generated.entete.Entete;
import fr.paris.lutece.plugins.ods.business.ordredujour.generated.pieddepage.PiedDePage;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.util.xml.XmlUtil;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import java.util.HashMap;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;


/**
 * ModeleUtils
 * Classe contenant des mthodes utilitaires pour les gestion des modles
 */
public final class ModeleUtils
{
    public static final String TAG_ENTETE = "entete";
    public static final String TAG_PIED_DE_PAGE = "piedDePage";
    public static final int XML_PARAMETER_CODE_DATE_SEANCE = 0;
    public static final int XML_PARAMETER_CODE_DATE_JOUR = 1;
    public static final int XML_PARAMETER_CODE_FORMATION_CONSEIL = 2;
    public static final int XML_PARAMETER_CODE_DATE_TRANSMISSION_CTRL = 3;
    public static final int XML_PARAMETER_CODE_NOM_SECRETAIRE_GENERAL = 4;
    public static final int XML_PARAMETER_CODE_DATE_SEANCE_AVEC_JOUR = 5;
    public static final int XML_PARAMETER_CODE_HEURE_SEANCE = 6;
    private static final String PROPERTY_CODE_DATE_SEANCE = "ods.xmlParameter.codeDateSeance";
    private static final String PROPERTY_CODE_DATE_SEANCE_AVEC_JOUR = "ods.xmlParameter.codeDateSeanceAvecJour";
    private static final String PROPERTY_CODE_DATE_JOUR = "ods.xmlParameter.codeDateJour";
    private static final String PROPERTY_CODE_HEURE_SEANCE = "ods.xmlParameter.codeHeureSeance";
    private static final String PROPERTY_CODE_DATE_TRANSMISSIION_CTRL = "ods.xmlParameter.codeDateTransmissionCtrl";
    private static final String PROPERTY_CODE_FORMATION_CONSEIL = "ods.xmlParameter.codeFormationConseil";
    private static final String PROPERTY_CODE_NOM_SECRETAIRE_GENERAL = "ods.xmlParameter.codeNomSecretaireGeneral";
    private static final String CONSTANTE_PACKAGE_ENTETE_GENERATION = "fr.paris.lutece.plugins.ods.business.ordredujour.generated.entete";
    private static final String CONSTANTE_PACKAGE_PIEDDEPAGE_GENERATION = "fr.paris.lutece.plugins.ods.business.ordredujour.generated.pieddepage";
    private static final String TAG_ENTREE_ENTETE = "entreeEntete";
    private static final String TAG_STYLE = "style";
    private static final String TAG_TEXTE_LIBRE = "texteLibre";
    private static final String TAG_ENTREE_PIED_DE_PAGE = "entreePiedDePage";

    /**
     * Constructeur vide
     */
    private ModeleUtils(  )
    {
    }

    /**
     * Retourne l'objet en-tte correspondant au flux pass en paramtre
     * @param  fEntete l'entete au format xml
     *  @return l'en-tte
     */
    public static Entete getEntete( byte[] fEntete )
    {
        Entete entete = null;

        if ( fEntete != null )
        {
            try
            {
                entete = analyzeEnteteXML( fEntete );
            }
            catch ( JAXBException e )
            {
                AppLogService.error( "getEntete - " + e.getClass(  ) + " : " + e.getMessage(  ) );
            }
            catch ( Exception e )
            {
                AppLogService.error( "getEntete - " + e.getClass(  ) + " : " + e.getMessage(  ) );
            }
        }
        else
        {
            entete = new Entete(  );
        }

        return entete;
    }

    /**
     * retourne l'objet pied de page correspondant au flux pass en paramtre
     * @param  fPiedPage le pied de page au format xml
     * @return le pied de page
     */
    public static PiedDePage getPiedDePage( byte[] fPiedPage )
    {
        PiedDePage piedDePage = null;

        if ( fPiedPage != null )
        {
            try
            {
                piedDePage = analyzePiedDePageXML( fPiedPage );
            }
            catch ( JAXBException e )
            {
                AppLogService.error( e );
            }

            catch ( Exception e )
            {
                AppLogService.error( e );
            }
        }
        else
        {
            piedDePage = new PiedDePage(  );
        }

        return piedDePage;
    }

    /**
     * Cette mthode permet de valider le XMl pass en paramtre  l'aide
     * d'un fichier XSD et de retourner l'objet PiedDePage correspondant.
     * Si le fichier est valide alors l'objet PiedDePage est instanci et retourn
     *
     * @param  byteArrayXML  Flux xml qui va etre analis
     * @return PiedDePage Objet qui contient toutes les donnes du fichier XML PiedDePage
     * @throws JAXBException JAXBException
     * @throws IOException IOException
     */
    private static PiedDePage analyzePiedDePageXML( byte[] byteArrayXML )
        throws JAXBException, IOException
    {
        /* create a JAXBContext capable of handling classes generated into */
        JAXBContext jc = JAXBContext.newInstance( CONSTANTE_PACKAGE_PIEDDEPAGE_GENERATION );

        /* create an Unmarshaller */
        Unmarshaller u = jc.createUnmarshaller(  );
        ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream( byteArrayXML );

        /* unmarshal an invalid po instance document into a tree of Java
         * content objects composed of classes from the primer.po package */
        PiedDePage piedDePage = (PiedDePage) u.unmarshal( byteArrayInputStream );
        byteArrayInputStream.close(  );

        return piedDePage;
    }

    /**
     * Cette mthode permet de valider le XMl pass en paramtre  l'aide
     * d'un fichier XSD et de retourner l'objet entete coorespondant.
     * Si le fichier est valide alors l'objet Affaire est instanci et retourn
     *
     * @param  byteArrayXML  Flux xml qui va etre analis
     * @return Affaire Objet qui contient toutes les donnes du fichier XML
     * @throws JAXBException JAXBException
     * @throws IOException IOException
     */
    private static Entete analyzeEnteteXML( byte[] byteArrayXML )
        throws JAXBException, IOException
    {
        /* create a JAXBContext capable of handling classes generated into */
        JAXBContext jc = JAXBContext.newInstance( CONSTANTE_PACKAGE_ENTETE_GENERATION );

        /* create an Unmarshaller */
        Unmarshaller u = jc.createUnmarshaller(  );
        ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream( byteArrayXML );

        /* unmarshal an invalid po instance document into a tree of Java
         * content objects composed of classes from the primer.po package */
        Entete entete = (Entete) u.unmarshal( byteArrayInputStream );
        byteArrayInputStream.close(  );

        return entete;
    }

    /**
     * @param entete  l'entete
     * @param xmlParams le HashMap contenant les valeurs correspondant aux balises  remplacer
     * @return retourne le fragment de code xml representant l'entete a partir de l'objet entete
     */
    public static String getXmlEntete( Entete entete, HashMap<Integer, Object> xmlParams )
    {
        StringBuffer buffer = new StringBuffer(  );
        XmlUtil.beginElement( buffer, TAG_ENTETE );

        for ( Entete.EntreeEntete entree : entete.getEntreeEntete(  ) )
        {
            XmlUtil.beginElement( buffer, TAG_ENTREE_ENTETE );
            OdsUtils.addElement( buffer, TAG_STYLE, entree.getStyle(  ) );
            OdsUtils.addElement( buffer, TAG_TEXTE_LIBRE, getXmlEntreeValue( entree.getTexteLibre(  ), xmlParams ) );
            XmlUtil.endElement( buffer, TAG_ENTREE_ENTETE );
        }

        XmlUtil.endElement( buffer, TAG_ENTETE );

        return buffer.toString(  );
    }

    /**
     * @param strEntreeTexteLibre le texte  remplacer
     * @param xmlParams le HashMap contenant les valeurs correspondant aux balises  remplacer
     * @return retourne le fragment de code xml representant une entree a partir de l'objet entree
     */
    private static String getXmlEntreeValue( String strEntreeTexteLibre, HashMap<Integer, Object> xmlParams )
    {
        String strEntreeTexte = strEntreeTexteLibre;

        if ( strEntreeTexte != null )
        {
            if ( ( strEntreeTexte.indexOf( AppPropertiesService.getProperty( PROPERTY_CODE_NOM_SECRETAIRE_GENERAL ) ) >= 0 ) )
            {
                strEntreeTexte = replaceWithXmlParams( strEntreeTexte, xmlParams,
                        XML_PARAMETER_CODE_NOM_SECRETAIRE_GENERAL );
            }

            if ( ( strEntreeTexte.indexOf( AppPropertiesService.getProperty( PROPERTY_CODE_DATE_SEANCE ) ) >= 0 ) )
            {
                strEntreeTexte = replaceWithXmlParams( strEntreeTexte, xmlParams, XML_PARAMETER_CODE_DATE_SEANCE );
            }

            if ( ( strEntreeTexte.indexOf( AppPropertiesService.getProperty( PROPERTY_CODE_DATE_JOUR ) ) >= 0 ) )
            {
                strEntreeTexte = replaceWithXmlParams( strEntreeTexte, xmlParams, XML_PARAMETER_CODE_DATE_JOUR );
            }

            if ( ( strEntreeTexte.indexOf( AppPropertiesService.getProperty( PROPERTY_CODE_FORMATION_CONSEIL ) ) >= 0 ) )
            {
                strEntreeTexte = replaceWithXmlParams( strEntreeTexte, xmlParams, XML_PARAMETER_CODE_FORMATION_CONSEIL );
            }

            if ( ( strEntreeTexte.indexOf( AppPropertiesService.getProperty( PROPERTY_CODE_DATE_TRANSMISSIION_CTRL ) ) >= 0 ) )
            {
                strEntreeTexte = replaceWithXmlParams( strEntreeTexte, xmlParams,
                        XML_PARAMETER_CODE_DATE_TRANSMISSION_CTRL );
            }

            if ( ( strEntreeTexte.indexOf( AppPropertiesService.getProperty( PROPERTY_CODE_HEURE_SEANCE ) ) >= 0 ) )
            {
                strEntreeTexte = replaceWithXmlParams( strEntreeTexte, xmlParams, XML_PARAMETER_CODE_HEURE_SEANCE );
            }

            if ( ( strEntreeTexte.indexOf( AppPropertiesService.getProperty( PROPERTY_CODE_DATE_SEANCE_AVEC_JOUR ) ) >= 0 ) )
            {
                strEntreeTexte = replaceWithXmlParams( strEntreeTexte, xmlParams,
                        XML_PARAMETER_CODE_DATE_SEANCE_AVEC_JOUR );
            }
        }

        return strEntreeTexte;
    }

    /**
     *
     * @param piedDePage PiedDePage
     * @param xmlParams le HashMap contenant les valeurs correspondant aux balises  remplacer
     * @return retourne le fragment de code xml representant le pied de page  a partir de l'objet piedDePage
     */
    public static String getXmlPiedDePage( PiedDePage piedDePage, HashMap<Integer, Object> xmlParams )
    {
        StringBuffer buffer = new StringBuffer(  );
        XmlUtil.beginElement( buffer, TAG_PIED_DE_PAGE );

        for ( PiedDePage.EntreePiedDePage entree : piedDePage.getEntreePiedDePage(  ) )
        {
            XmlUtil.beginElement( buffer, TAG_ENTREE_PIED_DE_PAGE );
            OdsUtils.addElement( buffer, TAG_STYLE, entree.getStyle(  ) );
            OdsUtils.addElement( buffer, TAG_TEXTE_LIBRE, getXmlEntreeValue( entree.getTexteLibre(  ), xmlParams ) );
            XmlUtil.endElement( buffer, TAG_ENTREE_PIED_DE_PAGE );
        }

        XmlUtil.endElement( buffer, TAG_PIED_DE_PAGE );

        return buffer.toString(  );
    }

    /**
     * Remplace dans la chaine de caractre 'strEntree',
     * toutes les constantes reprsentants la date du conseil
     * par la date du conseil de la sance passe en paramtre
     * @param strEntreeTexteLibre le texte  remplacer
     * @param xmlParams le HashMap contenant les valeurs correspondant aux balises  remplacer
     * @param nCodeParameter le code du paramtre  rempalcer
     * @return la chaine de caractre modifie
     */
    private static String replaceWithXmlParams( String strEntreeTexteLibre, HashMap<Integer, Object> xmlParams,
        int nCodeParameter )
    {
        String strXmlCodeParameter = null;
        String strParameterValue = null;
        String strEntreeTexte = strEntreeTexteLibre;

        switch ( nCodeParameter )
        {
            case XML_PARAMETER_CODE_NOM_SECRETAIRE_GENERAL:
                strXmlCodeParameter = AppPropertiesService.getProperty( PROPERTY_CODE_NOM_SECRETAIRE_GENERAL );
                strParameterValue = (String) xmlParams.get( XML_PARAMETER_CODE_NOM_SECRETAIRE_GENERAL );

                break;

            case XML_PARAMETER_CODE_DATE_JOUR:
                strXmlCodeParameter = AppPropertiesService.getProperty( PROPERTY_CODE_DATE_JOUR );
                strParameterValue = (String) xmlParams.get( XML_PARAMETER_CODE_DATE_JOUR );

                break;

            case XML_PARAMETER_CODE_DATE_SEANCE:
                strXmlCodeParameter = AppPropertiesService.getProperty( PROPERTY_CODE_DATE_SEANCE );
                strParameterValue = (String) xmlParams.get( XML_PARAMETER_CODE_DATE_SEANCE );

                break;

            case XML_PARAMETER_CODE_DATE_SEANCE_AVEC_JOUR:
                strXmlCodeParameter = AppPropertiesService.getProperty( PROPERTY_CODE_DATE_SEANCE_AVEC_JOUR );
                strParameterValue = (String) xmlParams.get( XML_PARAMETER_CODE_DATE_SEANCE_AVEC_JOUR );

                break;

            case XML_PARAMETER_CODE_HEURE_SEANCE:
                strXmlCodeParameter = AppPropertiesService.getProperty( PROPERTY_CODE_HEURE_SEANCE );
                strParameterValue = (String) xmlParams.get( XML_PARAMETER_CODE_HEURE_SEANCE );

                break;

            case XML_PARAMETER_CODE_DATE_TRANSMISSION_CTRL:
                strXmlCodeParameter = AppPropertiesService.getProperty( PROPERTY_CODE_DATE_TRANSMISSIION_CTRL );
                strParameterValue = (String) xmlParams.get( XML_PARAMETER_CODE_DATE_TRANSMISSION_CTRL );

                break;

            case XML_PARAMETER_CODE_FORMATION_CONSEIL:
                strXmlCodeParameter = AppPropertiesService.getProperty( PROPERTY_CODE_FORMATION_CONSEIL );
                strParameterValue = (String) xmlParams.get( XML_PARAMETER_CODE_FORMATION_CONSEIL );

                break;

            default:
                break;
        }

        if ( ( strXmlCodeParameter != null ) && ( strParameterValue != null ) )
        {
            strEntreeTexte = strEntreeTexteLibre.replace( strXmlCodeParameter, strParameterValue );
        }

        return strEntreeTexte;
    }
}
