/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.utils.business;

import de.schlichtherle.io.File;
import de.schlichtherle.io.FileOutputStream;

import de.schlichtherle.util.zip.ZipOutputStream;

import fr.paris.lutece.plugins.ods.business.ordredujour.generated.entete.Entete;
import fr.paris.lutece.plugins.ods.business.ordredujour.generated.pieddepage.PiedDePage;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJour;
import fr.paris.lutece.plugins.ods.utils.commons.FileUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import java.sql.Timestamp;

import java.text.SimpleDateFormat;

import java.util.Locale;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;


/**
 * OrdreDuJourUtils
 * Classe contenant des mthodes utilitaires pour les gestion des ordres du jour
 */

/**
 * @author OrdreDuJourUtils
 *
 */
public final class OrdreDuJourUtils
{
    public static final String TAG_ENTETE = "entete";
    public static final String TAG_PIED_DE_PAGE = "piedDePage";
    public static final String CONSTANTE_ETOILE = "=*=*=*";
    public static final String PROPERTY_SEANCE = "ods.all.label.seance";
    public static final String PROPERTY_DU = "ods.all.label.from";
    public static final String PROPERTY_AU = "ods.all.label.to";
    public static final int CONSTANTE_TYPE_DATE_SEANCE = 0;
    public static final int CONSTANTE_TYPE_DATE_SEANCE_REU1 = 1;
    public static final int CONSTANTE_TYPE_DATE_SEANCE_REU2 = 2;
    public static final int CONSTANTE_TYPE_DATE_SEANCE_REU3 = 3;
    public static final int CONSTANTE_TYPE_DATE_SEANCE_REU = 3;
    public static final String PROPERTY_ERE = "ods.all.label.ere";
    public static final String PROPERTY_EME = "ods.all.label.eme";
    public static final String PROPERTY_CODE_DATE_SEANCE = "ods.xmlParameter.codeDateSeance";
    public static final String TAG_ENTREE_ENTETE = "entreeEntete";
    public static final String TAG_STYLE = "style";
    public static final String TAG_TEXTE_LIBRE = "texteLibre";
    public static final String TAG_ENTREE_PIED_DE_PAGE = "entreePiedDePage";
    public static final String TAG_LISTE_ENTREES = "listeEntrees";
    public static final String TAG_EST_SAUVEGARDE = "estSauvegarde";
    public static final String TAG_EST_PUBLIE = "estPublie";
    public static final String TAG_DATE_PUBLICATION = "datePublication";
    public static final String TAG_MODE_CLASSEMENT = "modeClassement";
    public static final String TAG_INTITULE = "intitule";
    public static final String CONSTANTE_RAPPORTEUR_FEMININ_SINGULIER = "ods.odj.modificationOdj.label.rapporteure";
    public static final String CONSTANTE_RAPPORTEUR_FEMININ_PLURIEL = "ods.odj.modificationOdj.label.rapporteures";
    public static final String CONSTANTE_RAPPORTEUR_SINGULIER = "ods.odj.modificationOdj.label.rapporteur";
    public static final String CONSTANTE_RAPPORTEUR_PLURIEL = "ods.odj.modificationOdj.label.rapporteurs";
    private static final String CONSTANTE_PACKAGE_ENTETE_GENERATION = "fr.paris.lutece.plugins.ods.business.ordredujour.generated.entete";
    private static final String CONSTANTE_PACKAGE_PIEDDEPAGE_GENERATION = "fr.paris.lutece.plugins.ods.business.ordredujour.generated.pieddepage";
    private static final String ZIP_DIR_PATH = "ods.genereODJ.zip.directory";
    private static final String ZIP_FILE_NAME = "ods.genereODJ.zip.name";
    private static final String ZIP_FILE_EXTENSION = "ods.genereODJ.zip.extension";

    /**
     * Constructeur vide
     */
    private OrdreDuJourUtils(  )
    {
        // rien
    }

    /**
     * Retourne l'en-tte de l'ordre du jour pass en paramtre
     * @param odj l'ordre du jour
     * @param plugin le plugin
     * @return l'en-tte de l'ordre du jour pass en paramtre
     */
    public static Entete getEntete( IOrdreDuJour odj, Plugin plugin )
    {
        Entete entete = null;

        if ( odj.getXmlEntete(  ) != null )
        {
            try
            {
                entete = analyzeEnteteXML( odj.getXmlEntete(  ).getBytes( OdsConstants.UTF8 ) );
            }
            catch ( JAXBException e )
            {
                AppLogService.error( "getEntete - " + e.getClass(  ) + " : " + e.getMessage(  ) );
            }
            catch ( Exception e )
            {
                AppLogService.error( "getEntete - " + e.getClass(  ) + " : " + e.getMessage(  ) );
            }
        }
        else
        {
            entete = new Entete(  );
        }

        return entete;
    }

    /**
     * retourne le pied de page  de l'ordre du jour pass en parametre
     * @param odj l'ordre du jour
     * @param plugin le plugin
     * @return le pied de page
     */
    public static PiedDePage getPiedDePage( IOrdreDuJour odj, Plugin plugin )
    {
        PiedDePage piedDePage = null;

        if ( odj.getXmlPiedDePage(  ) != null )
        {
            try
            {
                piedDePage = analyzePiedDePageXML( odj.getXmlPiedDePage(  ).getBytes( OdsConstants.UTF8 ) );
            }
            catch ( JAXBException e )
            {
                AppLogService.error( e );
            }

            catch ( Exception e )
            {
                AppLogService.error( e );
            }
        }
        else
        {
            piedDePage = new PiedDePage(  );
        }

        return piedDePage;
    }

    /**
     * Cette mthode permet de valider le XMl pass en paramtre  l'aide
     * d'un fichier XSD et de retourner l'objet PiedDePage correspondant.
     * Si le fichier est valide alors l'objet PiedDePage est instanci et retourn
     *
     * @param  byteArrayXML  Flux xml qui va etre analis
     * @return PiedDePage Objet qui contient toutes les donnes du fichier XML PiedDePage
     * @throws JAXBException JAXBException
     * @throws IOException IOException
     */
    public static PiedDePage analyzePiedDePageXML( byte[] byteArrayXML )
        throws JAXBException, IOException
    {
        /* create a JAXBContext capable of handling classes generated into */
        JAXBContext jc = JAXBContext.newInstance( CONSTANTE_PACKAGE_PIEDDEPAGE_GENERATION );

        /* create an Unmarshaller */
        Unmarshaller u = jc.createUnmarshaller(  );
        ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream( byteArrayXML );

        /* unmarshal an invalid po instance document into a tree of Java
         * content objects composed of classes from the primer.po package */
        PiedDePage piedDePage = (PiedDePage) u.unmarshal( byteArrayInputStream );
        byteArrayInputStream.close(  );

        return piedDePage;
    }

    /**
     * Cette mthode permet de valider le XMl pass en paramtre  l'aide
     * d'un fichier XSD et de retourner l'objet entete coorespondant.
     * Si le fichier est valide alors l'objet Affaire est instanci et retourn
     *
     * @param  byteArrayXML  Flux xml qui va etre analis
     * @return Affaire Objet qui contient toutes les donnes du fichier XML
     * @throws JAXBException JAXBException
     * @throws IOException IOException
     */
    public static Entete analyzeEnteteXML( byte[] byteArrayXML )
        throws JAXBException, IOException
    {
        /* create a JAXBContext capable of handling classes generated into */
        JAXBContext jc = JAXBContext.newInstance( CONSTANTE_PACKAGE_ENTETE_GENERATION );

        /* create an Unmarshaller */
        Unmarshaller u = jc.createUnmarshaller(  );
        ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream( byteArrayXML );

        /* unmarshal an invalid po instance document into a tree of Java
         * content objects composed of classes from the primer.po package */
        Entete entete = (Entete) u.unmarshal( byteArrayInputStream );
        byteArrayInputStream.close(  );

        return entete;
    }

    /**
     * Permet d'ajouter tous les ODJ gnrs en Word dans un Zip
     * @param dossier Rpertoire contenant les fichiers  archiver
     * @return L'archive zip
     */
    public static java.io.File createZipODJ( String dossier, String strPrefix )
    {
        String strPathZip = AppPropertiesService.getProperty( ZIP_DIR_PATH );
        String strFileName = AppPropertiesService.getProperty( ZIP_FILE_NAME );
        String strFileExtension = AppPropertiesService.getProperty( ZIP_FILE_EXTENSION );

        SimpleDateFormat dateFormat = new SimpleDateFormat( "yyyyMMdd_HHmmss", Locale.FRENCH );
        String strDate = dateFormat.format( new Timestamp( System.currentTimeMillis(  ) ) );

        String zipName = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( !strPrefix.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            zipName = strPathZip + OdsConstants.PATH_SEPARATOR + strFileName + OdsConstants.CONSTANTE_UNDERSCORE +
                strPrefix + OdsConstants.CONSTANTE_UNDERSCORE + strDate + OdsConstants.CONSTANTE_POINT +
                strFileExtension;
        }
        else
        {
            zipName = strPathZip + OdsConstants.PATH_SEPARATOR + strFileName + OdsConstants.CONSTANTE_UNDERSCORE +
                strDate + OdsConstants.CONSTANTE_POINT + strFileExtension;
        }

        File tmpDir = new File( dossier );

        try
        {
            FileOutputStream fout = new FileOutputStream( zipName );
            ZipOutputStream zout = new ZipOutputStream( fout, "IBM437" );

            FileUtils.compressProfondeur( (File[]) tmpDir.listFiles(  ), zout, dossier );

            zout.finish(  );
            zout.close(  );

            fout.flush(  );
            fout.close(  );

            return new java.io.File( zipName );
        }
        catch ( Exception e )
        {
            AppLogService.error( "Creation Zip : " + e.getMessage(  ) );
        }

        return null;
    }
}
